/* Generated by Cython 0.29.30 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "name": "MorphologicalDisambiguation.AutoDisambiguator",
        "sources": [
            "MorphologicalDisambiguation/AutoDisambiguator.pyx"
        ]
    },
    "module_name": "MorphologicalDisambiguation.AutoDisambiguator"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_30"
#define CYTHON_HEX_VERSION 0x001D1EF0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC (PYPY_VERSION_HEX >= 0x07030900)
  #endif
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 0
  #elif !defined(CYTHON_FAST_THREAD_STATE)
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL (PY_VERSION_HEX < 0x030A0000)
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 0
  #elif !defined(CYTHON_USE_EXC_INFO_STACK)
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if PY_VERSION_HEX >= 0x030B00A1
    static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *kwds=NULL, *argcount=NULL, *posonlyargcount=NULL, *kwonlyargcount=NULL;
        PyObject *nlocals=NULL, *stacksize=NULL, *flags=NULL, *replace=NULL, *call_result=NULL, *empty=NULL;
        const char *fn_cstr=NULL;
        const char *name_cstr=NULL;
        PyCodeObject* co=NULL;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        if (!(kwds=PyDict_New())) goto end;
        if (!(argcount=PyLong_FromLong(a))) goto end;
        if (PyDict_SetItemString(kwds, "co_argcount", argcount) != 0) goto end;
        if (!(posonlyargcount=PyLong_FromLong(0))) goto end;
        if (PyDict_SetItemString(kwds, "co_posonlyargcount", posonlyargcount) != 0) goto end;
        if (!(kwonlyargcount=PyLong_FromLong(k))) goto end;
        if (PyDict_SetItemString(kwds, "co_kwonlyargcount", kwonlyargcount) != 0) goto end;
        if (!(nlocals=PyLong_FromLong(l))) goto end;
        if (PyDict_SetItemString(kwds, "co_nlocals", nlocals) != 0) goto end;
        if (!(stacksize=PyLong_FromLong(s))) goto end;
        if (PyDict_SetItemString(kwds, "co_stacksize", stacksize) != 0) goto end;
        if (!(flags=PyLong_FromLong(f))) goto end;
        if (PyDict_SetItemString(kwds, "co_flags", flags) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_code", code) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_consts", c) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_names", n) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_varnames", v) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_freevars", fv) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_cellvars", cell) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_linetable", lnos) != 0) goto end;
        if (!(fn_cstr=PyUnicode_AsUTF8AndSize(fn, NULL))) goto end;
        if (!(name_cstr=PyUnicode_AsUTF8AndSize(name, NULL))) goto end;
        if (!(co = PyCode_NewEmpty(fn_cstr, name_cstr, fline))) goto end;
        if (!(replace = PyObject_GetAttrString((PyObject*)co, "replace"))) goto cleanup_code_too;
        if (!(empty = PyTuple_New(0))) goto cleanup_code_too; // unfortunately __pyx_empty_tuple isn't available here
        if (!(call_result = PyObject_Call(replace, empty, kwds))) goto cleanup_code_too;
        Py_XDECREF((PyObject*)co);
        co = (PyCodeObject*)call_result;
        call_result = NULL;
        if (0) {
            cleanup_code_too:
            Py_XDECREF((PyObject*)co);
            co = NULL;
        }
        end:
        Py_XDECREF(kwds);
        Py_XDECREF(argcount);
        Py_XDECREF(posonlyargcount);
        Py_XDECREF(kwonlyargcount);
        Py_XDECREF(nlocals);
        Py_XDECREF(stacksize);
        Py_XDECREF(replace);
        Py_XDECREF(call_result);
        Py_XDECREF(empty);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return co;
    }
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if defined(PyUnicode_IS_READY)
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #else
  #define __Pyx_PyUnicode_READY(op)       (0)
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #endif
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #if !defined(_USE_MATH_DEFINES)
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__MorphologicalDisambiguation__AutoDisambiguator
#define __PYX_HAVE_API__MorphologicalDisambiguation__AutoDisambiguator
/* Early includes */
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "MorphologicalDisambiguation/AutoDisambiguator.pyx",
  "stringsource",
  "venv/lib/python3.7/site-packages/Dictionary/Word.pxd",
  "venv/lib/python3.7/site-packages/Corpus/Sentence.pxd",
  "venv/lib/python3.7/site-packages/Dictionary/Trie/TrieNode.pxd",
  "venv/lib/python3.7/site-packages/Dictionary/TxtWord.pxd",
  "venv/lib/python3.7/site-packages/Dictionary/Trie/Trie.pxd",
  "venv/lib/python3.7/site-packages/Dictionary/Dictionary.pxd",
  "venv/lib/python3.7/site-packages/Dictionary/TxtDictionary.pxd",
  "venv/lib/python3.7/site-packages/DataStructure/Cache/LRUCache.pxd",
  "venv/lib/python3.7/site-packages/MorphologicalAnalysis/InflectionalGroup.pxd",
  "venv/lib/python3.7/site-packages/MorphologicalAnalysis/MorphologicalParse.pxd",
  "venv/lib/python3.7/site-packages/MorphologicalAnalysis/State.pxd",
  "venv/lib/python3.7/site-packages/MorphologicalAnalysis/FsmParse.pxd",
  "venv/lib/python3.7/site-packages/MorphologicalAnalysis/FiniteStateMachine.pxd",
  "venv/lib/python3.7/site-packages/MorphologicalAnalysis/MetamorphicParse.pxd",
  "venv/lib/python3.7/site-packages/MorphologicalAnalysis/FsmMorphologicalAnalyzer.pxd",
  "venv/lib/python3.7/site-packages/DataStructure/CounterHashMap.pxd",
  "venv/lib/python3.7/site-packages/MorphologicalAnalysis/FsmParseList.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_10Dictionary_4Word_Word;
struct __pyx_obj_6Corpus_8Sentence_Sentence;
struct __pyx_obj_10Dictionary_4Trie_8TrieNode_TrieNode;
struct __pyx_obj_10Dictionary_7TxtWord_TxtWord;
struct __pyx_obj_10Dictionary_4Trie_4Trie_Trie;
struct __pyx_obj_10Dictionary_10Dictionary_Dictionary;
struct __pyx_obj_10Dictionary_13TxtDictionary_TxtDictionary;
struct __pyx_obj_13DataStructure_5Cache_8LRUCache_LRUCache;
struct __pyx_obj_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup;
struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse;
struct __pyx_obj_21MorphologicalAnalysis_5State_State;
struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse;
struct __pyx_obj_21MorphologicalAnalysis_18FiniteStateMachine_FiniteStateMachine;
struct __pyx_obj_21MorphologicalAnalysis_16MetamorphicParse_MetamorphicParse;
struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer;
struct __pyx_obj_13DataStructure_14CounterHashMap_CounterHashMap;
struct __pyx_obj_21MorphologicalAnalysis_12FsmParseList_FsmParseList;
struct __pyx_obj_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator;
struct __pyx_opt_args_10Dictionary_4Trie_8TrieNode_8TrieNode_addWord;

/* "Dictionary/Trie/TrieNode.pxd":9
 *     cdef set __words
 * 
 *     cpdef addWord(self, str word, Word root, int index=*)             # <<<<<<<<<<<<<<
 *     cpdef TrieNode getChild(self, char ch)
 *     cpdef set getWords(self)
 */
struct __pyx_opt_args_10Dictionary_4Trie_8TrieNode_8TrieNode_addWord {
  int __pyx_n;
  int index;
};
struct __pyx_opt_args_21MorphologicalAnalysis_18FiniteStateMachine_18FiniteStateMachine_addTransition;

/* "MorphologicalAnalysis/FiniteStateMachine.pxd":12
 *     cpdef list getStates(self)
 *     cpdef State getState(self, str name)
 *     cpdef addTransition(self, State fromState, State toState, str _with, str withName, toPos=*)             # <<<<<<<<<<<<<<
 *     cpdef list getTransitions(self, State state)
 */
struct __pyx_opt_args_21MorphologicalAnalysis_18FiniteStateMachine_18FiniteStateMachine_addTransition {
  int __pyx_n;
  PyObject *toPos;
};
struct __pyx_opt_args_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_24FsmMorphologicalAnalyzer_morphologicalAnalysisRoot;

/* "MorphologicalAnalysis/FsmMorphologicalAnalyzer.pxd":34
 *     cpdef bint __parseExists(self, list fsmParse, str surfaceForm)
 *     cpdef list __parseWord(self, list fsmParse, maxLengthOrSurfaceForm)
 *     cpdef list morphologicalAnalysisRoot(self, str surfaceForm, TxtWord root, state=*)             # <<<<<<<<<<<<<<
 *     cpdef list generateAllParses(self, TxtWord root, int maxLength)
 *     cpdef str replaceRootWord(self, FsmParse parse, TxtWord newRoot)
 */
struct __pyx_opt_args_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_24FsmMorphologicalAnalyzer_morphologicalAnalysisRoot {
  int __pyx_n;
  PyObject *state;
};
struct __pyx_opt_args_13DataStructure_14CounterHashMap_14CounterHashMap_max;

/* "DataStructure/CounterHashMap.pxd":7
 *     cpdef int count(self, object key)
 *     cpdef int sumOfCounts(self)
 *     cpdef object max(self, double threshold = *)             # <<<<<<<<<<<<<<
 *     cpdef add(self, CounterHashMap toBeAdded)
 *     cpdef list topN(self, int N)
 */
struct __pyx_opt_args_13DataStructure_14CounterHashMap_14CounterHashMap_max {
  int __pyx_n;
  double threshold;
};

/* "Dictionary/Word.pxd":1
 * cdef class Word:             # <<<<<<<<<<<<<<
 * 
 *     cdef str name
 */
struct __pyx_obj_10Dictionary_4Word_Word {
  PyObject_HEAD
  struct __pyx_vtabstruct_10Dictionary_4Word_Word *__pyx_vtab;
  PyObject *name;
};


/* "Corpus/Sentence.pxd":4
 * 
 * 
 * cdef class Sentence:             # <<<<<<<<<<<<<<
 * 
 *     cdef list words
 */
struct __pyx_obj_6Corpus_8Sentence_Sentence {
  PyObject_HEAD
  struct __pyx_vtabstruct_6Corpus_8Sentence_Sentence *__pyx_vtab;
  PyObject *words;
};


/* "Dictionary/Trie/TrieNode.pxd":4
 * 
 * 
 * cdef class TrieNode:             # <<<<<<<<<<<<<<
 * 
 *     cdef dict __children
 */
struct __pyx_obj_10Dictionary_4Trie_8TrieNode_TrieNode {
  PyObject_HEAD
  struct __pyx_vtabstruct_10Dictionary_4Trie_8TrieNode_TrieNode *__pyx_vtab;
  PyObject *__pyx___children;
  PyObject *__pyx___words;
};


/* "Dictionary/TxtWord.pxd":4
 * 
 * 
 * cdef class TxtWord(Word):             # <<<<<<<<<<<<<<
 * 
 *     cdef list __flags
 */
struct __pyx_obj_10Dictionary_7TxtWord_TxtWord {
  struct __pyx_obj_10Dictionary_4Word_Word __pyx_base;
  PyObject *__pyx___flags;
};


/* "Dictionary/Trie/Trie.pxd":6
 * 
 * 
 * cdef class Trie:             # <<<<<<<<<<<<<<
 * 
 *     cdef TrieNode __rootNode
 */
struct __pyx_obj_10Dictionary_4Trie_4Trie_Trie {
  PyObject_HEAD
  struct __pyx_vtabstruct_10Dictionary_4Trie_4Trie_Trie *__pyx_vtab;
  struct __pyx_obj_10Dictionary_4Trie_8TrieNode_TrieNode *__pyx___rootNode;
};


/* "Dictionary/Dictionary.pxd":4
 * 
 * 
 * cdef class Dictionary:             # <<<<<<<<<<<<<<
 * 
 *     cdef list words
 */
struct __pyx_obj_10Dictionary_10Dictionary_Dictionary {
  PyObject_HEAD
  struct __pyx_vtabstruct_10Dictionary_10Dictionary_Dictionary *__pyx_vtab;
  PyObject *words;
  PyObject *filename;
  PyObject *comparator;
};


/* "Dictionary/TxtDictionary.pxd":7
 * 
 * 
 * cdef class TxtDictionary(Dictionary):             # <<<<<<<<<<<<<<
 * 
 *     cdef dict __misspelledWords
 */
struct __pyx_obj_10Dictionary_13TxtDictionary_TxtDictionary {
  struct __pyx_obj_10Dictionary_10Dictionary_Dictionary __pyx_base;
  PyObject *__pyx___misspelledWords;
};


/* "DataStructure/Cache/LRUCache.pxd":1
 * cdef class LRUCache(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef int __cacheSize
 */
struct __pyx_obj_13DataStructure_5Cache_8LRUCache_LRUCache {
  PyObject_HEAD
  struct __pyx_vtabstruct_13DataStructure_5Cache_8LRUCache_LRUCache *__pyx_vtab;
  int __pyx___cacheSize;
  PyObject *__pyx___map;
};


/* "MorphologicalAnalysis/InflectionalGroup.pxd":1
 * cdef class InflectionalGroup:             # <<<<<<<<<<<<<<
 * 
 *     cdef list __IG
 */
struct __pyx_obj_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup {
  PyObject_HEAD
  struct __pyx_vtabstruct_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup *__pyx_vtab;
  PyObject *__pyx___IG;
};


/* "MorphologicalAnalysis/MorphologicalParse.pxd":7
 * 
 * 
 * cdef class MorphologicalParse:             # <<<<<<<<<<<<<<
 * 
 *     cdef list inflectionalGroups
 */
struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse {
  PyObject_HEAD
  struct __pyx_vtabstruct_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *__pyx_vtab;
  PyObject *inflectionalGroups;
  struct __pyx_obj_10Dictionary_4Word_Word *root;
};


/* "MorphologicalAnalysis/State.pxd":1
 * cdef class State:             # <<<<<<<<<<<<<<
 * 
 *     cdef bint __startState, __endState
 */
struct __pyx_obj_21MorphologicalAnalysis_5State_State {
  PyObject_HEAD
  struct __pyx_vtabstruct_21MorphologicalAnalysis_5State_State *__pyx_vtab;
  int __pyx___startState;
  int __pyx___endState;
  PyObject *__pyx___name;
  PyObject *__pyx___pos;
};


/* "MorphologicalAnalysis/FsmParse.pxd":5
 * 
 * 
 * cdef class FsmParse(MorphologicalParse):             # <<<<<<<<<<<<<<
 * 
 *     cdef list __suffixList
 */
struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse {
  struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse __pyx_base;
  PyObject *__pyx___suffixList;
  PyObject *__pyx___formList;
  PyObject *__pyx___transitionList;
  PyObject *__pyx___withList;
  PyObject *__pyx___initialPos;
  PyObject *__pyx___pos;
  PyObject *__pyx___form;
  PyObject *__pyx___verbAgreement;
  PyObject *__pyx___possesiveAgreement;
};


/* "MorphologicalAnalysis/FiniteStateMachine.pxd":4
 * 
 * 
 * cdef class FiniteStateMachine:             # <<<<<<<<<<<<<<
 * 
 *     cdef list __states
 */
struct __pyx_obj_21MorphologicalAnalysis_18FiniteStateMachine_FiniteStateMachine {
  PyObject_HEAD
  struct __pyx_vtabstruct_21MorphologicalAnalysis_18FiniteStateMachine_FiniteStateMachine *__pyx_vtab;
  PyObject *__pyx___states;
  PyObject *__pyx___transitions;
};


/* "MorphologicalAnalysis/MetamorphicParse.pxd":6
 * 
 * 
 * cdef class MetamorphicParse:             # <<<<<<<<<<<<<<
 * 
 *     cdef list __metaMorphemeList
 */
struct __pyx_obj_21MorphologicalAnalysis_16MetamorphicParse_MetamorphicParse {
  PyObject_HEAD
  struct __pyx_vtabstruct_21MorphologicalAnalysis_16MetamorphicParse_MetamorphicParse *__pyx_vtab;
  PyObject *__pyx___metaMorphemeList;
  struct __pyx_obj_10Dictionary_4Word_Word *__pyx___root;
};


/* "MorphologicalAnalysis/FsmMorphologicalAnalyzer.pxd":13
 * 
 * 
 * cdef class FsmMorphologicalAnalyzer:             # <<<<<<<<<<<<<<
 * 
 *     cdef Trie __dictionaryTrie
 */
struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer {
  PyObject_HEAD
  struct __pyx_vtabstruct_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *__pyx_vtab;
  struct __pyx_obj_10Dictionary_4Trie_4Trie_Trie *__pyx___dictionaryTrie;
  struct __pyx_obj_21MorphologicalAnalysis_18FiniteStateMachine_FiniteStateMachine *__pyx___finiteStateMachine;
  struct __pyx_obj_10Dictionary_13TxtDictionary_TxtDictionary *__pyx___dictionary;
  struct __pyx_obj_13DataStructure_5Cache_8LRUCache_LRUCache *__pyx___cache;
  PyObject *__pyx___mostUsedPatterns;
  PyObject *__pyx___parsedSurfaceForms;
};


/* "DataStructure/CounterHashMap.pxd":1
 * cdef class CounterHashMap(dict):             # <<<<<<<<<<<<<<
 * 
 *     cpdef put(self, object key)
 */
struct __pyx_obj_13DataStructure_14CounterHashMap_CounterHashMap {
  PyDictObject __pyx_base;
  struct __pyx_vtabstruct_13DataStructure_14CounterHashMap_CounterHashMap *__pyx_vtab;
};


/* "MorphologicalAnalysis/FsmParseList.pxd":4
 * from MorphologicalAnalysis.FsmParse cimport FsmParse
 * 
 * cdef class FsmParseList:             # <<<<<<<<<<<<<<
 * 
 *     cdef list __fsmParses
 */
struct __pyx_obj_21MorphologicalAnalysis_12FsmParseList_FsmParseList {
  PyObject_HEAD
  struct __pyx_vtabstruct_21MorphologicalAnalysis_12FsmParseList_FsmParseList *__pyx_vtab;
  PyObject *__pyx___fsmParses;
};


/* "MorphologicalDisambiguation/AutoDisambiguator.pxd":5
 * 
 * 
 * cdef class AutoDisambiguator:             # <<<<<<<<<<<<<<
 * 
 *     cdef FsmMorphologicalAnalyzer morphologicalAnalyzer
 */
struct __pyx_obj_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator {
  PyObject_HEAD
  struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *morphologicalAnalyzer;
};



/* "Dictionary/Word.pxd":1
 * cdef class Word:             # <<<<<<<<<<<<<<
 * 
 *     cdef str name
 */

struct __pyx_vtabstruct_10Dictionary_4Word_Word {
  int (*charCount)(struct __pyx_obj_10Dictionary_4Word_Word *, int __pyx_skip_dispatch);
  PyObject *(*getName)(struct __pyx_obj_10Dictionary_4Word_Word *, int __pyx_skip_dispatch);
  PyObject *(*setName)(struct __pyx_obj_10Dictionary_4Word_Word *, PyObject *, int __pyx_skip_dispatch);
  int (*isPunctuation)(struct __pyx_obj_10Dictionary_4Word_Word *, int __pyx_skip_dispatch);
  PyObject *(*toCharacters)(struct __pyx_obj_10Dictionary_4Word_Word *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_10Dictionary_4Word_Word *__pyx_vtabptr_10Dictionary_4Word_Word;


/* "Corpus/Sentence.pxd":4
 * 
 * 
 * cdef class Sentence:             # <<<<<<<<<<<<<<
 * 
 *     cdef list words
 */

struct __pyx_vtabstruct_6Corpus_8Sentence_Sentence {
  struct __pyx_obj_10Dictionary_4Word_Word *(*getWord)(struct __pyx_obj_6Corpus_8Sentence_Sentence *, int, int __pyx_skip_dispatch);
  PyObject *(*getWords)(struct __pyx_obj_6Corpus_8Sentence_Sentence *, int __pyx_skip_dispatch);
  PyObject *(*getStrings)(struct __pyx_obj_6Corpus_8Sentence_Sentence *, int __pyx_skip_dispatch);
  int (*getIndex)(struct __pyx_obj_6Corpus_8Sentence_Sentence *, struct __pyx_obj_10Dictionary_4Word_Word *, int __pyx_skip_dispatch);
  int (*wordCount)(struct __pyx_obj_6Corpus_8Sentence_Sentence *, int __pyx_skip_dispatch);
  PyObject *(*addWord)(struct __pyx_obj_6Corpus_8Sentence_Sentence *, struct __pyx_obj_10Dictionary_4Word_Word *, int __pyx_skip_dispatch);
  int (*charCount)(struct __pyx_obj_6Corpus_8Sentence_Sentence *, int __pyx_skip_dispatch);
  PyObject *(*insertWord)(struct __pyx_obj_6Corpus_8Sentence_Sentence *, int, struct __pyx_obj_10Dictionary_4Word_Word *, int __pyx_skip_dispatch);
  PyObject *(*replaceWord)(struct __pyx_obj_6Corpus_8Sentence_Sentence *, int, struct __pyx_obj_10Dictionary_4Word_Word *, int __pyx_skip_dispatch);
  int (*safeIndex)(struct __pyx_obj_6Corpus_8Sentence_Sentence *, int, int __pyx_skip_dispatch);
  PyObject *(*toString)(struct __pyx_obj_6Corpus_8Sentence_Sentence *, int __pyx_skip_dispatch);
  PyObject *(*writeToFile)(struct __pyx_obj_6Corpus_8Sentence_Sentence *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6Corpus_8Sentence_Sentence *__pyx_vtabptr_6Corpus_8Sentence_Sentence;


/* "Dictionary/Trie/TrieNode.pxd":4
 * 
 * 
 * cdef class TrieNode:             # <<<<<<<<<<<<<<
 * 
 *     cdef dict __children
 */

struct __pyx_vtabstruct_10Dictionary_4Trie_8TrieNode_TrieNode {
  PyObject *(*addWord)(struct __pyx_obj_10Dictionary_4Trie_8TrieNode_TrieNode *, PyObject *, struct __pyx_obj_10Dictionary_4Word_Word *, int __pyx_skip_dispatch, struct __pyx_opt_args_10Dictionary_4Trie_8TrieNode_8TrieNode_addWord *__pyx_optional_args);
  struct __pyx_obj_10Dictionary_4Trie_8TrieNode_TrieNode *(*getChild)(struct __pyx_obj_10Dictionary_4Trie_8TrieNode_TrieNode *, char, int __pyx_skip_dispatch);
  PyObject *(*getWords)(struct __pyx_obj_10Dictionary_4Trie_8TrieNode_TrieNode *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_10Dictionary_4Trie_8TrieNode_TrieNode *__pyx_vtabptr_10Dictionary_4Trie_8TrieNode_TrieNode;


/* "Dictionary/TxtWord.pxd":4
 * 
 * 
 * cdef class TxtWord(Word):             # <<<<<<<<<<<<<<
 * 
 *     cdef list __flags
 */

struct __pyx_vtabstruct_10Dictionary_7TxtWord_TxtWord {
  struct __pyx_vtabstruct_10Dictionary_4Word_Word __pyx_base;
  PyObject *(*addFlag)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*removeFlag)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*verbType)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*samePos)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isNominal)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isPassive)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isAbbreviation)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isInterjection)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isDuplicate)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isHeader)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isAdjective)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isPureAdjective)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isPronoun)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isQuestion)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isVerb)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isPortmanteau)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isDeterminer)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isConjunction)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isAdverb)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isPostP)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isPortmanteauEndingWithSI)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isPortmanteauFacedVowelEllipsis)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isPortmanteauFacedSoftening)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isSuffix)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isProperNoun)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isPlural)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isNumeral)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isReal)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isFraction)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isTime)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isDate)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isPercent)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isRange)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isOrdinal)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*notObeysVowelHarmonyDuringAgglutination)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*obeysAndNotObeysVowelHarmonyDuringAgglutination)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*rootSoftenDuringSuffixation)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*rootSoftenAndNotSoftenDuringSuffixation)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*verbSoftenDuringSuffixation)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*nounSoftenDuringSuffixation)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*endingKChangesIntoG)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*isExceptional)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*duplicatesDuringSuffixation)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*duplicatesAndNotDuplicatesDuringSuffixation)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*lastIdropsDuringSuffixation)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*lastIDropsAndNotDropDuringSuffixation)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*takesRelativeSuffixKi)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*takesRelativeSuffixKu)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*consonantSMayInsertedDuringPossesiveSuffixation)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*lastIdropsDuringPassiveSuffixation)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*vowelAChangesToIDuringYSuffixation)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*vowelEChangesToIDuringYSuffixation)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*takesSuffixIRAsAorist)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*takesSuffixDIRAsFactitive)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*showsSuRegularities)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*containsFlag)(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_10Dictionary_7TxtWord_TxtWord *__pyx_vtabptr_10Dictionary_7TxtWord_TxtWord;


/* "Dictionary/Trie/Trie.pxd":6
 * 
 * 
 * cdef class Trie:             # <<<<<<<<<<<<<<
 * 
 *     cdef TrieNode __rootNode
 */

struct __pyx_vtabstruct_10Dictionary_4Trie_4Trie_Trie {
  PyObject *(*addWord)(struct __pyx_obj_10Dictionary_4Trie_4Trie_Trie *, PyObject *, struct __pyx_obj_10Dictionary_4Word_Word *, int __pyx_skip_dispatch);
  PyObject *(*getWordsWithPrefix)(struct __pyx_obj_10Dictionary_4Trie_4Trie_Trie *, PyObject *, int __pyx_skip_dispatch);
  struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *(*getCompundWordStartingWith)(struct __pyx_obj_10Dictionary_4Trie_4Trie_Trie *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_10Dictionary_4Trie_4Trie_Trie *__pyx_vtabptr_10Dictionary_4Trie_4Trie_Trie;


/* "Dictionary/Dictionary.pxd":4
 * 
 * 
 * cdef class Dictionary:             # <<<<<<<<<<<<<<
 * 
 *     cdef list words
 */

struct __pyx_vtabstruct_10Dictionary_10Dictionary_Dictionary {
  struct __pyx_obj_10Dictionary_4Word_Word *(*getWord)(struct __pyx_obj_10Dictionary_10Dictionary_Dictionary *, PyObject *, int __pyx_skip_dispatch);
  int (*getWordIndex)(struct __pyx_obj_10Dictionary_10Dictionary_Dictionary *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*removeWord)(struct __pyx_obj_10Dictionary_10Dictionary_Dictionary *, PyObject *, int __pyx_skip_dispatch);
  int (*size)(struct __pyx_obj_10Dictionary_10Dictionary_Dictionary *, int __pyx_skip_dispatch);
  struct __pyx_obj_10Dictionary_4Word_Word *(*getWordWithIndex)(struct __pyx_obj_10Dictionary_10Dictionary_Dictionary *, int, int __pyx_skip_dispatch);
  int (*longestWordSize)(struct __pyx_obj_10Dictionary_10Dictionary_Dictionary *, int __pyx_skip_dispatch);
  int (*__pyx___getPosition)(struct __pyx_obj_10Dictionary_10Dictionary_Dictionary *, struct __pyx_obj_10Dictionary_4Word_Word *, int __pyx_skip_dispatch);
  int (*getWordStartingWith)(struct __pyx_obj_10Dictionary_10Dictionary_Dictionary *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_10Dictionary_10Dictionary_Dictionary *__pyx_vtabptr_10Dictionary_10Dictionary_Dictionary;


/* "Dictionary/TxtDictionary.pxd":7
 * 
 * 
 * cdef class TxtDictionary(Dictionary):             # <<<<<<<<<<<<<<
 * 
 *     cdef dict __misspelledWords
 */

struct __pyx_vtabstruct_10Dictionary_13TxtDictionary_TxtDictionary {
  struct __pyx_vtabstruct_10Dictionary_10Dictionary_Dictionary __pyx_base;
  PyObject *(*addNumber)(struct __pyx_obj_10Dictionary_13TxtDictionary_TxtDictionary *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*addRealNumber)(struct __pyx_obj_10Dictionary_13TxtDictionary_TxtDictionary *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*addFraction)(struct __pyx_obj_10Dictionary_13TxtDictionary_TxtDictionary *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*addTime)(struct __pyx_obj_10Dictionary_13TxtDictionary_TxtDictionary *, PyObject *, int __pyx_skip_dispatch);
  int (*addProperNoun)(struct __pyx_obj_10Dictionary_13TxtDictionary_TxtDictionary *, PyObject *, int __pyx_skip_dispatch);
  int (*addNoun)(struct __pyx_obj_10Dictionary_13TxtDictionary_TxtDictionary *, PyObject *, int __pyx_skip_dispatch);
  int (*addVerb)(struct __pyx_obj_10Dictionary_13TxtDictionary_TxtDictionary *, PyObject *, int __pyx_skip_dispatch);
  int (*addAdjective)(struct __pyx_obj_10Dictionary_13TxtDictionary_TxtDictionary *, PyObject *, int __pyx_skip_dispatch);
  int (*addAdverb)(struct __pyx_obj_10Dictionary_13TxtDictionary_TxtDictionary *, PyObject *, int __pyx_skip_dispatch);
  int (*addPronoun)(struct __pyx_obj_10Dictionary_13TxtDictionary_TxtDictionary *, PyObject *, int __pyx_skip_dispatch);
  int (*addWithFlag)(struct __pyx_obj_10Dictionary_13TxtDictionary_TxtDictionary *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*mergeDictionary)(struct __pyx_obj_10Dictionary_13TxtDictionary_TxtDictionary *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*__pyx___loadFromText)(struct __pyx_obj_10Dictionary_13TxtDictionary_TxtDictionary *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*__pyx___loadMisspelledWords)(struct __pyx_obj_10Dictionary_13TxtDictionary_TxtDictionary *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*getCorrectForm)(struct __pyx_obj_10Dictionary_13TxtDictionary_TxtDictionary *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*saveAsTxt)(struct __pyx_obj_10Dictionary_13TxtDictionary_TxtDictionary *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*__pyx___addWordWhenRootSoften)(struct __pyx_obj_10Dictionary_13TxtDictionary_TxtDictionary *, struct __pyx_obj_10Dictionary_4Trie_4Trie_Trie *, Py_UNICODE, PyObject *, struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  struct __pyx_obj_10Dictionary_4Trie_4Trie_Trie *(*prepareTrie)(struct __pyx_obj_10Dictionary_13TxtDictionary_TxtDictionary *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_10Dictionary_13TxtDictionary_TxtDictionary *__pyx_vtabptr_10Dictionary_13TxtDictionary_TxtDictionary;


/* "DataStructure/Cache/LRUCache.pxd":1
 * cdef class LRUCache(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef int __cacheSize
 */

struct __pyx_vtabstruct_13DataStructure_5Cache_8LRUCache_LRUCache {
  int (*contains)(struct __pyx_obj_13DataStructure_5Cache_8LRUCache_LRUCache *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*get)(struct __pyx_obj_13DataStructure_5Cache_8LRUCache_LRUCache *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*add)(struct __pyx_obj_13DataStructure_5Cache_8LRUCache_LRUCache *, PyObject *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_13DataStructure_5Cache_8LRUCache_LRUCache *__pyx_vtabptr_13DataStructure_5Cache_8LRUCache_LRUCache;


/* "MorphologicalAnalysis/InflectionalGroup.pxd":1
 * cdef class InflectionalGroup:             # <<<<<<<<<<<<<<
 * 
 *     cdef list __IG
 */

struct __pyx_vtabstruct_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup {
  PyObject *(*getTag)(struct __pyx_obj_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup *, int, int __pyx_skip_dispatch);
  int (*size)(struct __pyx_obj_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup *, int __pyx_skip_dispatch);
  PyObject *(*containsCase)(struct __pyx_obj_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup *, int __pyx_skip_dispatch);
  int (*containsPlural)(struct __pyx_obj_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup *, int __pyx_skip_dispatch);
  int (*containsTag)(struct __pyx_obj_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup *, PyObject *, int __pyx_skip_dispatch);
  int (*containsPossessive)(struct __pyx_obj_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup *__pyx_vtabptr_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup;


/* "MorphologicalAnalysis/MorphologicalParse.pxd":7
 * 
 * 
 * cdef class MorphologicalParse:             # <<<<<<<<<<<<<<
 * 
 *     cdef list inflectionalGroups
 */

struct __pyx_vtabstruct_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse {
  struct __pyx_obj_10Dictionary_4Word_Word *(*getWord)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  PyObject *(*getTransitionList)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  PyObject *(*getInflectionalGroupString)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int, int __pyx_skip_dispatch);
  struct __pyx_obj_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup *(*getInflectionalGroup)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int, int __pyx_skip_dispatch);
  struct __pyx_obj_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup *(*getLastInflectionalGroup)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  PyObject *(*getTag)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int, int __pyx_skip_dispatch);
  int (*tagSize)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  int (*size)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  struct __pyx_obj_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup *(*firstInflectionalGroup)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  struct __pyx_obj_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup *(*lastInflectionalGroup)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  struct __pyx_obj_10Dictionary_4Word_Word *(*getWordWithPos)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  PyObject *(*getPos)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  PyObject *(*getRootPos)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  PyObject *(*lastIGContainsCase)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  int (*lastIGContainsTag)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, PyObject *, int __pyx_skip_dispatch);
  int (*lastIGContainsPossessive)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  int (*isCapitalWord)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  int (*isNoun)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  int (*isVerb)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  int (*isRootVerb)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  int (*isAdjective)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  int (*isProperNoun)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  int (*isPunctuation)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  int (*isCardinal)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  int (*isOrdinal)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  int (*isReal)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  int (*isNumber)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  int (*isTime)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  int (*isDate)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  int (*isHashTag)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  int (*isEmail)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  int (*isPercent)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  int (*isFraction)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  int (*isRange)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  int (*isPlural)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  int (*isAuxiliary)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  int (*containsTag)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*getTreePos)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  PyObject *(*getPronType)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  PyObject *(*getNumType)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  PyObject *(*getReflex)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  PyObject *(*getNumber)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  PyObject *(*getCase)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  PyObject *(*getDefinite)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  PyObject *(*getDegree)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  PyObject *(*getPolarity)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  PyObject *(*getPerson)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  PyObject *(*getVoice)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  PyObject *(*getAspect)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  PyObject *(*getTense)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  PyObject *(*getMood)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  PyObject *(*getVerbForm)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
  PyObject *(*getUniversalDependencyFeatures)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*getUniversalDependencyPos)(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *__pyx_vtabptr_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse;


/* "MorphologicalAnalysis/State.pxd":1
 * cdef class State:             # <<<<<<<<<<<<<<
 * 
 *     cdef bint __startState, __endState
 */

struct __pyx_vtabstruct_21MorphologicalAnalysis_5State_State {
  PyObject *(*getName)(struct __pyx_obj_21MorphologicalAnalysis_5State_State *, int __pyx_skip_dispatch);
  PyObject *(*getPos)(struct __pyx_obj_21MorphologicalAnalysis_5State_State *, int __pyx_skip_dispatch);
  int (*isEndState)(struct __pyx_obj_21MorphologicalAnalysis_5State_State *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_21MorphologicalAnalysis_5State_State *__pyx_vtabptr_21MorphologicalAnalysis_5State_State;


/* "MorphologicalAnalysis/FsmParse.pxd":5
 * 
 * 
 * cdef class FsmParse(MorphologicalParse):             # <<<<<<<<<<<<<<
 * 
 *     cdef list __suffixList
 */

struct __pyx_vtabstruct_21MorphologicalAnalysis_8FsmParse_FsmParse {
  struct __pyx_vtabstruct_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse __pyx_base;
  PyObject *(*constructInflectionalGroups)(struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *, int __pyx_skip_dispatch);
  PyObject *(*getVerbAgreement)(struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *, int __pyx_skip_dispatch);
  PyObject *(*getPossesiveAgreement)(struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *, int __pyx_skip_dispatch);
  PyObject *(*setAgreement)(struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*getLastLemmaWithTag)(struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*getLastLemma)(struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *, int __pyx_skip_dispatch);
  PyObject *(*addSuffix)(struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *, struct __pyx_obj_21MorphologicalAnalysis_5State_State *, PyObject *, PyObject *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*getSurfaceForm)(struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *, int __pyx_skip_dispatch);
  struct __pyx_obj_21MorphologicalAnalysis_5State_State *(*getStartState)(struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *, int __pyx_skip_dispatch);
  PyObject *(*getFinalPos)(struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *, int __pyx_skip_dispatch);
  PyObject *(*getInitialPos)(struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *, int __pyx_skip_dispatch);
  PyObject *(*setForm)(struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *, PyObject *, int __pyx_skip_dispatch);
  struct __pyx_obj_21MorphologicalAnalysis_5State_State *(*getFinalSuffix)(struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *, int __pyx_skip_dispatch);
  PyObject *(*headerTransition)(struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *, int __pyx_skip_dispatch);
  PyObject *(*pronounTransition)(struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *, int __pyx_skip_dispatch);
  PyObject *(*transitionList)(struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *, int __pyx_skip_dispatch);
  PyObject *(*suffixList)(struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *, int __pyx_skip_dispatch);
  PyObject *(*withList)(struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *, int __pyx_skip_dispatch);
  PyObject *(*getWithList)(struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_21MorphologicalAnalysis_8FsmParse_FsmParse *__pyx_vtabptr_21MorphologicalAnalysis_8FsmParse_FsmParse;


/* "MorphologicalAnalysis/FiniteStateMachine.pxd":4
 * 
 * 
 * cdef class FiniteStateMachine:             # <<<<<<<<<<<<<<
 * 
 *     cdef list __states
 */

struct __pyx_vtabstruct_21MorphologicalAnalysis_18FiniteStateMachine_FiniteStateMachine {
  int (*isValidTransition)(struct __pyx_obj_21MorphologicalAnalysis_18FiniteStateMachine_FiniteStateMachine *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*getStates)(struct __pyx_obj_21MorphologicalAnalysis_18FiniteStateMachine_FiniteStateMachine *, int __pyx_skip_dispatch);
  struct __pyx_obj_21MorphologicalAnalysis_5State_State *(*getState)(struct __pyx_obj_21MorphologicalAnalysis_18FiniteStateMachine_FiniteStateMachine *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*addTransition)(struct __pyx_obj_21MorphologicalAnalysis_18FiniteStateMachine_FiniteStateMachine *, struct __pyx_obj_21MorphologicalAnalysis_5State_State *, struct __pyx_obj_21MorphologicalAnalysis_5State_State *, PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_21MorphologicalAnalysis_18FiniteStateMachine_18FiniteStateMachine_addTransition *__pyx_optional_args);
  PyObject *(*getTransitions)(struct __pyx_obj_21MorphologicalAnalysis_18FiniteStateMachine_FiniteStateMachine *, struct __pyx_obj_21MorphologicalAnalysis_5State_State *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_21MorphologicalAnalysis_18FiniteStateMachine_FiniteStateMachine *__pyx_vtabptr_21MorphologicalAnalysis_18FiniteStateMachine_FiniteStateMachine;


/* "MorphologicalAnalysis/MetamorphicParse.pxd":6
 * 
 * 
 * cdef class MetamorphicParse:             # <<<<<<<<<<<<<<
 * 
 *     cdef list __metaMorphemeList
 */

struct __pyx_vtabstruct_21MorphologicalAnalysis_16MetamorphicParse_MetamorphicParse {
  PyObject *(*getMetaMorphemeTag)(struct __pyx_obj_21MorphologicalAnalysis_16MetamorphicParse_MetamorphicParse *, PyObject *, int __pyx_skip_dispatch);
  struct __pyx_obj_10Dictionary_4Word_Word *(*getWord)(struct __pyx_obj_21MorphologicalAnalysis_16MetamorphicParse_MetamorphicParse *, int __pyx_skip_dispatch);
  PyObject *(*getMetaMorphemeTagForParse)(struct __pyx_obj_21MorphologicalAnalysis_16MetamorphicParse_MetamorphicParse *, struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, PyObject *, int __pyx_skip_dispatch);
  int (*size)(struct __pyx_obj_21MorphologicalAnalysis_16MetamorphicParse_MetamorphicParse *, int __pyx_skip_dispatch);
  PyObject *(*addMetaMorphemeList)(struct __pyx_obj_21MorphologicalAnalysis_16MetamorphicParse_MetamorphicParse *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*removeMetaMorphemeFromIndex)(struct __pyx_obj_21MorphologicalAnalysis_16MetamorphicParse_MetamorphicParse *, int, int __pyx_skip_dispatch);
  PyObject *(*getMetaMorpheme)(struct __pyx_obj_21MorphologicalAnalysis_16MetamorphicParse_MetamorphicParse *, int, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_21MorphologicalAnalysis_16MetamorphicParse_MetamorphicParse *__pyx_vtabptr_21MorphologicalAnalysis_16MetamorphicParse_MetamorphicParse;


/* "MorphologicalAnalysis/FsmMorphologicalAnalyzer.pxd":13
 * 
 * 
 * cdef class FsmMorphologicalAnalyzer:             # <<<<<<<<<<<<<<
 * 
 *     cdef Trie __dictionaryTrie
 */

struct __pyx_vtabstruct_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer {
  PyObject *(*addParsedSurfaceForms)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*getPossibleWords)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *, struct __pyx_obj_21MorphologicalAnalysis_16MetamorphicParse_MetamorphicParse *, int __pyx_skip_dispatch);
  struct __pyx_obj_10Dictionary_13TxtDictionary_TxtDictionary *(*getDictionary)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, int __pyx_skip_dispatch);
  struct __pyx_obj_21MorphologicalAnalysis_18FiniteStateMachine_FiniteStateMachine *(*getFiniteStateMachine)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, int __pyx_skip_dispatch);
  int (*__pyx___isPossibleSubstring)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, PyObject *, PyObject *, struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  PyObject *(*__pyx___initializeParseList)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, PyObject *, struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int, int __pyx_skip_dispatch);
  PyObject *(*__pyx___initializeParseListFromRoot)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, PyObject *, struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int, int __pyx_skip_dispatch);
  PyObject *(*__pyx___initializeParseListFromSurfaceForm)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, PyObject *, int, int __pyx_skip_dispatch);
  PyObject *(*__pyx___addNewParsesFromCurrentParse)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *, PyObject *, PyObject *, struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  int (*__pyx___parseExists)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*__pyx___parseWord)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*morphologicalAnalysisRoot)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, PyObject *, struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch, struct __pyx_opt_args_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_24FsmMorphologicalAnalyzer_morphologicalAnalysisRoot *__pyx_optional_args);
  PyObject *(*generateAllParses)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int, int __pyx_skip_dispatch);
  PyObject *(*replaceRootWord)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *, struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, int __pyx_skip_dispatch);
  struct __pyx_obj_6Corpus_8Sentence_Sentence *(*replaceWord)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, struct __pyx_obj_6Corpus_8Sentence_Sentence *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  int (*__pyx___analysisExists)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, PyObject *, int, int __pyx_skip_dispatch);
  PyObject *(*__pyx___analysis)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, PyObject *, int, int __pyx_skip_dispatch);
  int (*patternMatches)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  int (*isProperNoun)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*morphologicalAnalysis)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*robustMorphologicalAnalysis)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, PyObject *, int __pyx_skip_dispatch);
  int (*__pyx___isInteger)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, PyObject *, int __pyx_skip_dispatch);
  int (*__pyx___isDouble)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, PyObject *, int __pyx_skip_dispatch);
  int (*__pyx___isNumber)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, PyObject *, int __pyx_skip_dispatch);
  int (*__pyx___isPercent)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, PyObject *, int __pyx_skip_dispatch);
  int (*__pyx___isTime)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, PyObject *, int __pyx_skip_dispatch);
  int (*__pyx___isRange)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, PyObject *, int __pyx_skip_dispatch);
  int (*__pyx___isDate)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*__pyx___toLower)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, PyObject *, int __pyx_skip_dispatch);
  int (*morphologicalAnalysisExists)(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *, struct __pyx_obj_10Dictionary_7TxtWord_TxtWord *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *__pyx_vtabptr_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer;


/* "DataStructure/CounterHashMap.pxd":1
 * cdef class CounterHashMap(dict):             # <<<<<<<<<<<<<<
 * 
 *     cpdef put(self, object key)
 */

struct __pyx_vtabstruct_13DataStructure_14CounterHashMap_CounterHashMap {
  PyObject *(*put)(struct __pyx_obj_13DataStructure_14CounterHashMap_CounterHashMap *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*putNTimes)(struct __pyx_obj_13DataStructure_14CounterHashMap_CounterHashMap *, PyObject *, int, int __pyx_skip_dispatch);
  int (*count)(struct __pyx_obj_13DataStructure_14CounterHashMap_CounterHashMap *, PyObject *, int __pyx_skip_dispatch);
  int (*sumOfCounts)(struct __pyx_obj_13DataStructure_14CounterHashMap_CounterHashMap *, int __pyx_skip_dispatch);
  PyObject *(*max)(struct __pyx_obj_13DataStructure_14CounterHashMap_CounterHashMap *, int __pyx_skip_dispatch, struct __pyx_opt_args_13DataStructure_14CounterHashMap_14CounterHashMap_max *__pyx_optional_args);
  PyObject *(*add)(struct __pyx_obj_13DataStructure_14CounterHashMap_CounterHashMap *, struct __pyx_obj_13DataStructure_14CounterHashMap_CounterHashMap *, int __pyx_skip_dispatch);
  PyObject *(*topN)(struct __pyx_obj_13DataStructure_14CounterHashMap_CounterHashMap *, int, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_13DataStructure_14CounterHashMap_CounterHashMap *__pyx_vtabptr_13DataStructure_14CounterHashMap_CounterHashMap;


/* "MorphologicalAnalysis/FsmParseList.pxd":4
 * from MorphologicalAnalysis.FsmParse cimport FsmParse
 * 
 * cdef class FsmParseList:             # <<<<<<<<<<<<<<
 * 
 *     cdef list __fsmParses
 */

struct __pyx_vtabstruct_21MorphologicalAnalysis_12FsmParseList_FsmParseList {
  int (*size)(struct __pyx_obj_21MorphologicalAnalysis_12FsmParseList_FsmParseList *, int __pyx_skip_dispatch);
  PyObject *(*addFsmParse)(struct __pyx_obj_21MorphologicalAnalysis_12FsmParseList_FsmParseList *, struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *, int __pyx_skip_dispatch);
  struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *(*getFsmParse)(struct __pyx_obj_21MorphologicalAnalysis_12FsmParseList_FsmParseList *, int, int __pyx_skip_dispatch);
  PyObject *(*rootWords)(struct __pyx_obj_21MorphologicalAnalysis_12FsmParseList_FsmParseList *, int __pyx_skip_dispatch);
  PyObject *(*reduceToParsesWithSameRootAndPos)(struct __pyx_obj_21MorphologicalAnalysis_12FsmParseList_FsmParseList *, struct __pyx_obj_10Dictionary_4Word_Word *, int __pyx_skip_dispatch);
  struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *(*getParseWithLongestRootWord)(struct __pyx_obj_21MorphologicalAnalysis_12FsmParseList_FsmParseList *, int __pyx_skip_dispatch);
  PyObject *(*reduceToParsesWithSameRoot)(struct __pyx_obj_21MorphologicalAnalysis_12FsmParseList_FsmParseList *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*constructParseListForDifferentRootWithPos)(struct __pyx_obj_21MorphologicalAnalysis_12FsmParseList_FsmParseList *, int __pyx_skip_dispatch);
  PyObject *(*parsesWithoutPrefixAndSuffix)(struct __pyx_obj_21MorphologicalAnalysis_12FsmParseList_FsmParseList *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_21MorphologicalAnalysis_12FsmParseList_FsmParseList *__pyx_vtabptr_21MorphologicalAnalysis_12FsmParseList_FsmParseList;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_SubtractObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_SubtractObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceSubtract(op1, op2) : PyNumber_Subtract(op1, op2))
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if CYTHON_FAST_PYCALL
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif // CYTHON_FAST_PYCALL
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_AddObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* PyIntCompare.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* PyUnicodeContains.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_ContainsTF(PyObject* substring, PyObject* text, int eq) {
    int result = PyUnicode_Contains(text, substring);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* GetNameInClass.proto */
#define __Pyx_GetNameInClass(var, nmspace, name)  (var) = __Pyx__GetNameInClass(nmspace, name)
static PyObject *__Pyx__GetNameInClass(PyObject *nmspace, PyObject *name);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'Dictionary.Word' */
static PyTypeObject *__pyx_ptype_10Dictionary_4Word_Word = 0;

/* Module declarations from 'Corpus.Sentence' */
static PyTypeObject *__pyx_ptype_6Corpus_8Sentence_Sentence = 0;

/* Module declarations from 'Dictionary.Trie.TrieNode' */
static PyTypeObject *__pyx_ptype_10Dictionary_4Trie_8TrieNode_TrieNode = 0;

/* Module declarations from 'Dictionary.TxtWord' */
static PyTypeObject *__pyx_ptype_10Dictionary_7TxtWord_TxtWord = 0;

/* Module declarations from 'Dictionary.Trie.Trie' */
static PyTypeObject *__pyx_ptype_10Dictionary_4Trie_4Trie_Trie = 0;

/* Module declarations from 'Dictionary.Dictionary' */
static PyTypeObject *__pyx_ptype_10Dictionary_10Dictionary_Dictionary = 0;

/* Module declarations from 'Dictionary.TxtDictionary' */
static PyTypeObject *__pyx_ptype_10Dictionary_13TxtDictionary_TxtDictionary = 0;

/* Module declarations from 'DataStructure.Cache.LRUCache' */
static PyTypeObject *__pyx_ptype_13DataStructure_5Cache_8LRUCache_LRUCache = 0;

/* Module declarations from 'MorphologicalAnalysis.InflectionalGroup' */
static PyTypeObject *__pyx_ptype_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup = 0;

/* Module declarations from 'MorphologicalAnalysis.MorphologicalParse' */
static PyTypeObject *__pyx_ptype_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse = 0;

/* Module declarations from 'MorphologicalAnalysis.State' */
static PyTypeObject *__pyx_ptype_21MorphologicalAnalysis_5State_State = 0;

/* Module declarations from 'MorphologicalAnalysis.FsmParse' */
static PyTypeObject *__pyx_ptype_21MorphologicalAnalysis_8FsmParse_FsmParse = 0;

/* Module declarations from 'MorphologicalAnalysis.FiniteStateMachine' */
static PyTypeObject *__pyx_ptype_21MorphologicalAnalysis_18FiniteStateMachine_FiniteStateMachine = 0;

/* Module declarations from 'MorphologicalAnalysis.MetamorphicParse' */
static PyTypeObject *__pyx_ptype_21MorphologicalAnalysis_16MetamorphicParse_MetamorphicParse = 0;

/* Module declarations from 'MorphologicalAnalysis.FsmMorphologicalAnalyzer' */
static PyTypeObject *__pyx_ptype_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer = 0;

/* Module declarations from 'DataStructure.CounterHashMap' */
static PyTypeObject *__pyx_ptype_13DataStructure_14CounterHashMap_CounterHashMap = 0;

/* Module declarations from 'MorphologicalAnalysis.FsmParseList' */
static PyTypeObject *__pyx_ptype_21MorphologicalAnalysis_12FsmParseList_FsmParseList = 0;

/* Module declarations from 'MorphologicalDisambiguation.AutoDisambiguator' */
static PyTypeObject *__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator = 0;
static PyObject *__pyx_f_27MorphologicalDisambiguation_17AutoDisambiguator___pyx_unpickle_AutoDisambiguator__set_state(struct __pyx_obj_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator *, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "MorphologicalDisambiguation.AutoDisambiguator"
extern int __pyx_module_is_main_MorphologicalDisambiguation__AutoDisambiguator;
int __pyx_module_is_main_MorphologicalDisambiguation__AutoDisambiguator = 0;

/* Implementation of 'MorphologicalDisambiguation.AutoDisambiguator' */
static PyObject *__pyx_builtin_staticmethod;
static PyObject *__pyx_builtin_range;
static const char __pyx_k_[] = "?";
static const char __pyx_k_i[] = "i";
static const char __pyx_k__2[] = "";
static const char __pyx_k_ay[] = "ay";
static const char __pyx_k_da[] = "da";
static const char __pyx_k_dk[] = "d\303\274\305\237\303\274k";
static const char __pyx_k_ne[] = "ne";
static const char __pyx_k_sk[] = "s\304\261k";
static const char __pyx_k_ya[] = "ya";
static const char __pyx_k_yl[] = "y\304\261l";
static const char __pyx_k_ADJ[] = "ADJ";
static const char __pyx_k_ADV[] = "ADV";
static const char __pyx_k_DET[] = "DET";
static const char __pyx_k_NUM[] = "NUM";
static const char __pyx_k_kar[] = "kar\304\261";
static const char __pyx_k_map[] = "_map";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_sre[] = "s\303\274re";
static const char __pyx_k_tag[] = "tag";
static const char __pyx_k_var[] = "var";
static const char __pyx_k_yok[] = "yok";
static const char __pyx_k_A2SG[] = "A2SG";
static const char __pyx_k_CONJ[] = "CONJ";
static const char __pyx_k_NOUN[] = "NOUN";
static const char __pyx_k_P2SG[] = "P2SG";
static const char __pyx_k_P3PL[] = "P3PL";
static const char __pyx_k_P3SG[] = "P3SG";
static const char __pyx_k_PERS[] = "PERS";
static const char __pyx_k_PNON[] = "PNON";
static const char __pyx_k_PROP[] = "PROP";
static const char __pyx_k_QUES[] = "QUES";
static const char __pyx_k_VERB[] = "VERB";
static const char __pyx_k_artk[] = "art\304\261k";
static const char __pyx_k_deil[] = "de\304\237il";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_orta[] = "orta";
static const char __pyx_k_root[] = "root";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_word[] = "word";
static const char __pyx_k_count[] = "count";
static const char __pyx_k_eksik[] = "eksik";
static const char __pyx_k_gerek[] = "gerek";
static const char __pyx_k_index[] = "index";
static const char __pyx_k_rahat[] = "rahat";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_vasat[] = "vasat";
static const char __pyx_k_zaman[] = "zaman";
static const char __pyx_k_ADJ_DB[] = "ADJ^DB";
static const char __pyx_k_ADV_LY[] = "ADV+LY";
static const char __pyx_k_DATIVE[] = "DATIVE";
static const char __pyx_k_INTERJ[] = "INTERJ";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_isNoun[] = "isNoun";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_ADJ_ADV[] = "ADJ$ADV";
static const char __pyx_k_ADJ_AGT[] = "ADJ+AGT";
static const char __pyx_k_allmadk[] = "al\304\261\305\237\304\261lmad\304\261k";
static const char __pyx_k_fsmPars[] = "fsmPars";
static const char __pyx_k_getName[] = "getName";
static const char __pyx_k_getWord[] = "getWord";
static const char __pyx_k_ABLATIVE[] = "ABLATIVE";
static const char __pyx_k_ADJ_ASIF[] = "ADJ+ASIF";
static const char __pyx_k_ADV_CONJ[] = "ADV$CONJ";
static const char __pyx_k_AOR_A1SG[] = "AOR+A1SG";
static const char __pyx_k_AOR_A2PL[] = "AOR+A2PL";
static const char __pyx_k_AOR_A2SG[] = "AOR+A2SG";
static const char __pyx_k_AOR_A3PL[] = "AOR+A3PL";
static const char __pyx_k_AOR_A3SG[] = "AOR+A3SG";
static const char __pyx_k_NUM_CARD[] = "NUM+CARD";
static const char __pyx_k_P1PL_NOM[] = "P1PL+NOM";
static const char __pyx_k_P1SG_NOM[] = "P1SG+NOM";
static const char __pyx_k_P2PL_NOM[] = "P2PL+NOM";
static const char __pyx_k_P2SG_NOM[] = "P2SG+NOM";
static const char __pyx_k_P3PL_NOM[] = "P3PL+NOM";
static const char __pyx_k_P3SG_NOM[] = "P3SG+NOM";
static const char __pyx_k_PNON_ACC[] = "PNON+ACC";
static const char __pyx_k_PNON_GEN[] = "PNON+GEN";
static const char __pyx_k_VERB_POS[] = "VERB+POS";
static const char __pyx_k_fsmParse[] = "fsmParse";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_isPlural[] = "isPlural";
static const char __pyx_k_lastWord[] = "lastWord";
static const char __pyx_k_nextWord[] = "nextWord";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_A2SG_P2SG[] = "A2SG+P2SG";
static const char __pyx_k_A3PL_P2SG[] = "A3PL+P2SG";
static const char __pyx_k_A3PL_P3SG[] = "A3PL+P3SG";
static const char __pyx_k_A3SG_P3PL[] = "A3SG+P3PL";
static const char __pyx_k_A3SG_P3SG[] = "A3SG+P3SG";
static const char __pyx_k_ADV_SINCE[] = "ADV+SINCE";
static const char __pyx_k_NARR_A3SG[] = "NARR+A3SG";
static const char __pyx_k_NEG_PROG2[] = "NEG+PROG2";
static const char __pyx_k_NOUN_PROP[] = "NOUN+PROP";
static const char __pyx_k_P2SG_P3SG[] = "P2SG$P3SG";
static const char __pyx_k_P2SG_PNON[] = "P2SG$PNON";
static const char __pyx_k_P3SG_PNON[] = "P3SG$PNON";
static const char __pyx_k_POS_NECES[] = "POS+NECES";
static const char __pyx_k_POS_PROG2[] = "POS+PROG2";
static const char __pyx_k_PROP_A3SG[] = "PROP+A3SG";
static const char __pyx_k_fsmParses[] = "fsmParses";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_ADJ_INTERJ[] = "ADJ$INTERJ";
static const char __pyx_k_PRON_QUESP[] = "PRON+QUESP";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_ADJ_NUM_ORD[] = "ADJ$NUM+ORD";
static const char __pyx_k_CONJ_INTERJ[] = "CONJ$INTERJ";
static const char __pyx_k_POSTP_PCABL[] = "POSTP+PCABL";
static const char __pyx_k_POSTP_PCDAT[] = "POSTP+PCDAT";
static const char __pyx_k_POSTP_PCINS[] = "POSTP+PCINS";
static const char __pyx_k_POSTP_PCNOM[] = "POSTP+PCNOM";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_containsTag[] = "containsTag";
static const char __pyx_k_defaultCase[] = "defaultCase";
static const char __pyx_k_getFinalPos[] = "getFinalPos";
static const char __pyx_k_getFsmParse[] = "getFsmParse";
static const char __pyx_k_isFirstWord[] = "isFirstWord";
static const char __pyx_k_nextWordPos[] = "nextWordPos";
static const char __pyx_k_parseString[] = "parseString";
static const char __pyx_k_surfaceForm[] = "surfaceForm";
static const char __pyx_k_DEMONSP_PERS[] = "DEMONSP$PERS";
static const char __pyx_k_INSTRUMENTAL[] = "INSTRUMENTAL";
static const char __pyx_k_NEG_AOR_A1SG[] = "NEG+AOR+A1SG";
static const char __pyx_k_NEG_FUT_A1SG[] = "NEG+FUT+A1SG";
static const char __pyx_k_NEG_FUT_A3SG[] = "NEG+FUT+A3SG";
static const char __pyx_k_NUM_FRACTION[] = "NUM+FRACTION";
static const char __pyx_k_POS_FUT_A1SG[] = "POS+FUT+A1SG";
static const char __pyx_k_POS_FUT_A3PL[] = "POS+FUT+A3PL";
static const char __pyx_k_POS_FUT_A3SG[] = "POS+FUT+A3SG";
static const char __pyx_k_VERB_POS_AOR[] = "VERB+POS+AOR";
static const char __pyx_k_fsmParseList[] = "fsmParseList";
static const char __pyx_k_previousWord[] = "previousWord";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_staticmethod[] = "staticmethod";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_A1PL_P1PL_NOM[] = "A1PL+P1PL+NOM";
static const char __pyx_k_A3PL_P3SG_NOM[] = "A3PL+P3SG+NOM";
static const char __pyx_k_A3PL_PNON_NOM[] = "A3PL+PNON+NOM";
static const char __pyx_k_A3SG_P3PL_NOM[] = "A3SG+P3PL+NOM";
static const char __pyx_k_A3SG_PNON_ACC[] = "A3SG+PNON+ACC";
static const char __pyx_k_ABLE_FUT_A3SG[] = "ABLE+FUT+A3SG";
static const char __pyx_k_NEG_PAST_A1PL[] = "NEG+PAST+A1PL";
static const char __pyx_k_POS_PAST_A1PL[] = "POS+PAST+A1PL";
static const char __pyx_k_correctParses[] = "correctParses";
static const char __pyx_k_isCapitalWord[] = "isCapitalWord";
static const char __pyx_k_isNextWordNum[] = "isNextWordNum";
static const char __pyx_k_nextParseList[] = "nextParseList";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_A3SG_PROP_A3SG[] = "A3SG$PROP+A3SG";
static const char __pyx_k_NOUN_A3SG_P3SG[] = "NOUN+A3SG+P3SG";
static const char __pyx_k_P2SG_P3PL_P3SG[] = "P2SG$P3PL$P3SG";
static const char __pyx_k_POSTP_PCABL_DB[] = "POSTP+PCABL^DB";
static const char __pyx_k_POS_NECES_A3SG[] = "POS+NECES+A3SG";
static const char __pyx_k_POS_PROG2_A3SG[] = "POS+PROG2+A3SG";
static const char __pyx_k_PRON_PERS_A1PL[] = "PRON+PERS+A1PL";
static const char __pyx_k_PRON_PERS_A1SG[] = "PRON+PERS+A1SG";
static const char __pyx_k_PROP_A3PL_P3PL[] = "PROP+A3PL+P3PL";
static const char __pyx_k_QUES_PRES_A3SG[] = "QUES+PRES+A3SG";
static const char __pyx_k_getSurfaceForm[] = "getSurfaceForm";
static const char __pyx_k_isNextWordNoun[] = "isNextWordNoun";
static const char __pyx_k_nextWordExists[] = "nextWordExists";
static const char __pyx_k_ADJ_ASIF_ADV_LY[] = "ADJ+ASIF$ADV+LY";
static const char __pyx_k_ADJ_POSTP_PCABL[] = "ADJ$POSTP+PCABL";
static const char __pyx_k_ADV_POSTP_PCINS[] = "ADV$POSTP+PCINS";
static const char __pyx_k_AOR_DB_ADJ_ZERO[] = "AOR^DB+ADJ+ZERO";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_ADJ_DB_NOUN_ZERO[] = "ADJ^DB+NOUN+ZERO";
static const char __pyx_k_ADJ_DB_VERB_ZERO[] = "ADJ^DB+VERB+ZERO";
static const char __pyx_k_ADJ_FUTPART_P2PL[] = "ADJ+FUTPART+P2PL";
static const char __pyx_k_ADJ_FUTPART_P3PL[] = "ADJ+FUTPART+P3PL";
static const char __pyx_k_ADJ_FUTPART_P3SG[] = "ADJ+FUTPART+P3SG";
static const char __pyx_k_CONJ_POSTP_PCNOM[] = "CONJ$POSTP+PCNOM";
static const char __pyx_k_MorphologicalTag[] = "MorphologicalTag";
static const char __pyx_k_NARR_DB_ADJ_ZERO[] = "NARR^DB+ADJ+ZERO";
static const char __pyx_k_isBeforeLastWord[] = "isBeforeLastWord";
static const char __pyx_k_ADJ_PASTPART_P1PL[] = "ADJ+PASTPART+P1PL";
static const char __pyx_k_ADJ_PASTPART_P1SG[] = "ADJ+PASTPART+P1SG";
static const char __pyx_k_ADJ_PASTPART_P2PL[] = "ADJ+PASTPART+P2PL";
static const char __pyx_k_ADJ_PASTPART_P3PL[] = "ADJ+PASTPART+P3PL";
static const char __pyx_k_ADJ_PASTPART_P3SG[] = "ADJ+PASTPART+P3SG";
static const char __pyx_k_AutoDisambiguator[] = "AutoDisambiguator";
static const char __pyx_k_DATE_NUM_FRACTION[] = "DATE$NUM+FRACTION";
static const char __pyx_k_P2SG_NOM_PNON_GEN[] = "P2SG+NOM$PNON+GEN";
static const char __pyx_k_P3SG_NOM_PNON_ACC[] = "P3SG+NOM$PNON+ACC";
static const char __pyx_k_VERB_DB_VERB_PASS[] = "VERB^DB+VERB+PASS";
static const char __pyx_k_VERB_POS_IMP_A2PL[] = "VERB+POS+IMP+A2PL";
static const char __pyx_k_caseDisambiguator[] = "caseDisambiguator";
static const char __pyx_k_containsTwoNeOrYa[] = "containsTwoNeOrYa";
static const char __pyx_k_INF2_A3SG_P3SG_NOM[] = "INF2+A3SG+P3SG+NOM";
static const char __pyx_k_NOUN_A3PL_PNON_NOM[] = "NOUN+A3PL+PNON+NOM";
static const char __pyx_k_NOUN_A3SG_P1SG_DAT[] = "NOUN+A3SG+P1SG+DAT";
static const char __pyx_k_NOUN_A3SG_P1SG_NOM[] = "NOUN+A3SG+P1SG+NOM";
static const char __pyx_k_NOUN_A3SG_P2SG_DAT[] = "NOUN+A3SG+P2SG+DAT";
static const char __pyx_k_NOUN_A3SG_P2SG_NOM[] = "NOUN+A3SG+P2SG+NOM";
static const char __pyx_k_NOUN_A3SG_P3SG_DAT[] = "NOUN+A3SG+P3SG+DAT";
static const char __pyx_k_NOUN_A3SG_P3SG_GEN[] = "NOUN+A3SG+P3SG+GEN";
static const char __pyx_k_NOUN_A3SG_P3SG_NOM[] = "NOUN+A3SG+P3SG+NOM";
static const char __pyx_k_NOUN_A3SG_PNON_DAT[] = "NOUN+A3SG+PNON+DAT";
static const char __pyx_k_NOUN_A3SG_PNON_GEN[] = "NOUN+A3SG+PNON+GEN";
static const char __pyx_k_NOUN_A3SG_PNON_NOM[] = "NOUN+A3SG+PNON+NOM";
static const char __pyx_k_PROP_A3PL_PNON_NOM[] = "PROP+A3PL+PNON+NOM";
static const char __pyx_k_VERB_POS_NARR_A3SG[] = "VERB+POS+NARR+A3SG";
static const char __pyx_k_VERB_POS_PAST_A1PL[] = "VERB+POS+PAST+A1PL";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_hasPreviousWordTag[] = "hasPreviousWordTag";
static const char __pyx_k_isPossessivePlural[] = "isPossessivePlural";
static const char __pyx_k_A2SG_P2SG_A3SG_P3SG[] = "A2SG+P2SG$A3SG+P3SG";
static const char __pyx_k_ADJ_ADV_POSTP_PCABL[] = "ADJ$ADV$POSTP+PCABL";
static const char __pyx_k_ADJ_ADV_POSTP_PCINS[] = "ADJ$ADV$POSTP+PCINS";
static const char __pyx_k_NOUN_ZERO_A3SG_P3SG[] = "NOUN+ZERO+A3SG+P3SG";
static const char __pyx_k_PRON_PERS_A1PL_PNON[] = "PRON+PERS+A1PL+PNON";
static const char __pyx_k_PRON_PERS_A1SG_PNON[] = "PRON+PERS+A1SG+PNON";
static const char __pyx_k_VERB_ZERO_PRES_A1SG[] = "VERB+ZERO+PRES+A1SG";
static const char __pyx_k_ADJ_ADV_DET_NUM_CARD[] = "ADJ$ADV$DET$NUM+CARD";
static const char __pyx_k_NOUN_PROP_PRON_QUESP[] = "NOUN+PROP$PRON+QUESP";
static const char __pyx_k_ADJ_DB_NOUN_ZERO_NOUN[] = "ADJ^DB+NOUN+ZERO$NOUN";
static const char __pyx_k_ADJ_DB_POSTP_PCABL_DB[] = "ADJ^DB$POSTP+PCABL^DB";
static const char __pyx_k_ADJ_DB_VERB_ZERO_COND[] = "ADJ^DB+VERB+ZERO+COND";
static const char __pyx_k_ADJ_VERB_POS_IMP_A2SG[] = "ADJ$VERB+POS+IMP+A2SG";
static const char __pyx_k_ADV_VERB_POS_IMP_A2SG[] = "ADV$VERB+POS+IMP+A2SG";
static const char __pyx_k_NOUN_A3SG_PNON_NOM_DB[] = "NOUN+A3SG+PNON+NOM^DB";
static const char __pyx_k_PRON_QUANTP_A3PL_P3PL[] = "PRON+QUANTP+A3PL+P3PL";
static const char __pyx_k_PRON_QUANTP_A3SG_P3SG[] = "PRON+QUANTP+A3SG+P3SG";
static const char __pyx_k_VERB_DB_VERB_PASS_POS[] = "VERB^DB+VERB+PASS+POS";
static const char __pyx_k_isAnyWordSecondPerson[] = "isAnyWordSecondPerson";
static const char __pyx_k_ADJ_NOUN_A3SG_PNON_NOM[] = "ADJ$NOUN+A3SG+PNON+NOM";
static const char __pyx_k_ADV_NOUN_A3SG_PNON_NOM[] = "ADV$NOUN+A3SG+PNON+NOM";
static const char __pyx_k_CONJ_VERB_POS_IMP_A2SG[] = "CONJ$VERB+POS+IMP+A2SG";
static const char __pyx_k_DET_NOUN_A3SG_PNON_NOM[] = "DET$NOUN+A3SG+PNON+NOM";
static const char __pyx_k_NOUN_AGT_A3SG_P3SG_NOM[] = "NOUN+AGT+A3SG+P3SG+NOM";
static const char __pyx_k_NOUN_AGT_A3SG_PNON_NOM[] = "NOUN+AGT+A3SG+PNON+NOM";
static const char __pyx_k_VERB_POS_NARR_COP_A3SG[] = "VERB+POS+NARR+COP+A3SG";
static const char __pyx_k_ADJ_ADV_DET_POSTP_PCABL[] = "ADJ$ADV$DET$POSTP+PCABL";
static const char __pyx_k_ADV_WITHOUTHAVINGDONESO[] = "ADV+WITHOUTHAVINGDONESO";
static const char __pyx_k_NOUN_PROP_A3SG_PNON_NOM[] = "NOUN+PROP+A3SG+PNON+NOM";
static const char __pyx_k_NOUN_ZERO_A3PL_PNON_NOM[] = "NOUN+ZERO+A3PL+PNON+NOM";
static const char __pyx_k_NOUN_ZERO_A3SG_P3SG_NOM[] = "NOUN+ZERO+A3SG+P3SG+NOM";
static const char __pyx_k_POS_DB_ADJ_FUTPART_PNON[] = "POS^DB+ADJ+FUTPART+PNON";
static const char __pyx_k_PRON_PERS_A1PL_PNON_ACC[] = "PRON+PERS+A1PL+PNON+ACC";
static const char __pyx_k_PRON_PERS_A1PL_PNON_GEN[] = "PRON+PERS+A1PL+PNON+GEN";
static const char __pyx_k_PRON_PERS_A1SG_PNON_ACC[] = "PRON+PERS+A1SG+PNON+ACC";
static const char __pyx_k_PRON_PERS_A1SG_PNON_GEN[] = "PRON+PERS+A1SG+PNON+GEN";
static const char __pyx_k_PRON_PERS_A3SG_PNON_NOM[] = "PRON+PERS+A3SG+PNON+NOM";
static const char __pyx_k_VERB_ZERO_PRES_COP_A3SG[] = "VERB+ZERO+PRES+COP+A3SG";
static const char __pyx_k_ABLE_DB_ADJ_FUTPART_PNON[] = "ABLE^DB+ADJ+FUTPART+PNON";
static const char __pyx_k_AOR_A3SG_AOR_DB_ADJ_ZERO[] = "AOR+A3SG$AOR^DB+ADJ+ZERO";
static const char __pyx_k_NEG_DB_ADJ_PASTPART_PNON[] = "NEG^DB+ADJ+PASTPART+PNON";
static const char __pyx_k_NOUN_A3SG_PRON_PERS_A1PL[] = "NOUN+A3SG$PRON+PERS+A1PL";
static const char __pyx_k_NOUN_A3SG_PRON_PERS_A1SG[] = "NOUN+A3SG$PRON+PERS+A1SG";
static const char __pyx_k_PRON_QUESP_A3SG_PNON_NOM[] = "PRON+QUESP+A3SG+PNON+NOM";
static const char __pyx_k_selectCaseForParseString[] = "selectCaseForParseString";
static const char __pyx_k_ADJ_ADV_VERB_POS_IMP_A2SG[] = "ADJ$ADV$VERB+POS+IMP+A2SG";
static const char __pyx_k_ADJ_DB_VERB_ZERO_VERB_POS[] = "ADJ^DB+VERB+ZERO$VERB+POS";
static const char __pyx_k_PRON_QUANTP_A3PL_P3PL_GEN[] = "PRON+QUANTP+A3PL+P3PL+GEN";
static const char __pyx_k_VERB_POS_NARR_DB_ADJ_ZERO[] = "VERB+POS+NARR^DB+ADJ+ZERO";
static const char __pyx_k_isNextWordNounOrAdjective[] = "isNextWordNounOrAdjective";
static const char __pyx_k_A3PL_PNON_NOM_DB_ADV_SINCE[] = "A3PL+PNON+NOM^DB+ADV+SINCE";
static const char __pyx_k_ADJ_ADV_NOUN_A3SG_PNON_NOM[] = "ADJ$ADV$NOUN+A3SG+PNON+NOM";
static const char __pyx_k_NARR_A3SG_NARR_DB_ADJ_ZERO[] = "NARR+A3SG$NARR^DB+ADJ+ZERO";
static const char __pyx_k_NOUN_FUTPART_A3PL_P3SG_NOM[] = "NOUN+FUTPART+A3PL+P3SG+NOM";
static const char __pyx_k_NOUN_FUTPART_A3SG_P2PL_NOM[] = "NOUN+FUTPART+A3SG+P2PL+NOM";
static const char __pyx_k_NOUN_FUTPART_A3SG_P3PL_NOM[] = "NOUN+FUTPART+A3SG+P3PL+NOM";
static const char __pyx_k_NOUN_FUTPART_A3SG_P3SG_NOM[] = "NOUN+FUTPART+A3SG+P3SG+NOM";
static const char __pyx_k_NOUN_NUM_CARD_DB_NOUN_ZERO[] = "NOUN$NUM+CARD^DB+NOUN+ZERO";
static const char __pyx_k_NOUN_ZERO_A3SG_P3SG_NOM_DB[] = "NOUN+ZERO+A3SG+P3SG+NOM^DB";
static const char __pyx_k_NUM_CARD_VERB_POS_IMP_A2SG[] = "NUM+CARD$VERB+POS+IMP+A2SG";
static const char __pyx_k_P3PL_NOM_P3SG_NOM_PNON_ACC[] = "P3PL+NOM$P3SG+NOM$PNON+ACC";
static const char __pyx_k_PRON_DEMONSP_A3SG_PNON_NOM[] = "PRON+DEMONSP+A3SG+PNON+NOM";
static const char __pyx_k_ADJ_NOUN_PROP_A3SG_PNON_NOM[] = "ADJ$NOUN+PROP+A3SG+PNON+NOM";
static const char __pyx_k_NOUN_PASTPART_A3PL_P3SG_NOM[] = "NOUN+PASTPART+A3PL+P3SG+NOM";
static const char __pyx_k_NOUN_PASTPART_A3SG_P1SG_NOM[] = "NOUN+PASTPART+A3SG+P1SG+NOM";
static const char __pyx_k_NOUN_PASTPART_A3SG_P2PL_NOM[] = "NOUN+PASTPART+A3SG+P2PL+NOM";
static const char __pyx_k_NOUN_PASTPART_A3SG_P3PL_NOM[] = "NOUN+PASTPART+A3SG+P3PL+NOM";
static const char __pyx_k_NOUN_PASTPART_A3SG_P3SG_NOM[] = "NOUN+PASTPART+A3SG+P3SG+NOM";
static const char __pyx_k_ADJ_DB_NOUN_A3SG_PNON_NOM_DB[] = "ADJ^DB$NOUN+A3SG+PNON+NOM^DB";
static const char __pyx_k_ADJ_PRON_QUESP_A3SG_PNON_NOM[] = "ADJ$PRON+QUESP+A3SG+PNON+NOM";
static const char __pyx_k_parsesWithoutPrefixAndSuffix[] = "parsesWithoutPrefixAndSuffix";
static const char __pyx_k_A3PL_P3PL_A3PL_P3SG_A3SG_P3PL[] = "A3PL+P3PL$A3PL+P3SG$A3SG+P3PL";
static const char __pyx_k_ADJ_DB_NOUN_AGT_A3SG_PNON_NOM[] = "ADJ^DB+NOUN+AGT+A3SG+PNON+NOM";
static const char __pyx_k_ADV_PRON_QUANTP_A3SG_P3SG_NOM[] = "ADV$PRON+QUANTP+A3SG+P3SG+NOM";
static const char __pyx_k_NOUN_A3SG_PNON_NOM_DB_ADJ_AGT[] = "NOUN+A3SG+PNON+NOM^DB+ADJ+AGT";
static const char __pyx_k_VERB_REFLEX_VERB_DB_VERB_PASS[] = "VERB+REFLEX$VERB^DB+VERB+PASS";
static const char __pyx_k_ADJ_AGT_NOUN_AGT_A3SG_PNON_NOM[] = "ADJ+AGT$NOUN+AGT+A3SG+PNON+NOM";
static const char __pyx_k_ADJ_DB_NOUN_ZERO_A3SG_P3SG_NOM[] = "ADJ^DB+NOUN+ZERO+A3SG+P3SG+NOM";
static const char __pyx_k_ADJ_DB_VERB_ZERO_PRES_COP_A3SG[] = "ADJ^DB+VERB+ZERO+PRES+COP+A3SG";
static const char __pyx_k_DET_PRON_DEMONSP_A3SG_PNON_NOM[] = "DET$PRON+DEMONSP+A3SG+PNON+NOM";
static const char __pyx_k_NOUN_A3SG_PNON_NOM_POSTP_PCABL[] = "NOUN+A3SG+PNON+NOM$POSTP+PCABL";
static const char __pyx_k_NOUN_A3SG_PNON_NOM_POSTP_PCDAT[] = "NOUN+A3SG+PNON+NOM$POSTP+PCDAT";
static const char __pyx_k_POS_DB_NOUN_INF2_A3SG_PNON_DAT[] = "POS^DB+NOUN+INF2+A3SG+PNON+DAT";
static const char __pyx_k_POS_DB_NOUN_INF2_A3SG_PNON_NOM[] = "POS^DB+NOUN+INF2+A3SG+PNON+NOM";
static const char __pyx_k_POS_DB_NOUN_INF3_A3SG_P2SG_NOM[] = "POS^DB+NOUN+INF3+A3SG+P2SG+NOM";
static const char __pyx_k_POS_DB_NOUN_INF3_A3SG_PNON_DAT[] = "POS^DB+NOUN+INF3+A3SG+PNON+DAT";
static const char __pyx_k_POS_DB_NOUN_INF3_A3SG_PNON_GEN[] = "POS^DB+NOUN+INF3+A3SG+PNON+GEN";
static const char __pyx_k_POS_DB_NOUN_INF3_A3SG_PNON_NOM[] = "POS^DB+NOUN+INF3+A3SG+PNON+NOM";
static const char __pyx_k_pyx_unpickle_AutoDisambiguator[] = "__pyx_unpickle_AutoDisambiguator";
static const char __pyx_k_A3PL_PNON_NOM_DB_ADV_SINCE_A3PL[] = "A3PL+PNON+NOM^DB+ADV+SINCE$A3PL+PNON+NOM^DB+VERB+ZERO+PRES+COP+A3SG$A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A3PL+COP";
static const char __pyx_k_A3PL_PNON_NOM_DB_VERB_ZERO_PRES[] = "A3PL+PNON+NOM^DB+VERB+ZERO+PRES+COP+A3SG";
static const char __pyx_k_ADJ_ADV_NOUN_A3SG_PNON_NOM_NOUN[] = "ADJ$ADV$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM";
static const char __pyx_k_ADJ_DB_VERB_ZERO_NARR_A3SG_VERB[] = "ADJ^DB+VERB+ZERO+NARR+A3SG$VERB+POS+NARR+A3SG$VERB+POS+NARR^DB+ADJ+ZERO";
static const char __pyx_k_ADJ_DB_VERB_ZERO_NOUN_A3SG_PNON[] = "ADJ^DB+VERB+ZERO$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO$VERB+POS";
static const char __pyx_k_ADJ_DB_VERB_ZERO_PAST_A1PL_VERB[] = "ADJ^DB+VERB+ZERO+PAST+A1PL$VERB+POS+PAST+A1PL$VERB+POS^DB+ADJ+PASTPART+PNON$VERB+POS^DB+NOUN+PASTPART+A3SG+PNON+NOM";
static const char __pyx_k_ADJ_DB_VERB_ZERO_POSTP_PCABL_DB[] = "ADJ^DB+VERB+ZERO$POSTP+PCABL^DB+VERB+ZERO$VERB+POS";
static const char __pyx_k_ADJ_DB_VERB_ZERO_PRES_A1SG_NOUN[] = "ADJ^DB+VERB+ZERO+PRES+A1SG$NOUN+A3SG+P1SG+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A1SG";
static const char __pyx_k_ADJ_DB_VERB_ZERO_PRES_A3PL_NOUN[] = "ADJ^DB+VERB+ZERO+PRES+A3PL$NOUN+A3PL+PNON+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A3PL";
static const char __pyx_k_ADJ_NOUN_A3SG_PNON_NOM_VERB_POS[] = "ADJ$NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG";
static const char __pyx_k_ADJ_PASTPART_P1PL_NOUN_PASTPART[] = "ADJ+PASTPART+P1PL$NOUN+PASTPART+A3SG+P1PL+NOM";
static const char __pyx_k_ADJ_PASTPART_P1SG_NOUN_PASTPART[] = "ADJ+PASTPART+P1SG$NOUN+PASTPART+A3SG+P1SG+NOM";
static const char __pyx_k_ADJ_PASTPART_P2PL_NOUN_PASTPART[] = "ADJ+PASTPART+P2PL$NOUN+PASTPART+A3SG+P2PL+NOM$NOUN+PASTPART+A3SG+PNON+GEN^DB+VERB+ZERO+PRES+A1PL";
static const char __pyx_k_ADJ_PASTPART_P3PL_NOUN_PASTPART[] = "ADJ+PASTPART+P3PL$NOUN+PASTPART+A3PL+P3PL+NOM$NOUN+PASTPART+A3PL+P3SG+NOM$NOUN+PASTPART+A3SG+P3PL+NOM";
static const char __pyx_k_ADJ_PASTPART_P3SG_NOUN_PASTPART[] = "ADJ+PASTPART+P3SG$NOUN+PASTPART+A3SG+P3SG+NOM";
static const char __pyx_k_AOR_A2PL_AOR_DB_ADJ_ZERO_DB_ADJ[] = "AOR+A2PL$AOR^DB+ADJ+ZERO^DB+ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2PL+NOM";
static const char __pyx_k_AOR_A2SG_AOR_DB_ADJ_ZERO_DB_ADJ[] = "AOR+A2SG$AOR^DB+ADJ+ZERO^DB+ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2SG+NOM";
static const char __pyx_k_NEG_FUT_A1SG_NEG_DB_ADJ_FUTPART[] = "NEG+FUT+A1SG$NEG^DB+ADJ+FUTPART+P1SG$NEG^DB+NOUN+FUTPART+A3SG+P1SG+NOM";
static const char __pyx_k_NEG_FUT_A3SG_NEG_DB_ADJ_FUTPART[] = "NEG+FUT+A3SG$NEG^DB+ADJ+FUTPART+PNON";
static const char __pyx_k_NOUN_A3SG_P1SG_DAT_VERB_NEG_IMP[] = "NOUN+A3SG+P1SG+DAT$VERB+NEG+IMP+A2SG$VERB+POS^DB+NOUN+INF2+A3SG+PNON+NOM";
static const char __pyx_k_NOUN_A3SG_PNON_DAT_VERB_POS_OPT[] = "NOUN+A3SG+PNON+DAT$VERB+POS+OPT+A3SG";
static const char __pyx_k_NOUN_A3SG_PNON_NOM_DB_ADV_SINCE[] = "NOUN+A3SG+PNON+NOM^DB+ADV+SINCE$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+COP+A3SG$VERB^DB+VERB+CAUS+POS+IMP+A2SG";
static const char __pyx_k_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO[] = "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO$VERB+POS";
static const char __pyx_k_NOUN_A3SG_PNON_NOM_VERB_POS_IMP[] = "NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG";
static const char __pyx_k_NOUN_A3SG_PNON_PRON_QUANTP_A3PL[] = "NOUN+A3SG+PNON$PRON+QUANTP+A3PL+P3PL";
static const char __pyx_k_NOUN_A3SG_PNON_PRON_QUANTP_A3SG[] = "NOUN+A3SG+PNON$PRON+QUANTP+A3SG+P3SG";
static const char __pyx_k_NUM_CARD_DB_NOUN_ZERO_A3SG_P3SG[] = "NUM+CARD^DB+NOUN+ZERO+A3SG+P3SG";
static const char __pyx_k_PNON_NOM_DB_VERB_ZERO_PRES_A1SG[] = "PNON+NOM^DB+VERB+ZERO+PRES+A1SG";
static const char __pyx_k_POS_FUT_A1SG_POS_DB_ADJ_FUTPART[] = "POS+FUT+A1SG$POS^DB+ADJ+FUTPART+P1SG$POS^DB+NOUN+FUTPART+A3SG+P1SG+NOM";
static const char __pyx_k_POS_FUT_A3SG_POS_DB_ADJ_FUTPART[] = "POS+FUT+A3SG$POS^DB+ADJ+FUTPART+PNON";
static const char __pyx_k_POS_NECES_A3SG_POS_DB_NOUN_INF2[] = "POS+NECES+A3SG$POS^DB+NOUN+INF2+A3SG+PNON+NOM^DB+ADJ+WITH";
static const char __pyx_k_POS_NECES_POS_DB_NOUN_INF2_A3SG[] = "POS+NECES$POS^DB+NOUN+INF2+A3SG+PNON+NOM^DB+ADJ+WITH^DB+VERB+ZERO+PRES";
static const char __pyx_k_VERB_POS_DB_NOUN_INF2_A3SG_PNON[] = "VERB+POS^DB+NOUN+INF2+A3SG+PNON+NOM";
static const char __pyx_k_A1PL_P1PL_NOM_A3SG_P3SG_GEN_DB_V[] = "A1PL+P1PL+NOM$A3SG+P3SG+GEN^DB+VERB+ZERO+PRES+A1PL";
static const char __pyx_k_A3PL_P2SG_A3PL_P3PL_A3PL_P3SG_A3[] = "A3PL+P2SG$A3PL+P3PL$A3PL+P3SG$A3SG+P3PL";
static const char __pyx_k_A3PL_P3PL_NOM_A3PL_P3SG_NOM_A3PL[] = "A3PL+P3PL+NOM$A3PL+P3SG+NOM$A3PL+PNON+ACC$A3SG+P3PL+NOM";
static const char __pyx_k_A3PL_P3SG_A3SG_P3PL_PROP_A3PL_P3[] = "A3PL+P3SG$A3SG+P3PL$PROP+A3PL+P3PL";
static const char __pyx_k_A3PL_PNON_NOM_A3SG_PNON_NOM_DB_V[] = "A3PL+PNON+NOM$A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A3PL$PROP+A3PL+PNON+NOM";
static const char __pyx_k_A3SG_PNON_ACC_PROP_A3SG_PNON_NOM[] = "A3SG+PNON+ACC$PROP+A3SG+PNON+NOM";
static const char __pyx_k_ABLE_FUT_A3SG_ABLE_DB_ADJ_FUTPAR[] = "ABLE+FUT+A3SG$ABLE^DB+ADJ+FUTPART+PNON";
static const char __pyx_k_ADJ_ADV_CONJ_PRON_QUESP_A3SG_PNO[] = "ADJ$ADV$CONJ$PRON+QUESP+A3SG+PNON+NOM";
static const char __pyx_k_ADJ_ADV_NOUN_A3SG_PNON_NOM_POSTP[] = "ADJ$ADV$NOUN+A3SG+PNON+NOM$POSTP+PCDAT";
static const char __pyx_k_ADJ_ADV_POSTP_PCABL_VERB_POS_IMP[] = "ADJ$ADV$POSTP+PCABL$VERB+POS+IMP+A2SG";
static const char __pyx_k_ADJ_AGT_DB_ADJ_JUSTLIKE_NOUN_AGT[] = "ADJ+AGT^DB+ADJ+JUSTLIKE$NOUN+AGT+A3SG+P3SG+NOM$NOUN+AGT^DB+ADJ+ALMOST";
static const char __pyx_k_ADJ_ASIF_ADV_LY_NOUN_ZERO_A3SG_P[] = "ADJ+ASIF$ADV+LY$NOUN+ZERO+A3SG+PNON+EQU";
static const char __pyx_k_ADJ_DB_NOUN_AGT_A3SG_PNON_NOM_NO[] = "ADJ^DB+NOUN+AGT+A3SG+PNON+NOM$NOUN+A3SG+PNON+NOM^DB+ADJ+AGT";
static const char __pyx_k_ADJ_DB_NOUN_ZERO_A3PL_PNON_NOM_A[] = "ADJ^DB+NOUN+ZERO+A3PL+PNON+NOM$ADJ^DB+VERB+ZERO+PRES+A3PL$NOUN+A3PL+PNON+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A3PL";
static const char __pyx_k_ADJ_DB_NOUN_ZERO_A3SG_P2SG_ADJ_D[] = "ADJ^DB+NOUN+ZERO+A3SG+P2SG$ADJ^DB+NOUN+ZERO+A3SG+P3SG$NOUN+A3SG+P2SG$NOUN+A3SG+P3SG";
static const char __pyx_k_ADJ_DB_NOUN_ZERO_A3SG_P2SG_NOM_D[] = "ADJ^DB+NOUN+ZERO+A3SG+P2SG+NOM^DB+VERB+ZERO$ADJ^DB+NOUN+ZERO+A3SG+PNON+GEN^DB+VERB+ZERO$VERB^DB+VERB+PASS+POS";
static const char __pyx_k_ADJ_DB_NOUN_ZERO_A3SG_P3SG_NOM_A[] = "ADJ^DB+NOUN+ZERO+A3SG+P3SG+NOM$ADJ^DB+NOUN+ZERO+A3SG+PNON+ACC$NUM+CARD^DB+NOUN+ZERO+A3SG+P3SG+NOM$NUM+CARD^DB+NOUN+ZERO+A3SG+PNON+ACC";
static const char __pyx_k_ADJ_DB_VERB_ZERO_COND_VERB_POS_D[] = "ADJ^DB+VERB+ZERO+COND$VERB+POS+DESR";
static const char __pyx_k_ADJ_DB_VERB_ZERO_PRES_COP_A3SG_N[] = "ADJ^DB+VERB+ZERO+PRES+COP+A3SG$NOUN+A3SG+PNON+NOM^DB+ADV+SINCE$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+COP+A3SG";
static const char __pyx_k_ADJ_DB_VERB_ZERO_PRES_COP_A3SG_V[] = "ADJ^DB+VERB+ZERO+PRES+COP+A3SG$VERB^DB+VERB+CAUS+POS+IMP+A2SG";
static const char __pyx_k_ADJ_DET_PRON_QUANTP_A3SG_P3SG_NO[] = "ADJ$DET$PRON+QUANTP+A3SG+P3SG+NOM";
static const char __pyx_k_ADJ_FUTPART_P2PL_NOUN_FUTPART_A3[] = "ADJ+FUTPART+P2PL$NOUN+FUTPART+A3SG+P2PL+NOM$NOUN+FUTPART+A3SG+PNON+GEN^DB+VERB+ZERO+PRES+A1PL";
static const char __pyx_k_ADJ_FUTPART_P3PL_NOUN_FUTPART_A3[] = "ADJ+FUTPART+P3PL$NOUN+FUTPART+A3PL+P3PL+NOM$NOUN+FUTPART+A3PL+P3SG+NOM$NOUN+FUTPART+A3PL+PNON+ACC$NOUN+FUTPART+A3SG+P3PL+NOM";
static const char __pyx_k_ADJ_FUTPART_P3SG_NOUN_FUTPART_A3[] = "ADJ+FUTPART+P3SG$NOUN+FUTPART+A3SG+P3SG+NOM";
static const char __pyx_k_ADJ_INTERJ_NOUN_PROP_A3SG_PNON_N[] = "ADJ$INTERJ$NOUN+PROP+A3SG+PNON+NOM";
static const char __pyx_k_ADJ_JUSTLIKE_DB_NOUN_ZERO_A3SG_P[] = "ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2SG$NOUN+ZERO+A3SG+P3SG";
static const char __pyx_k_ADJ_JUSTLIKE_NOUN_ZERO_A3SG_P3SG[] = "ADJ+JUSTLIKE$NOUN+ZERO+A3SG+P3SG+NOM";
static const char __pyx_k_ADJ_NOUN_A3SG_PNON_NOM_NOUN_PROP[] = "ADJ$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM";
static const char __pyx_k_ADJ_NOUN_A3SG_PNON_NOM_POSTP_PCD[] = "ADJ$NOUN+A3SG+PNON+NOM$POSTP+PCDAT";
static const char __pyx_k_ADV_NOUN_A3SG_PNON_NOM_POSTP_PCA[] = "ADV$NOUN+A3SG+PNON+NOM$POSTP+PCABL";
static const char __pyx_k_ADV_SINCE_VERB_ZERO_PRES_COP_A3S[] = "ADV+SINCE$VERB+ZERO+PRES+COP+A3SG";
static const char __pyx_k_ADV_WITHOUTHAVINGDONESO_NOUN_INF[] = "ADV+WITHOUTHAVINGDONESO$NOUN+INF2+A3SG+PNON+ABL";
static const char __pyx_k_AOR_A1SG_AOR_DB_ADJ_ZERO_DB_NOUN[] = "AOR+A1SG$AOR^DB+ADJ+ZERO^DB+NOUN+ZERO+A3SG+P1SG+NOM";
static const char __pyx_k_AOR_A3PL_AOR_DB_ADJ_ZERO_DB_NOUN[] = "AOR+A3PL$AOR^DB+ADJ+ZERO^DB+NOUN+ZERO+A3PL+PNON+NOM";
static const char __pyx_k_CONJ_NOUN_A3SG_PNON_NOM_VERB_POS[] = "CONJ$NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG";
static const char __pyx_k_CONJ_NOUN_PROP_A3SG_PNON_NOM_VER[] = "CONJ$NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG";
static const char __pyx_k_DET_NOUN_PROP_A3SG_PNON_NOM_PRON[] = "DET$NOUN+PROP+A3SG+PNON+NOM$PRON+DEMONSP+A3SG+PNON+NOM$PRON+PERS+A3SG+PNON+NOM";
static const char __pyx_k_DET_PRON_DEMONSP_A3SG_PNON_NOM_P[] = "DET$PRON+DEMONSP+A3SG+PNON+NOM$PRON+PERS+A3SG+PNON+NOM";
static const char __pyx_k_INF2_A3SG_P3SG_NOM_INF2_DB_ADJ_A[] = "INF2+A3SG+P3SG+NOM$INF2^DB+ADJ+ALMOST$";
static const char __pyx_k_INTERJ_NOUN_A3SG_PNON_NOM_VERB_P[] = "INTERJ$NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG";
static const char __pyx_k_Incompatible_checksums_0x_x_vs_0[] = "Incompatible checksums (0x%x vs (0x8b03656, 0xbc895e2, 0x8b1515e) = (morphologicalAnalyzer))";
static const char __pyx_k_MorphologicalAnalysis_FsmMorphol[] = "MorphologicalAnalysis.FsmMorphologicalAnalyzer";
static const char __pyx_k_MorphologicalDisambiguation_Auto[] = "MorphologicalDisambiguation/AutoDisambiguator.pyx";
static const char __pyx_k_NEG_AOR_A1SG_POS_DB_NOUN_INF2_A3[] = "NEG+AOR+A1SG$POS^DB+NOUN+INF2+A3SG+P1SG+NOM";
static const char __pyx_k_NEG_IMP_A2SG_POS_DB_NOUN_INF2_A3[] = "NEG+IMP+A2SG$POS^DB+NOUN+INF2+A3SG+PNON+NOM";
static const char __pyx_k_NEG_OPT_A3SG_POS_DB_NOUN_INF2_A3[] = "NEG+OPT+A3SG$POS^DB+NOUN+INF2+A3SG+PNON+DAT";
static const char __pyx_k_NEG_PAST_A1PL_NEG_DB_ADJ_PASTPAR[] = "NEG+PAST+A1PL$NEG^DB+ADJ+PASTPART+PNON$NEG^DB+NOUN+PASTPART+A3SG+PNON+NOM";
static const char __pyx_k_NEG_PROG2_NEG_DB_NOUN_INF_A3SG_P[] = "NEG+PROG2$NEG^DB+NOUN+INF+A3SG+PNON+LOC^DB+VERB+ZERO+PRES";
static const char __pyx_k_NESS_A3SG_PNON_NOM_ZERO_A3SG_PNO[] = "NESS+A3SG+PNON+NOM$ZERO+A3SG+PNON+NOM^DB+ADJ+FITFOR";
static const char __pyx_k_NOUN_A3SG_NOUN_PROP_A3SG_PRON_PE[] = "NOUN+A3SG$NOUN+PROP+A3SG$PRON+PERS+A1SG";
static const char __pyx_k_NOUN_A3SG_P1SG_NOM_NOUN_A3SG_PNO[] = "NOUN+A3SG+P1SG+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A1SG$PRON+PERS+A1SG+PNON+GEN$PRON+PERS+A1SG+PNON+NOM^DB+VERB+ZERO+PRES+A1SG";
static const char __pyx_k_NOUN_A3SG_P1SG_PRON_PERS_A1PL_PN[] = "NOUN+A3SG+P1SG$PRON+PERS+A1PL+PNON";
static const char __pyx_k_NOUN_A3SG_P1SG_PRON_PERS_A1SG_PN[] = "NOUN+A3SG+P1SG$PRON+PERS+A1SG+PNON";
static const char __pyx_k_NOUN_A3SG_P2SG_DAT_NOUN_A3SG_P3S[] = "NOUN+A3SG+P2SG+DAT$NOUN+A3SG+P3SG+DAT$VERB^DB+VERB+PASS+POS+OPT+A3SG";
static const char __pyx_k_NOUN_A3SG_P2SG_GEN_NOUN_A3SG_P3S[] = "NOUN+A3SG+P2SG+GEN$NOUN+A3SG+P3SG+GEN$VERB^DB+VERB+PASS+POS+IMP+A2PL";
static const char __pyx_k_NOUN_A3SG_P2SG_NOM_NOUN_A3SG_PNO[] = "NOUN+A3SG+P2SG+NOM$NOUN+A3SG+PNON+GEN$VERB+POS+IMP+A2PL$VERB^DB+VERB+PASS+POS+IMP+A2SG";
static const char __pyx_k_NOUN_A3SG_P2SG_NOUN_A3SG_PNON_PR[] = "NOUN+A3SG+P2SG$NOUN+A3SG+PNON$PRON+QUANTP+A3SG+P3SG";
static const char __pyx_k_NOUN_A3SG_P3SG_NOM_NOUN_A3SG_PNO[] = "NOUN+A3SG+P3SG+NOM$NOUN+A3SG+PNON+ACC$PRON+PERS+A1PL+PNON+ACC";
static const char __pyx_k_NOUN_A3SG_P3SG_NOM_NOUN_DB_ADJ_A[] = "NOUN+A3SG+P3SG+NOM$NOUN^DB+ADJ+ALMOST";
static const char __pyx_k_NOUN_A3SG_PNON_NOM_NOUN_PROP_A3S[] = "NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG";
static const char __pyx_k_NOUN_A3SG_PNON_NOM_NUM_CARD_VERB[] = "NOUN+A3SG+PNON+NOM$NUM+CARD$VERB+POS+IMP+A2SG";
static const char __pyx_k_NOUN_A3SG_PNON_NOM_QUES_PRES_A3S[] = "NOUN+A3SG+PNON+NOM$QUES+PRES+A3SG";
static const char __pyx_k_NOUN_PROP_A3SG_PNON_NOM_VERB_POS[] = "NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG";
static const char __pyx_k_NOUN_ZERO_A3PL_PNON_NOM_VERB_ZER[] = "NOUN+ZERO+A3PL+PNON+NOM$VERB+ZERO+PRES+A3PL";
static const char __pyx_k_NOUN_ZERO_A3SG_P1SG_NOM_VERB_ZER[] = "NOUN+ZERO+A3SG+P1SG+NOM$VERB+ZERO+PRES+A1SG";
static const char __pyx_k_P1PL_NOM_P1SG_NOM_DB_VERB_ZERO_P[] = "P1PL+NOM$P1SG+NOM^DB+VERB+ZERO+PRES+A1PL";
static const char __pyx_k_P1SG_NOM_PNON_NOM_DB_VERB_ZERO_P[] = "P1SG+NOM$PNON+NOM^DB+VERB+ZERO+PRES+A1SG";
static const char __pyx_k_P2PL_NOM_PNON_GEN_DB_VERB_ZERO_P[] = "P2PL+NOM$PNON+GEN^DB+VERB+ZERO+PRES+A1PL";
static const char __pyx_k_POS_DB_NOUN_INF2_A3SG_PNON_NOM_D[] = "POS^DB+NOUN+INF2+A3SG+PNON+NOM^DB+ADJ+WITH";
static const char __pyx_k_POS_DB_NOUN_INF3_A3SG_P2SG_NOM_P[] = "POS^DB+NOUN+INF3+A3SG+P2SG+NOM$POS^DB+NOUN+INF3+A3SG+PNON+GEN$RECIP+POS+IMP+A2PL";
static const char __pyx_k_POS_DB_NOUN_INF3_A3SG_PNON_DAT_R[] = "POS^DB+NOUN+INF3+A3SG+PNON+DAT$RECIP+POS+OPT+A3SG";
static const char __pyx_k_POS_DB_NOUN_INF3_A3SG_PNON_NOM_R[] = "POS^DB+NOUN+INF3+A3SG+PNON+NOM$RECIP+POS+IMP+A2SG";
static const char __pyx_k_POS_FUT_A3PL_POS_DB_NOUN_FUTPART[] = "POS+FUT+A3PL$POS^DB+NOUN+FUTPART+A3PL+PNON+NOM";
static const char __pyx_k_POS_PAST_A1PL_POS_DB_ADJ_PASTPAR[] = "POS+PAST+A1PL$POS^DB+ADJ+PASTPART+PNON$POS^DB+NOUN+PASTPART+A3SG+PNON+NOM";
static const char __pyx_k_POS_PROG2_A3SG_POS_DB_NOUN_INF_A[] = "POS+PROG2+A3SG$POS^DB+NOUN+INF+A3SG+PNON+LOC";
static const char __pyx_k_POS_PROG2_POS_DB_NOUN_INF_A3SG_P[] = "POS+PROG2$POS^DB+NOUN+INF+A3SG+PNON+LOC^DB+VERB+ZERO+PRES";
static const char __pyx_k_VERB_POS_AOR_VERB_DB_VERB_CAUS_P[] = "VERB+POS+AOR$VERB^DB+VERB+CAUS+POS";
static const char __pyx_k_VERB_POS_IMP_A2PL_VERB_DB_VERB_P[] = "VERB+POS+IMP+A2PL$VERB^DB+VERB+PASS+POS+IMP+A2SG";
static const char __pyx_k_ZERO_A3SG_PNON_NOM_DB_ADJ_FITFOR[] = "ZERO+A3SG+PNON+NOM^DB+ADJ+FITFOR";
static const char __pyx_k_ADJ_DB_VERB_ZERO_PRES_A3PL_NOUN_2[] = "ADJ^DB+VERB+ZERO+PRES+A3PL$NOUN+A3PL+PNON+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A3PL$NOUN+PROP+A3PL+PNON+NOM";
static const char __pyx_k_NOUN_A3SG_PNON_NOM_DB_ADV_SINCE_2[] = "NOUN+A3SG+PNON+NOM^DB+ADV+SINCE";
static const char __pyx_k_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_2[] = "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PAST+A1PL$VERB+POS+PAST+A1PL$VERB+POS^DB+ADJ+PASTPART+PNON$VERB+POS^DB+NOUN+PASTPART+A3SG+PNON+NOM";
static const char __pyx_k_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_3[] = "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+NARR+A3SG$VERB+POS+NARR+A3SG$VERB+POS+NARR^DB+ADJ+ZERO";
static const char __pyx_k_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_4[] = "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+COP+A3SG";
static const char __pyx_k_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_5[] = "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+NARR+A3SG+COP$VERB+POS+NARR+COP+A3SG";
static const char __pyx_k_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_6[] = "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO$QUES";
static const char __pyx_k_NUM_CARD_DB_NOUN_ZERO_A3SG_P3SG_2[] = "NUM+CARD^DB+NOUN+ZERO+A3SG+P3SG+NOM";
static const char __pyx_k_POS_NECES_POS_DB_NOUN_INF2_A3SG_2[] = "POS+NECES$POS^DB+NOUN+INF2+A3SG+PNON+NOM^DB+ADJ+WITH^DB+VERB+ZERO";
static const char __pyx_k_A3PL_PNON_NOM_A3SG_PNON_NOM_DB_V_2[] = "A3PL+PNON+NOM$A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A3PL";
static const char __pyx_k_ADJ_ADV_NOUN_A3SG_PNON_NOM_POSTP_2[] = "ADJ$ADV$NOUN+A3SG+PNON+NOM$POSTP+PCABL";
static const char __pyx_k_ADJ_DB_NOUN_ZERO_A3SG_P2SG_ADJ_D_2[] = "ADJ^DB+NOUN+ZERO+A3SG+P2SG$ADJ^DB+NOUN+ZERO+A3SG+P3SG$NUM+CARD^DB+NOUN+ZERO+A3SG+P2SG$NUM+CARD^DB+NOUN+ZERO+A3SG+P3SG";
static const char __pyx_k_ADJ_DB_NOUN_ZERO_A3SG_P3SG_NOM_A_2[] = "ADJ^DB+NOUN+ZERO+A3SG+P3SG+NOM$ADJ^DB+NOUN+ZERO+A3SG+PNON+ACC$NOUN+A3SG+P3SG+NOM$NOUN+A3SG+PNON+ACC";
static const char __pyx_k_ADJ_JUSTLIKE_DB_NOUN_ZERO_A3SG_P_2[] = "ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2SG$ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+PNON$NOUN+ZERO+A3SG+P3SG";
static const char __pyx_k_ADJ_JUSTLIKE_DB_NOUN_ZERO_A3SG_P_3[] = "ADJ+JUSTLIKE^DB$NOUN+ZERO+A3SG+P3SG+NOM^DB";
static const char __pyx_k_ADJ_JUSTLIKE_NOUN_ZERO_A3SG_P3SG_2[] = "ADJ+JUSTLIKE$NOUN+ZERO+A3SG+P3SG+NOM$NOUN+ZERO^DB+ADJ+ALMOST";
static const char __pyx_k_ADJ_NOUN_A3SG_PNON_NOM_NOUN_PROP_2[] = "ADJ$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG";
static const char __pyx_k_ADJ_NOUN_A3SG_PNON_NOM_NOUN_PROP_3[] = "ADJ$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM$POSTP+PCDAT";
static const char __pyx_k_MorphologicalDisambiguation_Auto_2[] = "MorphologicalDisambiguation.AutoDisambiguator";
static const char __pyx_k_NOUN_A3SG_P1SG_NOM_NOUN_A3SG_PNO_2[] = "NOUN+A3SG+P1SG+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A1SG$PRON+PERS+A1PL+PNON+GEN$PRON+PERS+A1PL+PNON+NOM^DB+VERB+ZERO+PRES+A1SG";
static const char __pyx_k_NOUN_A3SG_P2SG_NOM_NOUN_A3SG_PNO_2[] = "NOUN+A3SG+P2SG+NOM$NOUN+A3SG+PNON+GEN$VERB+POS+IMP+A2PL";
static const char __pyx_k_NOUN_A3SG_P2SG_NOM_NOUN_A3SG_PNO_3[] = "NOUN+A3SG+P2SG+NOM$NOUN+A3SG+PNON+GEN$PRON+QUANTP+A3PL+P3PL+GEN";
static const char __pyx_k_NOUN_A3SG_P3SG_NOM_NOUN_A3SG_PNO_2[] = "NOUN+A3SG+P3SG+NOM$NOUN+A3SG+PNON+ACC$PRON+PERS+A1SG+PNON+ACC";
static const char __pyx_k_POS_PROG2_POS_DB_NOUN_INF_A3SG_P_2[] = "POS+PROG2$POS^DB+NOUN+INF+A3SG+PNON+LOC^DB+VERB+ZERO";
static PyObject *__pyx_kp_u_;
static PyObject *__pyx_kp_u_A1PL_P1PL_NOM;
static PyObject *__pyx_kp_u_A1PL_P1PL_NOM_A3SG_P3SG_GEN_DB_V;
static PyObject *__pyx_n_s_A2SG;
static PyObject *__pyx_kp_u_A2SG_P2SG;
static PyObject *__pyx_kp_u_A2SG_P2SG_A3SG_P3SG;
static PyObject *__pyx_kp_u_A3PL_P2SG;
static PyObject *__pyx_kp_u_A3PL_P2SG_A3PL_P3PL_A3PL_P3SG_A3;
static PyObject *__pyx_kp_u_A3PL_P3PL_A3PL_P3SG_A3SG_P3PL;
static PyObject *__pyx_kp_u_A3PL_P3PL_NOM_A3PL_P3SG_NOM_A3PL;
static PyObject *__pyx_kp_u_A3PL_P3SG;
static PyObject *__pyx_kp_u_A3PL_P3SG_A3SG_P3PL_PROP_A3PL_P3;
static PyObject *__pyx_kp_u_A3PL_P3SG_NOM;
static PyObject *__pyx_kp_u_A3PL_PNON_NOM;
static PyObject *__pyx_kp_u_A3PL_PNON_NOM_A3SG_PNON_NOM_DB_V;
static PyObject *__pyx_kp_u_A3PL_PNON_NOM_A3SG_PNON_NOM_DB_V_2;
static PyObject *__pyx_kp_u_A3PL_PNON_NOM_DB_ADV_SINCE;
static PyObject *__pyx_kp_u_A3PL_PNON_NOM_DB_ADV_SINCE_A3PL;
static PyObject *__pyx_kp_u_A3PL_PNON_NOM_DB_VERB_ZERO_PRES;
static PyObject *__pyx_kp_u_A3SG_P3PL;
static PyObject *__pyx_kp_u_A3SG_P3PL_NOM;
static PyObject *__pyx_kp_u_A3SG_P3SG;
static PyObject *__pyx_kp_u_A3SG_PNON_ACC;
static PyObject *__pyx_kp_u_A3SG_PNON_ACC_PROP_A3SG_PNON_NOM;
static PyObject *__pyx_kp_u_A3SG_PROP_A3SG;
static PyObject *__pyx_n_s_ABLATIVE;
static PyObject *__pyx_kp_u_ABLE_DB_ADJ_FUTPART_PNON;
static PyObject *__pyx_kp_u_ABLE_FUT_A3SG;
static PyObject *__pyx_kp_u_ABLE_FUT_A3SG_ABLE_DB_ADJ_FUTPAR;
static PyObject *__pyx_n_u_ADJ;
static PyObject *__pyx_kp_u_ADJ_ADV;
static PyObject *__pyx_kp_u_ADJ_ADV_CONJ_PRON_QUESP_A3SG_PNO;
static PyObject *__pyx_kp_u_ADJ_ADV_DET_NUM_CARD;
static PyObject *__pyx_kp_u_ADJ_ADV_DET_POSTP_PCABL;
static PyObject *__pyx_kp_u_ADJ_ADV_NOUN_A3SG_PNON_NOM;
static PyObject *__pyx_kp_u_ADJ_ADV_NOUN_A3SG_PNON_NOM_NOUN;
static PyObject *__pyx_kp_u_ADJ_ADV_NOUN_A3SG_PNON_NOM_POSTP;
static PyObject *__pyx_kp_u_ADJ_ADV_NOUN_A3SG_PNON_NOM_POSTP_2;
static PyObject *__pyx_kp_u_ADJ_ADV_POSTP_PCABL;
static PyObject *__pyx_kp_u_ADJ_ADV_POSTP_PCABL_VERB_POS_IMP;
static PyObject *__pyx_kp_u_ADJ_ADV_POSTP_PCINS;
static PyObject *__pyx_kp_u_ADJ_ADV_VERB_POS_IMP_A2SG;
static PyObject *__pyx_kp_u_ADJ_AGT;
static PyObject *__pyx_kp_u_ADJ_AGT_DB_ADJ_JUSTLIKE_NOUN_AGT;
static PyObject *__pyx_kp_u_ADJ_AGT_NOUN_AGT_A3SG_PNON_NOM;
static PyObject *__pyx_kp_u_ADJ_ASIF;
static PyObject *__pyx_kp_u_ADJ_ASIF_ADV_LY;
static PyObject *__pyx_kp_u_ADJ_ASIF_ADV_LY_NOUN_ZERO_A3SG_P;
static PyObject *__pyx_kp_u_ADJ_DB;
static PyObject *__pyx_kp_u_ADJ_DB_NOUN_A3SG_PNON_NOM_DB;
static PyObject *__pyx_kp_u_ADJ_DB_NOUN_AGT_A3SG_PNON_NOM;
static PyObject *__pyx_kp_u_ADJ_DB_NOUN_AGT_A3SG_PNON_NOM_NO;
static PyObject *__pyx_kp_u_ADJ_DB_NOUN_ZERO;
static PyObject *__pyx_kp_u_ADJ_DB_NOUN_ZERO_A3PL_PNON_NOM_A;
static PyObject *__pyx_kp_u_ADJ_DB_NOUN_ZERO_A3SG_P2SG_ADJ_D;
static PyObject *__pyx_kp_u_ADJ_DB_NOUN_ZERO_A3SG_P2SG_ADJ_D_2;
static PyObject *__pyx_kp_u_ADJ_DB_NOUN_ZERO_A3SG_P2SG_NOM_D;
static PyObject *__pyx_kp_u_ADJ_DB_NOUN_ZERO_A3SG_P3SG_NOM;
static PyObject *__pyx_kp_u_ADJ_DB_NOUN_ZERO_A3SG_P3SG_NOM_A;
static PyObject *__pyx_kp_u_ADJ_DB_NOUN_ZERO_A3SG_P3SG_NOM_A_2;
static PyObject *__pyx_kp_u_ADJ_DB_NOUN_ZERO_NOUN;
static PyObject *__pyx_kp_u_ADJ_DB_POSTP_PCABL_DB;
static PyObject *__pyx_kp_u_ADJ_DB_VERB_ZERO;
static PyObject *__pyx_kp_u_ADJ_DB_VERB_ZERO_COND;
static PyObject *__pyx_kp_u_ADJ_DB_VERB_ZERO_COND_VERB_POS_D;
static PyObject *__pyx_kp_u_ADJ_DB_VERB_ZERO_NARR_A3SG_VERB;
static PyObject *__pyx_kp_u_ADJ_DB_VERB_ZERO_NOUN_A3SG_PNON;
static PyObject *__pyx_kp_u_ADJ_DB_VERB_ZERO_PAST_A1PL_VERB;
static PyObject *__pyx_kp_u_ADJ_DB_VERB_ZERO_POSTP_PCABL_DB;
static PyObject *__pyx_kp_u_ADJ_DB_VERB_ZERO_PRES_A1SG_NOUN;
static PyObject *__pyx_kp_u_ADJ_DB_VERB_ZERO_PRES_A3PL_NOUN;
static PyObject *__pyx_kp_u_ADJ_DB_VERB_ZERO_PRES_A3PL_NOUN_2;
static PyObject *__pyx_kp_u_ADJ_DB_VERB_ZERO_PRES_COP_A3SG;
static PyObject *__pyx_kp_u_ADJ_DB_VERB_ZERO_PRES_COP_A3SG_N;
static PyObject *__pyx_kp_u_ADJ_DB_VERB_ZERO_PRES_COP_A3SG_V;
static PyObject *__pyx_kp_u_ADJ_DB_VERB_ZERO_VERB_POS;
static PyObject *__pyx_kp_u_ADJ_DET_PRON_QUANTP_A3SG_P3SG_NO;
static PyObject *__pyx_kp_u_ADJ_FUTPART_P2PL;
static PyObject *__pyx_kp_u_ADJ_FUTPART_P2PL_NOUN_FUTPART_A3;
static PyObject *__pyx_kp_u_ADJ_FUTPART_P3PL;
static PyObject *__pyx_kp_u_ADJ_FUTPART_P3PL_NOUN_FUTPART_A3;
static PyObject *__pyx_kp_u_ADJ_FUTPART_P3SG;
static PyObject *__pyx_kp_u_ADJ_FUTPART_P3SG_NOUN_FUTPART_A3;
static PyObject *__pyx_kp_u_ADJ_INTERJ;
static PyObject *__pyx_kp_u_ADJ_INTERJ_NOUN_PROP_A3SG_PNON_N;
static PyObject *__pyx_kp_u_ADJ_JUSTLIKE_DB_NOUN_ZERO_A3SG_P;
static PyObject *__pyx_kp_u_ADJ_JUSTLIKE_DB_NOUN_ZERO_A3SG_P_2;
static PyObject *__pyx_kp_u_ADJ_JUSTLIKE_DB_NOUN_ZERO_A3SG_P_3;
static PyObject *__pyx_kp_u_ADJ_JUSTLIKE_NOUN_ZERO_A3SG_P3SG;
static PyObject *__pyx_kp_u_ADJ_JUSTLIKE_NOUN_ZERO_A3SG_P3SG_2;
static PyObject *__pyx_kp_u_ADJ_NOUN_A3SG_PNON_NOM;
static PyObject *__pyx_kp_u_ADJ_NOUN_A3SG_PNON_NOM_NOUN_PROP;
static PyObject *__pyx_kp_u_ADJ_NOUN_A3SG_PNON_NOM_NOUN_PROP_2;
static PyObject *__pyx_kp_u_ADJ_NOUN_A3SG_PNON_NOM_NOUN_PROP_3;
static PyObject *__pyx_kp_u_ADJ_NOUN_A3SG_PNON_NOM_POSTP_PCD;
static PyObject *__pyx_kp_u_ADJ_NOUN_A3SG_PNON_NOM_VERB_POS;
static PyObject *__pyx_kp_u_ADJ_NOUN_PROP_A3SG_PNON_NOM;
static PyObject *__pyx_kp_u_ADJ_NUM_ORD;
static PyObject *__pyx_kp_u_ADJ_PASTPART_P1PL;
static PyObject *__pyx_kp_u_ADJ_PASTPART_P1PL_NOUN_PASTPART;
static PyObject *__pyx_kp_u_ADJ_PASTPART_P1SG;
static PyObject *__pyx_kp_u_ADJ_PASTPART_P1SG_NOUN_PASTPART;
static PyObject *__pyx_kp_u_ADJ_PASTPART_P2PL;
static PyObject *__pyx_kp_u_ADJ_PASTPART_P2PL_NOUN_PASTPART;
static PyObject *__pyx_kp_u_ADJ_PASTPART_P3PL;
static PyObject *__pyx_kp_u_ADJ_PASTPART_P3PL_NOUN_PASTPART;
static PyObject *__pyx_kp_u_ADJ_PASTPART_P3SG;
static PyObject *__pyx_kp_u_ADJ_PASTPART_P3SG_NOUN_PASTPART;
static PyObject *__pyx_kp_u_ADJ_POSTP_PCABL;
static PyObject *__pyx_kp_u_ADJ_PRON_QUESP_A3SG_PNON_NOM;
static PyObject *__pyx_kp_u_ADJ_VERB_POS_IMP_A2SG;
static PyObject *__pyx_n_u_ADV;
static PyObject *__pyx_kp_u_ADV_CONJ;
static PyObject *__pyx_kp_u_ADV_LY;
static PyObject *__pyx_kp_u_ADV_NOUN_A3SG_PNON_NOM;
static PyObject *__pyx_kp_u_ADV_NOUN_A3SG_PNON_NOM_POSTP_PCA;
static PyObject *__pyx_kp_u_ADV_POSTP_PCINS;
static PyObject *__pyx_kp_u_ADV_PRON_QUANTP_A3SG_P3SG_NOM;
static PyObject *__pyx_kp_u_ADV_SINCE;
static PyObject *__pyx_kp_u_ADV_SINCE_VERB_ZERO_PRES_COP_A3S;
static PyObject *__pyx_kp_u_ADV_VERB_POS_IMP_A2SG;
static PyObject *__pyx_kp_u_ADV_WITHOUTHAVINGDONESO;
static PyObject *__pyx_kp_u_ADV_WITHOUTHAVINGDONESO_NOUN_INF;
static PyObject *__pyx_kp_u_AOR_A1SG;
static PyObject *__pyx_kp_u_AOR_A1SG_AOR_DB_ADJ_ZERO_DB_NOUN;
static PyObject *__pyx_kp_u_AOR_A2PL;
static PyObject *__pyx_kp_u_AOR_A2PL_AOR_DB_ADJ_ZERO_DB_ADJ;
static PyObject *__pyx_kp_u_AOR_A2SG;
static PyObject *__pyx_kp_u_AOR_A2SG_AOR_DB_ADJ_ZERO_DB_ADJ;
static PyObject *__pyx_kp_u_AOR_A3PL;
static PyObject *__pyx_kp_u_AOR_A3PL_AOR_DB_ADJ_ZERO_DB_NOUN;
static PyObject *__pyx_kp_u_AOR_A3SG;
static PyObject *__pyx_kp_u_AOR_A3SG_AOR_DB_ADJ_ZERO;
static PyObject *__pyx_kp_u_AOR_DB_ADJ_ZERO;
static PyObject *__pyx_n_s_AutoDisambiguator;
static PyObject *__pyx_n_u_CONJ;
static PyObject *__pyx_kp_u_CONJ_INTERJ;
static PyObject *__pyx_kp_u_CONJ_NOUN_A3SG_PNON_NOM_VERB_POS;
static PyObject *__pyx_kp_u_CONJ_NOUN_PROP_A3SG_PNON_NOM_VER;
static PyObject *__pyx_kp_u_CONJ_POSTP_PCNOM;
static PyObject *__pyx_kp_u_CONJ_VERB_POS_IMP_A2SG;
static PyObject *__pyx_kp_u_DATE_NUM_FRACTION;
static PyObject *__pyx_n_s_DATIVE;
static PyObject *__pyx_kp_u_DEMONSP_PERS;
static PyObject *__pyx_n_u_DET;
static PyObject *__pyx_kp_u_DET_NOUN_A3SG_PNON_NOM;
static PyObject *__pyx_kp_u_DET_NOUN_PROP_A3SG_PNON_NOM_PRON;
static PyObject *__pyx_kp_u_DET_PRON_DEMONSP_A3SG_PNON_NOM;
static PyObject *__pyx_kp_u_DET_PRON_DEMONSP_A3SG_PNON_NOM_P;
static PyObject *__pyx_kp_u_INF2_A3SG_P3SG_NOM;
static PyObject *__pyx_kp_u_INF2_A3SG_P3SG_NOM_INF2_DB_ADJ_A;
static PyObject *__pyx_n_s_INSTRUMENTAL;
static PyObject *__pyx_n_u_INTERJ;
static PyObject *__pyx_kp_u_INTERJ_NOUN_A3SG_PNON_NOM_VERB_P;
static PyObject *__pyx_kp_s_Incompatible_checksums_0x_x_vs_0;
static PyObject *__pyx_n_s_MorphologicalAnalysis_FsmMorphol;
static PyObject *__pyx_kp_s_MorphologicalDisambiguation_Auto;
static PyObject *__pyx_n_s_MorphologicalDisambiguation_Auto_2;
static PyObject *__pyx_n_s_MorphologicalTag;
static PyObject *__pyx_kp_u_NARR_A3SG;
static PyObject *__pyx_kp_u_NARR_A3SG_NARR_DB_ADJ_ZERO;
static PyObject *__pyx_kp_u_NARR_DB_ADJ_ZERO;
static PyObject *__pyx_kp_u_NEG_AOR_A1SG;
static PyObject *__pyx_kp_u_NEG_AOR_A1SG_POS_DB_NOUN_INF2_A3;
static PyObject *__pyx_kp_u_NEG_DB_ADJ_PASTPART_PNON;
static PyObject *__pyx_kp_u_NEG_FUT_A1SG;
static PyObject *__pyx_kp_u_NEG_FUT_A1SG_NEG_DB_ADJ_FUTPART;
static PyObject *__pyx_kp_u_NEG_FUT_A3SG;
static PyObject *__pyx_kp_u_NEG_FUT_A3SG_NEG_DB_ADJ_FUTPART;
static PyObject *__pyx_kp_u_NEG_IMP_A2SG_POS_DB_NOUN_INF2_A3;
static PyObject *__pyx_kp_u_NEG_OPT_A3SG_POS_DB_NOUN_INF2_A3;
static PyObject *__pyx_kp_u_NEG_PAST_A1PL;
static PyObject *__pyx_kp_u_NEG_PAST_A1PL_NEG_DB_ADJ_PASTPAR;
static PyObject *__pyx_kp_u_NEG_PROG2;
static PyObject *__pyx_kp_u_NEG_PROG2_NEG_DB_NOUN_INF_A3SG_P;
static PyObject *__pyx_kp_u_NESS_A3SG_PNON_NOM_ZERO_A3SG_PNO;
static PyObject *__pyx_n_u_NOUN;
static PyObject *__pyx_kp_u_NOUN_A3PL_PNON_NOM;
static PyObject *__pyx_kp_u_NOUN_A3SG_NOUN_PROP_A3SG_PRON_PE;
static PyObject *__pyx_kp_u_NOUN_A3SG_P1SG_DAT;
static PyObject *__pyx_kp_u_NOUN_A3SG_P1SG_DAT_VERB_NEG_IMP;
static PyObject *__pyx_kp_u_NOUN_A3SG_P1SG_NOM;
static PyObject *__pyx_kp_u_NOUN_A3SG_P1SG_NOM_NOUN_A3SG_PNO;
static PyObject *__pyx_kp_u_NOUN_A3SG_P1SG_NOM_NOUN_A3SG_PNO_2;
static PyObject *__pyx_kp_u_NOUN_A3SG_P1SG_PRON_PERS_A1PL_PN;
static PyObject *__pyx_kp_u_NOUN_A3SG_P1SG_PRON_PERS_A1SG_PN;
static PyObject *__pyx_kp_u_NOUN_A3SG_P2SG_DAT;
static PyObject *__pyx_kp_u_NOUN_A3SG_P2SG_DAT_NOUN_A3SG_P3S;
static PyObject *__pyx_kp_u_NOUN_A3SG_P2SG_GEN_NOUN_A3SG_P3S;
static PyObject *__pyx_kp_u_NOUN_A3SG_P2SG_NOM;
static PyObject *__pyx_kp_u_NOUN_A3SG_P2SG_NOM_NOUN_A3SG_PNO;
static PyObject *__pyx_kp_u_NOUN_A3SG_P2SG_NOM_NOUN_A3SG_PNO_2;
static PyObject *__pyx_kp_u_NOUN_A3SG_P2SG_NOM_NOUN_A3SG_PNO_3;
static PyObject *__pyx_kp_u_NOUN_A3SG_P2SG_NOUN_A3SG_PNON_PR;
static PyObject *__pyx_kp_u_NOUN_A3SG_P3SG;
static PyObject *__pyx_kp_u_NOUN_A3SG_P3SG_DAT;
static PyObject *__pyx_kp_u_NOUN_A3SG_P3SG_GEN;
static PyObject *__pyx_kp_u_NOUN_A3SG_P3SG_NOM;
static PyObject *__pyx_kp_u_NOUN_A3SG_P3SG_NOM_NOUN_A3SG_PNO;
static PyObject *__pyx_kp_u_NOUN_A3SG_P3SG_NOM_NOUN_A3SG_PNO_2;
static PyObject *__pyx_kp_u_NOUN_A3SG_P3SG_NOM_NOUN_DB_ADJ_A;
static PyObject *__pyx_kp_u_NOUN_A3SG_PNON_DAT;
static PyObject *__pyx_kp_u_NOUN_A3SG_PNON_DAT_VERB_POS_OPT;
static PyObject *__pyx_kp_u_NOUN_A3SG_PNON_GEN;
static PyObject *__pyx_kp_u_NOUN_A3SG_PNON_NOM;
static PyObject *__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB;
static PyObject *__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_ADJ_AGT;
static PyObject *__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_ADV_SINCE;
static PyObject *__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_ADV_SINCE_2;
static PyObject *__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO;
static PyObject *__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_2;
static PyObject *__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_3;
static PyObject *__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_4;
static PyObject *__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_5;
static PyObject *__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_6;
static PyObject *__pyx_kp_u_NOUN_A3SG_PNON_NOM_NOUN_PROP_A3S;
static PyObject *__pyx_kp_u_NOUN_A3SG_PNON_NOM_NUM_CARD_VERB;
static PyObject *__pyx_kp_u_NOUN_A3SG_PNON_NOM_POSTP_PCABL;
static PyObject *__pyx_kp_u_NOUN_A3SG_PNON_NOM_POSTP_PCDAT;
static PyObject *__pyx_kp_u_NOUN_A3SG_PNON_NOM_QUES_PRES_A3S;
static PyObject *__pyx_kp_u_NOUN_A3SG_PNON_NOM_VERB_POS_IMP;
static PyObject *__pyx_kp_u_NOUN_A3SG_PNON_PRON_QUANTP_A3PL;
static PyObject *__pyx_kp_u_NOUN_A3SG_PNON_PRON_QUANTP_A3SG;
static PyObject *__pyx_kp_u_NOUN_A3SG_PRON_PERS_A1PL;
static PyObject *__pyx_kp_u_NOUN_A3SG_PRON_PERS_A1SG;
static PyObject *__pyx_kp_u_NOUN_AGT_A3SG_P3SG_NOM;
static PyObject *__pyx_kp_u_NOUN_AGT_A3SG_PNON_NOM;
static PyObject *__pyx_kp_u_NOUN_FUTPART_A3PL_P3SG_NOM;
static PyObject *__pyx_kp_u_NOUN_FUTPART_A3SG_P2PL_NOM;
static PyObject *__pyx_kp_u_NOUN_FUTPART_A3SG_P3PL_NOM;
static PyObject *__pyx_kp_u_NOUN_FUTPART_A3SG_P3SG_NOM;
static PyObject *__pyx_kp_u_NOUN_NUM_CARD_DB_NOUN_ZERO;
static PyObject *__pyx_kp_u_NOUN_PASTPART_A3PL_P3SG_NOM;
static PyObject *__pyx_kp_u_NOUN_PASTPART_A3SG_P1SG_NOM;
static PyObject *__pyx_kp_u_NOUN_PASTPART_A3SG_P2PL_NOM;
static PyObject *__pyx_kp_u_NOUN_PASTPART_A3SG_P3PL_NOM;
static PyObject *__pyx_kp_u_NOUN_PASTPART_A3SG_P3SG_NOM;
static PyObject *__pyx_kp_u_NOUN_PROP;
static PyObject *__pyx_kp_u_NOUN_PROP_A3SG_PNON_NOM;
static PyObject *__pyx_kp_u_NOUN_PROP_A3SG_PNON_NOM_VERB_POS;
static PyObject *__pyx_kp_u_NOUN_PROP_PRON_QUESP;
static PyObject *__pyx_kp_u_NOUN_ZERO_A3PL_PNON_NOM;
static PyObject *__pyx_kp_u_NOUN_ZERO_A3PL_PNON_NOM_VERB_ZER;
static PyObject *__pyx_kp_u_NOUN_ZERO_A3SG_P1SG_NOM_VERB_ZER;
static PyObject *__pyx_kp_u_NOUN_ZERO_A3SG_P3SG;
static PyObject *__pyx_kp_u_NOUN_ZERO_A3SG_P3SG_NOM;
static PyObject *__pyx_kp_u_NOUN_ZERO_A3SG_P3SG_NOM_DB;
static PyObject *__pyx_n_u_NUM;
static PyObject *__pyx_kp_u_NUM_CARD;
static PyObject *__pyx_kp_u_NUM_CARD_DB_NOUN_ZERO_A3SG_P3SG;
static PyObject *__pyx_kp_u_NUM_CARD_DB_NOUN_ZERO_A3SG_P3SG_2;
static PyObject *__pyx_kp_u_NUM_CARD_VERB_POS_IMP_A2SG;
static PyObject *__pyx_kp_u_NUM_FRACTION;
static PyObject *__pyx_kp_u_P1PL_NOM;
static PyObject *__pyx_kp_u_P1PL_NOM_P1SG_NOM_DB_VERB_ZERO_P;
static PyObject *__pyx_kp_u_P1SG_NOM;
static PyObject *__pyx_kp_u_P1SG_NOM_PNON_NOM_DB_VERB_ZERO_P;
static PyObject *__pyx_kp_u_P2PL_NOM;
static PyObject *__pyx_kp_u_P2PL_NOM_PNON_GEN_DB_VERB_ZERO_P;
static PyObject *__pyx_n_s_P2SG;
static PyObject *__pyx_n_u_P2SG;
static PyObject *__pyx_kp_u_P2SG_NOM;
static PyObject *__pyx_kp_u_P2SG_NOM_PNON_GEN;
static PyObject *__pyx_kp_u_P2SG_P3PL_P3SG;
static PyObject *__pyx_kp_u_P2SG_P3SG;
static PyObject *__pyx_kp_u_P2SG_PNON;
static PyObject *__pyx_n_u_P3PL;
static PyObject *__pyx_kp_u_P3PL_NOM;
static PyObject *__pyx_kp_u_P3PL_NOM_P3SG_NOM_PNON_ACC;
static PyObject *__pyx_n_u_P3SG;
static PyObject *__pyx_kp_u_P3SG_NOM;
static PyObject *__pyx_kp_u_P3SG_NOM_PNON_ACC;
static PyObject *__pyx_kp_u_P3SG_PNON;
static PyObject *__pyx_n_u_PERS;
static PyObject *__pyx_n_u_PNON;
static PyObject *__pyx_kp_u_PNON_ACC;
static PyObject *__pyx_kp_u_PNON_GEN;
static PyObject *__pyx_kp_u_PNON_NOM_DB_VERB_ZERO_PRES_A1SG;
static PyObject *__pyx_kp_u_POSTP_PCABL;
static PyObject *__pyx_kp_u_POSTP_PCABL_DB;
static PyObject *__pyx_kp_u_POSTP_PCDAT;
static PyObject *__pyx_kp_u_POSTP_PCINS;
static PyObject *__pyx_kp_u_POSTP_PCNOM;
static PyObject *__pyx_kp_u_POS_DB_ADJ_FUTPART_PNON;
static PyObject *__pyx_kp_u_POS_DB_NOUN_INF2_A3SG_PNON_DAT;
static PyObject *__pyx_kp_u_POS_DB_NOUN_INF2_A3SG_PNON_NOM;
static PyObject *__pyx_kp_u_POS_DB_NOUN_INF2_A3SG_PNON_NOM_D;
static PyObject *__pyx_kp_u_POS_DB_NOUN_INF3_A3SG_P2SG_NOM;
static PyObject *__pyx_kp_u_POS_DB_NOUN_INF3_A3SG_P2SG_NOM_P;
static PyObject *__pyx_kp_u_POS_DB_NOUN_INF3_A3SG_PNON_DAT;
static PyObject *__pyx_kp_u_POS_DB_NOUN_INF3_A3SG_PNON_DAT_R;
static PyObject *__pyx_kp_u_POS_DB_NOUN_INF3_A3SG_PNON_GEN;
static PyObject *__pyx_kp_u_POS_DB_NOUN_INF3_A3SG_PNON_NOM;
static PyObject *__pyx_kp_u_POS_DB_NOUN_INF3_A3SG_PNON_NOM_R;
static PyObject *__pyx_kp_u_POS_FUT_A1SG;
static PyObject *__pyx_kp_u_POS_FUT_A1SG_POS_DB_ADJ_FUTPART;
static PyObject *__pyx_kp_u_POS_FUT_A3PL;
static PyObject *__pyx_kp_u_POS_FUT_A3PL_POS_DB_NOUN_FUTPART;
static PyObject *__pyx_kp_u_POS_FUT_A3SG;
static PyObject *__pyx_kp_u_POS_FUT_A3SG_POS_DB_ADJ_FUTPART;
static PyObject *__pyx_kp_u_POS_NECES;
static PyObject *__pyx_kp_u_POS_NECES_A3SG;
static PyObject *__pyx_kp_u_POS_NECES_A3SG_POS_DB_NOUN_INF2;
static PyObject *__pyx_kp_u_POS_NECES_POS_DB_NOUN_INF2_A3SG;
static PyObject *__pyx_kp_u_POS_NECES_POS_DB_NOUN_INF2_A3SG_2;
static PyObject *__pyx_kp_u_POS_PAST_A1PL;
static PyObject *__pyx_kp_u_POS_PAST_A1PL_POS_DB_ADJ_PASTPAR;
static PyObject *__pyx_kp_u_POS_PROG2;
static PyObject *__pyx_kp_u_POS_PROG2_A3SG;
static PyObject *__pyx_kp_u_POS_PROG2_A3SG_POS_DB_NOUN_INF_A;
static PyObject *__pyx_kp_u_POS_PROG2_POS_DB_NOUN_INF_A3SG_P;
static PyObject *__pyx_kp_u_POS_PROG2_POS_DB_NOUN_INF_A3SG_P_2;
static PyObject *__pyx_kp_u_PRON_DEMONSP_A3SG_PNON_NOM;
static PyObject *__pyx_kp_u_PRON_PERS_A1PL;
static PyObject *__pyx_kp_u_PRON_PERS_A1PL_PNON;
static PyObject *__pyx_kp_u_PRON_PERS_A1PL_PNON_ACC;
static PyObject *__pyx_kp_u_PRON_PERS_A1PL_PNON_GEN;
static PyObject *__pyx_kp_u_PRON_PERS_A1SG;
static PyObject *__pyx_kp_u_PRON_PERS_A1SG_PNON;
static PyObject *__pyx_kp_u_PRON_PERS_A1SG_PNON_ACC;
static PyObject *__pyx_kp_u_PRON_PERS_A1SG_PNON_GEN;
static PyObject *__pyx_kp_u_PRON_PERS_A3SG_PNON_NOM;
static PyObject *__pyx_kp_u_PRON_QUANTP_A3PL_P3PL;
static PyObject *__pyx_kp_u_PRON_QUANTP_A3PL_P3PL_GEN;
static PyObject *__pyx_kp_u_PRON_QUANTP_A3SG_P3SG;
static PyObject *__pyx_kp_u_PRON_QUESP;
static PyObject *__pyx_kp_u_PRON_QUESP_A3SG_PNON_NOM;
static PyObject *__pyx_n_u_PROP;
static PyObject *__pyx_kp_u_PROP_A3PL_P3PL;
static PyObject *__pyx_kp_u_PROP_A3PL_PNON_NOM;
static PyObject *__pyx_kp_u_PROP_A3SG;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_n_u_QUES;
static PyObject *__pyx_kp_u_QUES_PRES_A3SG;
static PyObject *__pyx_n_u_VERB;
static PyObject *__pyx_kp_u_VERB_DB_VERB_PASS;
static PyObject *__pyx_kp_u_VERB_DB_VERB_PASS_POS;
static PyObject *__pyx_kp_u_VERB_POS;
static PyObject *__pyx_kp_u_VERB_POS_AOR;
static PyObject *__pyx_kp_u_VERB_POS_AOR_VERB_DB_VERB_CAUS_P;
static PyObject *__pyx_kp_u_VERB_POS_DB_NOUN_INF2_A3SG_PNON;
static PyObject *__pyx_kp_u_VERB_POS_IMP_A2PL;
static PyObject *__pyx_kp_u_VERB_POS_IMP_A2PL_VERB_DB_VERB_P;
static PyObject *__pyx_kp_u_VERB_POS_NARR_A3SG;
static PyObject *__pyx_kp_u_VERB_POS_NARR_COP_A3SG;
static PyObject *__pyx_kp_u_VERB_POS_NARR_DB_ADJ_ZERO;
static PyObject *__pyx_kp_u_VERB_POS_PAST_A1PL;
static PyObject *__pyx_kp_u_VERB_REFLEX_VERB_DB_VERB_PASS;
static PyObject *__pyx_kp_u_VERB_ZERO_PRES_A1SG;
static PyObject *__pyx_kp_u_VERB_ZERO_PRES_COP_A3SG;
static PyObject *__pyx_kp_u_ZERO_A3SG_PNON_NOM_DB_ADJ_FITFOR;
static PyObject *__pyx_kp_u__2;
static PyObject *__pyx_n_u_allmadk;
static PyObject *__pyx_n_u_artk;
static PyObject *__pyx_n_u_ay;
static PyObject *__pyx_n_s_caseDisambiguator;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_containsTag;
static PyObject *__pyx_n_s_containsTwoNeOrYa;
static PyObject *__pyx_n_s_correctParses;
static PyObject *__pyx_n_s_count;
static PyObject *__pyx_n_u_da;
static PyObject *__pyx_n_s_defaultCase;
static PyObject *__pyx_n_u_deil;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_u_dk;
static PyObject *__pyx_n_u_eksik;
static PyObject *__pyx_n_s_fsmPars;
static PyObject *__pyx_n_s_fsmParse;
static PyObject *__pyx_n_s_fsmParseList;
static PyObject *__pyx_n_s_fsmParses;
static PyObject *__pyx_n_u_gerek;
static PyObject *__pyx_n_s_getFinalPos;
static PyObject *__pyx_n_s_getFsmParse;
static PyObject *__pyx_n_s_getName;
static PyObject *__pyx_n_s_getSurfaceForm;
static PyObject *__pyx_n_s_getWord;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_hasPreviousWordTag;
static PyObject *__pyx_n_s_i;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_index;
static PyObject *__pyx_n_s_isAnyWordSecondPerson;
static PyObject *__pyx_n_s_isBeforeLastWord;
static PyObject *__pyx_n_s_isCapitalWord;
static PyObject *__pyx_n_s_isFirstWord;
static PyObject *__pyx_n_s_isNextWordNoun;
static PyObject *__pyx_n_s_isNextWordNounOrAdjective;
static PyObject *__pyx_n_s_isNextWordNum;
static PyObject *__pyx_n_s_isNoun;
static PyObject *__pyx_n_s_isPlural;
static PyObject *__pyx_n_s_isPossessivePlural;
static PyObject *__pyx_n_u_kar;
static PyObject *__pyx_n_s_lastWord;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_map;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_u_ne;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_nextParseList;
static PyObject *__pyx_n_s_nextWord;
static PyObject *__pyx_n_s_nextWordExists;
static PyObject *__pyx_n_s_nextWordPos;
static PyObject *__pyx_n_u_orta;
static PyObject *__pyx_n_s_parseString;
static PyObject *__pyx_n_s_parsesWithoutPrefixAndSuffix;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_previousWord;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_AutoDisambiguator;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_u_rahat;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_root;
static PyObject *__pyx_n_s_selectCaseForParseString;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_u_sk;
static PyObject *__pyx_n_u_sre;
static PyObject *__pyx_n_s_staticmethod;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_surfaceForm;
static PyObject *__pyx_n_s_tag;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_u_var;
static PyObject *__pyx_n_u_vasat;
static PyObject *__pyx_n_s_word;
static PyObject *__pyx_n_u_ya;
static PyObject *__pyx_n_u_yl;
static PyObject *__pyx_n_u_yok;
static PyObject *__pyx_n_u_zaman;
static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_isAnyWordSecondPerson(PyObject *__pyx_v_index, PyObject *__pyx_v_correctParses); /* proto */
static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_2isPossessivePlural(PyObject *__pyx_v_index, PyObject *__pyx_v_correctParses); /* proto */
static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_4nextWordPos(struct __pyx_obj_21MorphologicalAnalysis_12FsmParseList_FsmParseList *__pyx_v_nextParseList); /* proto */
static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_6isBeforeLastWord(PyObject *__pyx_v_index, PyObject *__pyx_v_fsmParses); /* proto */
static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_8nextWordExists(PyObject *__pyx_v_index, PyObject *__pyx_v_fsmParses); /* proto */
static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_10isNextWordNoun(PyObject *__pyx_v_index, PyObject *__pyx_v_fsmParses); /* proto */
static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_12isNextWordNum(PyObject *__pyx_v_index, PyObject *__pyx_v_fsmParses); /* proto */
static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_14isNextWordNounOrAdjective(PyObject *__pyx_v_index, PyObject *__pyx_v_fsmParses); /* proto */
static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_16isFirstWord(PyObject *__pyx_v_index); /* proto */
static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_18containsTwoNeOrYa(PyObject *__pyx_v_fsmParses, PyObject *__pyx_v_word); /* proto */
static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_20hasPreviousWordTag(PyObject *__pyx_v_index, PyObject *__pyx_v_correctParses, PyObject *__pyx_v_tag); /* proto */
static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_22selectCaseForParseString(PyObject *__pyx_v_parseString, PyObject *__pyx_v_index, PyObject *__pyx_v_fsmParses, PyObject *__pyx_v_correctParses); /* proto */
static struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_24caseDisambiguator(PyObject *__pyx_v_index, PyObject *__pyx_v_fsmParses, PyObject *__pyx_v_correctParses); /* proto */
static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_26__reduce_cython__(struct __pyx_obj_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_28__setstate_cython__(struct __pyx_obj_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator___pyx_unpickle_AutoDisambiguator(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_145765974;
static PyObject *__pyx_int_145838430;
static PyObject *__pyx_int_197694946;
static PyObject *__pyx_int_neg_1;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__20;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__24;
static PyObject *__pyx_tuple__26;
static PyObject *__pyx_tuple__28;
static PyObject *__pyx_tuple__30;
static PyObject *__pyx_codeobj__5;
static PyObject *__pyx_codeobj__7;
static PyObject *__pyx_codeobj__9;
static PyObject *__pyx_codeobj__11;
static PyObject *__pyx_codeobj__13;
static PyObject *__pyx_codeobj__15;
static PyObject *__pyx_codeobj__17;
static PyObject *__pyx_codeobj__19;
static PyObject *__pyx_codeobj__21;
static PyObject *__pyx_codeobj__23;
static PyObject *__pyx_codeobj__25;
static PyObject *__pyx_codeobj__27;
static PyObject *__pyx_codeobj__29;
static PyObject *__pyx_codeobj__31;
/* Late includes */

/* "MorphologicalDisambiguation/AutoDisambiguator.pyx":8
 * 
 *     @staticmethod
 *     def isAnyWordSecondPerson(index: int, correctParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         cdef int count, i
 *         count = 0
 */

/* Python wrapper */
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_1isAnyWordSecondPerson(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_1isAnyWordSecondPerson = {"isAnyWordSecondPerson", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_1isAnyWordSecondPerson, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_1isAnyWordSecondPerson(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_correctParses = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("isAnyWordSecondPerson (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_correctParses,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_correctParses)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("isAnyWordSecondPerson", 1, 2, 2, 1); __PYX_ERR(0, 8, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "isAnyWordSecondPerson") < 0)) __PYX_ERR(0, 8, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_index = values[0];
    __pyx_v_correctParses = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("isAnyWordSecondPerson", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 8, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.isAnyWordSecondPerson", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_correctParses), (&PyList_Type), 1, "correctParses", 1))) __PYX_ERR(0, 8, __pyx_L1_error)
  __pyx_r = __pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_isAnyWordSecondPerson(__pyx_v_index, __pyx_v_correctParses);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_isAnyWordSecondPerson(PyObject *__pyx_v_index, PyObject *__pyx_v_correctParses) {
  int __pyx_v_count;
  int __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  long __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("isAnyWordSecondPerson", 0);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":10
 *     def isAnyWordSecondPerson(index: int, correctParses: list) -> bool:
 *         cdef int count, i
 *         count = 0             # <<<<<<<<<<<<<<
 *         for i in range(index - 1, -1, -1):
 *             if correctParses[i].containsTag(MorphologicalTag.A2SG) or \
 */
  __pyx_v_count = 0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":11
 *         cdef int count, i
 *         count = 0
 *         for i in range(index - 1, -1, -1):             # <<<<<<<<<<<<<<
 *             if correctParses[i].containsTag(MorphologicalTag.A2SG) or \
 *                     correctParses[i].containsTag(MorphologicalTag.P2SG):
 */
  __pyx_t_1 = __Pyx_PyInt_SubtractObjC(__pyx_v_index, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_long(__pyx_t_1); if (unlikely((__pyx_t_2 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (__pyx_t_3 = __pyx_t_2; __pyx_t_3 > -1; __pyx_t_3-=1) {
    __pyx_v_i = __pyx_t_3;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":12
 *         count = 0
 *         for i in range(index - 1, -1, -1):
 *             if correctParses[i].containsTag(MorphologicalTag.A2SG) or \             # <<<<<<<<<<<<<<
 *                     correctParses[i].containsTag(MorphologicalTag.P2SG):
 *                 count = count + 1
 */
    if (unlikely(__pyx_v_correctParses == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 12, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_GetItemInt_List(__pyx_v_correctParses, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_containsTag); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_MorphologicalTag); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_A2SG); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
      }
    }
    __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_5, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 12, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (!__pyx_t_8) {
    } else {
      __pyx_t_4 = __pyx_t_8;
      goto __pyx_L6_bool_binop_done;
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":13
 *         for i in range(index - 1, -1, -1):
 *             if correctParses[i].containsTag(MorphologicalTag.A2SG) or \
 *                     correctParses[i].containsTag(MorphologicalTag.P2SG):             # <<<<<<<<<<<<<<
 *                 count = count + 1
 *         return count >= 1
 */
    if (unlikely(__pyx_v_correctParses == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 13, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_List(__pyx_v_correctParses, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_containsTag); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_MorphologicalTag); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_P2SG); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_5);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 13, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_4 = __pyx_t_8;
    __pyx_L6_bool_binop_done:;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":12
 *         count = 0
 *         for i in range(index - 1, -1, -1):
 *             if correctParses[i].containsTag(MorphologicalTag.A2SG) or \             # <<<<<<<<<<<<<<
 *                     correctParses[i].containsTag(MorphologicalTag.P2SG):
 *                 count = count + 1
 */
    if (__pyx_t_4) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":14
 *             if correctParses[i].containsTag(MorphologicalTag.A2SG) or \
 *                     correctParses[i].containsTag(MorphologicalTag.P2SG):
 *                 count = count + 1             # <<<<<<<<<<<<<<
 *         return count >= 1
 * 
 */
      __pyx_v_count = (__pyx_v_count + 1);

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":12
 *         count = 0
 *         for i in range(index - 1, -1, -1):
 *             if correctParses[i].containsTag(MorphologicalTag.A2SG) or \             # <<<<<<<<<<<<<<
 *                     correctParses[i].containsTag(MorphologicalTag.P2SG):
 *                 count = count + 1
 */
    }
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":15
 *                     correctParses[i].containsTag(MorphologicalTag.P2SG):
 *                 count = count + 1
 *         return count >= 1             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong((__pyx_v_count >= 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":8
 * 
 *     @staticmethod
 *     def isAnyWordSecondPerson(index: int, correctParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         cdef int count, i
 *         count = 0
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.isAnyWordSecondPerson", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "MorphologicalDisambiguation/AutoDisambiguator.pyx":18
 * 
 *     @staticmethod
 *     def isPossessivePlural(index: int, correctParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         cdef int i
 *         for i in range(index - 1, -1, -1):
 */

/* Python wrapper */
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_3isPossessivePlural(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_3isPossessivePlural = {"isPossessivePlural", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_3isPossessivePlural, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_3isPossessivePlural(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_correctParses = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("isPossessivePlural (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_correctParses,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_correctParses)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("isPossessivePlural", 1, 2, 2, 1); __PYX_ERR(0, 18, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "isPossessivePlural") < 0)) __PYX_ERR(0, 18, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_index = values[0];
    __pyx_v_correctParses = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("isPossessivePlural", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 18, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.isPossessivePlural", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_correctParses), (&PyList_Type), 1, "correctParses", 1))) __PYX_ERR(0, 18, __pyx_L1_error)
  __pyx_r = __pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_2isPossessivePlural(__pyx_v_index, __pyx_v_correctParses);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_2isPossessivePlural(PyObject *__pyx_v_index, PyObject *__pyx_v_correctParses) {
  int __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  long __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("isPossessivePlural", 0);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":20
 *     def isPossessivePlural(index: int, correctParses: list) -> bool:
 *         cdef int i
 *         for i in range(index - 1, -1, -1):             # <<<<<<<<<<<<<<
 *             if correctParses[i].isNoun():
 *                 return correctParses[i].isPlural()
 */
  __pyx_t_1 = __Pyx_PyInt_SubtractObjC(__pyx_v_index, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_long(__pyx_t_1); if (unlikely((__pyx_t_2 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 20, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (__pyx_t_3 = __pyx_t_2; __pyx_t_3 > -1; __pyx_t_3-=1) {
    __pyx_v_i = __pyx_t_3;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":21
 *         cdef int i
 *         for i in range(index - 1, -1, -1):
 *             if correctParses[i].isNoun():             # <<<<<<<<<<<<<<
 *                 return correctParses[i].isPlural()
 *         return False
 */
    if (unlikely(__pyx_v_correctParses == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 21, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_GetItemInt_List(__pyx_v_correctParses, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 21, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_isNoun); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 21, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 21, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 21, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_6) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":22
 *         for i in range(index - 1, -1, -1):
 *             if correctParses[i].isNoun():
 *                 return correctParses[i].isPlural()             # <<<<<<<<<<<<<<
 *         return False
 * 
 */
      __Pyx_XDECREF(__pyx_r);
      if (unlikely(__pyx_v_correctParses == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 22, __pyx_L1_error)
      }
      __pyx_t_5 = __Pyx_GetItemInt_List(__pyx_v_correctParses, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 22, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_isPlural); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 22, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 22, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_r = __pyx_t_1;
      __pyx_t_1 = 0;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":21
 *         cdef int i
 *         for i in range(index - 1, -1, -1):
 *             if correctParses[i].isNoun():             # <<<<<<<<<<<<<<
 *                 return correctParses[i].isPlural()
 *         return False
 */
    }
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":23
 *             if correctParses[i].isNoun():
 *                 return correctParses[i].isPlural()
 *         return False             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(Py_False);
  __pyx_r = Py_False;
  goto __pyx_L0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":18
 * 
 *     @staticmethod
 *     def isPossessivePlural(index: int, correctParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         cdef int i
 *         for i in range(index - 1, -1, -1):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.isPossessivePlural", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "MorphologicalDisambiguation/AutoDisambiguator.pyx":26
 * 
 *     @staticmethod
 *     def nextWordPos(nextParseList: FsmParseList) -> str:             # <<<<<<<<<<<<<<
 *         cdef CounterHashMap _map
 *         cdef int i
 */

/* Python wrapper */
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_5nextWordPos(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_5nextWordPos = {"nextWordPos", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_5nextWordPos, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_5nextWordPos(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_21MorphologicalAnalysis_12FsmParseList_FsmParseList *__pyx_v_nextParseList = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("nextWordPos (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_nextParseList,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_nextParseList)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "nextWordPos") < 0)) __PYX_ERR(0, 26, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_nextParseList = ((struct __pyx_obj_21MorphologicalAnalysis_12FsmParseList_FsmParseList *)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("nextWordPos", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 26, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.nextWordPos", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_nextParseList), __pyx_ptype_21MorphologicalAnalysis_12FsmParseList_FsmParseList, 1, "nextParseList", 0))) __PYX_ERR(0, 26, __pyx_L1_error)
  __pyx_r = __pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_4nextWordPos(__pyx_v_nextParseList);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_4nextWordPos(struct __pyx_obj_21MorphologicalAnalysis_12FsmParseList_FsmParseList *__pyx_v_nextParseList) {
  struct __pyx_obj_13DataStructure_14CounterHashMap_CounterHashMap *__pyx_v__map = 0;
  int __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("nextWordPos", 0);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":29
 *         cdef CounterHashMap _map
 *         cdef int i
 *         _map = CounterHashMap()             # <<<<<<<<<<<<<<
 *         for i in range(nextParseList.size()):
 *             _map.put(nextParseList.getFsmParse(i).getPos())
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_13DataStructure_14CounterHashMap_CounterHashMap)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__map = ((struct __pyx_obj_13DataStructure_14CounterHashMap_CounterHashMap *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":30
 *         cdef int i
 *         _map = CounterHashMap()
 *         for i in range(nextParseList.size()):             # <<<<<<<<<<<<<<
 *             _map.put(nextParseList.getFsmParse(i).getPos())
 *         return _map.max()
 */
  __pyx_t_2 = ((struct __pyx_vtabstruct_21MorphologicalAnalysis_12FsmParseList_FsmParseList *)__pyx_v_nextParseList->__pyx_vtab)->size(__pyx_v_nextParseList, 0);
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":31
 *         _map = CounterHashMap()
 *         for i in range(nextParseList.size()):
 *             _map.put(nextParseList.getFsmParse(i).getPos())             # <<<<<<<<<<<<<<
 *         return _map.max()
 * 
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_21MorphologicalAnalysis_12FsmParseList_FsmParseList *)__pyx_v_nextParseList->__pyx_vtab)->getFsmParse(__pyx_v_nextParseList, __pyx_v_i, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = ((struct __pyx_vtabstruct_21MorphologicalAnalysis_8FsmParse_FsmParse *)((struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *)__pyx_t_1)->__pyx_base.__pyx_vtab)->__pyx_base.getPos(((struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse *)__pyx_t_1), 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 31, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = ((struct __pyx_vtabstruct_13DataStructure_14CounterHashMap_CounterHashMap *)__pyx_v__map->__pyx_vtab)->put(__pyx_v__map, __pyx_t_5, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":32
 *         for i in range(nextParseList.size()):
 *             _map.put(nextParseList.getFsmParse(i).getPos())
 *         return _map.max()             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_13DataStructure_14CounterHashMap_CounterHashMap *)__pyx_v__map->__pyx_vtab)->max(__pyx_v__map, 0, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 32, __pyx_L1_error)
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":26
 * 
 *     @staticmethod
 *     def nextWordPos(nextParseList: FsmParseList) -> str:             # <<<<<<<<<<<<<<
 *         cdef CounterHashMap _map
 *         cdef int i
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.nextWordPos", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v__map);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "MorphologicalDisambiguation/AutoDisambiguator.pyx":35
 * 
 *     @staticmethod
 *     def isBeforeLastWord(index: int, fsmParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         return index + 2 == len(fsmParses)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_7isBeforeLastWord(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_7isBeforeLastWord = {"isBeforeLastWord", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_7isBeforeLastWord, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_7isBeforeLastWord(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_fsmParses = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("isBeforeLastWord (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_fsmParses,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_fsmParses)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("isBeforeLastWord", 1, 2, 2, 1); __PYX_ERR(0, 35, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "isBeforeLastWord") < 0)) __PYX_ERR(0, 35, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_index = values[0];
    __pyx_v_fsmParses = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("isBeforeLastWord", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 35, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.isBeforeLastWord", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fsmParses), (&PyList_Type), 1, "fsmParses", 1))) __PYX_ERR(0, 35, __pyx_L1_error)
  __pyx_r = __pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_6isBeforeLastWord(__pyx_v_index, __pyx_v_fsmParses);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_6isBeforeLastWord(PyObject *__pyx_v_index, PyObject *__pyx_v_fsmParses) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("isBeforeLastWord", 0);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":36
 *     @staticmethod
 *     def isBeforeLastWord(index: int, fsmParses: list) -> bool:
 *         return index + 2 == len(fsmParses)             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_AddObjC(__pyx_v_index, __pyx_int_2, 2, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(__pyx_v_fsmParses == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 36, __pyx_L1_error)
  }
  __pyx_t_2 = PyList_GET_SIZE(__pyx_v_fsmParses); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 36, __pyx_L1_error)
  __pyx_t_3 = PyInt_FromSsize_t(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyObject_RichCompare(__pyx_t_1, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":35
 * 
 *     @staticmethod
 *     def isBeforeLastWord(index: int, fsmParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         return index + 2 == len(fsmParses)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.isBeforeLastWord", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "MorphologicalDisambiguation/AutoDisambiguator.pyx":39
 * 
 *     @staticmethod
 *     def nextWordExists(index: int, fsmParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         return index + 1 < len(fsmParses)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_9nextWordExists(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_9nextWordExists = {"nextWordExists", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_9nextWordExists, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_9nextWordExists(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_fsmParses = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("nextWordExists (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_fsmParses,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_fsmParses)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("nextWordExists", 1, 2, 2, 1); __PYX_ERR(0, 39, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "nextWordExists") < 0)) __PYX_ERR(0, 39, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_index = values[0];
    __pyx_v_fsmParses = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("nextWordExists", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 39, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.nextWordExists", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fsmParses), (&PyList_Type), 1, "fsmParses", 1))) __PYX_ERR(0, 39, __pyx_L1_error)
  __pyx_r = __pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_8nextWordExists(__pyx_v_index, __pyx_v_fsmParses);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_8nextWordExists(PyObject *__pyx_v_index, PyObject *__pyx_v_fsmParses) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("nextWordExists", 0);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":40
 *     @staticmethod
 *     def nextWordExists(index: int, fsmParses: list) -> bool:
 *         return index + 1 < len(fsmParses)             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_AddObjC(__pyx_v_index, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(__pyx_v_fsmParses == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 40, __pyx_L1_error)
  }
  __pyx_t_2 = PyList_GET_SIZE(__pyx_v_fsmParses); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 40, __pyx_L1_error)
  __pyx_t_3 = PyInt_FromSsize_t(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyObject_RichCompare(__pyx_t_1, __pyx_t_3, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":39
 * 
 *     @staticmethod
 *     def nextWordExists(index: int, fsmParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         return index + 1 < len(fsmParses)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.nextWordExists", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "MorphologicalDisambiguation/AutoDisambiguator.pyx":43
 * 
 *     @staticmethod
 *     def isNextWordNoun(index: int, fsmParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         return index + 1 < len(fsmParses) and AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN"
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_11isNextWordNoun(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_11isNextWordNoun = {"isNextWordNoun", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_11isNextWordNoun, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_11isNextWordNoun(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_fsmParses = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("isNextWordNoun (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_fsmParses,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_fsmParses)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("isNextWordNoun", 1, 2, 2, 1); __PYX_ERR(0, 43, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "isNextWordNoun") < 0)) __PYX_ERR(0, 43, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_index = values[0];
    __pyx_v_fsmParses = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("isNextWordNoun", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 43, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.isNextWordNoun", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fsmParses), (&PyList_Type), 1, "fsmParses", 1))) __PYX_ERR(0, 43, __pyx_L1_error)
  __pyx_r = __pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_10isNextWordNoun(__pyx_v_index, __pyx_v_fsmParses);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_10isNextWordNoun(PyObject *__pyx_v_index, PyObject *__pyx_v_fsmParses) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("isNextWordNoun", 0);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":44
 *     @staticmethod
 *     def isNextWordNoun(index: int, fsmParses: list) -> bool:
 *         return index + 1 < len(fsmParses) and AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN"             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_AddObjC(__pyx_v_index, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (unlikely(__pyx_v_fsmParses == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 44, __pyx_L1_error)
  }
  __pyx_t_3 = PyList_GET_SIZE(__pyx_v_fsmParses); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 44, __pyx_L1_error)
  __pyx_t_4 = PyInt_FromSsize_t(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyObject_RichCompare(__pyx_t_2, __pyx_t_4, Py_LT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 44, __pyx_L1_error)
  if (__pyx_t_6) {
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else {
    __Pyx_INCREF(__pyx_t_5);
    __pyx_t_1 = __pyx_t_5;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_nextWordPos); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (unlikely(__pyx_v_fsmParses == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 44, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_PyInt_AddObjC(__pyx_v_index, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyObject_GetItem(__pyx_v_fsmParses, __pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_5 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_2, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_7);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyObject_RichCompare(__pyx_t_5, __pyx_n_u_NOUN, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_1 = __pyx_t_4;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":43
 * 
 *     @staticmethod
 *     def isNextWordNoun(index: int, fsmParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         return index + 1 < len(fsmParses) and AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN"
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.isNextWordNoun", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "MorphologicalDisambiguation/AutoDisambiguator.pyx":47
 * 
 *     @staticmethod
 *     def isNextWordNum(index: int, fsmParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         return index + 1 < len(fsmParses) and AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NUM"
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_13isNextWordNum(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_13isNextWordNum = {"isNextWordNum", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_13isNextWordNum, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_13isNextWordNum(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_fsmParses = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("isNextWordNum (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_fsmParses,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_fsmParses)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("isNextWordNum", 1, 2, 2, 1); __PYX_ERR(0, 47, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "isNextWordNum") < 0)) __PYX_ERR(0, 47, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_index = values[0];
    __pyx_v_fsmParses = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("isNextWordNum", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 47, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.isNextWordNum", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fsmParses), (&PyList_Type), 1, "fsmParses", 1))) __PYX_ERR(0, 47, __pyx_L1_error)
  __pyx_r = __pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_12isNextWordNum(__pyx_v_index, __pyx_v_fsmParses);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_12isNextWordNum(PyObject *__pyx_v_index, PyObject *__pyx_v_fsmParses) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("isNextWordNum", 0);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":48
 *     @staticmethod
 *     def isNextWordNum(index: int, fsmParses: list) -> bool:
 *         return index + 1 < len(fsmParses) and AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NUM"             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_AddObjC(__pyx_v_index, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (unlikely(__pyx_v_fsmParses == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 48, __pyx_L1_error)
  }
  __pyx_t_3 = PyList_GET_SIZE(__pyx_v_fsmParses); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 48, __pyx_L1_error)
  __pyx_t_4 = PyInt_FromSsize_t(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyObject_RichCompare(__pyx_t_2, __pyx_t_4, Py_LT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 48, __pyx_L1_error)
  if (__pyx_t_6) {
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else {
    __Pyx_INCREF(__pyx_t_5);
    __pyx_t_1 = __pyx_t_5;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_nextWordPos); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (unlikely(__pyx_v_fsmParses == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 48, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_PyInt_AddObjC(__pyx_v_index, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyObject_GetItem(__pyx_v_fsmParses, __pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_5 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_2, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_7);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyObject_RichCompare(__pyx_t_5, __pyx_n_u_NUM, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_1 = __pyx_t_4;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":47
 * 
 *     @staticmethod
 *     def isNextWordNum(index: int, fsmParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         return index + 1 < len(fsmParses) and AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NUM"
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.isNextWordNum", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "MorphologicalDisambiguation/AutoDisambiguator.pyx":51
 * 
 *     @staticmethod
 *     def isNextWordNounOrAdjective(index: int, fsmParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         return index + 1 < len(fsmParses) and (AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN" or
 *                                                AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "ADJ" or
 */

/* Python wrapper */
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_15isNextWordNounOrAdjective(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_15isNextWordNounOrAdjective = {"isNextWordNounOrAdjective", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_15isNextWordNounOrAdjective, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_15isNextWordNounOrAdjective(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_fsmParses = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("isNextWordNounOrAdjective (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_fsmParses,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_fsmParses)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("isNextWordNounOrAdjective", 1, 2, 2, 1); __PYX_ERR(0, 51, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "isNextWordNounOrAdjective") < 0)) __PYX_ERR(0, 51, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_index = values[0];
    __pyx_v_fsmParses = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("isNextWordNounOrAdjective", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 51, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.isNextWordNounOrAdjective", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fsmParses), (&PyList_Type), 1, "fsmParses", 1))) __PYX_ERR(0, 51, __pyx_L1_error)
  __pyx_r = __pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_14isNextWordNounOrAdjective(__pyx_v_index, __pyx_v_fsmParses);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_14isNextWordNounOrAdjective(PyObject *__pyx_v_index, PyObject *__pyx_v_fsmParses) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("isNextWordNounOrAdjective", 0);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":52
 *     @staticmethod
 *     def isNextWordNounOrAdjective(index: int, fsmParses: list) -> bool:
 *         return index + 1 < len(fsmParses) and (AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN" or             # <<<<<<<<<<<<<<
 *                                                AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "ADJ" or
 *                                                AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "DET")
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_AddObjC(__pyx_v_index, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (unlikely(__pyx_v_fsmParses == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 52, __pyx_L1_error)
  }
  __pyx_t_3 = PyList_GET_SIZE(__pyx_v_fsmParses); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 52, __pyx_L1_error)
  __pyx_t_4 = PyInt_FromSsize_t(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyObject_RichCompare(__pyx_t_2, __pyx_t_4, Py_LT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 52, __pyx_L1_error)
  if (__pyx_t_6) {
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else {
    __Pyx_INCREF(__pyx_t_5);
    __pyx_t_1 = __pyx_t_5;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_nextWordPos); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (unlikely(__pyx_v_fsmParses == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 52, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_PyInt_AddObjC(__pyx_v_index, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyObject_GetItem(__pyx_v_fsmParses, __pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_5 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_2, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_7);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyObject_RichCompare(__pyx_t_5, __pyx_n_u_NOUN, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 52, __pyx_L1_error)
  if (!__pyx_t_6) {
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else {
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_1 = __pyx_t_4;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L3_bool_binop_done;
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":53
 *     def isNextWordNounOrAdjective(index: int, fsmParses: list) -> bool:
 *         return index + 1 < len(fsmParses) and (AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN" or
 *                                                AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "ADJ" or             # <<<<<<<<<<<<<<
 *                                                AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "DET")
 * 
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_nextWordPos); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (unlikely(__pyx_v_fsmParses == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 53, __pyx_L1_error)
  }
  __pyx_t_7 = __Pyx_PyInt_AddObjC(__pyx_v_index, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_fsmParses, __pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_4 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_7, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyObject_RichCompare(__pyx_t_4, __pyx_n_u_ADJ, Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 53, __pyx_L1_error)
  if (!__pyx_t_6) {
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else {
    __Pyx_INCREF(__pyx_t_5);
    __pyx_t_1 = __pyx_t_5;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L3_bool_binop_done;
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":54
 *         return index + 1 < len(fsmParses) and (AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN" or
 *                                                AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "ADJ" or
 *                                                AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "DET")             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_nextWordPos); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (unlikely(__pyx_v_fsmParses == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 54, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_PyInt_AddObjC(__pyx_v_index, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyObject_GetItem(__pyx_v_fsmParses, __pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_5 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_2, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_7);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyObject_RichCompare(__pyx_t_5, __pyx_n_u_DET, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_1 = __pyx_t_4;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":51
 * 
 *     @staticmethod
 *     def isNextWordNounOrAdjective(index: int, fsmParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         return index + 1 < len(fsmParses) and (AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN" or
 *                                                AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "ADJ" or
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.isNextWordNounOrAdjective", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "MorphologicalDisambiguation/AutoDisambiguator.pyx":57
 * 
 *     @staticmethod
 *     def isFirstWord(index: int) -> bool:             # <<<<<<<<<<<<<<
 *         return index == 0
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_17isFirstWord(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_17isFirstWord = {"isFirstWord", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_17isFirstWord, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_17isFirstWord(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_index = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("isFirstWord (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "isFirstWord") < 0)) __PYX_ERR(0, 57, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_index = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("isFirstWord", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 57, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.isFirstWord", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_16isFirstWord(__pyx_v_index);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_16isFirstWord(PyObject *__pyx_v_index) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("isFirstWord", 0);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":58
 *     @staticmethod
 *     def isFirstWord(index: int) -> bool:
 *         return index == 0             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_index, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":57
 * 
 *     @staticmethod
 *     def isFirstWord(index: int) -> bool:             # <<<<<<<<<<<<<<
 *         return index == 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.isFirstWord", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "MorphologicalDisambiguation/AutoDisambiguator.pyx":61
 * 
 *     @staticmethod
 *     def containsTwoNeOrYa(fsmParses: list, word: str) -> bool:             # <<<<<<<<<<<<<<
 *         cdef int count
 *         cdef FsmParseList fsmPars
 */

/* Python wrapper */
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_19containsTwoNeOrYa(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_19containsTwoNeOrYa = {"containsTwoNeOrYa", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_19containsTwoNeOrYa, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_19containsTwoNeOrYa(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_fsmParses = 0;
  PyObject *__pyx_v_word = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("containsTwoNeOrYa (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_fsmParses,&__pyx_n_s_word,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_fsmParses)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_word)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("containsTwoNeOrYa", 1, 2, 2, 1); __PYX_ERR(0, 61, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "containsTwoNeOrYa") < 0)) __PYX_ERR(0, 61, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_fsmParses = ((PyObject*)values[0]);
    __pyx_v_word = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("containsTwoNeOrYa", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 61, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.containsTwoNeOrYa", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fsmParses), (&PyList_Type), 1, "fsmParses", 1))) __PYX_ERR(0, 61, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_word), (&PyUnicode_Type), 1, "word", 1))) __PYX_ERR(0, 61, __pyx_L1_error)
  __pyx_r = __pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_18containsTwoNeOrYa(__pyx_v_fsmParses, __pyx_v_word);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_18containsTwoNeOrYa(PyObject *__pyx_v_fsmParses, PyObject *__pyx_v_word) {
  int __pyx_v_count;
  struct __pyx_obj_21MorphologicalAnalysis_12FsmParseList_FsmParseList *__pyx_v_fsmPars = 0;
  PyObject *__pyx_v_surfaceForm = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("containsTwoNeOrYa", 0);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":65
 *         cdef FsmParseList fsmPars
 *         cdef str surfaceForm
 *         count = 0             # <<<<<<<<<<<<<<
 *         for fsmPars in fsmParses:
 *             surfaceForm = fsmPars.getFsmParse(0).getSurfaceForm()
 */
  __pyx_v_count = 0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":66
 *         cdef str surfaceForm
 *         count = 0
 *         for fsmPars in fsmParses:             # <<<<<<<<<<<<<<
 *             surfaceForm = fsmPars.getFsmParse(0).getSurfaceForm()
 *             if surfaceForm == word:
 */
  if (unlikely(__pyx_v_fsmParses == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 66, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_fsmParses; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 66, __pyx_L1_error)
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 66, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_21MorphologicalAnalysis_12FsmParseList_FsmParseList))))) __PYX_ERR(0, 66, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_fsmPars, ((struct __pyx_obj_21MorphologicalAnalysis_12FsmParseList_FsmParseList *)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":67
 *         count = 0
 *         for fsmPars in fsmParses:
 *             surfaceForm = fsmPars.getFsmParse(0).getSurfaceForm()             # <<<<<<<<<<<<<<
 *             if surfaceForm == word:
 *                 count = count + 1
 */
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_21MorphologicalAnalysis_12FsmParseList_FsmParseList *)__pyx_v_fsmPars->__pyx_vtab)->getFsmParse(__pyx_v_fsmPars, 0, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 67, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = ((struct __pyx_vtabstruct_21MorphologicalAnalysis_8FsmParse_FsmParse *)((struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *)__pyx_t_3)->__pyx_base.__pyx_vtab)->getSurfaceForm(((struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *)__pyx_t_3), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 67, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_surfaceForm, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":68
 *         for fsmPars in fsmParses:
 *             surfaceForm = fsmPars.getFsmParse(0).getSurfaceForm()
 *             if surfaceForm == word:             # <<<<<<<<<<<<<<
 *                 count = count + 1
 *         return count == 2
 */
    __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_surfaceForm, __pyx_v_word, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 68, __pyx_L1_error)
    __pyx_t_6 = (__pyx_t_5 != 0);
    if (__pyx_t_6) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":69
 *             surfaceForm = fsmPars.getFsmParse(0).getSurfaceForm()
 *             if surfaceForm == word:
 *                 count = count + 1             # <<<<<<<<<<<<<<
 *         return count == 2
 * 
 */
      __pyx_v_count = (__pyx_v_count + 1);

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":68
 *         for fsmPars in fsmParses:
 *             surfaceForm = fsmPars.getFsmParse(0).getSurfaceForm()
 *             if surfaceForm == word:             # <<<<<<<<<<<<<<
 *                 count = count + 1
 *         return count == 2
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":66
 *         cdef str surfaceForm
 *         count = 0
 *         for fsmPars in fsmParses:             # <<<<<<<<<<<<<<
 *             surfaceForm = fsmPars.getFsmParse(0).getSurfaceForm()
 *             if surfaceForm == word:
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":70
 *             if surfaceForm == word:
 *                 count = count + 1
 *         return count == 2             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong((__pyx_v_count == 2)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":61
 * 
 *     @staticmethod
 *     def containsTwoNeOrYa(fsmParses: list, word: str) -> bool:             # <<<<<<<<<<<<<<
 *         cdef int count
 *         cdef FsmParseList fsmPars
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.containsTwoNeOrYa", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_fsmPars);
  __Pyx_XDECREF(__pyx_v_surfaceForm);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "MorphologicalDisambiguation/AutoDisambiguator.pyx":73
 * 
 *     @staticmethod
 *     def hasPreviousWordTag(index: int, correctParses: list, tag: MorphologicalTag) -> bool:             # <<<<<<<<<<<<<<
 *         return index > 0 and correctParses[index - 1].containsTag(tag)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_21hasPreviousWordTag(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_21hasPreviousWordTag = {"hasPreviousWordTag", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_21hasPreviousWordTag, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_21hasPreviousWordTag(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_correctParses = 0;
  PyObject *__pyx_v_tag = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("hasPreviousWordTag (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_correctParses,&__pyx_n_s_tag,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_correctParses)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("hasPreviousWordTag", 1, 3, 3, 1); __PYX_ERR(0, 73, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tag)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("hasPreviousWordTag", 1, 3, 3, 2); __PYX_ERR(0, 73, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "hasPreviousWordTag") < 0)) __PYX_ERR(0, 73, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_index = values[0];
    __pyx_v_correctParses = ((PyObject*)values[1]);
    __pyx_v_tag = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("hasPreviousWordTag", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 73, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.hasPreviousWordTag", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_correctParses), (&PyList_Type), 1, "correctParses", 1))) __PYX_ERR(0, 73, __pyx_L1_error)
  __pyx_r = __pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_20hasPreviousWordTag(__pyx_v_index, __pyx_v_correctParses, __pyx_v_tag);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_20hasPreviousWordTag(PyObject *__pyx_v_index, PyObject *__pyx_v_correctParses, PyObject *__pyx_v_tag) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("hasPreviousWordTag", 0);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":74
 *     @staticmethod
 *     def hasPreviousWordTag(index: int, correctParses: list, tag: MorphologicalTag) -> bool:
 *         return index > 0 and correctParses[index - 1].containsTag(tag)             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_index, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 74, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 74, __pyx_L1_error)
  if (__pyx_t_3) {
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_1 = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L3_bool_binop_done;
  }
  if (unlikely(__pyx_v_correctParses == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 74, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyInt_SubtractObjC(__pyx_v_index, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetItem(__pyx_v_correctParses, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_containsTag); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_v_tag) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_tag);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_1 = __pyx_t_2;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":73
 * 
 *     @staticmethod
 *     def hasPreviousWordTag(index: int, correctParses: list, tag: MorphologicalTag) -> bool:             # <<<<<<<<<<<<<<
 *         return index > 0 and correctParses[index - 1].containsTag(tag)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.hasPreviousWordTag", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "MorphologicalDisambiguation/AutoDisambiguator.pyx":77
 * 
 *     @staticmethod
 *     def selectCaseForParseString(parseString: str, index: int, fsmParses: list, correctParses: list) -> str:             # <<<<<<<<<<<<<<
 *         cdef str surfaceForm, root, lastWord
 *         surfaceForm = fsmParses[index].getFsmParse(0).getSurfaceForm()
 */

/* Python wrapper */
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_23selectCaseForParseString(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_23selectCaseForParseString = {"selectCaseForParseString", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_23selectCaseForParseString, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_23selectCaseForParseString(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_parseString = 0;
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_fsmParses = 0;
  PyObject *__pyx_v_correctParses = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selectCaseForParseString (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_parseString,&__pyx_n_s_index,&__pyx_n_s_fsmParses,&__pyx_n_s_correctParses,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_parseString)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("selectCaseForParseString", 1, 4, 4, 1); __PYX_ERR(0, 77, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_fsmParses)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("selectCaseForParseString", 1, 4, 4, 2); __PYX_ERR(0, 77, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_correctParses)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("selectCaseForParseString", 1, 4, 4, 3); __PYX_ERR(0, 77, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "selectCaseForParseString") < 0)) __PYX_ERR(0, 77, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_parseString = ((PyObject*)values[0]);
    __pyx_v_index = values[1];
    __pyx_v_fsmParses = ((PyObject*)values[2]);
    __pyx_v_correctParses = ((PyObject*)values[3]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("selectCaseForParseString", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 77, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.selectCaseForParseString", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_parseString), (&PyUnicode_Type), 1, "parseString", 1))) __PYX_ERR(0, 77, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fsmParses), (&PyList_Type), 1, "fsmParses", 1))) __PYX_ERR(0, 77, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_correctParses), (&PyList_Type), 1, "correctParses", 1))) __PYX_ERR(0, 77, __pyx_L1_error)
  __pyx_r = __pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_22selectCaseForParseString(__pyx_v_parseString, __pyx_v_index, __pyx_v_fsmParses, __pyx_v_correctParses);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_22selectCaseForParseString(PyObject *__pyx_v_parseString, PyObject *__pyx_v_index, PyObject *__pyx_v_fsmParses, PyObject *__pyx_v_correctParses) {
  PyObject *__pyx_v_surfaceForm = 0;
  PyObject *__pyx_v_root = 0;
  PyObject *__pyx_v_lastWord = 0;
  PyObject *__pyx_v_previousWord = NULL;
  PyObject *__pyx_v_nextWord = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("selectCaseForParseString", 0);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":79
 *     def selectCaseForParseString(parseString: str, index: int, fsmParses: list, correctParses: list) -> str:
 *         cdef str surfaceForm, root, lastWord
 *         surfaceForm = fsmParses[index].getFsmParse(0).getSurfaceForm()             # <<<<<<<<<<<<<<
 *         root = fsmParses[index].getFsmParse(0).getWord().getName()
 *         lastWord = fsmParses[len(fsmParses) - 1].getFsmParse(0).getSurfaceForm()
 */
  if (unlikely(__pyx_v_fsmParses == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 79, __pyx_L1_error)
  }
  __pyx_t_3 = __Pyx_PyObject_GetItem(__pyx_v_fsmParses, __pyx_v_index); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_getFsmParse); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_3, __pyx_int_0) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_int_0);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_getSurfaceForm); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 79, __pyx_L1_error)
  __pyx_v_surfaceForm = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":80
 *         cdef str surfaceForm, root, lastWord
 *         surfaceForm = fsmParses[index].getFsmParse(0).getSurfaceForm()
 *         root = fsmParses[index].getFsmParse(0).getWord().getName()             # <<<<<<<<<<<<<<
 *         lastWord = fsmParses[len(fsmParses) - 1].getFsmParse(0).getSurfaceForm()
 *         # ksmn, duracan, grubunun #
 */
  if (unlikely(__pyx_v_fsmParses == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 80, __pyx_L1_error)
  }
  __pyx_t_3 = __Pyx_PyObject_GetItem(__pyx_v_fsmParses, __pyx_v_index); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 80, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_getFsmParse); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 80, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_3, __pyx_int_0) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_int_0);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 80, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_getWord); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 80, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_4 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 80, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_getName); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 80, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 80, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 80, __pyx_L1_error)
  __pyx_v_root = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":81
 *         surfaceForm = fsmParses[index].getFsmParse(0).getSurfaceForm()
 *         root = fsmParses[index].getFsmParse(0).getWord().getName()
 *         lastWord = fsmParses[len(fsmParses) - 1].getFsmParse(0).getSurfaceForm()             # <<<<<<<<<<<<<<
 *         # ksmn, duracan, grubunun #
 *         if parseString == "P2SG$P3SG":
 */
  if (unlikely(__pyx_v_fsmParses == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 81, __pyx_L1_error)
  }
  if (unlikely(__pyx_v_fsmParses == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 81, __pyx_L1_error)
  }
  __pyx_t_6 = PyList_GET_SIZE(__pyx_v_fsmParses); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 81, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 - 1);
  __pyx_t_4 = __Pyx_GetItemInt_List(__pyx_v_fsmParses, __pyx_t_7, Py_ssize_t, 1, PyInt_FromSsize_t, 1, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_getFsmParse); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_5 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_int_0) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_int_0);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_getSurfaceForm); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 81, __pyx_L1_error)
  __pyx_v_lastWord = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":83
 *         lastWord = fsmParses[len(fsmParses) - 1].getFsmParse(0).getSurfaceForm()
 *         # ksmn, duracan, grubunun #
 *         if parseString == "P2SG$P3SG":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "P2SG"
 */
  __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_P2SG_P3SG, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 83, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":84
 *         # ksmn, duracan, grubunun #
 *         if parseString == "P2SG$P3SG":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "P2SG"
 *             return "P3SG"
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isAnyWordSecondPerson); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 84, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 84, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 84, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 84, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_10, __pyx_v_correctParses);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 84, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 84, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":85
 *         if parseString == "P2SG$P3SG":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "P2SG"             # <<<<<<<<<<<<<<
 *             return "P3SG"
 *         elif parseString == "A2SG+P2SG$A3SG+P3SG":
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_n_u_P2SG);
      __pyx_r = __pyx_n_u_P2SG;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":84
 *         # ksmn, duracan, grubunun #
 *         if parseString == "P2SG$P3SG":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "P2SG"
 *             return "P3SG"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":86
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "P2SG"
 *             return "P3SG"             # <<<<<<<<<<<<<<
 *         elif parseString == "A2SG+P2SG$A3SG+P3SG":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_P3SG);
    __pyx_r = __pyx_n_u_P3SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":83
 *         lastWord = fsmParses[len(fsmParses) - 1].getFsmParse(0).getSurfaceForm()
 *         # ksmn, duracan, grubunun #
 *         if parseString == "P2SG$P3SG":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "P2SG"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":87
 *                 return "P2SG"
 *             return "P3SG"
 *         elif parseString == "A2SG+P2SG$A3SG+P3SG":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "A2SG+P2SG"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_A2SG_P2SG_A3SG_P3SG, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 87, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_9 != 0);
  if (__pyx_t_8) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":88
 *             return "P3SG"
 *         elif parseString == "A2SG+P2SG$A3SG+P3SG":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "A2SG+P2SG"
 *             return "A3SG+P3SG"
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isAnyWordSecondPerson); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 88, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 88, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 88, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 88, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_10, __pyx_v_correctParses);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 88, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 88, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_8) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":89
 *         elif parseString == "A2SG+P2SG$A3SG+P3SG":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "A2SG+P2SG"             # <<<<<<<<<<<<<<
 *             return "A3SG+P3SG"
 *             # BR #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_A2SG_P2SG);
      __pyx_r = __pyx_kp_u_A2SG_P2SG;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":88
 *             return "P3SG"
 *         elif parseString == "A2SG+P2SG$A3SG+P3SG":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "A2SG+P2SG"
 *             return "A3SG+P3SG"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":90
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "A2SG+P2SG"
 *             return "A3SG+P3SG"             # <<<<<<<<<<<<<<
 *             # BR #
 *         elif parseString == "ADJ$ADV$DET$NUM+CARD":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_A3SG_P3SG);
    __pyx_r = __pyx_kp_u_A3SG_P3SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":87
 *                 return "P2SG"
 *             return "P3SG"
 *         elif parseString == "A2SG+P2SG$A3SG+P3SG":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "A2SG+P2SG"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":92
 *             return "A3SG+P3SG"
 *             # BR #
 *         elif parseString == "ADJ$ADV$DET$NUM+CARD":             # <<<<<<<<<<<<<<
 *             return "DET"
 *             # tahminleri, ileri, hisseleri #
 */
  __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_ADV_DET_NUM_CARD, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 92, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":93
 *             # BR #
 *         elif parseString == "ADJ$ADV$DET$NUM+CARD":
 *             return "DET"             # <<<<<<<<<<<<<<
 *             # tahminleri, ileri, hisseleri #
 *         elif parseString == "A3PL+P3PL+NOM$A3PL+P3SG+NOM$A3PL+PNON+ACC$A3SG+P3PL+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_DET);
    __pyx_r = __pyx_n_u_DET;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":92
 *             return "A3SG+P3SG"
 *             # BR #
 *         elif parseString == "ADJ$ADV$DET$NUM+CARD":             # <<<<<<<<<<<<<<
 *             return "DET"
 *             # tahminleri, ileri, hisseleri #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":95
 *             return "DET"
 *             # tahminleri, ileri, hisseleri #
 *         elif parseString == "A3PL+P3PL+NOM$A3PL+P3SG+NOM$A3PL+PNON+ACC$A3SG+P3PL+NOM":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "A3SG+P3PL+NOM"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_A3PL_P3PL_NOM_A3PL_P3SG_NOM_A3PL, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 95, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_9 != 0);
  if (__pyx_t_8) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":96
 *             # tahminleri, ileri, hisseleri #
 *         elif parseString == "A3PL+P3PL+NOM$A3PL+P3SG+NOM$A3PL+PNON+ACC$A3SG+P3PL+NOM":
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "A3SG+P3PL+NOM"
 *             return "A3PL+P3SG+NOM"
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isPossessivePlural); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 96, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 96, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 96, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 96, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_10, __pyx_v_correctParses);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 96, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 96, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_8) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":97
 *         elif parseString == "A3PL+P3PL+NOM$A3PL+P3SG+NOM$A3PL+PNON+ACC$A3SG+P3PL+NOM":
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "A3SG+P3PL+NOM"             # <<<<<<<<<<<<<<
 *             return "A3PL+P3SG+NOM"
 *             # Ocak, Cuma, ABD #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_A3SG_P3PL_NOM);
      __pyx_r = __pyx_kp_u_A3SG_P3PL_NOM;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":96
 *             # tahminleri, ileri, hisseleri #
 *         elif parseString == "A3PL+P3PL+NOM$A3PL+P3SG+NOM$A3PL+PNON+ACC$A3SG+P3PL+NOM":
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "A3SG+P3PL+NOM"
 *             return "A3PL+P3SG+NOM"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":98
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "A3SG+P3PL+NOM"
 *             return "A3PL+P3SG+NOM"             # <<<<<<<<<<<<<<
 *             # Ocak, Cuma, ABD #
 *         elif parseString == "A3SG$PROP+A3SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_A3PL_P3SG_NOM);
    __pyx_r = __pyx_kp_u_A3PL_P3SG_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":95
 *             return "DET"
 *             # tahminleri, ileri, hisseleri #
 *         elif parseString == "A3PL+P3PL+NOM$A3PL+P3SG+NOM$A3PL+PNON+ACC$A3SG+P3PL+NOM":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "A3SG+P3PL+NOM"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":100
 *             return "A3PL+P3SG+NOM"
 *             # Ocak, Cuma, ABD #
 *         elif parseString == "A3SG$PROP+A3SG":             # <<<<<<<<<<<<<<
 *             if index > 0:
 *                 return "PROP+A3SG"
 */
  __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_A3SG_PROP_A3SG, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 100, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":101
 *             # Ocak, Cuma, ABD #
 *         elif parseString == "A3SG$PROP+A3SG":
 *             if index > 0:             # <<<<<<<<<<<<<<
 *                 return "PROP+A3SG"
 *             # irketin, seimlerin, borsaclarn, kitaplarn #
 */
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_index, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 101, __pyx_L1_error)
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 101, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":102
 *         elif parseString == "A3SG$PROP+A3SG":
 *             if index > 0:
 *                 return "PROP+A3SG"             # <<<<<<<<<<<<<<
 *             # irketin, seimlerin, borsaclarn, kitaplarn #
 *         elif parseString == "P2SG+NOM$PNON+GEN":
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_PROP_A3SG);
      __pyx_r = __pyx_kp_u_PROP_A3SG;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":101
 *             # Ocak, Cuma, ABD #
 *         elif parseString == "A3SG$PROP+A3SG":
 *             if index > 0:             # <<<<<<<<<<<<<<
 *                 return "PROP+A3SG"
 *             # irketin, seimlerin, borsaclarn, kitaplarn #
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":100
 *             return "A3PL+P3SG+NOM"
 *             # Ocak, Cuma, ABD #
 *         elif parseString == "A3SG$PROP+A3SG":             # <<<<<<<<<<<<<<
 *             if index > 0:
 *                 return "PROP+A3SG"
 */
    goto __pyx_L3;
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":104
 *                 return "PROP+A3SG"
 *             # irketin, seimlerin, borsaclarn, kitaplarn #
 *         elif parseString == "P2SG+NOM$PNON+GEN":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "P2SG+NOM"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_P2SG_NOM_PNON_GEN, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 104, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_9 != 0);
  if (__pyx_t_8) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":105
 *             # irketin, seimlerin, borsaclarn, kitaplarn #
 *         elif parseString == "P2SG+NOM$PNON+GEN":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "P2SG+NOM"
 *             return "PNON+GEN"
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isAnyWordSecondPerson); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 105, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 105, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 105, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 105, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_10, __pyx_v_correctParses);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 105, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 105, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_8) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":106
 *         elif parseString == "P2SG+NOM$PNON+GEN":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "P2SG+NOM"             # <<<<<<<<<<<<<<
 *             return "PNON+GEN"
 *             # FAZLA #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_P2SG_NOM);
      __pyx_r = __pyx_kp_u_P2SG_NOM;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":105
 *             # irketin, seimlerin, borsaclarn, kitaplarn #
 *         elif parseString == "P2SG+NOM$PNON+GEN":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "P2SG+NOM"
 *             return "PNON+GEN"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":107
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "P2SG+NOM"
 *             return "PNON+GEN"             # <<<<<<<<<<<<<<
 *             # FAZLA #
 *             # OK #
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_PNON_GEN);
    __pyx_r = __pyx_kp_u_PNON_GEN;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":104
 *                 return "PROP+A3SG"
 *             # irketin, seimlerin, borsaclarn, kitaplarn #
 *         elif parseString == "P2SG+NOM$PNON+GEN":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "P2SG+NOM"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":110
 *             # FAZLA #
 *             # OK #
 *         elif parseString == "ADJ$ADV$DET$POSTP+PCABL" or parseString == "ADJ$ADV$POSTP+PCABL":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_ADV_DET_POSTP_PCABL, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 110, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (!__pyx_t_11) {
  } else {
    __pyx_t_8 = __pyx_t_11;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_ADV_POSTP_PCABL, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 110, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  __pyx_t_8 = __pyx_t_9;
  __pyx_L9_bool_binop_done:;
  if (__pyx_t_8) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":111
 *             # OK #
 *         elif parseString == "ADJ$ADV$DET$POSTP+PCABL" or parseString == "ADJ$ADV$POSTP+PCABL":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCABL"
 *             if index + 1 < len(fsmParses):
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_hasPreviousWordTag); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 111, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_MorphologicalTag); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 111, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_ABLATIVE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 111, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_index, __pyx_v_correctParses, __pyx_t_4};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 111, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_index, __pyx_v_correctParses, __pyx_t_4};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 111, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else
    #endif
    {
      __pyx_t_3 = PyTuple_New(3+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 111, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_10, __pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_3, 2+__pyx_t_10, __pyx_t_4);
      __pyx_t_4 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 111, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 111, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_8) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":112
 *         elif parseString == "ADJ$ADV$DET$POSTP+PCABL" or parseString == "ADJ$ADV$POSTP+PCABL":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"             # <<<<<<<<<<<<<<
 *             if index + 1 < len(fsmParses):
 *                 if AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN":
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_POSTP_PCABL);
      __pyx_r = __pyx_kp_u_POSTP_PCABL;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":111
 *             # OK #
 *         elif parseString == "ADJ$ADV$DET$POSTP+PCABL" or parseString == "ADJ$ADV$POSTP+PCABL":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCABL"
 *             if index + 1 < len(fsmParses):
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":113
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"
 *             if index + 1 < len(fsmParses):             # <<<<<<<<<<<<<<
 *                 if AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN":
 *                     return "ADJ"
 */
    __pyx_t_1 = __Pyx_PyInt_AddObjC(__pyx_v_index, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 113, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(__pyx_v_fsmParses == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 113, __pyx_L1_error)
    }
    __pyx_t_7 = PyList_GET_SIZE(__pyx_v_fsmParses); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 113, __pyx_L1_error)
    __pyx_t_2 = PyInt_FromSsize_t(__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 113, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_t_2, Py_LT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 113, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 113, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_8) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":114
 *                 return "POSTP+PCABL"
 *             if index + 1 < len(fsmParses):
 *                 if AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN":             # <<<<<<<<<<<<<<
 *                     return "ADJ"
 *                 elif AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "ADJ" or \
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_nextWordPos); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 114, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (unlikely(__pyx_v_fsmParses == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 114, __pyx_L1_error)
      }
      __pyx_t_1 = __Pyx_PyInt_AddObjC(__pyx_v_index, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 114, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_v_fsmParses, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 114, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_3 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 114, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_n_u_NOUN, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 114, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_8) {

        /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":115
 *             if index + 1 < len(fsmParses):
 *                 if AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN":
 *                     return "ADJ"             # <<<<<<<<<<<<<<
 *                 elif AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "ADJ" or \
 *                         AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "ADV" or \
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_n_u_ADJ);
        __pyx_r = __pyx_n_u_ADJ;
        goto __pyx_L0;

        /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":114
 *                 return "POSTP+PCABL"
 *             if index + 1 < len(fsmParses):
 *                 if AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN":             # <<<<<<<<<<<<<<
 *                     return "ADJ"
 *                 elif AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "ADJ" or \
 */
      }

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":116
 *                 if AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN":
 *                     return "ADJ"
 *                 elif AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "ADJ" or \             # <<<<<<<<<<<<<<
 *                         AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "ADV" or \
 *                         AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "VERB":
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_nextWordPos); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 116, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (unlikely(__pyx_v_fsmParses == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 116, __pyx_L1_error)
      }
      __pyx_t_4 = __Pyx_PyInt_AddObjC(__pyx_v_index, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 116, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_fsmParses, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 116, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_1);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 116, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_n_u_ADJ, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 116, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (!__pyx_t_9) {
      } else {
        __pyx_t_8 = __pyx_t_9;
        goto __pyx_L14_bool_binop_done;
      }

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":117
 *                     return "ADJ"
 *                 elif AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "ADJ" or \
 *                         AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "ADV" or \             # <<<<<<<<<<<<<<
 *                         AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "VERB":
 *                     return "ADV"
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_nextWordPos); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 117, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (unlikely(__pyx_v_fsmParses == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 117, __pyx_L1_error)
      }
      __pyx_t_1 = __Pyx_PyInt_AddObjC(__pyx_v_index, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 117, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_v_fsmParses, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 117, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_3 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 117, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_n_u_ADV, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 117, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (!__pyx_t_9) {
      } else {
        __pyx_t_8 = __pyx_t_9;
        goto __pyx_L14_bool_binop_done;
      }

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":118
 *                 elif AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "ADJ" or \
 *                         AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "ADV" or \
 *                         AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "VERB":             # <<<<<<<<<<<<<<
 *                     return "ADV"
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM":
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_nextWordPos); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 118, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (unlikely(__pyx_v_fsmParses == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 118, __pyx_L1_error)
      }
      __pyx_t_4 = __Pyx_PyInt_AddObjC(__pyx_v_index, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 118, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_fsmParses, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 118, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_1);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 118, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_n_u_VERB, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 118, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_8 = __pyx_t_9;
      __pyx_L14_bool_binop_done:;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":116
 *                 if AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN":
 *                     return "ADJ"
 *                 elif AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "ADJ" or \             # <<<<<<<<<<<<<<
 *                         AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "ADV" or \
 *                         AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "VERB":
 */
      if (__pyx_t_8) {

        /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":119
 *                         AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "ADV" or \
 *                         AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "VERB":
 *                     return "ADV"             # <<<<<<<<<<<<<<
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_n_u_ADV);
        __pyx_r = __pyx_n_u_ADV;
        goto __pyx_L0;

        /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":116
 *                 if AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN":
 *                     return "ADJ"
 *                 elif AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "ADJ" or \             # <<<<<<<<<<<<<<
 *                         AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "ADV" or \
 *                         AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "VERB":
 */
      }

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":113
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"
 *             if index + 1 < len(fsmParses):             # <<<<<<<<<<<<<<
 *                 if AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN":
 *                     return "ADJ"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":110
 *             # FAZLA #
 *             # OK #
 *         elif parseString == "ADJ$ADV$DET$POSTP+PCABL" or parseString == "ADJ$ADV$POSTP+PCABL":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"
 */
    goto __pyx_L3;
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":120
 *                         AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "VERB":
 *                     return "ADV"
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ"
 */
  __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_NOUN_A3SG_PNON_NOM, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 120, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":121
 *                     return "ADV"
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ"
 *             return "NOUN+A3SG+PNON+NOM"
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNounOrAdjective); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 121, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 121, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 121, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 121, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_1) {
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1); __pyx_t_1 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 121, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 121, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":122
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ"             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+PNON+NOM"
 *             # fanatiklerini, senetlerini, olduklarn #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_n_u_ADJ);
      __pyx_r = __pyx_n_u_ADJ;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":121
 *                     return "ADV"
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ"
 *             return "NOUN+A3SG+PNON+NOM"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":123
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ"
 *             return "NOUN+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *             # fanatiklerini, senetlerini, olduklarn #
 *         elif parseString == "A3PL+P2SG$A3PL+P3PL$A3PL+P3SG$A3SG+P3PL":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_A3SG_PNON_NOM);
    __pyx_r = __pyx_kp_u_NOUN_A3SG_PNON_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":120
 *                         AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "VERB":
 *                     return "ADV"
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":125
 *             return "NOUN+A3SG+PNON+NOM"
 *             # fanatiklerini, senetlerini, olduklarn #
 *         elif parseString == "A3PL+P2SG$A3PL+P3PL$A3PL+P3SG$A3SG+P3PL":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "A3PL+P2SG"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_A3PL_P2SG_A3PL_P3PL_A3PL_P3SG_A3, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 125, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_9 != 0);
  if (__pyx_t_8) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":126
 *             # fanatiklerini, senetlerini, olduklarn #
 *         elif parseString == "A3PL+P2SG$A3PL+P3PL$A3PL+P3SG$A3SG+P3PL":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "A3PL+P2SG"
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isAnyWordSecondPerson); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 126, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 126, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 126, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_1 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 126, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_10, __pyx_v_correctParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 126, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 126, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_8) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":127
 *         elif parseString == "A3PL+P2SG$A3PL+P3PL$A3PL+P3SG$A3SG+P3PL":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "A3PL+P2SG"             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "A3SG+P3PL"
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_A3PL_P2SG);
      __pyx_r = __pyx_kp_u_A3PL_P2SG;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":126
 *             # fanatiklerini, senetlerini, olduklarn #
 *         elif parseString == "A3PL+P2SG$A3PL+P3PL$A3PL+P3SG$A3SG+P3PL":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "A3PL+P2SG"
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":128
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "A3PL+P2SG"
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "A3SG+P3PL"
 *             else:
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isPossessivePlural); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 128, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 128, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 128, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 128, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_1) {
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1); __pyx_t_1 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_10, __pyx_v_correctParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 128, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 128, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_8) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":129
 *                 return "A3PL+P2SG"
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "A3SG+P3PL"             # <<<<<<<<<<<<<<
 *             else:
 *                 return "A3PL+P3SG"
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_A3SG_P3PL);
      __pyx_r = __pyx_kp_u_A3SG_P3PL;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":128
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "A3PL+P2SG"
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "A3SG+P3PL"
 *             else:
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":131
 *                 return "A3SG+P3PL"
 *             else:
 *                 return "A3PL+P3SG"             # <<<<<<<<<<<<<<
 *         elif parseString == "ADJ$NOUN+PROP+A3SG+PNON+NOM":
 *             if index > 0:
 */
    /*else*/ {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_A3PL_P3SG);
      __pyx_r = __pyx_kp_u_A3PL_P3SG;
      goto __pyx_L0;
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":125
 *             return "NOUN+A3SG+PNON+NOM"
 *             # fanatiklerini, senetlerini, olduklarn #
 *         elif parseString == "A3PL+P2SG$A3PL+P3PL$A3PL+P3SG$A3SG+P3PL":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "A3PL+P2SG"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":132
 *             else:
 *                 return "A3PL+P3SG"
 *         elif parseString == "ADJ$NOUN+PROP+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if index > 0:
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 */
  __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_NOUN_PROP_A3SG_PNON_NOM, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 132, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":133
 *                 return "A3PL+P3SG"
 *         elif parseString == "ADJ$NOUN+PROP+A3SG+PNON+NOM":
 *             if index > 0:             # <<<<<<<<<<<<<<
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 *             # BU, U #
 */
    __pyx_t_3 = PyObject_RichCompare(__pyx_v_index, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 133, __pyx_L1_error)
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 133, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":134
 *         elif parseString == "ADJ$NOUN+PROP+A3SG+PNON+NOM":
 *             if index > 0:
 *                 return "NOUN+PROP+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *             # BU, U #
 *         elif parseString == "DET$PRON+DEMONSP+A3SG+PNON+NOM":
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_NOUN_PROP_A3SG_PNON_NOM);
      __pyx_r = __pyx_kp_u_NOUN_PROP_A3SG_PNON_NOM;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":133
 *                 return "A3PL+P3SG"
 *         elif parseString == "ADJ$NOUN+PROP+A3SG+PNON+NOM":
 *             if index > 0:             # <<<<<<<<<<<<<<
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 *             # BU, U #
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":132
 *             else:
 *                 return "A3PL+P3SG"
 *         elif parseString == "ADJ$NOUN+PROP+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if index > 0:
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 */
    goto __pyx_L3;
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":136
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 *             # BU, U #
 *         elif parseString == "DET$PRON+DEMONSP+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "DET"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_DET_PRON_DEMONSP_A3SG_PNON_NOM, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 136, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_9 != 0);
  if (__pyx_t_8) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":137
 *             # BU, U #
 *         elif parseString == "DET$PRON+DEMONSP+A3SG+PNON+NOM":
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "DET"
 *             return "PRON+DEMONSP+A3SG+PNON+NOM"
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNoun); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 137, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 137, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 137, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_1 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 137, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 137, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 137, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_8) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":138
 *         elif parseString == "DET$PRON+DEMONSP+A3SG+PNON+NOM":
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "DET"             # <<<<<<<<<<<<<<
 *             return "PRON+DEMONSP+A3SG+PNON+NOM"
 *             # gelebilir #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_n_u_DET);
      __pyx_r = __pyx_n_u_DET;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":137
 *             # BU, U #
 *         elif parseString == "DET$PRON+DEMONSP+A3SG+PNON+NOM":
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "DET"
 *             return "PRON+DEMONSP+A3SG+PNON+NOM"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":139
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "DET"
 *             return "PRON+DEMONSP+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *             # gelebilir #
 *         elif parseString == "AOR+A3SG$AOR^DB+ADJ+ZERO":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_PRON_DEMONSP_A3SG_PNON_NOM);
    __pyx_r = __pyx_kp_u_PRON_DEMONSP_A3SG_PNON_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":136
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 *             # BU, U #
 *         elif parseString == "DET$PRON+DEMONSP+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "DET"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":141
 *             return "PRON+DEMONSP+A3SG+PNON+NOM"
 *             # gelebilir #
 *         elif parseString == "AOR+A3SG$AOR^DB+ADJ+ZERO":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "AOR+A3SG"
 */
  __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_AOR_A3SG_AOR_DB_ADJ_ZERO, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 141, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":142
 *             # gelebilir #
 *         elif parseString == "AOR+A3SG$AOR^DB+ADJ+ZERO":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "AOR+A3SG"
 *             elif AutoDisambiguator.isFirstWord(index):
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isBeforeLastWord); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 142, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 142, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 142, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 142, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_1) {
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1); __pyx_t_1 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 142, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 142, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":143
 *         elif parseString == "AOR+A3SG$AOR^DB+ADJ+ZERO":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "AOR+A3SG"             # <<<<<<<<<<<<<<
 *             elif AutoDisambiguator.isFirstWord(index):
 *                 return "AOR^DB+ADJ+ZERO"
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_AOR_A3SG);
      __pyx_r = __pyx_kp_u_AOR_A3SG;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":142
 *             # gelebilir #
 *         elif parseString == "AOR+A3SG$AOR^DB+ADJ+ZERO":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "AOR+A3SG"
 *             elif AutoDisambiguator.isFirstWord(index):
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":144
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "AOR+A3SG"
 *             elif AutoDisambiguator.isFirstWord(index):             # <<<<<<<<<<<<<<
 *                 return "AOR^DB+ADJ+ZERO"
 *             elif AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isFirstWord); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 144, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v_index) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_index);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 144, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 144, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":145
 *                 return "AOR+A3SG"
 *             elif AutoDisambiguator.isFirstWord(index):
 *                 return "AOR^DB+ADJ+ZERO"             # <<<<<<<<<<<<<<
 *             elif AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "AOR^DB+ADJ+ZERO"
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_AOR_DB_ADJ_ZERO);
      __pyx_r = __pyx_kp_u_AOR_DB_ADJ_ZERO;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":144
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "AOR+A3SG"
 *             elif AutoDisambiguator.isFirstWord(index):             # <<<<<<<<<<<<<<
 *                 return "AOR^DB+ADJ+ZERO"
 *             elif AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":146
 *             elif AutoDisambiguator.isFirstWord(index):
 *                 return "AOR^DB+ADJ+ZERO"
 *             elif AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "AOR^DB+ADJ+ZERO"
 *             else:
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNounOrAdjective); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 146, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 146, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_1 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 146, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 146, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 146, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":147
 *                 return "AOR^DB+ADJ+ZERO"
 *             elif AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "AOR^DB+ADJ+ZERO"             # <<<<<<<<<<<<<<
 *             else:
 *                 return "AOR+A3SG"
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_AOR_DB_ADJ_ZERO);
      __pyx_r = __pyx_kp_u_AOR_DB_ADJ_ZERO;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":146
 *             elif AutoDisambiguator.isFirstWord(index):
 *                 return "AOR^DB+ADJ+ZERO"
 *             elif AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "AOR^DB+ADJ+ZERO"
 *             else:
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":149
 *                 return "AOR^DB+ADJ+ZERO"
 *             else:
 *                 return "AOR+A3SG"             # <<<<<<<<<<<<<<
 *         elif parseString == "ADV$NOUN+A3SG+PNON+NOM":
 *             return "ADV"
 */
    /*else*/ {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_AOR_A3SG);
      __pyx_r = __pyx_kp_u_AOR_A3SG;
      goto __pyx_L0;
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":141
 *             return "PRON+DEMONSP+A3SG+PNON+NOM"
 *             # gelebilir #
 *         elif parseString == "AOR+A3SG$AOR^DB+ADJ+ZERO":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "AOR+A3SG"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":150
 *             else:
 *                 return "AOR+A3SG"
 *         elif parseString == "ADV$NOUN+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             return "ADV"
 *         elif parseString == "ADJ$ADV":
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADV_NOUN_A3SG_PNON_NOM, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 150, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_9 != 0);
  if (__pyx_t_8) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":151
 *                 return "AOR+A3SG"
 *         elif parseString == "ADV$NOUN+A3SG+PNON+NOM":
 *             return "ADV"             # <<<<<<<<<<<<<<
 *         elif parseString == "ADJ$ADV":
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ADV);
    __pyx_r = __pyx_n_u_ADV;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":150
 *             else:
 *                 return "AOR+A3SG"
 *         elif parseString == "ADV$NOUN+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             return "ADV"
 *         elif parseString == "ADJ$ADV":
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":152
 *         elif parseString == "ADV$NOUN+A3SG+PNON+NOM":
 *             return "ADV"
 *         elif parseString == "ADJ$ADV":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "ADJ"
 */
  __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_ADV, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 152, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":153
 *             return "ADV"
 *         elif parseString == "ADJ$ADV":
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ"
 *             return "ADV"
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNoun); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 153, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 153, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 153, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 153, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_1) {
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1); __pyx_t_1 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 153, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 153, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":154
 *         elif parseString == "ADJ$ADV":
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "ADJ"             # <<<<<<<<<<<<<<
 *             return "ADV"
 *         elif parseString == "P2SG$PNON":
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_n_u_ADJ);
      __pyx_r = __pyx_n_u_ADJ;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":153
 *             return "ADV"
 *         elif parseString == "ADJ$ADV":
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ"
 *             return "ADV"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":155
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "ADJ"
 *             return "ADV"             # <<<<<<<<<<<<<<
 *         elif parseString == "P2SG$PNON":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ADV);
    __pyx_r = __pyx_n_u_ADV;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":152
 *         elif parseString == "ADV$NOUN+A3SG+PNON+NOM":
 *             return "ADV"
 *         elif parseString == "ADJ$ADV":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "ADJ"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":156
 *                 return "ADJ"
 *             return "ADV"
 *         elif parseString == "P2SG$PNON":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "P2SG"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_P2SG_PNON, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 156, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_9 != 0);
  if (__pyx_t_8) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":157
 *             return "ADV"
 *         elif parseString == "P2SG$PNON":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "P2SG"
 *             return "PNON"
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isAnyWordSecondPerson); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 157, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 157, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 157, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_1 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 157, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_10, __pyx_v_correctParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 157, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 157, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_8) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":158
 *         elif parseString == "P2SG$PNON":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "P2SG"             # <<<<<<<<<<<<<<
 *             return "PNON"
 *             # etti, krd #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_n_u_P2SG);
      __pyx_r = __pyx_n_u_P2SG;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":157
 *             return "ADV"
 *         elif parseString == "P2SG$PNON":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "P2SG"
 *             return "PNON"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":159
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "P2SG"
 *             return "PNON"             # <<<<<<<<<<<<<<
 *             # etti, krd #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO$VERB+POS":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_PNON);
    __pyx_r = __pyx_n_u_PNON;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":156
 *                 return "ADJ"
 *             return "ADV"
 *         elif parseString == "P2SG$PNON":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "P2SG"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":161
 *             return "PNON"
 *             # etti, krd #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO$VERB+POS":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "VERB+POS"
 */
  __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 161, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":162
 *             # etti, krd #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO$VERB+POS":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "VERB+POS"
 *             # LE #
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isBeforeLastWord); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 162, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 162, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 162, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 162, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_1) {
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1); __pyx_t_1 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 162, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 162, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":163
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO$VERB+POS":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "VERB+POS"             # <<<<<<<<<<<<<<
 *             # LE #
 *         elif parseString == "CONJ$POSTP+PCNOM":
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_VERB_POS);
      __pyx_r = __pyx_kp_u_VERB_POS;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":162
 *             # etti, krd #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO$VERB+POS":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "VERB+POS"
 *             # LE #
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":161
 *             return "PNON"
 *             # etti, krd #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO$VERB+POS":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "VERB+POS"
 */
    goto __pyx_L3;
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":165
 *                 return "VERB+POS"
 *             # LE #
 *         elif parseString == "CONJ$POSTP+PCNOM":             # <<<<<<<<<<<<<<
 *             return "POSTP+PCNOM"
 *             # gelecek #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_CONJ_POSTP_PCNOM, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 165, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_9 != 0);
  if (__pyx_t_8) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":166
 *             # LE #
 *         elif parseString == "CONJ$POSTP+PCNOM":
 *             return "POSTP+PCNOM"             # <<<<<<<<<<<<<<
 *             # gelecek #
 *         elif parseString == "POS+FUT+A3SG$POS^DB+ADJ+FUTPART+PNON":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_POSTP_PCNOM);
    __pyx_r = __pyx_kp_u_POSTP_PCNOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":165
 *                 return "VERB+POS"
 *             # LE #
 *         elif parseString == "CONJ$POSTP+PCNOM":             # <<<<<<<<<<<<<<
 *             return "POSTP+PCNOM"
 *             # gelecek #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":168
 *             return "POSTP+PCNOM"
 *             # gelecek #
 *         elif parseString == "POS+FUT+A3SG$POS^DB+ADJ+FUTPART+PNON":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "POS+FUT+A3SG"
 */
  __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_POS_FUT_A3SG_POS_DB_ADJ_FUTPART, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 168, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":169
 *             # gelecek #
 *         elif parseString == "POS+FUT+A3SG$POS^DB+ADJ+FUTPART+PNON":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "POS+FUT+A3SG"
 *             return "POS^DB+ADJ+FUTPART+PNON"
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isBeforeLastWord); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 169, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 169, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 169, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_1 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 169, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 169, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 169, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":170
 *         elif parseString == "POS+FUT+A3SG$POS^DB+ADJ+FUTPART+PNON":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "POS+FUT+A3SG"             # <<<<<<<<<<<<<<
 *             return "POS^DB+ADJ+FUTPART+PNON"
 *         elif parseString == "ADJ^DB$NOUN+A3SG+PNON+NOM^DB":
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_POS_FUT_A3SG);
      __pyx_r = __pyx_kp_u_POS_FUT_A3SG;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":169
 *             # gelecek #
 *         elif parseString == "POS+FUT+A3SG$POS^DB+ADJ+FUTPART+PNON":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "POS+FUT+A3SG"
 *             return "POS^DB+ADJ+FUTPART+PNON"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":171
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "POS+FUT+A3SG"
 *             return "POS^DB+ADJ+FUTPART+PNON"             # <<<<<<<<<<<<<<
 *         elif parseString == "ADJ^DB$NOUN+A3SG+PNON+NOM^DB":
 *             if root == "yok" or root == "dk" or root == "eksik" or root == "rahat" or root == "orta" \
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_POS_DB_ADJ_FUTPART_PNON);
    __pyx_r = __pyx_kp_u_POS_DB_ADJ_FUTPART_PNON;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":168
 *             return "POSTP+PCNOM"
 *             # gelecek #
 *         elif parseString == "POS+FUT+A3SG$POS^DB+ADJ+FUTPART+PNON":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "POS+FUT+A3SG"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":172
 *                 return "POS+FUT+A3SG"
 *             return "POS^DB+ADJ+FUTPART+PNON"
 *         elif parseString == "ADJ^DB$NOUN+A3SG+PNON+NOM^DB":             # <<<<<<<<<<<<<<
 *             if root == "yok" or root == "dk" or root == "eksik" or root == "rahat" or root == "orta" \
 *                     or root == "vasat":
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_DB_NOUN_A3SG_PNON_NOM_DB, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 172, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_9 != 0);
  if (__pyx_t_8) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":173
 *             return "POS^DB+ADJ+FUTPART+PNON"
 *         elif parseString == "ADJ^DB$NOUN+A3SG+PNON+NOM^DB":
 *             if root == "yok" or root == "dk" or root == "eksik" or root == "rahat" or root == "orta" \             # <<<<<<<<<<<<<<
 *                     or root == "vasat":
 *                 return "ADJ^DB"
 */
    __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_root, __pyx_n_u_yok, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 173, __pyx_L1_error)
    __pyx_t_11 = (__pyx_t_9 != 0);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_8 = __pyx_t_11;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_root, __pyx_n_u_dk, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 173, __pyx_L1_error)
    __pyx_t_9 = (__pyx_t_11 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_root, __pyx_n_u_eksik, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 173, __pyx_L1_error)
    __pyx_t_11 = (__pyx_t_9 != 0);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_8 = __pyx_t_11;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_root, __pyx_n_u_rahat, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 173, __pyx_L1_error)
    __pyx_t_9 = (__pyx_t_11 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_8 = __pyx_t_9;
      goto __pyx_L28_bool_binop_done;
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":174
 *         elif parseString == "ADJ^DB$NOUN+A3SG+PNON+NOM^DB":
 *             if root == "yok" or root == "dk" or root == "eksik" or root == "rahat" or root == "orta" \
 *                     or root == "vasat":             # <<<<<<<<<<<<<<
 *                 return "ADJ^DB"
 *             return "NOUN+A3SG+PNON+NOM^DB"
 */
    __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_root, __pyx_n_u_orta, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 173, __pyx_L1_error)

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":173
 *             return "POS^DB+ADJ+FUTPART+PNON"
 *         elif parseString == "ADJ^DB$NOUN+A3SG+PNON+NOM^DB":
 *             if root == "yok" or root == "dk" or root == "eksik" or root == "rahat" or root == "orta" \             # <<<<<<<<<<<<<<
 *                     or root == "vasat":
 *                 return "ADJ^DB"
 */
    __pyx_t_11 = (__pyx_t_9 != 0);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_8 = __pyx_t_11;
      goto __pyx_L28_bool_binop_done;
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":174
 *         elif parseString == "ADJ^DB$NOUN+A3SG+PNON+NOM^DB":
 *             if root == "yok" or root == "dk" or root == "eksik" or root == "rahat" or root == "orta" \
 *                     or root == "vasat":             # <<<<<<<<<<<<<<
 *                 return "ADJ^DB"
 *             return "NOUN+A3SG+PNON+NOM^DB"
 */
    __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_root, __pyx_n_u_vasat, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 174, __pyx_L1_error)
    __pyx_t_9 = (__pyx_t_11 != 0);
    __pyx_t_8 = __pyx_t_9;
    __pyx_L28_bool_binop_done:;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":173
 *             return "POS^DB+ADJ+FUTPART+PNON"
 *         elif parseString == "ADJ^DB$NOUN+A3SG+PNON+NOM^DB":
 *             if root == "yok" or root == "dk" or root == "eksik" or root == "rahat" or root == "orta" \             # <<<<<<<<<<<<<<
 *                     or root == "vasat":
 *                 return "ADJ^DB"
 */
    if (__pyx_t_8) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":175
 *             if root == "yok" or root == "dk" or root == "eksik" or root == "rahat" or root == "orta" \
 *                     or root == "vasat":
 *                 return "ADJ^DB"             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+PNON+NOM^DB"
 *             # yaptk, phelendik #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_ADJ_DB);
      __pyx_r = __pyx_kp_u_ADJ_DB;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":173
 *             return "POS^DB+ADJ+FUTPART+PNON"
 *         elif parseString == "ADJ^DB$NOUN+A3SG+PNON+NOM^DB":
 *             if root == "yok" or root == "dk" or root == "eksik" or root == "rahat" or root == "orta" \             # <<<<<<<<<<<<<<
 *                     or root == "vasat":
 *                 return "ADJ^DB"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":176
 *                     or root == "vasat":
 *                 return "ADJ^DB"
 *             return "NOUN+A3SG+PNON+NOM^DB"             # <<<<<<<<<<<<<<
 *             # yaptk, phelendik #
 *         elif parseString == "POS+PAST+A1PL$POS^DB+ADJ+PASTPART+PNON$POS^DB+NOUN+PASTPART+A3SG+PNON+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB);
    __pyx_r = __pyx_kp_u_NOUN_A3SG_PNON_NOM_DB;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":172
 *                 return "POS+FUT+A3SG"
 *             return "POS^DB+ADJ+FUTPART+PNON"
 *         elif parseString == "ADJ^DB$NOUN+A3SG+PNON+NOM^DB":             # <<<<<<<<<<<<<<
 *             if root == "yok" or root == "dk" or root == "eksik" or root == "rahat" or root == "orta" \
 *                     or root == "vasat":
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":178
 *             return "NOUN+A3SG+PNON+NOM^DB"
 *             # yaptk, phelendik #
 *         elif parseString == "POS+PAST+A1PL$POS^DB+ADJ+PASTPART+PNON$POS^DB+NOUN+PASTPART+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             return "POS+PAST+A1PL"
 *             # ederim, yaparm #
 */
  __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_POS_PAST_A1PL_POS_DB_ADJ_PASTPAR, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 178, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":179
 *             # yaptk, phelendik #
 *         elif parseString == "POS+PAST+A1PL$POS^DB+ADJ+PASTPART+PNON$POS^DB+NOUN+PASTPART+A3SG+PNON+NOM":
 *             return "POS+PAST+A1PL"             # <<<<<<<<<<<<<<
 *             # ederim, yaparm #
 *         elif parseString == "AOR+A1SG$AOR^DB+ADJ+ZERO^DB+NOUN+ZERO+A3SG+P1SG+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_POS_PAST_A1PL);
    __pyx_r = __pyx_kp_u_POS_PAST_A1PL;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":178
 *             return "NOUN+A3SG+PNON+NOM^DB"
 *             # yaptk, phelendik #
 *         elif parseString == "POS+PAST+A1PL$POS^DB+ADJ+PASTPART+PNON$POS^DB+NOUN+PASTPART+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             return "POS+PAST+A1PL"
 *             # ederim, yaparm #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":181
 *             return "POS+PAST+A1PL"
 *             # ederim, yaparm #
 *         elif parseString == "AOR+A1SG$AOR^DB+ADJ+ZERO^DB+NOUN+ZERO+A3SG+P1SG+NOM":             # <<<<<<<<<<<<<<
 *             return "AOR+A1SG"
 *             # geti, vard, ald #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_AOR_A1SG_AOR_DB_ADJ_ZERO_DB_NOUN, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 181, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_9 != 0);
  if (__pyx_t_8) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":182
 *             # ederim, yaparm #
 *         elif parseString == "AOR+A1SG$AOR^DB+ADJ+ZERO^DB+NOUN+ZERO+A3SG+P1SG+NOM":
 *             return "AOR+A1SG"             # <<<<<<<<<<<<<<
 *             # geti, vard, ald #
 *         elif parseString == "ADJ^DB+VERB+ZERO$VERB+POS":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_AOR_A1SG);
    __pyx_r = __pyx_kp_u_AOR_A1SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":181
 *             return "POS+PAST+A1PL"
 *             # ederim, yaparm #
 *         elif parseString == "AOR+A1SG$AOR^DB+ADJ+ZERO^DB+NOUN+ZERO+A3SG+P1SG+NOM":             # <<<<<<<<<<<<<<
 *             return "AOR+A1SG"
 *             # geti, vard, ald #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":184
 *             return "AOR+A1SG"
 *             # geti, vard, ald #
 *         elif parseString == "ADJ^DB+VERB+ZERO$VERB+POS":             # <<<<<<<<<<<<<<
 *             if root == "var" and not AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "ADJ^DB+VERB+ZERO"
 */
  __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_DB_VERB_ZERO_VERB_POS, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 184, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":185
 *             # geti, vard, ald #
 *         elif parseString == "ADJ^DB+VERB+ZERO$VERB+POS":
 *             if root == "var" and not AutoDisambiguator.isPossessivePlural(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ^DB+VERB+ZERO"
 *             return "VERB+POS"
 */
    __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_root, __pyx_n_u_var, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 185, __pyx_L1_error)
    __pyx_t_11 = (__pyx_t_8 != 0);
    if (__pyx_t_11) {
    } else {
      __pyx_t_9 = __pyx_t_11;
      goto __pyx_L35_bool_binop_done;
    }
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isPossessivePlural); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 185, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 185, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 185, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 185, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_1) {
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1); __pyx_t_1 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_10, __pyx_v_correctParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 185, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 185, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_8 = ((!__pyx_t_11) != 0);
    __pyx_t_9 = __pyx_t_8;
    __pyx_L35_bool_binop_done:;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":186
 *         elif parseString == "ADJ^DB+VERB+ZERO$VERB+POS":
 *             if root == "var" and not AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "ADJ^DB+VERB+ZERO"             # <<<<<<<<<<<<<<
 *             return "VERB+POS"
 *             # ancak #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_ADJ_DB_VERB_ZERO);
      __pyx_r = __pyx_kp_u_ADJ_DB_VERB_ZERO;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":185
 *             # geti, vard, ald #
 *         elif parseString == "ADJ^DB+VERB+ZERO$VERB+POS":
 *             if root == "var" and not AutoDisambiguator.isPossessivePlural(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ^DB+VERB+ZERO"
 *             return "VERB+POS"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":187
 *             if root == "var" and not AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "ADJ^DB+VERB+ZERO"
 *             return "VERB+POS"             # <<<<<<<<<<<<<<
 *             # ancak #
 *         elif parseString == "ADV$CONJ":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_VERB_POS);
    __pyx_r = __pyx_kp_u_VERB_POS;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":184
 *             return "AOR+A1SG"
 *             # geti, vard, ald #
 *         elif parseString == "ADJ^DB+VERB+ZERO$VERB+POS":             # <<<<<<<<<<<<<<
 *             if root == "var" and not AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "ADJ^DB+VERB+ZERO"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":189
 *             return "VERB+POS"
 *             # ancak #
 *         elif parseString == "ADV$CONJ":             # <<<<<<<<<<<<<<
 *             return "CONJ"
 *             # yapt, ettii #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADV_CONJ, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 189, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_9 != 0);
  if (__pyx_t_8) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":190
 *             # ancak #
 *         elif parseString == "ADV$CONJ":
 *             return "CONJ"             # <<<<<<<<<<<<<<
 *             # yapt, ettii #
 *         elif parseString == "ADJ+PASTPART+P3SG$NOUN+PASTPART+A3SG+P3SG+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_CONJ);
    __pyx_r = __pyx_n_u_CONJ;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":189
 *             return "VERB+POS"
 *             # ancak #
 *         elif parseString == "ADV$CONJ":             # <<<<<<<<<<<<<<
 *             return "CONJ"
 *             # yapt, ettii #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":192
 *             return "CONJ"
 *             # yapt, ettii #
 *         elif parseString == "ADJ+PASTPART+P3SG$NOUN+PASTPART+A3SG+P3SG+NOM":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+PASTPART+P3SG"
 */
  __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_PASTPART_P3SG_NOUN_PASTPART, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 192, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":193
 *             # yapt, ettii #
 *         elif parseString == "ADJ+PASTPART+P3SG$NOUN+PASTPART+A3SG+P3SG+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ+PASTPART+P3SG"
 *             return "NOUN+PASTPART+A3SG+P3SG+NOM"
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNounOrAdjective); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 193, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 193, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_1 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 193, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 193, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 193, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":194
 *         elif parseString == "ADJ+PASTPART+P3SG$NOUN+PASTPART+A3SG+P3SG+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+PASTPART+P3SG"             # <<<<<<<<<<<<<<
 *             return "NOUN+PASTPART+A3SG+P3SG+NOM"
 *             # NCE, SONRA #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_ADJ_PASTPART_P3SG);
      __pyx_r = __pyx_kp_u_ADJ_PASTPART_P3SG;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":193
 *             # yapt, ettii #
 *         elif parseString == "ADJ+PASTPART+P3SG$NOUN+PASTPART+A3SG+P3SG+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ+PASTPART+P3SG"
 *             return "NOUN+PASTPART+A3SG+P3SG+NOM"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":195
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+PASTPART+P3SG"
 *             return "NOUN+PASTPART+A3SG+P3SG+NOM"             # <<<<<<<<<<<<<<
 *             # NCE, SONRA #
 *         elif parseString == "ADV$NOUN+A3SG+PNON+NOM$POSTP+PCABL":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_PASTPART_A3SG_P3SG_NOM);
    __pyx_r = __pyx_kp_u_NOUN_PASTPART_A3SG_P3SG_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":192
 *             return "CONJ"
 *             # yapt, ettii #
 *         elif parseString == "ADJ+PASTPART+P3SG$NOUN+PASTPART+A3SG+P3SG+NOM":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+PASTPART+P3SG"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":197
 *             return "NOUN+PASTPART+A3SG+P3SG+NOM"
 *             # NCE, SONRA #
 *         elif parseString == "ADV$NOUN+A3SG+PNON+NOM$POSTP+PCABL":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADV_NOUN_A3SG_PNON_NOM_POSTP_PCA, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 197, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_9 != 0);
  if (__pyx_t_8) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":198
 *             # NCE, SONRA #
 *         elif parseString == "ADV$NOUN+A3SG+PNON+NOM$POSTP+PCABL":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCABL"
 *             return "ADV"
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_hasPreviousWordTag); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 198, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_MorphologicalTag); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 198, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_ABLATIVE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 198, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[4] = {__pyx_t_1, __pyx_v_index, __pyx_v_correctParses, __pyx_t_4};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 198, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[4] = {__pyx_t_1, __pyx_v_index, __pyx_v_correctParses, __pyx_t_4};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 198, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(3+__pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 198, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__pyx_t_1) {
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1); __pyx_t_1 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_10, __pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_10, __pyx_t_4);
      __pyx_t_4 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 198, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 198, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_8) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":199
 *         elif parseString == "ADV$NOUN+A3SG+PNON+NOM$POSTP+PCABL":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"             # <<<<<<<<<<<<<<
 *             return "ADV"
 *         elif parseString == "NARR+A3SG$NARR^DB+ADJ+ZERO":
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_POSTP_PCABL);
      __pyx_r = __pyx_kp_u_POSTP_PCABL;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":198
 *             # NCE, SONRA #
 *         elif parseString == "ADV$NOUN+A3SG+PNON+NOM$POSTP+PCABL":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCABL"
 *             return "ADV"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":200
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"
 *             return "ADV"             # <<<<<<<<<<<<<<
 *         elif parseString == "NARR+A3SG$NARR^DB+ADJ+ZERO":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ADV);
    __pyx_r = __pyx_n_u_ADV;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":197
 *             return "NOUN+PASTPART+A3SG+P3SG+NOM"
 *             # NCE, SONRA #
 *         elif parseString == "ADV$NOUN+A3SG+PNON+NOM$POSTP+PCABL":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":201
 *                 return "POSTP+PCABL"
 *             return "ADV"
 *         elif parseString == "NARR+A3SG$NARR^DB+ADJ+ZERO":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "NARR+A3SG"
 */
  __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NARR_A3SG_NARR_DB_ADJ_ZERO, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 201, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":202
 *             return "ADV"
 *         elif parseString == "NARR+A3SG$NARR^DB+ADJ+ZERO":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "NARR+A3SG"
 *             return "NARR^DB+ADJ+ZERO"
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isBeforeLastWord); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 202, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 202, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 202, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 202, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 202, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 202, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":203
 *         elif parseString == "NARR+A3SG$NARR^DB+ADJ+ZERO":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "NARR+A3SG"             # <<<<<<<<<<<<<<
 *             return "NARR^DB+ADJ+ZERO"
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM":
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_NARR_A3SG);
      __pyx_r = __pyx_kp_u_NARR_A3SG;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":202
 *             return "ADV"
 *         elif parseString == "NARR+A3SG$NARR^DB+ADJ+ZERO":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "NARR+A3SG"
 *             return "NARR^DB+ADJ+ZERO"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":204
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "NARR+A3SG"
 *             return "NARR^DB+ADJ+ZERO"             # <<<<<<<<<<<<<<
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM":
 *             if index > 0:
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NARR_DB_ADJ_ZERO);
    __pyx_r = __pyx_kp_u_NARR_DB_ADJ_ZERO;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":201
 *                 return "POSTP+PCABL"
 *             return "ADV"
 *         elif parseString == "NARR+A3SG$NARR^DB+ADJ+ZERO":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "NARR+A3SG"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":205
 *                 return "NARR+A3SG"
 *             return "NARR^DB+ADJ+ZERO"
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if index > 0:
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_NOUN_A3SG_PNON_NOM_NOUN_PROP, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 205, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_9 != 0);
  if (__pyx_t_8) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":206
 *             return "NARR^DB+ADJ+ZERO"
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM":
 *             if index > 0:             # <<<<<<<<<<<<<<
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 *             elif AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 */
    __pyx_t_3 = PyObject_RichCompare(__pyx_v_index, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 206, __pyx_L1_error)
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 206, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_8) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":207
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM":
 *             if index > 0:
 *                 return "NOUN+PROP+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *             elif AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ"
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_NOUN_PROP_A3SG_PNON_NOM);
      __pyx_r = __pyx_kp_u_NOUN_PROP_A3SG_PNON_NOM;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":206
 *             return "NARR^DB+ADJ+ZERO"
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM":
 *             if index > 0:             # <<<<<<<<<<<<<<
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 *             elif AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":208
 *             if index > 0:
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 *             elif AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ"
 *             return "NOUN+A3SG+PNON+NOM"
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNounOrAdjective); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 208, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 208, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 208, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 208, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 208, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 208, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_8) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":209
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 *             elif AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ"             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+PNON+NOM"
 *             # dediim #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_n_u_ADJ);
      __pyx_r = __pyx_n_u_ADJ;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":208
 *             if index > 0:
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 *             elif AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ"
 *             return "NOUN+A3SG+PNON+NOM"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":210
 *             elif AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ"
 *             return "NOUN+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *             # dediim #
 *         elif parseString == "ADJ+PASTPART+P1SG$NOUN+PASTPART+A3SG+P1SG+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_A3SG_PNON_NOM);
    __pyx_r = __pyx_kp_u_NOUN_A3SG_PNON_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":205
 *                 return "NARR+A3SG"
 *             return "NARR^DB+ADJ+ZERO"
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if index > 0:
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":212
 *             return "NOUN+A3SG+PNON+NOM"
 *             # dediim #
 *         elif parseString == "ADJ+PASTPART+P1SG$NOUN+PASTPART+A3SG+P1SG+NOM":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+PASTPART+P1SG"
 */
  __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_PASTPART_P1SG_NOUN_PASTPART, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 212, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":213
 *             # dediim #
 *         elif parseString == "ADJ+PASTPART+P1SG$NOUN+PASTPART+A3SG+P1SG+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ+PASTPART+P1SG"
 *             return "NOUN+PASTPART+A3SG+P1SG+NOM"
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNounOrAdjective); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 213, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 213, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 213, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 213, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 213, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 213, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":214
 *         elif parseString == "ADJ+PASTPART+P1SG$NOUN+PASTPART+A3SG+P1SG+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+PASTPART+P1SG"             # <<<<<<<<<<<<<<
 *             return "NOUN+PASTPART+A3SG+P1SG+NOM"
 *             # O #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_ADJ_PASTPART_P1SG);
      __pyx_r = __pyx_kp_u_ADJ_PASTPART_P1SG;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":213
 *             # dediim #
 *         elif parseString == "ADJ+PASTPART+P1SG$NOUN+PASTPART+A3SG+P1SG+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ+PASTPART+P1SG"
 *             return "NOUN+PASTPART+A3SG+P1SG+NOM"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":215
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+PASTPART+P1SG"
 *             return "NOUN+PASTPART+A3SG+P1SG+NOM"             # <<<<<<<<<<<<<<
 *             # O #
 *         elif parseString == "DET$PRON+DEMONSP+A3SG+PNON+NOM$PRON+PERS+A3SG+PNON+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_PASTPART_A3SG_P1SG_NOM);
    __pyx_r = __pyx_kp_u_NOUN_PASTPART_A3SG_P1SG_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":212
 *             return "NOUN+A3SG+PNON+NOM"
 *             # dediim #
 *         elif parseString == "ADJ+PASTPART+P1SG$NOUN+PASTPART+A3SG+P1SG+NOM":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+PASTPART+P1SG"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":217
 *             return "NOUN+PASTPART+A3SG+P1SG+NOM"
 *             # O #
 *         elif parseString == "DET$PRON+DEMONSP+A3SG+PNON+NOM$PRON+PERS+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "DET"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_DET_PRON_DEMONSP_A3SG_PNON_NOM_P, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 217, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_9 != 0);
  if (__pyx_t_8) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":218
 *             # O #
 *         elif parseString == "DET$PRON+DEMONSP+A3SG+PNON+NOM$PRON+PERS+A3SG+PNON+NOM":
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "DET"
 *             return "PRON+PERS+A3SG+PNON+NOM"
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNoun); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 218, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 218, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 218, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 218, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 218, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 218, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_8) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":219
 *         elif parseString == "DET$PRON+DEMONSP+A3SG+PNON+NOM$PRON+PERS+A3SG+PNON+NOM":
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "DET"             # <<<<<<<<<<<<<<
 *             return "PRON+PERS+A3SG+PNON+NOM"
 *             # BAZI #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_n_u_DET);
      __pyx_r = __pyx_n_u_DET;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":218
 *             # O #
 *         elif parseString == "DET$PRON+DEMONSP+A3SG+PNON+NOM$PRON+PERS+A3SG+PNON+NOM":
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "DET"
 *             return "PRON+PERS+A3SG+PNON+NOM"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":220
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "DET"
 *             return "PRON+PERS+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *             # BAZI #
 *         elif parseString == "ADJ$DET$PRON+QUANTP+A3SG+P3SG+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_PRON_PERS_A3SG_PNON_NOM);
    __pyx_r = __pyx_kp_u_PRON_PERS_A3SG_PNON_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":217
 *             return "NOUN+PASTPART+A3SG+P1SG+NOM"
 *             # O #
 *         elif parseString == "DET$PRON+DEMONSP+A3SG+PNON+NOM$PRON+PERS+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "DET"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":222
 *             return "PRON+PERS+A3SG+PNON+NOM"
 *             # BAZI #
 *         elif parseString == "ADJ$DET$PRON+QUANTP+A3SG+P3SG+NOM":             # <<<<<<<<<<<<<<
 *             return "DET"
 *             # ONUN, ONA, ONDAN, ONUNLA, OYDU, ONUNK #
 */
  __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_DET_PRON_QUANTP_A3SG_P3SG_NO, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 222, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":223
 *             # BAZI #
 *         elif parseString == "ADJ$DET$PRON+QUANTP+A3SG+P3SG+NOM":
 *             return "DET"             # <<<<<<<<<<<<<<
 *             # ONUN, ONA, ONDAN, ONUNLA, OYDU, ONUNK #
 *         elif parseString == "DEMONSP$PERS":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_DET);
    __pyx_r = __pyx_n_u_DET;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":222
 *             return "PRON+PERS+A3SG+PNON+NOM"
 *             # BAZI #
 *         elif parseString == "ADJ$DET$PRON+QUANTP+A3SG+P3SG+NOM":             # <<<<<<<<<<<<<<
 *             return "DET"
 *             # ONUN, ONA, ONDAN, ONUNLA, OYDU, ONUNK #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":225
 *             return "DET"
 *             # ONUN, ONA, ONDAN, ONUNLA, OYDU, ONUNK #
 *         elif parseString == "DEMONSP$PERS":             # <<<<<<<<<<<<<<
 *             return "PERS"
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_DEMONSP_PERS, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 225, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_9 != 0);
  if (__pyx_t_8) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":226
 *             # ONUN, ONA, ONDAN, ONUNLA, OYDU, ONUNK #
 *         elif parseString == "DEMONSP$PERS":
 *             return "PERS"             # <<<<<<<<<<<<<<
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_PERS);
    __pyx_r = __pyx_n_u_PERS;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":225
 *             return "DET"
 *             # ONUN, ONA, ONDAN, ONUNLA, OYDU, ONUNK #
 *         elif parseString == "DEMONSP$PERS":             # <<<<<<<<<<<<<<
 *             return "PERS"
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":227
 *         elif parseString == "DEMONSP$PERS":
 *             return "PERS"
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ"
 */
  __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_NOUN_A3SG_PNON_NOM_VERB_POS, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 227, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":228
 *             return "PERS"
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ"
 *             return "NOUN+A3SG+PNON+NOM"
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNounOrAdjective); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 228, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 228, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 228, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 228, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 228, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 228, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":229
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ"             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+PNON+NOM"
 *             # hazineler, kymetler #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_n_u_ADJ);
      __pyx_r = __pyx_n_u_ADJ;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":228
 *             return "PERS"
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ"
 *             return "NOUN+A3SG+PNON+NOM"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":230
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ"
 *             return "NOUN+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *             # hazineler, kymetler #
 *         elif parseString == "A3PL+PNON+NOM$A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A3PL$PROP+A3PL+PNON+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_A3SG_PNON_NOM);
    __pyx_r = __pyx_kp_u_NOUN_A3SG_PNON_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":227
 *         elif parseString == "DEMONSP$PERS":
 *             return "PERS"
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":232
 *             return "NOUN+A3SG+PNON+NOM"
 *             # hazineler, kymetler #
 *         elif parseString == "A3PL+PNON+NOM$A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A3PL$PROP+A3PL+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if index > 0:
 *                 if fsmParses[index].getFsmParse(0).isCapitalWord():
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_A3PL_PNON_NOM_A3SG_PNON_NOM_DB_V, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 232, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_9 != 0);
  if (__pyx_t_8) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":233
 *             # hazineler, kymetler #
 *         elif parseString == "A3PL+PNON+NOM$A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A3PL$PROP+A3PL+PNON+NOM":
 *             if index > 0:             # <<<<<<<<<<<<<<
 *                 if fsmParses[index].getFsmParse(0).isCapitalWord():
 *                     return "PROP+A3PL+PNON+NOM"
 */
    __pyx_t_3 = PyObject_RichCompare(__pyx_v_index, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 233, __pyx_L1_error)
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 233, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_8) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":234
 *         elif parseString == "A3PL+PNON+NOM$A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A3PL$PROP+A3PL+PNON+NOM":
 *             if index > 0:
 *                 if fsmParses[index].getFsmParse(0).isCapitalWord():             # <<<<<<<<<<<<<<
 *                     return "PROP+A3PL+PNON+NOM"
 *                 return "A3PL+PNON+NOM"
 */
      if (unlikely(__pyx_v_fsmParses == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 234, __pyx_L1_error)
      }
      __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_v_fsmParses, __pyx_v_index); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 234, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_getFsmParse); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 234, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_4, __pyx_int_0) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_int_0);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 234, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_isCapitalWord); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 234, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      __pyx_t_3 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 234, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 234, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_8) {

        /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":235
 *             if index > 0:
 *                 if fsmParses[index].getFsmParse(0).isCapitalWord():
 *                     return "PROP+A3PL+PNON+NOM"             # <<<<<<<<<<<<<<
 *                 return "A3PL+PNON+NOM"
 *             # ARTIK, GER #
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_kp_u_PROP_A3PL_PNON_NOM);
        __pyx_r = __pyx_kp_u_PROP_A3PL_PNON_NOM;
        goto __pyx_L0;

        /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":234
 *         elif parseString == "A3PL+PNON+NOM$A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A3PL$PROP+A3PL+PNON+NOM":
 *             if index > 0:
 *                 if fsmParses[index].getFsmParse(0).isCapitalWord():             # <<<<<<<<<<<<<<
 *                     return "PROP+A3PL+PNON+NOM"
 *                 return "A3PL+PNON+NOM"
 */
      }

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":236
 *                 if fsmParses[index].getFsmParse(0).isCapitalWord():
 *                     return "PROP+A3PL+PNON+NOM"
 *                 return "A3PL+PNON+NOM"             # <<<<<<<<<<<<<<
 *             # ARTIK, GER #
 *         elif parseString == "ADJ$ADV$NOUN+A3SG+PNON+NOM":
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_A3PL_PNON_NOM);
      __pyx_r = __pyx_kp_u_A3PL_PNON_NOM;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":233
 *             # hazineler, kymetler #
 *         elif parseString == "A3PL+PNON+NOM$A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A3PL$PROP+A3PL+PNON+NOM":
 *             if index > 0:             # <<<<<<<<<<<<<<
 *                 if fsmParses[index].getFsmParse(0).isCapitalWord():
 *                     return "PROP+A3PL+PNON+NOM"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":232
 *             return "NOUN+A3SG+PNON+NOM"
 *             # hazineler, kymetler #
 *         elif parseString == "A3PL+PNON+NOM$A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A3PL$PROP+A3PL+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if index > 0:
 *                 if fsmParses[index].getFsmParse(0).isCapitalWord():
 */
    goto __pyx_L3;
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":238
 *                 return "A3PL+PNON+NOM"
 *             # ARTIK, GER #
 *         elif parseString == "ADJ$ADV$NOUN+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if root == "artk":
 *                 return "ADV"
 */
  __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_ADV_NOUN_A3SG_PNON_NOM, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 238, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":239
 *             # ARTIK, GER #
 *         elif parseString == "ADJ$ADV$NOUN+A3SG+PNON+NOM":
 *             if root == "artk":             # <<<<<<<<<<<<<<
 *                 return "ADV"
 *             elif AutoDisambiguator.isNextWordNoun(index, fsmParses):
 */
    __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_root, __pyx_n_u_artk, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 239, __pyx_L1_error)
    __pyx_t_8 = (__pyx_t_9 != 0);
    if (__pyx_t_8) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":240
 *         elif parseString == "ADJ$ADV$NOUN+A3SG+PNON+NOM":
 *             if root == "artk":
 *                 return "ADV"             # <<<<<<<<<<<<<<
 *             elif AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "ADJ"
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_n_u_ADV);
      __pyx_r = __pyx_n_u_ADV;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":239
 *             # ARTIK, GER #
 *         elif parseString == "ADJ$ADV$NOUN+A3SG+PNON+NOM":
 *             if root == "artk":             # <<<<<<<<<<<<<<
 *                 return "ADV"
 *             elif AutoDisambiguator.isNextWordNoun(index, fsmParses):
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":241
 *             if root == "artk":
 *                 return "ADV"
 *             elif AutoDisambiguator.isNextWordNoun(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ"
 *             return "ADV"
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNoun); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 241, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 241, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 241, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 241, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_2) {
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2); __pyx_t_2 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 241, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 241, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_8) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":242
 *                 return "ADV"
 *             elif AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "ADJ"             # <<<<<<<<<<<<<<
 *             return "ADV"
 *         elif parseString == "P1SG+NOM$PNON+NOM^DB+VERB+ZERO+PRES+A1SG":
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_n_u_ADJ);
      __pyx_r = __pyx_n_u_ADJ;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":241
 *             if root == "artk":
 *                 return "ADV"
 *             elif AutoDisambiguator.isNextWordNoun(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ"
 *             return "ADV"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":243
 *             elif AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "ADJ"
 *             return "ADV"             # <<<<<<<<<<<<<<
 *         elif parseString == "P1SG+NOM$PNON+NOM^DB+VERB+ZERO+PRES+A1SG":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses) or root == "deil":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ADV);
    __pyx_r = __pyx_n_u_ADV;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":238
 *                 return "A3PL+PNON+NOM"
 *             # ARTIK, GER #
 *         elif parseString == "ADJ$ADV$NOUN+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if root == "artk":
 *                 return "ADV"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":244
 *                 return "ADJ"
 *             return "ADV"
 *         elif parseString == "P1SG+NOM$PNON+NOM^DB+VERB+ZERO+PRES+A1SG":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses) or root == "deil":
 *                 return "PNON+NOM^DB+VERB+ZERO+PRES+A1SG"
 */
  __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_P1SG_NOM_PNON_NOM_DB_VERB_ZERO_P, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 244, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":245
 *             return "ADV"
 *         elif parseString == "P1SG+NOM$PNON+NOM^DB+VERB+ZERO+PRES+A1SG":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses) or root == "deil":             # <<<<<<<<<<<<<<
 *                 return "PNON+NOM^DB+VERB+ZERO+PRES+A1SG"
 *             return "P1SG+NOM"
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isBeforeLastWord); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 245, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 245, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 245, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_2 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 245, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 245, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 245, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!__pyx_t_8) {
    } else {
      __pyx_t_9 = __pyx_t_8;
      goto __pyx_L48_bool_binop_done;
    }
    __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_root, __pyx_n_u_deil, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 245, __pyx_L1_error)
    __pyx_t_11 = (__pyx_t_8 != 0);
    __pyx_t_9 = __pyx_t_11;
    __pyx_L48_bool_binop_done:;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":246
 *         elif parseString == "P1SG+NOM$PNON+NOM^DB+VERB+ZERO+PRES+A1SG":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses) or root == "deil":
 *                 return "PNON+NOM^DB+VERB+ZERO+PRES+A1SG"             # <<<<<<<<<<<<<<
 *             return "P1SG+NOM"
 *             # grlmektedir #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_PNON_NOM_DB_VERB_ZERO_PRES_A1SG);
      __pyx_r = __pyx_kp_u_PNON_NOM_DB_VERB_ZERO_PRES_A1SG;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":245
 *             return "ADV"
 *         elif parseString == "P1SG+NOM$PNON+NOM^DB+VERB+ZERO+PRES+A1SG":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses) or root == "deil":             # <<<<<<<<<<<<<<
 *                 return "PNON+NOM^DB+VERB+ZERO+PRES+A1SG"
 *             return "P1SG+NOM"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":247
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses) or root == "deil":
 *                 return "PNON+NOM^DB+VERB+ZERO+PRES+A1SG"
 *             return "P1SG+NOM"             # <<<<<<<<<<<<<<
 *             # grlmektedir #
 *         elif parseString == "POS+PROG2$POS^DB+NOUN+INF+A3SG+PNON+LOC^DB+VERB+ZERO+PRES":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_P1SG_NOM);
    __pyx_r = __pyx_kp_u_P1SG_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":244
 *                 return "ADJ"
 *             return "ADV"
 *         elif parseString == "P1SG+NOM$PNON+NOM^DB+VERB+ZERO+PRES+A1SG":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses) or root == "deil":
 *                 return "PNON+NOM^DB+VERB+ZERO+PRES+A1SG"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":249
 *             return "P1SG+NOM"
 *             # grlmektedir #
 *         elif parseString == "POS+PROG2$POS^DB+NOUN+INF+A3SG+PNON+LOC^DB+VERB+ZERO+PRES":             # <<<<<<<<<<<<<<
 *             return "POS+PROG2"
 *             # NE #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_POS_PROG2_POS_DB_NOUN_INF_A3SG_P, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 249, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":250
 *             # grlmektedir #
 *         elif parseString == "POS+PROG2$POS^DB+NOUN+INF+A3SG+PNON+LOC^DB+VERB+ZERO+PRES":
 *             return "POS+PROG2"             # <<<<<<<<<<<<<<
 *             # NE #
 *         elif parseString == "ADJ$ADV$CONJ$PRON+QUESP+A3SG+PNON+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_POS_PROG2);
    __pyx_r = __pyx_kp_u_POS_PROG2;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":249
 *             return "P1SG+NOM"
 *             # grlmektedir #
 *         elif parseString == "POS+PROG2$POS^DB+NOUN+INF+A3SG+PNON+LOC^DB+VERB+ZERO+PRES":             # <<<<<<<<<<<<<<
 *             return "POS+PROG2"
 *             # NE #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":252
 *             return "POS+PROG2"
 *             # NE #
 *         elif parseString == "ADJ$ADV$CONJ$PRON+QUESP+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if lastWord == "?":
 *                 return "PRON+QUESP+A3SG+PNON+NOM"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_ADV_CONJ_PRON_QUESP_A3SG_PNO, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 252, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":253
 *             # NE #
 *         elif parseString == "ADJ$ADV$CONJ$PRON+QUESP+A3SG+PNON+NOM":
 *             if lastWord == "?":             # <<<<<<<<<<<<<<
 *                 return "PRON+QUESP+A3SG+PNON+NOM"
 *             if AutoDisambiguator.containsTwoNeOrYa(fsmParses, "ne"):
 */
    __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_lastWord, __pyx_kp_u_, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 253, __pyx_L1_error)
    __pyx_t_11 = (__pyx_t_9 != 0);
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":254
 *         elif parseString == "ADJ$ADV$CONJ$PRON+QUESP+A3SG+PNON+NOM":
 *             if lastWord == "?":
 *                 return "PRON+QUESP+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.containsTwoNeOrYa(fsmParses, "ne"):
 *                 return "CONJ"
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_PRON_QUESP_A3SG_PNON_NOM);
      __pyx_r = __pyx_kp_u_PRON_QUESP_A3SG_PNON_NOM;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":253
 *             # NE #
 *         elif parseString == "ADJ$ADV$CONJ$PRON+QUESP+A3SG+PNON+NOM":
 *             if lastWord == "?":             # <<<<<<<<<<<<<<
 *                 return "PRON+QUESP+A3SG+PNON+NOM"
 *             if AutoDisambiguator.containsTwoNeOrYa(fsmParses, "ne"):
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":255
 *             if lastWord == "?":
 *                 return "PRON+QUESP+A3SG+PNON+NOM"
 *             if AutoDisambiguator.containsTwoNeOrYa(fsmParses, "ne"):             # <<<<<<<<<<<<<<
 *                 return "CONJ"
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_containsTwoNeOrYa); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 255, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_fsmParses, __pyx_n_u_ne};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 255, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_fsmParses, __pyx_n_u_ne};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 255, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 255, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_2) {
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2); __pyx_t_2 = NULL;
      }
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_10, __pyx_v_fsmParses);
      __Pyx_INCREF(__pyx_n_u_ne);
      __Pyx_GIVEREF(__pyx_n_u_ne);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_10, __pyx_n_u_ne);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 255, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 255, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":256
 *                 return "PRON+QUESP+A3SG+PNON+NOM"
 *             if AutoDisambiguator.containsTwoNeOrYa(fsmParses, "ne"):
 *                 return "CONJ"             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "ADJ"
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_n_u_CONJ);
      __pyx_r = __pyx_n_u_CONJ;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":255
 *             if lastWord == "?":
 *                 return "PRON+QUESP+A3SG+PNON+NOM"
 *             if AutoDisambiguator.containsTwoNeOrYa(fsmParses, "ne"):             # <<<<<<<<<<<<<<
 *                 return "CONJ"
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":257
 *             if AutoDisambiguator.containsTwoNeOrYa(fsmParses, "ne"):
 *                 return "CONJ"
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ"
 *             return "ADV"
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNoun); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 257, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 257, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 257, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_2 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 257, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 257, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 257, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":258
 *                 return "CONJ"
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "ADJ"             # <<<<<<<<<<<<<<
 *             return "ADV"
 *             # TM #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_n_u_ADJ);
      __pyx_r = __pyx_n_u_ADJ;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":257
 *             if AutoDisambiguator.containsTwoNeOrYa(fsmParses, "ne"):
 *                 return "CONJ"
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ"
 *             return "ADV"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":259
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "ADJ"
 *             return "ADV"             # <<<<<<<<<<<<<<
 *             # TM #
 *         elif parseString == "DET$NOUN+A3SG+PNON+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ADV);
    __pyx_r = __pyx_n_u_ADV;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":252
 *             return "POS+PROG2"
 *             # NE #
 *         elif parseString == "ADJ$ADV$CONJ$PRON+QUESP+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if lastWord == "?":
 *                 return "PRON+QUESP+A3SG+PNON+NOM"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":261
 *             return "ADV"
 *             # TM #
 *         elif parseString == "DET$NOUN+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             return "DET"
 *             # AZ #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_DET_NOUN_A3SG_PNON_NOM, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 261, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":262
 *             # TM #
 *         elif parseString == "DET$NOUN+A3SG+PNON+NOM":
 *             return "DET"             # <<<<<<<<<<<<<<
 *             # AZ #
 *         elif parseString == "ADJ$ADV$POSTP+PCABL$VERB+POS+IMP+A2SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_DET);
    __pyx_r = __pyx_n_u_DET;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":261
 *             return "ADV"
 *             # TM #
 *         elif parseString == "DET$NOUN+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             return "DET"
 *             # AZ #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":264
 *             return "DET"
 *             # AZ #
 *         elif parseString == "ADJ$ADV$POSTP+PCABL$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_ADV_POSTP_PCABL_VERB_POS_IMP, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 264, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":265
 *             # AZ #
 *         elif parseString == "ADJ$ADV$POSTP+PCABL$VERB+POS+IMP+A2SG":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCABL"
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_hasPreviousWordTag); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 265, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_MorphologicalTag); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 265, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_ABLATIVE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 265, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_v_index, __pyx_v_correctParses, __pyx_t_4};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 265, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_v_index, __pyx_v_correctParses, __pyx_t_4};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 265, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else
    #endif
    {
      __pyx_t_1 = PyTuple_New(3+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 265, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__pyx_t_2) {
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2); __pyx_t_2 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_10, __pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_1, 2+__pyx_t_10, __pyx_t_4);
      __pyx_t_4 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 265, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 265, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":266
 *         elif parseString == "ADJ$ADV$POSTP+PCABL$VERB+POS+IMP+A2SG":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ"
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_POSTP_PCABL);
      __pyx_r = __pyx_kp_u_POSTP_PCABL;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":265
 *             # AZ #
 *         elif parseString == "ADJ$ADV$POSTP+PCABL$VERB+POS+IMP+A2SG":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCABL"
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":267
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ"
 *             return "ADV"
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNounOrAdjective); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 267, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 267, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 267, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 267, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_1) {
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1); __pyx_t_1 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 267, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 267, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":268
 *                 return "POSTP+PCABL"
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ"             # <<<<<<<<<<<<<<
 *             return "ADV"
 *             # grlmedik #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_n_u_ADJ);
      __pyx_r = __pyx_n_u_ADJ;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":267
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ"
 *             return "ADV"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":269
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ"
 *             return "ADV"             # <<<<<<<<<<<<<<
 *             # grlmedik #
 *         elif parseString == "NEG+PAST+A1PL$NEG^DB+ADJ+PASTPART+PNON$NEG^DB+NOUN+PASTPART+A3SG+PNON+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ADV);
    __pyx_r = __pyx_n_u_ADV;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":264
 *             return "DET"
 *             # AZ #
 *         elif parseString == "ADJ$ADV$POSTP+PCABL$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":271
 *             return "ADV"
 *             # grlmedik #
 *         elif parseString == "NEG+PAST+A1PL$NEG^DB+ADJ+PASTPART+PNON$NEG^DB+NOUN+PASTPART+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if surfaceForm == "allmadk":
 *                 return "NEG^DB+ADJ+PASTPART+PNON"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NEG_PAST_A1PL_NEG_DB_ADJ_PASTPAR, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 271, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":272
 *             # grlmedik #
 *         elif parseString == "NEG+PAST+A1PL$NEG^DB+ADJ+PASTPART+PNON$NEG^DB+NOUN+PASTPART+A3SG+PNON+NOM":
 *             if surfaceForm == "allmadk":             # <<<<<<<<<<<<<<
 *                 return "NEG^DB+ADJ+PASTPART+PNON"
 *             return "NEG+PAST+A1PL"
 */
    __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_surfaceForm, __pyx_n_u_allmadk, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 272, __pyx_L1_error)
    __pyx_t_11 = (__pyx_t_9 != 0);
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":273
 *         elif parseString == "NEG+PAST+A1PL$NEG^DB+ADJ+PASTPART+PNON$NEG^DB+NOUN+PASTPART+A3SG+PNON+NOM":
 *             if surfaceForm == "allmadk":
 *                 return "NEG^DB+ADJ+PASTPART+PNON"             # <<<<<<<<<<<<<<
 *             return "NEG+PAST+A1PL"
 *         elif parseString == "DATE$NUM+FRACTION":
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_NEG_DB_ADJ_PASTPART_PNON);
      __pyx_r = __pyx_kp_u_NEG_DB_ADJ_PASTPART_PNON;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":272
 *             # grlmedik #
 *         elif parseString == "NEG+PAST+A1PL$NEG^DB+ADJ+PASTPART+PNON$NEG^DB+NOUN+PASTPART+A3SG+PNON+NOM":
 *             if surfaceForm == "allmadk":             # <<<<<<<<<<<<<<
 *                 return "NEG^DB+ADJ+PASTPART+PNON"
 *             return "NEG+PAST+A1PL"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":274
 *             if surfaceForm == "allmadk":
 *                 return "NEG^DB+ADJ+PASTPART+PNON"
 *             return "NEG+PAST+A1PL"             # <<<<<<<<<<<<<<
 *         elif parseString == "DATE$NUM+FRACTION":
 *             return "NUM+FRACTION"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NEG_PAST_A1PL);
    __pyx_r = __pyx_kp_u_NEG_PAST_A1PL;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":271
 *             return "ADV"
 *             # grlmedik #
 *         elif parseString == "NEG+PAST+A1PL$NEG^DB+ADJ+PASTPART+PNON$NEG^DB+NOUN+PASTPART+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if surfaceForm == "allmadk":
 *                 return "NEG^DB+ADJ+PASTPART+PNON"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":275
 *                 return "NEG^DB+ADJ+PASTPART+PNON"
 *             return "NEG+PAST+A1PL"
 *         elif parseString == "DATE$NUM+FRACTION":             # <<<<<<<<<<<<<<
 *             return "NUM+FRACTION"
 *             # giri, sat, p, vuru #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_DATE_NUM_FRACTION, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 275, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":276
 *             return "NEG+PAST+A1PL"
 *         elif parseString == "DATE$NUM+FRACTION":
 *             return "NUM+FRACTION"             # <<<<<<<<<<<<<<
 *             # giri, sat, p, vuru #
 *         elif parseString == "POS^DB+NOUN+INF3+A3SG+PNON+NOM$RECIP+POS+IMP+A2SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NUM_FRACTION);
    __pyx_r = __pyx_kp_u_NUM_FRACTION;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":275
 *                 return "NEG^DB+ADJ+PASTPART+PNON"
 *             return "NEG+PAST+A1PL"
 *         elif parseString == "DATE$NUM+FRACTION":             # <<<<<<<<<<<<<<
 *             return "NUM+FRACTION"
 *             # giri, sat, p, vuru #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":278
 *             return "NUM+FRACTION"
 *             # giri, sat, p, vuru #
 *         elif parseString == "POS^DB+NOUN+INF3+A3SG+PNON+NOM$RECIP+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             return "POS^DB+NOUN+INF3+A3SG+PNON+NOM"
 *             # baka, yukar #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_POS_DB_NOUN_INF3_A3SG_PNON_NOM_R, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 278, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":279
 *             # giri, sat, p, vuru #
 *         elif parseString == "POS^DB+NOUN+INF3+A3SG+PNON+NOM$RECIP+POS+IMP+A2SG":
 *             return "POS^DB+NOUN+INF3+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *             # baka, yukar #
 *         elif parseString == "ADJ$POSTP+PCABL":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_POS_DB_NOUN_INF3_A3SG_PNON_NOM);
    __pyx_r = __pyx_kp_u_POS_DB_NOUN_INF3_A3SG_PNON_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":278
 *             return "NUM+FRACTION"
 *             # giri, sat, p, vuru #
 *         elif parseString == "POS^DB+NOUN+INF3+A3SG+PNON+NOM$RECIP+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             return "POS^DB+NOUN+INF3+A3SG+PNON+NOM"
 *             # baka, yukar #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":281
 *             return "POS^DB+NOUN+INF3+A3SG+PNON+NOM"
 *             # baka, yukar #
 *         elif parseString == "ADJ$POSTP+PCABL":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_POSTP_PCABL, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 281, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":282
 *             # baka, yukar #
 *         elif parseString == "ADJ$POSTP+PCABL":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCABL"
 *             return "ADJ"
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_hasPreviousWordTag); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 282, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_MorphologicalTag); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 282, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_ABLATIVE); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 282, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_index, __pyx_v_correctParses, __pyx_t_1};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 282, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_index, __pyx_v_correctParses, __pyx_t_1};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 282, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else
    #endif
    {
      __pyx_t_2 = PyTuple_New(3+__pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 282, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_10, __pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_2, 2+__pyx_t_10, __pyx_t_1);
      __pyx_t_1 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 282, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 282, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":283
 *         elif parseString == "ADJ$POSTP+PCABL":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"             # <<<<<<<<<<<<<<
 *             return "ADJ"
 *             # KARI #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_POSTP_PCABL);
      __pyx_r = __pyx_kp_u_POSTP_PCABL;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":282
 *             # baka, yukar #
 *         elif parseString == "ADJ$POSTP+PCABL":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCABL"
 *             return "ADJ"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":284
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"
 *             return "ADJ"             # <<<<<<<<<<<<<<
 *             # KARI #
 *         elif parseString == "ADJ$ADV$NOUN+A3SG+PNON+NOM$POSTP+PCDAT":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ADJ);
    __pyx_r = __pyx_n_u_ADJ;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":281
 *             return "POS^DB+NOUN+INF3+A3SG+PNON+NOM"
 *             # baka, yukar #
 *         elif parseString == "ADJ$POSTP+PCABL":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":286
 *             return "ADJ"
 *             # KARI #
 *         elif parseString == "ADJ$ADV$NOUN+A3SG+PNON+NOM$POSTP+PCDAT":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.DATIVE):
 *                 return "POSTP+PCDAT"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_ADV_NOUN_A3SG_PNON_NOM_POSTP, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 286, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":287
 *             # KARI #
 *         elif parseString == "ADJ$ADV$NOUN+A3SG+PNON+NOM$POSTP+PCDAT":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.DATIVE):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCDAT"
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_hasPreviousWordTag); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 287, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_MorphologicalTag); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 287, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_DATIVE); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 287, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_v_index, __pyx_v_correctParses, __pyx_t_1};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 287, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_v_index, __pyx_v_correctParses, __pyx_t_1};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 287, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(3+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 287, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_2) {
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2); __pyx_t_2 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_10, __pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_4, 2+__pyx_t_10, __pyx_t_1);
      __pyx_t_1 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 287, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 287, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":288
 *         elif parseString == "ADJ$ADV$NOUN+A3SG+PNON+NOM$POSTP+PCDAT":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.DATIVE):
 *                 return "POSTP+PCDAT"             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "ADJ"
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_POSTP_PCDAT);
      __pyx_r = __pyx_kp_u_POSTP_PCDAT;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":287
 *             # KARI #
 *         elif parseString == "ADJ$ADV$NOUN+A3SG+PNON+NOM$POSTP+PCDAT":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.DATIVE):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCDAT"
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":289
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.DATIVE):
 *                 return "POSTP+PCDAT"
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ"
 *             return "ADV"
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNoun); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 289, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 289, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 289, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_1 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 289, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 289, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 289, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":290
 *                 return "POSTP+PCDAT"
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "ADJ"             # <<<<<<<<<<<<<<
 *             return "ADV"
 *             # BEN #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_n_u_ADJ);
      __pyx_r = __pyx_n_u_ADJ;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":289
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.DATIVE):
 *                 return "POSTP+PCDAT"
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ"
 *             return "ADV"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":291
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "ADJ"
 *             return "ADV"             # <<<<<<<<<<<<<<
 *             # BEN #
 *         elif parseString == "NOUN+A3SG$NOUN+PROP+A3SG$PRON+PERS+A1SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ADV);
    __pyx_r = __pyx_n_u_ADV;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":286
 *             return "ADJ"
 *             # KARI #
 *         elif parseString == "ADJ$ADV$NOUN+A3SG+PNON+NOM$POSTP+PCDAT":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.DATIVE):
 *                 return "POSTP+PCDAT"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":293
 *             return "ADV"
 *             # BEN #
 *         elif parseString == "NOUN+A3SG$NOUN+PROP+A3SG$PRON+PERS+A1SG":             # <<<<<<<<<<<<<<
 *             return "PRON+PERS+A1SG"
 *             # yapc, verici #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_NOUN_PROP_A3SG_PRON_PE, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 293, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":294
 *             # BEN #
 *         elif parseString == "NOUN+A3SG$NOUN+PROP+A3SG$PRON+PERS+A1SG":
 *             return "PRON+PERS+A1SG"             # <<<<<<<<<<<<<<
 *             # yapc, verici #
 *         elif parseString == "ADJ+AGT$NOUN+AGT+A3SG+PNON+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_PRON_PERS_A1SG);
    __pyx_r = __pyx_kp_u_PRON_PERS_A1SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":293
 *             return "ADV"
 *             # BEN #
 *         elif parseString == "NOUN+A3SG$NOUN+PROP+A3SG$PRON+PERS+A1SG":             # <<<<<<<<<<<<<<
 *             return "PRON+PERS+A1SG"
 *             # yapc, verici #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":296
 *             return "PRON+PERS+A1SG"
 *             # yapc, verici #
 *         elif parseString == "ADJ+AGT$NOUN+AGT+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+AGT"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_AGT_NOUN_AGT_A3SG_PNON_NOM, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 296, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":297
 *             # yapc, verici #
 *         elif parseString == "ADJ+AGT$NOUN+AGT+A3SG+PNON+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ+AGT"
 *             return "NOUN+AGT+A3SG+PNON+NOM"
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNounOrAdjective); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 297, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 297, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 297, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 297, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_1) {
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1); __pyx_t_1 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 297, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 297, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":298
 *         elif parseString == "ADJ+AGT$NOUN+AGT+A3SG+PNON+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+AGT"             # <<<<<<<<<<<<<<
 *             return "NOUN+AGT+A3SG+PNON+NOM"
 *             # BLE #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_ADJ_AGT);
      __pyx_r = __pyx_kp_u_ADJ_AGT;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":297
 *             # yapc, verici #
 *         elif parseString == "ADJ+AGT$NOUN+AGT+A3SG+PNON+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ+AGT"
 *             return "NOUN+AGT+A3SG+PNON+NOM"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":299
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+AGT"
 *             return "NOUN+AGT+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *             # BLE #
 *         elif parseString == "ADV$VERB+POS+IMP+A2SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_AGT_A3SG_PNON_NOM);
    __pyx_r = __pyx_kp_u_NOUN_AGT_A3SG_PNON_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":296
 *             return "PRON+PERS+A1SG"
 *             # yapc, verici #
 *         elif parseString == "ADJ+AGT$NOUN+AGT+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+AGT"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":301
 *             return "NOUN+AGT+A3SG+PNON+NOM"
 *             # BLE #
 *         elif parseString == "ADV$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             return "ADV"
 *             # ortalamalar, uzayllar, demokratlar #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADV_VERB_POS_IMP_A2SG, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 301, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":302
 *             # BLE #
 *         elif parseString == "ADV$VERB+POS+IMP+A2SG":
 *             return "ADV"             # <<<<<<<<<<<<<<
 *             # ortalamalar, uzayllar, demokratlar #
 *         elif parseString == "NOUN+ZERO+A3PL+PNON+NOM$VERB+ZERO+PRES+A3PL":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ADV);
    __pyx_r = __pyx_n_u_ADV;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":301
 *             return "NOUN+AGT+A3SG+PNON+NOM"
 *             # BLE #
 *         elif parseString == "ADV$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             return "ADV"
 *             # ortalamalar, uzayllar, demokratlar #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":304
 *             return "ADV"
 *             # ortalamalar, uzayllar, demokratlar #
 *         elif parseString == "NOUN+ZERO+A3PL+PNON+NOM$VERB+ZERO+PRES+A3PL":             # <<<<<<<<<<<<<<
 *             return "NOUN+ZERO+A3PL+PNON+NOM"
 *             # yasa, diye, yla #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_ZERO_A3PL_PNON_NOM_VERB_ZER, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 304, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":305
 *             # ortalamalar, uzayllar, demokratlar #
 *         elif parseString == "NOUN+ZERO+A3PL+PNON+NOM$VERB+ZERO+PRES+A3PL":
 *             return "NOUN+ZERO+A3PL+PNON+NOM"             # <<<<<<<<<<<<<<
 *             # yasa, diye, yla #
 *         elif parseString == "NOUN+A3SG+PNON+DAT$VERB+POS+OPT+A3SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_ZERO_A3PL_PNON_NOM);
    __pyx_r = __pyx_kp_u_NOUN_ZERO_A3PL_PNON_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":304
 *             return "ADV"
 *             # ortalamalar, uzayllar, demokratlar #
 *         elif parseString == "NOUN+ZERO+A3PL+PNON+NOM$VERB+ZERO+PRES+A3PL":             # <<<<<<<<<<<<<<
 *             return "NOUN+ZERO+A3PL+PNON+NOM"
 *             # yasa, diye, yla #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":307
 *             return "NOUN+ZERO+A3PL+PNON+NOM"
 *             # yasa, diye, yla #
 *         elif parseString == "NOUN+A3SG+PNON+DAT$VERB+POS+OPT+A3SG":             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+PNON+DAT"
 *             # BZ, BZE #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_PNON_DAT_VERB_POS_OPT, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 307, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":308
 *             # yasa, diye, yla #
 *         elif parseString == "NOUN+A3SG+PNON+DAT$VERB+POS+OPT+A3SG":
 *             return "NOUN+A3SG+PNON+DAT"             # <<<<<<<<<<<<<<
 *             # BZ, BZE #
 *         elif parseString == "NOUN+A3SG$PRON+PERS+A1PL":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_A3SG_PNON_DAT);
    __pyx_r = __pyx_kp_u_NOUN_A3SG_PNON_DAT;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":307
 *             return "NOUN+ZERO+A3PL+PNON+NOM"
 *             # yasa, diye, yla #
 *         elif parseString == "NOUN+A3SG+PNON+DAT$VERB+POS+OPT+A3SG":             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+PNON+DAT"
 *             # BZ, BZE #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":310
 *             return "NOUN+A3SG+PNON+DAT"
 *             # BZ, BZE #
 *         elif parseString == "NOUN+A3SG$PRON+PERS+A1PL":             # <<<<<<<<<<<<<<
 *             return "PRON+PERS+A1PL"
 *             # AZDI #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_PRON_PERS_A1PL, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 310, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":311
 *             # BZ, BZE #
 *         elif parseString == "NOUN+A3SG$PRON+PERS+A1PL":
 *             return "PRON+PERS+A1PL"             # <<<<<<<<<<<<<<
 *             # AZDI #
 *         elif parseString == "ADJ^DB+VERB+ZERO$POSTP+PCABL^DB+VERB+ZERO$VERB+POS":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_PRON_PERS_A1PL);
    __pyx_r = __pyx_kp_u_PRON_PERS_A1PL;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":310
 *             return "NOUN+A3SG+PNON+DAT"
 *             # BZ, BZE #
 *         elif parseString == "NOUN+A3SG$PRON+PERS+A1PL":             # <<<<<<<<<<<<<<
 *             return "PRON+PERS+A1PL"
 *             # AZDI #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":313
 *             return "PRON+PERS+A1PL"
 *             # AZDI #
 *         elif parseString == "ADJ^DB+VERB+ZERO$POSTP+PCABL^DB+VERB+ZERO$VERB+POS":             # <<<<<<<<<<<<<<
 *             return "ADJ^DB+VERB+ZERO"
 *             # BRNC, KNC, NC, DRDNC, BENC #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_DB_VERB_ZERO_POSTP_PCABL_DB, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 313, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":314
 *             # AZDI #
 *         elif parseString == "ADJ^DB+VERB+ZERO$POSTP+PCABL^DB+VERB+ZERO$VERB+POS":
 *             return "ADJ^DB+VERB+ZERO"             # <<<<<<<<<<<<<<
 *             # BRNC, KNC, NC, DRDNC, BENC #
 *         elif parseString == "ADJ$NUM+ORD":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_ADJ_DB_VERB_ZERO);
    __pyx_r = __pyx_kp_u_ADJ_DB_VERB_ZERO;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":313
 *             return "PRON+PERS+A1PL"
 *             # AZDI #
 *         elif parseString == "ADJ^DB+VERB+ZERO$POSTP+PCABL^DB+VERB+ZERO$VERB+POS":             # <<<<<<<<<<<<<<
 *             return "ADJ^DB+VERB+ZERO"
 *             # BRNC, KNC, NC, DRDNC, BENC #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":316
 *             return "ADJ^DB+VERB+ZERO"
 *             # BRNC, KNC, NC, DRDNC, BENC #
 *         elif parseString == "ADJ$NUM+ORD":             # <<<<<<<<<<<<<<
 *             return "ADJ"
 *             # AY #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_NUM_ORD, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 316, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":317
 *             # BRNC, KNC, NC, DRDNC, BENC #
 *         elif parseString == "ADJ$NUM+ORD":
 *             return "ADJ"             # <<<<<<<<<<<<<<
 *             # AY #
 *         elif parseString == "INTERJ$NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ADJ);
    __pyx_r = __pyx_n_u_ADJ;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":316
 *             return "ADJ^DB+VERB+ZERO"
 *             # BRNC, KNC, NC, DRDNC, BENC #
 *         elif parseString == "ADJ$NUM+ORD":             # <<<<<<<<<<<<<<
 *             return "ADJ"
 *             # AY #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":319
 *             return "ADJ"
 *             # AY #
 *         elif parseString == "INTERJ$NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+PNON+NOM"
 *             # konumam, savunmam, etmem #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_INTERJ_NOUN_A3SG_PNON_NOM_VERB_P, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 319, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":320
 *             # AY #
 *         elif parseString == "INTERJ$NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             return "NOUN+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *             # konumam, savunmam, etmem #
 *         elif parseString == "NEG+AOR+A1SG$POS^DB+NOUN+INF2+A3SG+P1SG+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_A3SG_PNON_NOM);
    __pyx_r = __pyx_kp_u_NOUN_A3SG_PNON_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":319
 *             return "ADJ"
 *             # AY #
 *         elif parseString == "INTERJ$NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+PNON+NOM"
 *             # konumam, savunmam, etmem #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":322
 *             return "NOUN+A3SG+PNON+NOM"
 *             # konumam, savunmam, etmem #
 *         elif parseString == "NEG+AOR+A1SG$POS^DB+NOUN+INF2+A3SG+P1SG+NOM":             # <<<<<<<<<<<<<<
 *             return "NEG+AOR+A1SG"
 *             # YA #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NEG_AOR_A1SG_POS_DB_NOUN_INF2_A3, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 322, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":323
 *             # konumam, savunmam, etmem #
 *         elif parseString == "NEG+AOR+A1SG$POS^DB+NOUN+INF2+A3SG+P1SG+NOM":
 *             return "NEG+AOR+A1SG"             # <<<<<<<<<<<<<<
 *             # YA #
 *         elif parseString == "CONJ$INTERJ":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NEG_AOR_A1SG);
    __pyx_r = __pyx_kp_u_NEG_AOR_A1SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":322
 *             return "NOUN+A3SG+PNON+NOM"
 *             # konumam, savunmam, etmem #
 *         elif parseString == "NEG+AOR+A1SG$POS^DB+NOUN+INF2+A3SG+P1SG+NOM":             # <<<<<<<<<<<<<<
 *             return "NEG+AOR+A1SG"
 *             # YA #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":325
 *             return "NEG+AOR+A1SG"
 *             # YA #
 *         elif parseString == "CONJ$INTERJ":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.containsTwoNeOrYa(fsmParses, "ya"):
 *                 return "CONJ"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_CONJ_INTERJ, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 325, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":326
 *             # YA #
 *         elif parseString == "CONJ$INTERJ":
 *             if AutoDisambiguator.containsTwoNeOrYa(fsmParses, "ya"):             # <<<<<<<<<<<<<<
 *                 return "CONJ"
 *             if AutoDisambiguator.nextWordExists(index, fsmParses) and \
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_containsTwoNeOrYa); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 326, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_fsmParses, __pyx_n_u_ya};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 326, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_fsmParses, __pyx_n_u_ya};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 326, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_1 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 326, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_10, __pyx_v_fsmParses);
      __Pyx_INCREF(__pyx_n_u_ya);
      __Pyx_GIVEREF(__pyx_n_u_ya);
      PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_10, __pyx_n_u_ya);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 326, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 326, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":327
 *         elif parseString == "CONJ$INTERJ":
 *             if AutoDisambiguator.containsTwoNeOrYa(fsmParses, "ya"):
 *                 return "CONJ"             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.nextWordExists(index, fsmParses) and \
 *                     fsmParses[index + 1].getFsmParse(0).getSurfaceForm() == "da":
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_n_u_CONJ);
      __pyx_r = __pyx_n_u_CONJ;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":326
 *             # YA #
 *         elif parseString == "CONJ$INTERJ":
 *             if AutoDisambiguator.containsTwoNeOrYa(fsmParses, "ya"):             # <<<<<<<<<<<<<<
 *                 return "CONJ"
 *             if AutoDisambiguator.nextWordExists(index, fsmParses) and \
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":328
 *             if AutoDisambiguator.containsTwoNeOrYa(fsmParses, "ya"):
 *                 return "CONJ"
 *             if AutoDisambiguator.nextWordExists(index, fsmParses) and \             # <<<<<<<<<<<<<<
 *                     fsmParses[index + 1].getFsmParse(0).getSurfaceForm() == "da":
 *                 return "CONJ"
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_nextWordExists); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 328, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 328, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 328, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 328, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_1) {
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1); __pyx_t_1 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 328, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 328, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {
    } else {
      __pyx_t_9 = __pyx_t_11;
      goto __pyx_L62_bool_binop_done;
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":329
 *                 return "CONJ"
 *             if AutoDisambiguator.nextWordExists(index, fsmParses) and \
 *                     fsmParses[index + 1].getFsmParse(0).getSurfaceForm() == "da":             # <<<<<<<<<<<<<<
 *                 return "CONJ"
 *             return "INTERJ"
 */
    if (unlikely(__pyx_v_fsmParses == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 329, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_PyInt_AddObjC(__pyx_v_index, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 329, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_fsmParses, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 329, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_getFsmParse); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 329, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_5 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_1, __pyx_int_0) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_int_0);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 329, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_getSurfaceForm); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 329, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 329, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_n_u_da, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 329, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_9 = __pyx_t_11;
    __pyx_L62_bool_binop_done:;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":328
 *             if AutoDisambiguator.containsTwoNeOrYa(fsmParses, "ya"):
 *                 return "CONJ"
 *             if AutoDisambiguator.nextWordExists(index, fsmParses) and \             # <<<<<<<<<<<<<<
 *                     fsmParses[index + 1].getFsmParse(0).getSurfaceForm() == "da":
 *                 return "CONJ"
 */
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":330
 *             if AutoDisambiguator.nextWordExists(index, fsmParses) and \
 *                     fsmParses[index + 1].getFsmParse(0).getSurfaceForm() == "da":
 *                 return "CONJ"             # <<<<<<<<<<<<<<
 *             return "INTERJ"
 *         elif parseString == "A3PL+P3PL$A3PL+P3SG$A3SG+P3PL":
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_n_u_CONJ);
      __pyx_r = __pyx_n_u_CONJ;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":328
 *             if AutoDisambiguator.containsTwoNeOrYa(fsmParses, "ya"):
 *                 return "CONJ"
 *             if AutoDisambiguator.nextWordExists(index, fsmParses) and \             # <<<<<<<<<<<<<<
 *                     fsmParses[index + 1].getFsmParse(0).getSurfaceForm() == "da":
 *                 return "CONJ"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":331
 *                     fsmParses[index + 1].getFsmParse(0).getSurfaceForm() == "da":
 *                 return "CONJ"
 *             return "INTERJ"             # <<<<<<<<<<<<<<
 *         elif parseString == "A3PL+P3PL$A3PL+P3SG$A3SG+P3PL":
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_INTERJ);
    __pyx_r = __pyx_n_u_INTERJ;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":325
 *             return "NEG+AOR+A1SG"
 *             # YA #
 *         elif parseString == "CONJ$INTERJ":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.containsTwoNeOrYa(fsmParses, "ya"):
 *                 return "CONJ"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":332
 *                 return "CONJ"
 *             return "INTERJ"
 *         elif parseString == "A3PL+P3PL$A3PL+P3SG$A3SG+P3PL":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "A3SG+P3PL"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_A3PL_P3PL_A3PL_P3SG_A3SG_P3PL, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 332, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":333
 *             return "INTERJ"
 *         elif parseString == "A3PL+P3PL$A3PL+P3SG$A3SG+P3PL":
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "A3SG+P3PL"
 *             return "A3PL+P3SG"
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isPossessivePlural); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 333, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 333, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 333, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_1 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 333, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_10, __pyx_v_correctParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 333, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 333, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":334
 *         elif parseString == "A3PL+P3PL$A3PL+P3SG$A3SG+P3PL":
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "A3SG+P3PL"             # <<<<<<<<<<<<<<
 *             return "A3PL+P3SG"
 *             # YZDE, YZL #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_A3SG_P3PL);
      __pyx_r = __pyx_kp_u_A3SG_P3PL;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":333
 *             return "INTERJ"
 *         elif parseString == "A3PL+P3PL$A3PL+P3SG$A3SG+P3PL":
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "A3SG+P3PL"
 *             return "A3PL+P3SG"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":335
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "A3SG+P3PL"
 *             return "A3PL+P3SG"             # <<<<<<<<<<<<<<
 *             # YZDE, YZL #
 *         elif parseString == "NOUN$NUM+CARD^DB+NOUN+ZERO":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_A3PL_P3SG);
    __pyx_r = __pyx_kp_u_A3PL_P3SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":332
 *                 return "CONJ"
 *             return "INTERJ"
 *         elif parseString == "A3PL+P3PL$A3PL+P3SG$A3SG+P3PL":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "A3SG+P3PL"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":337
 *             return "A3PL+P3SG"
 *             # YZDE, YZL #
 *         elif parseString == "NOUN$NUM+CARD^DB+NOUN+ZERO":             # <<<<<<<<<<<<<<
 *             return "NOUN"
 *             # almanlar, uzmanlar, elmaslar, katiller #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_NUM_CARD_DB_NOUN_ZERO, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 337, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":338
 *             # YZDE, YZL #
 *         elif parseString == "NOUN$NUM+CARD^DB+NOUN+ZERO":
 *             return "NOUN"             # <<<<<<<<<<<<<<
 *             # almanlar, uzmanlar, elmaslar, katiller #
 *         elif parseString == "ADJ^DB+VERB+ZERO+PRES+A3PL$NOUN+A3PL+PNON+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A3PL":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_NOUN);
    __pyx_r = __pyx_n_u_NOUN;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":337
 *             return "A3PL+P3SG"
 *             # YZDE, YZL #
 *         elif parseString == "NOUN$NUM+CARD^DB+NOUN+ZERO":             # <<<<<<<<<<<<<<
 *             return "NOUN"
 *             # almanlar, uzmanlar, elmaslar, katiller #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":340
 *             return "NOUN"
 *             # almanlar, uzmanlar, elmaslar, katiller #
 *         elif parseString == "ADJ^DB+VERB+ZERO+PRES+A3PL$NOUN+A3PL+PNON+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A3PL":             # <<<<<<<<<<<<<<
 *             return "NOUN+A3PL+PNON+NOM"
 *             # fazlas, yetkilisi #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_DB_VERB_ZERO_PRES_A3PL_NOUN, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 340, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":341
 *             # almanlar, uzmanlar, elmaslar, katiller #
 *         elif parseString == "ADJ^DB+VERB+ZERO+PRES+A3PL$NOUN+A3PL+PNON+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A3PL":
 *             return "NOUN+A3PL+PNON+NOM"             # <<<<<<<<<<<<<<
 *             # fazlas, yetkilisi #
 *         elif parseString == "ADJ+JUSTLIKE$NOUN+ZERO+A3SG+P3SG+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_A3PL_PNON_NOM);
    __pyx_r = __pyx_kp_u_NOUN_A3PL_PNON_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":340
 *             return "NOUN"
 *             # almanlar, uzmanlar, elmaslar, katiller #
 *         elif parseString == "ADJ^DB+VERB+ZERO+PRES+A3PL$NOUN+A3PL+PNON+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A3PL":             # <<<<<<<<<<<<<<
 *             return "NOUN+A3PL+PNON+NOM"
 *             # fazlas, yetkilisi #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":343
 *             return "NOUN+A3PL+PNON+NOM"
 *             # fazlas, yetkilisi #
 *         elif parseString == "ADJ+JUSTLIKE$NOUN+ZERO+A3SG+P3SG+NOM":             # <<<<<<<<<<<<<<
 *             return "NOUN+ZERO+A3SG+P3SG+NOM"
 *             # HERKES, HERKESTEN, HERKESLE, HERKES #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_JUSTLIKE_NOUN_ZERO_A3SG_P3SG, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 343, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":344
 *             # fazlas, yetkilisi #
 *         elif parseString == "ADJ+JUSTLIKE$NOUN+ZERO+A3SG+P3SG+NOM":
 *             return "NOUN+ZERO+A3SG+P3SG+NOM"             # <<<<<<<<<<<<<<
 *             # HERKES, HERKESTEN, HERKESLE, HERKES #
 *         elif parseString == "NOUN+A3SG+PNON$PRON+QUANTP+A3PL+P3PL":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_ZERO_A3SG_P3SG_NOM);
    __pyx_r = __pyx_kp_u_NOUN_ZERO_A3SG_P3SG_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":343
 *             return "NOUN+A3PL+PNON+NOM"
 *             # fazlas, yetkilisi #
 *         elif parseString == "ADJ+JUSTLIKE$NOUN+ZERO+A3SG+P3SG+NOM":             # <<<<<<<<<<<<<<
 *             return "NOUN+ZERO+A3SG+P3SG+NOM"
 *             # HERKES, HERKESTEN, HERKESLE, HERKES #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":346
 *             return "NOUN+ZERO+A3SG+P3SG+NOM"
 *             # HERKES, HERKESTEN, HERKESLE, HERKES #
 *         elif parseString == "NOUN+A3SG+PNON$PRON+QUANTP+A3PL+P3PL":             # <<<<<<<<<<<<<<
 *             return "PRON+QUANTP+A3PL+P3PL"
 *             # BEN, BENDEN, BENCE, BANA, BENDE #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_PNON_PRON_QUANTP_A3PL, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 346, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":347
 *             # HERKES, HERKESTEN, HERKESLE, HERKES #
 *         elif parseString == "NOUN+A3SG+PNON$PRON+QUANTP+A3PL+P3PL":
 *             return "PRON+QUANTP+A3PL+P3PL"             # <<<<<<<<<<<<<<
 *             # BEN, BENDEN, BENCE, BANA, BENDE #
 *         elif parseString == "NOUN+A3SG$PRON+PERS+A1SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_PRON_QUANTP_A3PL_P3PL);
    __pyx_r = __pyx_kp_u_PRON_QUANTP_A3PL_P3PL;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":346
 *             return "NOUN+ZERO+A3SG+P3SG+NOM"
 *             # HERKES, HERKESTEN, HERKESLE, HERKES #
 *         elif parseString == "NOUN+A3SG+PNON$PRON+QUANTP+A3PL+P3PL":             # <<<<<<<<<<<<<<
 *             return "PRON+QUANTP+A3PL+P3PL"
 *             # BEN, BENDEN, BENCE, BANA, BENDE #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":349
 *             return "PRON+QUANTP+A3PL+P3PL"
 *             # BEN, BENDEN, BENCE, BANA, BENDE #
 *         elif parseString == "NOUN+A3SG$PRON+PERS+A1SG":             # <<<<<<<<<<<<<<
 *             return "PRON+PERS+A1SG"
 *             # karsndan, geriye, geride #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_PRON_PERS_A1SG, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 349, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":350
 *             # BEN, BENDEN, BENCE, BANA, BENDE #
 *         elif parseString == "NOUN+A3SG$PRON+PERS+A1SG":
 *             return "PRON+PERS+A1SG"             # <<<<<<<<<<<<<<
 *             # karsndan, geriye, geride #
 *         elif parseString == "ADJ^DB+NOUN+ZERO$NOUN":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_PRON_PERS_A1SG);
    __pyx_r = __pyx_kp_u_PRON_PERS_A1SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":349
 *             return "PRON+QUANTP+A3PL+P3PL"
 *             # BEN, BENDEN, BENCE, BANA, BENDE #
 *         elif parseString == "NOUN+A3SG$PRON+PERS+A1SG":             # <<<<<<<<<<<<<<
 *             return "PRON+PERS+A1SG"
 *             # karsndan, geriye, geride #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":352
 *             return "PRON+PERS+A1SG"
 *             # karsndan, geriye, geride #
 *         elif parseString == "ADJ^DB+NOUN+ZERO$NOUN":             # <<<<<<<<<<<<<<
 *             return "ADJ^DB+NOUN+ZERO"
 *             # gidecei, kalaca #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_DB_NOUN_ZERO_NOUN, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 352, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":353
 *             # karsndan, geriye, geride #
 *         elif parseString == "ADJ^DB+NOUN+ZERO$NOUN":
 *             return "ADJ^DB+NOUN+ZERO"             # <<<<<<<<<<<<<<
 *             # gidecei, kalaca #
 *         elif parseString == "ADJ+FUTPART+P3SG$NOUN+FUTPART+A3SG+P3SG+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_ADJ_DB_NOUN_ZERO);
    __pyx_r = __pyx_kp_u_ADJ_DB_NOUN_ZERO;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":352
 *             return "PRON+PERS+A1SG"
 *             # karsndan, geriye, geride #
 *         elif parseString == "ADJ^DB+NOUN+ZERO$NOUN":             # <<<<<<<<<<<<<<
 *             return "ADJ^DB+NOUN+ZERO"
 *             # gidecei, kalaca #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":355
 *             return "ADJ^DB+NOUN+ZERO"
 *             # gidecei, kalaca #
 *         elif parseString == "ADJ+FUTPART+P3SG$NOUN+FUTPART+A3SG+P3SG+NOM":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+FUTPART+P3SG"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_FUTPART_P3SG_NOUN_FUTPART_A3, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 355, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":356
 *             # gidecei, kalaca #
 *         elif parseString == "ADJ+FUTPART+P3SG$NOUN+FUTPART+A3SG+P3SG+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ+FUTPART+P3SG"
 *             return "NOUN+FUTPART+A3SG+P3SG+NOM"
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNounOrAdjective); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 356, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 356, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 356, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 356, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__pyx_t_1) {
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1); __pyx_t_1 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 356, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 356, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":357
 *         elif parseString == "ADJ+FUTPART+P3SG$NOUN+FUTPART+A3SG+P3SG+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+FUTPART+P3SG"             # <<<<<<<<<<<<<<
 *             return "NOUN+FUTPART+A3SG+P3SG+NOM"
 *             # bildiimiz, getiimiz, yaadmz #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_ADJ_FUTPART_P3SG);
      __pyx_r = __pyx_kp_u_ADJ_FUTPART_P3SG;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":356
 *             # gidecei, kalaca #
 *         elif parseString == "ADJ+FUTPART+P3SG$NOUN+FUTPART+A3SG+P3SG+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ+FUTPART+P3SG"
 *             return "NOUN+FUTPART+A3SG+P3SG+NOM"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":358
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+FUTPART+P3SG"
 *             return "NOUN+FUTPART+A3SG+P3SG+NOM"             # <<<<<<<<<<<<<<
 *             # bildiimiz, getiimiz, yaadmz #
 *         elif parseString == "ADJ+PASTPART+P1PL$NOUN+PASTPART+A3SG+P1PL+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_FUTPART_A3SG_P3SG_NOM);
    __pyx_r = __pyx_kp_u_NOUN_FUTPART_A3SG_P3SG_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":355
 *             return "ADJ^DB+NOUN+ZERO"
 *             # gidecei, kalaca #
 *         elif parseString == "ADJ+FUTPART+P3SG$NOUN+FUTPART+A3SG+P3SG+NOM":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+FUTPART+P3SG"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":360
 *             return "NOUN+FUTPART+A3SG+P3SG+NOM"
 *             # bildiimiz, getiimiz, yaadmz #
 *         elif parseString == "ADJ+PASTPART+P1PL$NOUN+PASTPART+A3SG+P1PL+NOM":             # <<<<<<<<<<<<<<
 *             return "ADJ+PASTPART+P1PL"
 *             # eminim, memnunum, am #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_PASTPART_P1PL_NOUN_PASTPART, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 360, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":361
 *             # bildiimiz, getiimiz, yaadmz #
 *         elif parseString == "ADJ+PASTPART+P1PL$NOUN+PASTPART+A3SG+P1PL+NOM":
 *             return "ADJ+PASTPART+P1PL"             # <<<<<<<<<<<<<<
 *             # eminim, memnunum, am #
 *         elif parseString == "NOUN+ZERO+A3SG+P1SG+NOM$VERB+ZERO+PRES+A1SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_ADJ_PASTPART_P1PL);
    __pyx_r = __pyx_kp_u_ADJ_PASTPART_P1PL;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":360
 *             return "NOUN+FUTPART+A3SG+P3SG+NOM"
 *             # bildiimiz, getiimiz, yaadmz #
 *         elif parseString == "ADJ+PASTPART+P1PL$NOUN+PASTPART+A3SG+P1PL+NOM":             # <<<<<<<<<<<<<<
 *             return "ADJ+PASTPART+P1PL"
 *             # eminim, memnunum, am #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":363
 *             return "ADJ+PASTPART+P1PL"
 *             # eminim, memnunum, am #
 *         elif parseString == "NOUN+ZERO+A3SG+P1SG+NOM$VERB+ZERO+PRES+A1SG":             # <<<<<<<<<<<<<<
 *             return "VERB+ZERO+PRES+A1SG"
 *             # yaparlar, olabilirler, deitirirler #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_ZERO_A3SG_P1SG_NOM_VERB_ZER, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 363, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":364
 *             # eminim, memnunum, am #
 *         elif parseString == "NOUN+ZERO+A3SG+P1SG+NOM$VERB+ZERO+PRES+A1SG":
 *             return "VERB+ZERO+PRES+A1SG"             # <<<<<<<<<<<<<<
 *             # yaparlar, olabilirler, deitirirler #
 *         elif parseString == "AOR+A3PL$AOR^DB+ADJ+ZERO^DB+NOUN+ZERO+A3PL+PNON+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_VERB_ZERO_PRES_A1SG);
    __pyx_r = __pyx_kp_u_VERB_ZERO_PRES_A1SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":363
 *             return "ADJ+PASTPART+P1PL"
 *             # eminim, memnunum, am #
 *         elif parseString == "NOUN+ZERO+A3SG+P1SG+NOM$VERB+ZERO+PRES+A1SG":             # <<<<<<<<<<<<<<
 *             return "VERB+ZERO+PRES+A1SG"
 *             # yaparlar, olabilirler, deitirirler #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":366
 *             return "VERB+ZERO+PRES+A1SG"
 *             # yaparlar, olabilirler, deitirirler #
 *         elif parseString == "AOR+A3PL$AOR^DB+ADJ+ZERO^DB+NOUN+ZERO+A3PL+PNON+NOM":             # <<<<<<<<<<<<<<
 *             return "AOR+A3PL"
 *             # san, yasa #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_AOR_A3PL_AOR_DB_ADJ_ZERO_DB_NOUN, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 366, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":367
 *             # yaparlar, olabilirler, deitirirler #
 *         elif parseString == "AOR+A3PL$AOR^DB+ADJ+ZERO^DB+NOUN+ZERO+A3PL+PNON+NOM":
 *             return "AOR+A3PL"             # <<<<<<<<<<<<<<
 *             # san, yasa #
 *         elif parseString == "NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_AOR_A3PL);
    __pyx_r = __pyx_kp_u_AOR_A3PL;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":366
 *             return "VERB+ZERO+PRES+A1SG"
 *             # yaparlar, olabilirler, deitirirler #
 *         elif parseString == "AOR+A3PL$AOR^DB+ADJ+ZERO^DB+NOUN+ZERO+A3PL+PNON+NOM":             # <<<<<<<<<<<<<<
 *             return "AOR+A3PL"
 *             # san, yasa #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":369
 *             return "AOR+A3PL"
 *             # san, yasa #
 *         elif parseString == "NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             if index > 0:
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_PNON_NOM_NOUN_PROP_A3S, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 369, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":370
 *             # san, yasa #
 *         elif parseString == "NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             if index > 0:             # <<<<<<<<<<<<<<
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 *             # etmeyecek, yapmayacak, komayacak #
 */
    __pyx_t_3 = PyObject_RichCompare(__pyx_v_index, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 370, __pyx_L1_error)
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 370, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":371
 *         elif parseString == "NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             if index > 0:
 *                 return "NOUN+PROP+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *             # etmeyecek, yapmayacak, komayacak #
 *         elif parseString == "NEG+FUT+A3SG$NEG^DB+ADJ+FUTPART+PNON":
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_NOUN_PROP_A3SG_PNON_NOM);
      __pyx_r = __pyx_kp_u_NOUN_PROP_A3SG_PNON_NOM;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":370
 *             # san, yasa #
 *         elif parseString == "NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             if index > 0:             # <<<<<<<<<<<<<<
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 *             # etmeyecek, yapmayacak, komayacak #
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":369
 *             return "AOR+A3PL"
 *             # san, yasa #
 *         elif parseString == "NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             if index > 0:
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 */
    goto __pyx_L3;
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":373
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 *             # etmeyecek, yapmayacak, komayacak #
 *         elif parseString == "NEG+FUT+A3SG$NEG^DB+ADJ+FUTPART+PNON":             # <<<<<<<<<<<<<<
 *             return "NEG+FUT+A3SG"
 *             # etmeli, olmal #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NEG_FUT_A3SG_NEG_DB_ADJ_FUTPART, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 373, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":374
 *             # etmeyecek, yapmayacak, komayacak #
 *         elif parseString == "NEG+FUT+A3SG$NEG^DB+ADJ+FUTPART+PNON":
 *             return "NEG+FUT+A3SG"             # <<<<<<<<<<<<<<
 *             # etmeli, olmal #
 *         elif parseString == "POS+NECES+A3SG$POS^DB+NOUN+INF2+A3SG+PNON+NOM^DB+ADJ+WITH":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NEG_FUT_A3SG);
    __pyx_r = __pyx_kp_u_NEG_FUT_A3SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":373
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 *             # etmeyecek, yapmayacak, komayacak #
 *         elif parseString == "NEG+FUT+A3SG$NEG^DB+ADJ+FUTPART+PNON":             # <<<<<<<<<<<<<<
 *             return "NEG+FUT+A3SG"
 *             # etmeli, olmal #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":376
 *             return "NEG+FUT+A3SG"
 *             # etmeli, olmal #
 *         elif parseString == "POS+NECES+A3SG$POS^DB+NOUN+INF2+A3SG+PNON+NOM^DB+ADJ+WITH":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "POS+NECES+A3SG"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_POS_NECES_A3SG_POS_DB_NOUN_INF2, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 376, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":377
 *             # etmeli, olmal #
 *         elif parseString == "POS+NECES+A3SG$POS^DB+NOUN+INF2+A3SG+PNON+NOM^DB+ADJ+WITH":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "POS+NECES+A3SG"
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isBeforeLastWord); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 377, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 377, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 377, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_1 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 377, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 377, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 377, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":378
 *         elif parseString == "POS+NECES+A3SG$POS^DB+NOUN+INF2+A3SG+PNON+NOM^DB+ADJ+WITH":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "POS+NECES+A3SG"             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "POS^DB+NOUN+INF2+A3SG+PNON+NOM^DB+ADJ+WITH"
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_POS_NECES_A3SG);
      __pyx_r = __pyx_kp_u_POS_NECES_A3SG;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":377
 *             # etmeli, olmal #
 *         elif parseString == "POS+NECES+A3SG$POS^DB+NOUN+INF2+A3SG+PNON+NOM^DB+ADJ+WITH":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "POS+NECES+A3SG"
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":379
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "POS+NECES+A3SG"
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "POS^DB+NOUN+INF2+A3SG+PNON+NOM^DB+ADJ+WITH"
 *             return "POS+NECES+A3SG"
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNounOrAdjective); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 379, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 379, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 379, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 379, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__pyx_t_1) {
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1); __pyx_t_1 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 379, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 379, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":380
 *                 return "POS+NECES+A3SG"
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "POS^DB+NOUN+INF2+A3SG+PNON+NOM^DB+ADJ+WITH"             # <<<<<<<<<<<<<<
 *             return "POS+NECES+A3SG"
 *             # DE #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_POS_DB_NOUN_INF2_A3SG_PNON_NOM_D);
      __pyx_r = __pyx_kp_u_POS_DB_NOUN_INF2_A3SG_PNON_NOM_D;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":379
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "POS+NECES+A3SG"
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "POS^DB+NOUN+INF2+A3SG+PNON+NOM^DB+ADJ+WITH"
 *             return "POS+NECES+A3SG"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":381
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "POS^DB+NOUN+INF2+A3SG+PNON+NOM^DB+ADJ+WITH"
 *             return "POS+NECES+A3SG"             # <<<<<<<<<<<<<<
 *             # DE #
 *         elif parseString == "CONJ$NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_POS_NECES_A3SG);
    __pyx_r = __pyx_kp_u_POS_NECES_A3SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":376
 *             return "NEG+FUT+A3SG"
 *             # etmeli, olmal #
 *         elif parseString == "POS+NECES+A3SG$POS^DB+NOUN+INF2+A3SG+PNON+NOM^DB+ADJ+WITH":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "POS+NECES+A3SG"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":383
 *             return "POS+NECES+A3SG"
 *             # DE #
 *         elif parseString == "CONJ$NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             if index > 0:
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_CONJ_NOUN_PROP_A3SG_PNON_NOM_VER, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 383, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":384
 *             # DE #
 *         elif parseString == "CONJ$NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             if index > 0:             # <<<<<<<<<<<<<<
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 *             # GE, SIK #
 */
    __pyx_t_3 = PyObject_RichCompare(__pyx_v_index, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 384, __pyx_L1_error)
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 384, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":385
 *         elif parseString == "CONJ$NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             if index > 0:
 *                 return "NOUN+PROP+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *             # GE, SIK #
 *         elif parseString == "ADJ$ADV$VERB+POS+IMP+A2SG":
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_NOUN_PROP_A3SG_PNON_NOM);
      __pyx_r = __pyx_kp_u_NOUN_PROP_A3SG_PNON_NOM;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":384
 *             # DE #
 *         elif parseString == "CONJ$NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             if index > 0:             # <<<<<<<<<<<<<<
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 *             # GE, SIK #
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":383
 *             return "POS+NECES+A3SG"
 *             # DE #
 *         elif parseString == "CONJ$NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             if index > 0:
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 */
    goto __pyx_L3;
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":387
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 *             # GE, SIK #
 *         elif parseString == "ADJ$ADV$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             if surfaceForm == "sk":
 *                 previousWord = ""
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_ADV_VERB_POS_IMP_A2SG, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 387, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":388
 *             # GE, SIK #
 *         elif parseString == "ADJ$ADV$VERB+POS+IMP+A2SG":
 *             if surfaceForm == "sk":             # <<<<<<<<<<<<<<
 *                 previousWord = ""
 *                 nextWord = ""
 */
    __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_surfaceForm, __pyx_n_u_sk, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 388, __pyx_L1_error)
    __pyx_t_11 = (__pyx_t_9 != 0);
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":389
 *         elif parseString == "ADJ$ADV$VERB+POS+IMP+A2SG":
 *             if surfaceForm == "sk":
 *                 previousWord = ""             # <<<<<<<<<<<<<<
 *                 nextWord = ""
 *                 if index - 1 > -1:
 */
      __Pyx_INCREF(__pyx_kp_u__2);
      __pyx_v_previousWord = __pyx_kp_u__2;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":390
 *             if surfaceForm == "sk":
 *                 previousWord = ""
 *                 nextWord = ""             # <<<<<<<<<<<<<<
 *                 if index - 1 > -1:
 *                     previousWord = fsmParses[index - 1].getFsmParse(0).getSurfaceForm()
 */
      __Pyx_INCREF(__pyx_kp_u__2);
      __pyx_v_nextWord = __pyx_kp_u__2;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":391
 *                 previousWord = ""
 *                 nextWord = ""
 *                 if index - 1 > -1:             # <<<<<<<<<<<<<<
 *                     previousWord = fsmParses[index - 1].getFsmParse(0).getSurfaceForm()
 *                 if index + 1 < len(fsmParses):
 */
      __pyx_t_3 = __Pyx_PyInt_SubtractObjC(__pyx_v_index, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 391, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_t_3, __pyx_int_neg_1, Py_GT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 391, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 391, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_11) {

        /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":392
 *                 nextWord = ""
 *                 if index - 1 > -1:
 *                     previousWord = fsmParses[index - 1].getFsmParse(0).getSurfaceForm()             # <<<<<<<<<<<<<<
 *                 if index + 1 < len(fsmParses):
 *                     nextWord = fsmParses[index + 1].getFsmParse(0).getSurfaceForm()
 */
        if (unlikely(__pyx_v_fsmParses == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 392, __pyx_L1_error)
        }
        __pyx_t_5 = __Pyx_PyInt_SubtractObjC(__pyx_v_index, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 392, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_fsmParses, __pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 392, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_getFsmParse); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 392, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_1 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_1)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_1);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
          }
        }
        __pyx_t_3 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_1, __pyx_int_0) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_int_0);
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 392, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_getSurfaceForm); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 392, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_3)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_3);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
          }
        }
        __pyx_t_4 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 392, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF_SET(__pyx_v_previousWord, __pyx_t_4);
        __pyx_t_4 = 0;

        /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":391
 *                 previousWord = ""
 *                 nextWord = ""
 *                 if index - 1 > -1:             # <<<<<<<<<<<<<<
 *                     previousWord = fsmParses[index - 1].getFsmParse(0).getSurfaceForm()
 *                 if index + 1 < len(fsmParses):
 */
      }

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":393
 *                 if index - 1 > -1:
 *                     previousWord = fsmParses[index - 1].getFsmParse(0).getSurfaceForm()
 *                 if index + 1 < len(fsmParses):             # <<<<<<<<<<<<<<
 *                     nextWord = fsmParses[index + 1].getFsmParse(0).getSurfaceForm()
 *                 if previousWord == "sk" or nextWord == "sk":
 */
      __pyx_t_4 = __Pyx_PyInt_AddObjC(__pyx_v_index, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 393, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (unlikely(__pyx_v_fsmParses == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 393, __pyx_L1_error)
      }
      __pyx_t_7 = PyList_GET_SIZE(__pyx_v_fsmParses); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 393, __pyx_L1_error)
      __pyx_t_5 = PyInt_FromSsize_t(__pyx_t_7); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 393, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_3 = PyObject_RichCompare(__pyx_t_4, __pyx_t_5, Py_LT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 393, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 393, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_11) {

        /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":394
 *                     previousWord = fsmParses[index - 1].getFsmParse(0).getSurfaceForm()
 *                 if index + 1 < len(fsmParses):
 *                     nextWord = fsmParses[index + 1].getFsmParse(0).getSurfaceForm()             # <<<<<<<<<<<<<<
 *                 if previousWord == "sk" or nextWord == "sk":
 *                     return "ADV"
 */
        if (unlikely(__pyx_v_fsmParses == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 394, __pyx_L1_error)
        }
        __pyx_t_4 = __Pyx_PyInt_AddObjC(__pyx_v_index, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 394, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_fsmParses, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 394, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_getFsmParse); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 394, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_1 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_1)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_1);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        __pyx_t_5 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_1, __pyx_int_0) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_int_0);
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 394, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_getSurfaceForm); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 394, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 394, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF_SET(__pyx_v_nextWord, __pyx_t_3);
        __pyx_t_3 = 0;

        /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":393
 *                 if index - 1 > -1:
 *                     previousWord = fsmParses[index - 1].getFsmParse(0).getSurfaceForm()
 *                 if index + 1 < len(fsmParses):             # <<<<<<<<<<<<<<
 *                     nextWord = fsmParses[index + 1].getFsmParse(0).getSurfaceForm()
 *                 if previousWord == "sk" or nextWord == "sk":
 */
      }

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":395
 *                 if index + 1 < len(fsmParses):
 *                     nextWord = fsmParses[index + 1].getFsmParse(0).getSurfaceForm()
 *                 if previousWord == "sk" or nextWord == "sk":             # <<<<<<<<<<<<<<
 *                     return "ADV"
 *                 if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 */
      __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_previousWord, __pyx_n_u_sk, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 395, __pyx_L1_error)
      if (!__pyx_t_9) {
      } else {
        __pyx_t_11 = __pyx_t_9;
        goto __pyx_L74_bool_binop_done;
      }
      __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_nextWord, __pyx_n_u_sk, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 395, __pyx_L1_error)
      __pyx_t_11 = __pyx_t_9;
      __pyx_L74_bool_binop_done:;
      if (__pyx_t_11) {

        /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":396
 *                     nextWord = fsmParses[index + 1].getFsmParse(0).getSurfaceForm()
 *                 if previousWord == "sk" or nextWord == "sk":
 *                     return "ADV"             # <<<<<<<<<<<<<<
 *                 if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                     return "ADJ"
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_n_u_ADV);
        __pyx_r = __pyx_n_u_ADV;
        goto __pyx_L0;

        /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":395
 *                 if index + 1 < len(fsmParses):
 *                     nextWord = fsmParses[index + 1].getFsmParse(0).getSurfaceForm()
 *                 if previousWord == "sk" or nextWord == "sk":             # <<<<<<<<<<<<<<
 *                     return "ADV"
 *                 if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 */
      }

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":397
 *                 if previousWord == "sk" or nextWord == "sk":
 *                     return "ADV"
 *                 if AutoDisambiguator.isNextWordNoun(index, fsmParses):             # <<<<<<<<<<<<<<
 *                     return "ADJ"
 *             return "ADV"
 */
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNoun); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 397, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = NULL;
      __pyx_t_10 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
          __pyx_t_10 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_fsmParses};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 397, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_fsmParses};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 397, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      {
        __pyx_t_1 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 397, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        if (__pyx_t_5) {
          __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_5); __pyx_t_5 = NULL;
        }
        __Pyx_INCREF(__pyx_v_index);
        __Pyx_GIVEREF(__pyx_v_index);
        PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_10, __pyx_v_index);
        __Pyx_INCREF(__pyx_v_fsmParses);
        __Pyx_GIVEREF(__pyx_v_fsmParses);
        PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_10, __pyx_v_fsmParses);
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 397, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 397, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_11) {

        /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":398
 *                     return "ADV"
 *                 if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                     return "ADJ"             # <<<<<<<<<<<<<<
 *             return "ADV"
 *             # BRLKTE #
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_n_u_ADJ);
        __pyx_r = __pyx_n_u_ADJ;
        goto __pyx_L0;

        /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":397
 *                 if previousWord == "sk" or nextWord == "sk":
 *                     return "ADV"
 *                 if AutoDisambiguator.isNextWordNoun(index, fsmParses):             # <<<<<<<<<<<<<<
 *                     return "ADJ"
 *             return "ADV"
 */
      }

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":388
 *             # GE, SIK #
 *         elif parseString == "ADJ$ADV$VERB+POS+IMP+A2SG":
 *             if surfaceForm == "sk":             # <<<<<<<<<<<<<<
 *                 previousWord = ""
 *                 nextWord = ""
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":399
 *                 if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                     return "ADJ"
 *             return "ADV"             # <<<<<<<<<<<<<<
 *             # BRLKTE #
 *         elif parseString == "ADV$POSTP+PCINS":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ADV);
    __pyx_r = __pyx_n_u_ADV;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":387
 *                 return "NOUN+PROP+A3SG+PNON+NOM"
 *             # GE, SIK #
 *         elif parseString == "ADJ$ADV$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             if surfaceForm == "sk":
 *                 previousWord = ""
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":401
 *             return "ADV"
 *             # BRLKTE #
 *         elif parseString == "ADV$POSTP+PCINS":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.INSTRUMENTAL):
 *                 return "POSTP+PCINS"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADV_POSTP_PCINS, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 401, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":402
 *             # BRLKTE #
 *         elif parseString == "ADV$POSTP+PCINS":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.INSTRUMENTAL):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCINS"
 *             return "ADV"
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_hasPreviousWordTag); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 402, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_MorphologicalTag); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 402, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_INSTRUMENTAL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 402, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[4] = {__pyx_t_1, __pyx_v_index, __pyx_v_correctParses, __pyx_t_5};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 402, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[4] = {__pyx_t_1, __pyx_v_index, __pyx_v_correctParses, __pyx_t_5};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 402, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    {
      __pyx_t_2 = PyTuple_New(3+__pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 402, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (__pyx_t_1) {
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1); __pyx_t_1 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_10, __pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_2, 2+__pyx_t_10, __pyx_t_5);
      __pyx_t_5 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 402, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 402, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":403
 *         elif parseString == "ADV$POSTP+PCINS":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.INSTRUMENTAL):
 *                 return "POSTP+PCINS"             # <<<<<<<<<<<<<<
 *             return "ADV"
 *             # yavaa, drste, fazlaca #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_POSTP_PCINS);
      __pyx_r = __pyx_kp_u_POSTP_PCINS;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":402
 *             # BRLKTE #
 *         elif parseString == "ADV$POSTP+PCINS":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.INSTRUMENTAL):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCINS"
 *             return "ADV"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":404
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.INSTRUMENTAL):
 *                 return "POSTP+PCINS"
 *             return "ADV"             # <<<<<<<<<<<<<<
 *             # yavaa, drste, fazlaca #
 *         elif parseString == "ADJ+ASIF$ADV+LY$NOUN+ZERO+A3SG+PNON+EQU":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ADV);
    __pyx_r = __pyx_n_u_ADV;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":401
 *             return "ADV"
 *             # BRLKTE #
 *         elif parseString == "ADV$POSTP+PCINS":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.INSTRUMENTAL):
 *                 return "POSTP+PCINS"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":406
 *             return "ADV"
 *             # yavaa, drste, fazlaca #
 *         elif parseString == "ADJ+ASIF$ADV+LY$NOUN+ZERO+A3SG+PNON+EQU":             # <<<<<<<<<<<<<<
 *             return "ADV+LY"
 *             # FAZLADIR, FAZLAYDI, OKTU, OKTUR #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_ASIF_ADV_LY_NOUN_ZERO_A3SG_P, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 406, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":407
 *             # yavaa, drste, fazlaca #
 *         elif parseString == "ADJ+ASIF$ADV+LY$NOUN+ZERO+A3SG+PNON+EQU":
 *             return "ADV+LY"             # <<<<<<<<<<<<<<
 *             # FAZLADIR, FAZLAYDI, OKTU, OKTUR #
 *         elif parseString == "ADJ^DB$POSTP+PCABL^DB":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_ADV_LY);
    __pyx_r = __pyx_kp_u_ADV_LY;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":406
 *             return "ADV"
 *             # yavaa, drste, fazlaca #
 *         elif parseString == "ADJ+ASIF$ADV+LY$NOUN+ZERO+A3SG+PNON+EQU":             # <<<<<<<<<<<<<<
 *             return "ADV+LY"
 *             # FAZLADIR, FAZLAYDI, OKTU, OKTUR #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":409
 *             return "ADV+LY"
 *             # FAZLADIR, FAZLAYDI, OKTU, OKTUR #
 *         elif parseString == "ADJ^DB$POSTP+PCABL^DB":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL^DB"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_DB_POSTP_PCABL_DB, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 409, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":410
 *             # FAZLADIR, FAZLAYDI, OKTU, OKTUR #
 *         elif parseString == "ADJ^DB$POSTP+PCABL^DB":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCABL^DB"
 *             return "ADJ^DB"
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_hasPreviousWordTag); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 410, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_MorphologicalTag); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 410, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_ABLATIVE); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 410, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_v_index, __pyx_v_correctParses, __pyx_t_5};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 410, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_v_index, __pyx_v_correctParses, __pyx_t_5};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 410, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    {
      __pyx_t_1 = PyTuple_New(3+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 410, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__pyx_t_2) {
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2); __pyx_t_2 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_10, __pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_1, 2+__pyx_t_10, __pyx_t_5);
      __pyx_t_5 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 410, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 410, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":411
 *         elif parseString == "ADJ^DB$POSTP+PCABL^DB":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL^DB"             # <<<<<<<<<<<<<<
 *             return "ADJ^DB"
 *             # kaybettikleri, umduklar, gsterdikleri #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_POSTP_PCABL_DB);
      __pyx_r = __pyx_kp_u_POSTP_PCABL_DB;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":410
 *             # FAZLADIR, FAZLAYDI, OKTU, OKTUR #
 *         elif parseString == "ADJ^DB$POSTP+PCABL^DB":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCABL^DB"
 *             return "ADJ^DB"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":412
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL^DB"
 *             return "ADJ^DB"             # <<<<<<<<<<<<<<
 *             # kaybettikleri, umduklar, gsterdikleri #
 *         elif parseString == "ADJ+PASTPART+P3PL$NOUN+PASTPART+A3PL+P3PL+NOM$NOUN+PASTPART+A3PL+P3SG+NOM$NOUN+PASTPART" \
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_ADJ_DB);
    __pyx_r = __pyx_kp_u_ADJ_DB;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":409
 *             return "ADV+LY"
 *             # FAZLADIR, FAZLAYDI, OKTU, OKTUR #
 *         elif parseString == "ADJ^DB$POSTP+PCABL^DB":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL^DB"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":414
 *             return "ADJ^DB"
 *             # kaybettikleri, umduklar, gsterdikleri #
 *         elif parseString == "ADJ+PASTPART+P3PL$NOUN+PASTPART+A3PL+P3PL+NOM$NOUN+PASTPART+A3PL+P3SG+NOM$NOUN+PASTPART" \             # <<<<<<<<<<<<<<
 *                             "+A3SG+P3PL+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_PASTPART_P3PL_NOUN_PASTPART, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 414, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":416
 *         elif parseString == "ADJ+PASTPART+P3PL$NOUN+PASTPART+A3PL+P3PL+NOM$NOUN+PASTPART+A3PL+P3SG+NOM$NOUN+PASTPART" \
 *                             "+A3SG+P3PL+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ+PASTPART+P3PL"
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNounOrAdjective); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 416, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 416, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 416, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 416, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__pyx_t_1) {
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1); __pyx_t_1 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 416, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 416, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":417
 *                             "+A3SG+P3PL+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+PASTPART+P3PL"             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "NOUN+PASTPART+A3SG+P3PL+NOM"
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_ADJ_PASTPART_P3PL);
      __pyx_r = __pyx_kp_u_ADJ_PASTPART_P3PL;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":416
 *         elif parseString == "ADJ+PASTPART+P3PL$NOUN+PASTPART+A3PL+P3PL+NOM$NOUN+PASTPART+A3PL+P3SG+NOM$NOUN+PASTPART" \
 *                             "+A3SG+P3PL+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ+PASTPART+P3PL"
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":418
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+PASTPART+P3PL"
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "NOUN+PASTPART+A3SG+P3PL+NOM"
 *             return "NOUN+PASTPART+A3PL+P3SG+NOM"
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isPossessivePlural); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 418, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 418, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 418, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_1 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 418, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_10, __pyx_v_correctParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 418, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 418, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":419
 *                 return "ADJ+PASTPART+P3PL"
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "NOUN+PASTPART+A3SG+P3PL+NOM"             # <<<<<<<<<<<<<<
 *             return "NOUN+PASTPART+A3PL+P3SG+NOM"
 *             # yln, yolun #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_NOUN_PASTPART_A3SG_P3PL_NOM);
      __pyx_r = __pyx_kp_u_NOUN_PASTPART_A3SG_P3PL_NOM;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":418
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+PASTPART+P3PL"
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "NOUN+PASTPART+A3SG+P3PL+NOM"
 *             return "NOUN+PASTPART+A3PL+P3SG+NOM"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":420
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "NOUN+PASTPART+A3SG+P3PL+NOM"
 *             return "NOUN+PASTPART+A3PL+P3SG+NOM"             # <<<<<<<<<<<<<<
 *             # yln, yolun #
 *         elif parseString == "NOUN+A3SG+P2SG+NOM$NOUN+A3SG+PNON+GEN$VERB+POS+IMP+A2PL$VERB^DB+VERB+PASS+POS+IMP+A2SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_PASTPART_A3PL_P3SG_NOM);
    __pyx_r = __pyx_kp_u_NOUN_PASTPART_A3PL_P3SG_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":414
 *             return "ADJ^DB"
 *             # kaybettikleri, umduklar, gsterdikleri #
 *         elif parseString == "ADJ+PASTPART+P3PL$NOUN+PASTPART+A3PL+P3PL+NOM$NOUN+PASTPART+A3PL+P3SG+NOM$NOUN+PASTPART" \             # <<<<<<<<<<<<<<
 *                             "+A3SG+P3PL+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":422
 *             return "NOUN+PASTPART+A3PL+P3SG+NOM"
 *             # yln, yolun #
 *         elif parseString == "NOUN+A3SG+P2SG+NOM$NOUN+A3SG+PNON+GEN$VERB+POS+IMP+A2PL$VERB^DB+VERB+PASS+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "NOUN+A3SG+P2SG+NOM"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_P2SG_NOM_NOUN_A3SG_PNO, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 422, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":423
 *             # yln, yolun #
 *         elif parseString == "NOUN+A3SG+P2SG+NOM$NOUN+A3SG+PNON+GEN$VERB+POS+IMP+A2PL$VERB^DB+VERB+PASS+POS+IMP+A2SG":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "NOUN+A3SG+P2SG+NOM"
 *             return "NOUN+A3SG+PNON+GEN"
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isAnyWordSecondPerson); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 423, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 423, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 423, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 423, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__pyx_t_1) {
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1); __pyx_t_1 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_10, __pyx_v_correctParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 423, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 423, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":424
 *         elif parseString == "NOUN+A3SG+P2SG+NOM$NOUN+A3SG+PNON+GEN$VERB+POS+IMP+A2PL$VERB^DB+VERB+PASS+POS+IMP+A2SG":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "NOUN+A3SG+P2SG+NOM"             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+PNON+GEN"
 *             # srmekte, beklenmekte, deimekte #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_NOUN_A3SG_P2SG_NOM);
      __pyx_r = __pyx_kp_u_NOUN_A3SG_P2SG_NOM;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":423
 *             # yln, yolun #
 *         elif parseString == "NOUN+A3SG+P2SG+NOM$NOUN+A3SG+PNON+GEN$VERB+POS+IMP+A2PL$VERB^DB+VERB+PASS+POS+IMP+A2SG":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "NOUN+A3SG+P2SG+NOM"
 *             return "NOUN+A3SG+PNON+GEN"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":425
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "NOUN+A3SG+P2SG+NOM"
 *             return "NOUN+A3SG+PNON+GEN"             # <<<<<<<<<<<<<<
 *             # srmekte, beklenmekte, deimekte #
 *         elif parseString == "POS+PROG2+A3SG$POS^DB+NOUN+INF+A3SG+PNON+LOC":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_A3SG_PNON_GEN);
    __pyx_r = __pyx_kp_u_NOUN_A3SG_PNON_GEN;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":422
 *             return "NOUN+PASTPART+A3PL+P3SG+NOM"
 *             # yln, yolun #
 *         elif parseString == "NOUN+A3SG+P2SG+NOM$NOUN+A3SG+PNON+GEN$VERB+POS+IMP+A2PL$VERB^DB+VERB+PASS+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "NOUN+A3SG+P2SG+NOM"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":427
 *             return "NOUN+A3SG+PNON+GEN"
 *             # srmekte, beklenmekte, deimekte #
 *         elif parseString == "POS+PROG2+A3SG$POS^DB+NOUN+INF+A3SG+PNON+LOC":             # <<<<<<<<<<<<<<
 *             return "POS+PROG2+A3SG"
 *             # KMSE, KMSEDE, KMSEYE #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_POS_PROG2_A3SG_POS_DB_NOUN_INF_A, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 427, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":428
 *             # srmekte, beklenmekte, deimekte #
 *         elif parseString == "POS+PROG2+A3SG$POS^DB+NOUN+INF+A3SG+PNON+LOC":
 *             return "POS+PROG2+A3SG"             # <<<<<<<<<<<<<<
 *             # KMSE, KMSEDE, KMSEYE #
 *         elif parseString == "NOUN+A3SG+PNON$PRON+QUANTP+A3SG+P3SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_POS_PROG2_A3SG);
    __pyx_r = __pyx_kp_u_POS_PROG2_A3SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":427
 *             return "NOUN+A3SG+PNON+GEN"
 *             # srmekte, beklenmekte, deimekte #
 *         elif parseString == "POS+PROG2+A3SG$POS^DB+NOUN+INF+A3SG+PNON+LOC":             # <<<<<<<<<<<<<<
 *             return "POS+PROG2+A3SG"
 *             # KMSE, KMSEDE, KMSEYE #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":430
 *             return "POS+PROG2+A3SG"
 *             # KMSE, KMSEDE, KMSEYE #
 *         elif parseString == "NOUN+A3SG+PNON$PRON+QUANTP+A3SG+P3SG":             # <<<<<<<<<<<<<<
 *             return "PRON+QUANTP+A3SG+P3SG"
 *             # DORU #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_PNON_PRON_QUANTP_A3SG, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 430, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":431
 *             # KMSE, KMSEDE, KMSEYE #
 *         elif parseString == "NOUN+A3SG+PNON$PRON+QUANTP+A3SG+P3SG":
 *             return "PRON+QUANTP+A3SG+P3SG"             # <<<<<<<<<<<<<<
 *             # DORU #
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$POSTP+PCDAT":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_PRON_QUANTP_A3SG_P3SG);
    __pyx_r = __pyx_kp_u_PRON_QUANTP_A3SG_P3SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":430
 *             return "POS+PROG2+A3SG"
 *             # KMSE, KMSEDE, KMSEYE #
 *         elif parseString == "NOUN+A3SG+PNON$PRON+QUANTP+A3SG+P3SG":             # <<<<<<<<<<<<<<
 *             return "PRON+QUANTP+A3SG+P3SG"
 *             # DORU #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":433
 *             return "PRON+QUANTP+A3SG+P3SG"
 *             # DORU #
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$POSTP+PCDAT":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.DATIVE):
 *                 return "POSTP+PCDAT"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_NOUN_A3SG_PNON_NOM_POSTP_PCD, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 433, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":434
 *             # DORU #
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$POSTP+PCDAT":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.DATIVE):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCDAT"
 *             return "ADJ"
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_hasPreviousWordTag); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 434, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_MorphologicalTag); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 434, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_DATIVE); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 434, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_index, __pyx_v_correctParses, __pyx_t_1};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 434, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_index, __pyx_v_correctParses, __pyx_t_1};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 434, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else
    #endif
    {
      __pyx_t_2 = PyTuple_New(3+__pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 434, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_10, __pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_2, 2+__pyx_t_10, __pyx_t_1);
      __pyx_t_1 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 434, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 434, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":435
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$POSTP+PCDAT":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.DATIVE):
 *                 return "POSTP+PCDAT"             # <<<<<<<<<<<<<<
 *             return "ADJ"
 *             # ikisini, ikisine, fazlasna #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_POSTP_PCDAT);
      __pyx_r = __pyx_kp_u_POSTP_PCDAT;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":434
 *             # DORU #
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$POSTP+PCDAT":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.DATIVE):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCDAT"
 *             return "ADJ"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":436
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.DATIVE):
 *                 return "POSTP+PCDAT"
 *             return "ADJ"             # <<<<<<<<<<<<<<
 *             # ikisini, ikisine, fazlasna #
 *         elif parseString == "ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2SG$NOUN+ZERO+A3SG+P3SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ADJ);
    __pyx_r = __pyx_n_u_ADJ;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":433
 *             return "PRON+QUANTP+A3SG+P3SG"
 *             # DORU #
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$POSTP+PCDAT":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.DATIVE):
 *                 return "POSTP+PCDAT"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":438
 *             return "ADJ"
 *             # ikisini, ikisine, fazlasna #
 *         elif parseString == "ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2SG$NOUN+ZERO+A3SG+P3SG":             # <<<<<<<<<<<<<<
 *             return "NOUN+ZERO+A3SG+P3SG"
 *             # kiilerdir, aylardr, yllardr #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_JUSTLIKE_DB_NOUN_ZERO_A3SG_P, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 438, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":439
 *             # ikisini, ikisine, fazlasna #
 *         elif parseString == "ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2SG$NOUN+ZERO+A3SG+P3SG":
 *             return "NOUN+ZERO+A3SG+P3SG"             # <<<<<<<<<<<<<<
 *             # kiilerdir, aylardr, yllardr #
 *         elif parseString == "A3PL+PNON+NOM^DB+ADV+SINCE$A3PL+PNON+NOM^DB+VERB+ZERO+PRES+COP+A3SG$A3SG+PNON+NOM^DB" \
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_ZERO_A3SG_P3SG);
    __pyx_r = __pyx_kp_u_NOUN_ZERO_A3SG_P3SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":438
 *             return "ADJ"
 *             # ikisini, ikisine, fazlasna #
 *         elif parseString == "ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2SG$NOUN+ZERO+A3SG+P3SG":             # <<<<<<<<<<<<<<
 *             return "NOUN+ZERO+A3SG+P3SG"
 *             # kiilerdir, aylardr, yllardr #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":441
 *             return "NOUN+ZERO+A3SG+P3SG"
 *             # kiilerdir, aylardr, yllardr #
 *         elif parseString == "A3PL+PNON+NOM^DB+ADV+SINCE$A3PL+PNON+NOM^DB+VERB+ZERO+PRES+COP+A3SG$A3SG+PNON+NOM^DB" \             # <<<<<<<<<<<<<<
 *                             "+VERB+ZERO+PRES+A3PL+COP":
 *             if root == "yl" or root == "sre" or root == "zaman" or root == "ay":
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_A3PL_PNON_NOM_DB_ADV_SINCE_A3PL, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 441, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":443
 *         elif parseString == "A3PL+PNON+NOM^DB+ADV+SINCE$A3PL+PNON+NOM^DB+VERB+ZERO+PRES+COP+A3SG$A3SG+PNON+NOM^DB" \
 *                             "+VERB+ZERO+PRES+A3PL+COP":
 *             if root == "yl" or root == "sre" or root == "zaman" or root == "ay":             # <<<<<<<<<<<<<<
 *                 return "A3PL+PNON+NOM^DB+ADV+SINCE"
 *             else:
 */
    __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_root, __pyx_n_u_yl, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 443, __pyx_L1_error)
    __pyx_t_8 = (__pyx_t_9 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_11 = __pyx_t_8;
      goto __pyx_L84_bool_binop_done;
    }
    __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_root, __pyx_n_u_sre, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 443, __pyx_L1_error)
    __pyx_t_9 = (__pyx_t_8 != 0);
    if (!__pyx_t_9) {
    } else {
      __pyx_t_11 = __pyx_t_9;
      goto __pyx_L84_bool_binop_done;
    }
    __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_root, __pyx_n_u_zaman, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 443, __pyx_L1_error)
    __pyx_t_8 = (__pyx_t_9 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_11 = __pyx_t_8;
      goto __pyx_L84_bool_binop_done;
    }
    __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_root, __pyx_n_u_ay, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 443, __pyx_L1_error)
    __pyx_t_9 = (__pyx_t_8 != 0);
    __pyx_t_11 = __pyx_t_9;
    __pyx_L84_bool_binop_done:;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":444
 *                             "+VERB+ZERO+PRES+A3PL+COP":
 *             if root == "yl" or root == "sre" or root == "zaman" or root == "ay":
 *                 return "A3PL+PNON+NOM^DB+ADV+SINCE"             # <<<<<<<<<<<<<<
 *             else:
 *                 return "A3PL+PNON+NOM^DB+VERB+ZERO+PRES+COP+A3SG"
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_A3PL_PNON_NOM_DB_ADV_SINCE);
      __pyx_r = __pyx_kp_u_A3PL_PNON_NOM_DB_ADV_SINCE;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":443
 *         elif parseString == "A3PL+PNON+NOM^DB+ADV+SINCE$A3PL+PNON+NOM^DB+VERB+ZERO+PRES+COP+A3SG$A3SG+PNON+NOM^DB" \
 *                             "+VERB+ZERO+PRES+A3PL+COP":
 *             if root == "yl" or root == "sre" or root == "zaman" or root == "ay":             # <<<<<<<<<<<<<<
 *                 return "A3PL+PNON+NOM^DB+ADV+SINCE"
 *             else:
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":446
 *                 return "A3PL+PNON+NOM^DB+ADV+SINCE"
 *             else:
 *                 return "A3PL+PNON+NOM^DB+VERB+ZERO+PRES+COP+A3SG"             # <<<<<<<<<<<<<<
 *             # HEP #
 *         elif parseString == "ADV$PRON+QUANTP+A3SG+P3SG+NOM":
 */
    /*else*/ {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_A3PL_PNON_NOM_DB_VERB_ZERO_PRES);
      __pyx_r = __pyx_kp_u_A3PL_PNON_NOM_DB_VERB_ZERO_PRES;
      goto __pyx_L0;
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":441
 *             return "NOUN+ZERO+A3SG+P3SG"
 *             # kiilerdir, aylardr, yllardr #
 *         elif parseString == "A3PL+PNON+NOM^DB+ADV+SINCE$A3PL+PNON+NOM^DB+VERB+ZERO+PRES+COP+A3SG$A3SG+PNON+NOM^DB" \             # <<<<<<<<<<<<<<
 *                             "+VERB+ZERO+PRES+A3PL+COP":
 *             if root == "yl" or root == "sre" or root == "zaman" or root == "ay":
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":448
 *                 return "A3PL+PNON+NOM^DB+VERB+ZERO+PRES+COP+A3SG"
 *             # HEP #
 *         elif parseString == "ADV$PRON+QUANTP+A3SG+P3SG+NOM":             # <<<<<<<<<<<<<<
 *             return "ADV"
 *             # O #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADV_PRON_QUANTP_A3SG_P3SG_NOM, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 448, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":449
 *             # HEP #
 *         elif parseString == "ADV$PRON+QUANTP+A3SG+P3SG+NOM":
 *             return "ADV"             # <<<<<<<<<<<<<<
 *             # O #
 *         elif parseString == "DET$NOUN+PROP+A3SG+PNON+NOM$PRON+DEMONSP+A3SG+PNON+NOM$PRON+PERS+A3SG+PNON+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ADV);
    __pyx_r = __pyx_n_u_ADV;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":448
 *                 return "A3PL+PNON+NOM^DB+VERB+ZERO+PRES+COP+A3SG"
 *             # HEP #
 *         elif parseString == "ADV$PRON+QUANTP+A3SG+P3SG+NOM":             # <<<<<<<<<<<<<<
 *             return "ADV"
 *             # O #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":451
 *             return "ADV"
 *             # O #
 *         elif parseString == "DET$NOUN+PROP+A3SG+PNON+NOM$PRON+DEMONSP+A3SG+PNON+NOM$PRON+PERS+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "DET"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_DET_NOUN_PROP_A3SG_PNON_NOM_PRON, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 451, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":452
 *             # O #
 *         elif parseString == "DET$NOUN+PROP+A3SG+PNON+NOM$PRON+DEMONSP+A3SG+PNON+NOM$PRON+PERS+A3SG+PNON+NOM":
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "DET"
 *             else:
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNoun); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 452, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 452, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 452, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_1 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 452, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__pyx_t_2) {
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2); __pyx_t_2 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 452, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 452, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":453
 *         elif parseString == "DET$NOUN+PROP+A3SG+PNON+NOM$PRON+DEMONSP+A3SG+PNON+NOM$PRON+PERS+A3SG+PNON+NOM":
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "DET"             # <<<<<<<<<<<<<<
 *             else:
 *                 return "PRON+PERS+A3SG+PNON+NOM"
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_n_u_DET);
      __pyx_r = __pyx_n_u_DET;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":452
 *             # O #
 *         elif parseString == "DET$NOUN+PROP+A3SG+PNON+NOM$PRON+DEMONSP+A3SG+PNON+NOM$PRON+PERS+A3SG+PNON+NOM":
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "DET"
 *             else:
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":455
 *                 return "DET"
 *             else:
 *                 return "PRON+PERS+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *             # yapmalyz, etmeliyiz, alnmaldr #
 *         elif parseString == "POS+NECES$POS^DB+NOUN+INF2+A3SG+PNON+NOM^DB+ADJ+WITH^DB+VERB+ZERO+PRES":
 */
    /*else*/ {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_PRON_PERS_A3SG_PNON_NOM);
      __pyx_r = __pyx_kp_u_PRON_PERS_A3SG_PNON_NOM;
      goto __pyx_L0;
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":451
 *             return "ADV"
 *             # O #
 *         elif parseString == "DET$NOUN+PROP+A3SG+PNON+NOM$PRON+DEMONSP+A3SG+PNON+NOM$PRON+PERS+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNoun(index, fsmParses):
 *                 return "DET"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":457
 *                 return "PRON+PERS+A3SG+PNON+NOM"
 *             # yapmalyz, etmeliyiz, alnmaldr #
 *         elif parseString == "POS+NECES$POS^DB+NOUN+INF2+A3SG+PNON+NOM^DB+ADJ+WITH^DB+VERB+ZERO+PRES":             # <<<<<<<<<<<<<<
 *             return "POS+NECES"
 *             # kzd, ekti, bozdu #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_POS_NECES_POS_DB_NOUN_INF2_A3SG, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 457, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":458
 *             # yapmalyz, etmeliyiz, alnmaldr #
 *         elif parseString == "POS+NECES$POS^DB+NOUN+INF2+A3SG+PNON+NOM^DB+ADJ+WITH^DB+VERB+ZERO+PRES":
 *             return "POS+NECES"             # <<<<<<<<<<<<<<
 *             # kzd, ekti, bozdu #
 *         elif parseString == "ADJ^DB+VERB+ZERO$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO$VERB+POS":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_POS_NECES);
    __pyx_r = __pyx_kp_u_POS_NECES;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":457
 *                 return "PRON+PERS+A3SG+PNON+NOM"
 *             # yapmalyz, etmeliyiz, alnmaldr #
 *         elif parseString == "POS+NECES$POS^DB+NOUN+INF2+A3SG+PNON+NOM^DB+ADJ+WITH^DB+VERB+ZERO+PRES":             # <<<<<<<<<<<<<<
 *             return "POS+NECES"
 *             # kzd, ekti, bozdu #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":460
 *             return "POS+NECES"
 *             # kzd, ekti, bozdu #
 *         elif parseString == "ADJ^DB+VERB+ZERO$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO$VERB+POS":             # <<<<<<<<<<<<<<
 *             return "VERB+POS"
 *             # BZMLE #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_DB_VERB_ZERO_NOUN_A3SG_PNON, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 460, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":461
 *             # kzd, ekti, bozdu #
 *         elif parseString == "ADJ^DB+VERB+ZERO$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO$VERB+POS":
 *             return "VERB+POS"             # <<<<<<<<<<<<<<
 *             # BZMLE #
 *         elif parseString == "NOUN+A3SG+P1SG$PRON+PERS+A1PL+PNON":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_VERB_POS);
    __pyx_r = __pyx_kp_u_VERB_POS;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":460
 *             return "POS+NECES"
 *             # kzd, ekti, bozdu #
 *         elif parseString == "ADJ^DB+VERB+ZERO$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO$VERB+POS":             # <<<<<<<<<<<<<<
 *             return "VERB+POS"
 *             # BZMLE #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":463
 *             return "VERB+POS"
 *             # BZMLE #
 *         elif parseString == "NOUN+A3SG+P1SG$PRON+PERS+A1PL+PNON":             # <<<<<<<<<<<<<<
 *             return "PRON+PERS+A1PL+PNON"
 *             # VARDIR #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_P1SG_PRON_PERS_A1PL_PN, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 463, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":464
 *             # BZMLE #
 *         elif parseString == "NOUN+A3SG+P1SG$PRON+PERS+A1PL+PNON":
 *             return "PRON+PERS+A1PL+PNON"             # <<<<<<<<<<<<<<
 *             # VARDIR #
 *         elif parseString == "ADJ^DB+VERB+ZERO+PRES+COP+A3SG$VERB^DB+VERB+CAUS+POS+IMP+A2SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_PRON_PERS_A1PL_PNON);
    __pyx_r = __pyx_kp_u_PRON_PERS_A1PL_PNON;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":463
 *             return "VERB+POS"
 *             # BZMLE #
 *         elif parseString == "NOUN+A3SG+P1SG$PRON+PERS+A1PL+PNON":             # <<<<<<<<<<<<<<
 *             return "PRON+PERS+A1PL+PNON"
 *             # VARDIR #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":466
 *             return "PRON+PERS+A1PL+PNON"
 *             # VARDIR #
 *         elif parseString == "ADJ^DB+VERB+ZERO+PRES+COP+A3SG$VERB^DB+VERB+CAUS+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             return "ADJ^DB+VERB+ZERO+PRES+COP+A3SG"
 *             # M #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_DB_VERB_ZERO_PRES_COP_A3SG_V, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 466, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":467
 *             # VARDIR #
 *         elif parseString == "ADJ^DB+VERB+ZERO+PRES+COP+A3SG$VERB^DB+VERB+CAUS+POS+IMP+A2SG":
 *             return "ADJ^DB+VERB+ZERO+PRES+COP+A3SG"             # <<<<<<<<<<<<<<
 *             # M #
 *         elif parseString == "NOUN+A3SG+PNON+NOM$QUES+PRES+A3SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_ADJ_DB_VERB_ZERO_PRES_COP_A3SG);
    __pyx_r = __pyx_kp_u_ADJ_DB_VERB_ZERO_PRES_COP_A3SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":466
 *             return "PRON+PERS+A1PL+PNON"
 *             # VARDIR #
 *         elif parseString == "ADJ^DB+VERB+ZERO+PRES+COP+A3SG$VERB^DB+VERB+CAUS+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             return "ADJ^DB+VERB+ZERO+PRES+COP+A3SG"
 *             # M #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":469
 *             return "ADJ^DB+VERB+ZERO+PRES+COP+A3SG"
 *             # M #
 *         elif parseString == "NOUN+A3SG+PNON+NOM$QUES+PRES+A3SG":             # <<<<<<<<<<<<<<
 *             return "QUES+PRES+A3SG"
 *             # BENM #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_PNON_NOM_QUES_PRES_A3S, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 469, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":470
 *             # M #
 *         elif parseString == "NOUN+A3SG+PNON+NOM$QUES+PRES+A3SG":
 *             return "QUES+PRES+A3SG"             # <<<<<<<<<<<<<<
 *             # BENM #
 *         elif parseString == "NOUN+A3SG+P1SG+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A1SG$PRON+PERS+A1SG+PNON" \
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_QUES_PRES_A3SG);
    __pyx_r = __pyx_kp_u_QUES_PRES_A3SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":469
 *             return "ADJ^DB+VERB+ZERO+PRES+COP+A3SG"
 *             # M #
 *         elif parseString == "NOUN+A3SG+PNON+NOM$QUES+PRES+A3SG":             # <<<<<<<<<<<<<<
 *             return "QUES+PRES+A3SG"
 *             # BENM #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":472
 *             return "QUES+PRES+A3SG"
 *             # BENM #
 *         elif parseString == "NOUN+A3SG+P1SG+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A1SG$PRON+PERS+A1SG+PNON" \             # <<<<<<<<<<<<<<
 *                             "+GEN$PRON+PERS+A1SG+PNON+NOM^DB+VERB+ZERO+PRES+A1SG":
 *             return "PRON+PERS+A1SG+PNON+GEN"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_P1SG_NOM_NOUN_A3SG_PNO, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 472, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":474
 *         elif parseString == "NOUN+A3SG+P1SG+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A1SG$PRON+PERS+A1SG+PNON" \
 *                             "+GEN$PRON+PERS+A1SG+PNON+NOM^DB+VERB+ZERO+PRES+A1SG":
 *             return "PRON+PERS+A1SG+PNON+GEN"             # <<<<<<<<<<<<<<
 *             # SUN #
 *         elif parseString == "NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_PRON_PERS_A1SG_PNON_GEN);
    __pyx_r = __pyx_kp_u_PRON_PERS_A1SG_PNON_GEN;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":472
 *             return "QUES+PRES+A3SG"
 *             # BENM #
 *         elif parseString == "NOUN+A3SG+P1SG+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A1SG$PRON+PERS+A1SG+PNON" \             # <<<<<<<<<<<<<<
 *                             "+GEN$PRON+PERS+A1SG+PNON+NOM^DB+VERB+ZERO+PRES+A1SG":
 *             return "PRON+PERS+A1SG+PNON+GEN"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":476
 *             return "PRON+PERS+A1SG+PNON+GEN"
 *             # SUN #
 *         elif parseString == "NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             return "NOUN+PROP+A3SG+PNON+NOM"
 *         elif parseString == "ADJ+JUSTLIKE$NOUN+ZERO+A3SG+P3SG+NOM$NOUN+ZERO^DB+ADJ+ALMOST":
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_PROP_A3SG_PNON_NOM_VERB_POS, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 476, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":477
 *             # SUN #
 *         elif parseString == "NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             return "NOUN+PROP+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *         elif parseString == "ADJ+JUSTLIKE$NOUN+ZERO+A3SG+P3SG+NOM$NOUN+ZERO^DB+ADJ+ALMOST":
 *             return "NOUN+ZERO+A3SG+P3SG+NOM"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_PROP_A3SG_PNON_NOM);
    __pyx_r = __pyx_kp_u_NOUN_PROP_A3SG_PNON_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":476
 *             return "PRON+PERS+A1SG+PNON+GEN"
 *             # SUN #
 *         elif parseString == "NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             return "NOUN+PROP+A3SG+PNON+NOM"
 *         elif parseString == "ADJ+JUSTLIKE$NOUN+ZERO+A3SG+P3SG+NOM$NOUN+ZERO^DB+ADJ+ALMOST":
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":478
 *         elif parseString == "NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             return "NOUN+PROP+A3SG+PNON+NOM"
 *         elif parseString == "ADJ+JUSTLIKE$NOUN+ZERO+A3SG+P3SG+NOM$NOUN+ZERO^DB+ADJ+ALMOST":             # <<<<<<<<<<<<<<
 *             return "NOUN+ZERO+A3SG+P3SG+NOM"
 *             # dndk, ettik, kazandk #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_JUSTLIKE_NOUN_ZERO_A3SG_P3SG_2, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 478, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":479
 *             return "NOUN+PROP+A3SG+PNON+NOM"
 *         elif parseString == "ADJ+JUSTLIKE$NOUN+ZERO+A3SG+P3SG+NOM$NOUN+ZERO^DB+ADJ+ALMOST":
 *             return "NOUN+ZERO+A3SG+P3SG+NOM"             # <<<<<<<<<<<<<<
 *             # dndk, ettik, kazandk #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PAST+A1PL$VERB+POS+PAST+A1PL$VERB+POS^DB+ADJ+PASTPART" \
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_ZERO_A3SG_P3SG_NOM);
    __pyx_r = __pyx_kp_u_NOUN_ZERO_A3SG_P3SG_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":478
 *         elif parseString == "NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             return "NOUN+PROP+A3SG+PNON+NOM"
 *         elif parseString == "ADJ+JUSTLIKE$NOUN+ZERO+A3SG+P3SG+NOM$NOUN+ZERO^DB+ADJ+ALMOST":             # <<<<<<<<<<<<<<
 *             return "NOUN+ZERO+A3SG+P3SG+NOM"
 *             # dndk, ettik, kazandk #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":481
 *             return "NOUN+ZERO+A3SG+P3SG+NOM"
 *             # dndk, ettik, kazandk #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PAST+A1PL$VERB+POS+PAST+A1PL$VERB+POS^DB+ADJ+PASTPART" \             # <<<<<<<<<<<<<<
 *                             "+PNON$VERB+POS^DB+NOUN+PASTPART+A3SG+PNON+NOM":
 *             return "VERB+POS+PAST+A1PL"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_2, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 481, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":483
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PAST+A1PL$VERB+POS+PAST+A1PL$VERB+POS^DB+ADJ+PASTPART" \
 *                             "+PNON$VERB+POS^DB+NOUN+PASTPART+A3SG+PNON+NOM":
 *             return "VERB+POS+PAST+A1PL"             # <<<<<<<<<<<<<<
 *             # komiktir, eksiktir, mevcuttur, yoktur #
 *         elif parseString == "ADJ^DB+VERB+ZERO+PRES+COP+A3SG$NOUN+A3SG+PNON+NOM^DB+ADV+SINCE$NOUN+A3SG+PNON+NOM^DB" \
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_VERB_POS_PAST_A1PL);
    __pyx_r = __pyx_kp_u_VERB_POS_PAST_A1PL;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":481
 *             return "NOUN+ZERO+A3SG+P3SG+NOM"
 *             # dndk, ettik, kazandk #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PAST+A1PL$VERB+POS+PAST+A1PL$VERB+POS^DB+ADJ+PASTPART" \             # <<<<<<<<<<<<<<
 *                             "+PNON$VERB+POS^DB+NOUN+PASTPART+A3SG+PNON+NOM":
 *             return "VERB+POS+PAST+A1PL"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":485
 *             return "VERB+POS+PAST+A1PL"
 *             # komiktir, eksiktir, mevcuttur, yoktur #
 *         elif parseString == "ADJ^DB+VERB+ZERO+PRES+COP+A3SG$NOUN+A3SG+PNON+NOM^DB+ADV+SINCE$NOUN+A3SG+PNON+NOM^DB" \             # <<<<<<<<<<<<<<
 *                             "+VERB+ZERO+PRES+COP+A3SG":
 *             return "ADJ^DB+VERB+ZERO+PRES+COP+A3SG"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_DB_VERB_ZERO_PRES_COP_A3SG_N, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 485, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":487
 *         elif parseString == "ADJ^DB+VERB+ZERO+PRES+COP+A3SG$NOUN+A3SG+PNON+NOM^DB+ADV+SINCE$NOUN+A3SG+PNON+NOM^DB" \
 *                             "+VERB+ZERO+PRES+COP+A3SG":
 *             return "ADJ^DB+VERB+ZERO+PRES+COP+A3SG"             # <<<<<<<<<<<<<<
 *             # edeceim, ekeceim, koacam, gideceim, savaacam, olacam  #
 *         elif parseString == "POS+FUT+A1SG$POS^DB+ADJ+FUTPART+P1SG$POS^DB+NOUN+FUTPART+A3SG+P1SG+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_ADJ_DB_VERB_ZERO_PRES_COP_A3SG);
    __pyx_r = __pyx_kp_u_ADJ_DB_VERB_ZERO_PRES_COP_A3SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":485
 *             return "VERB+POS+PAST+A1PL"
 *             # komiktir, eksiktir, mevcuttur, yoktur #
 *         elif parseString == "ADJ^DB+VERB+ZERO+PRES+COP+A3SG$NOUN+A3SG+PNON+NOM^DB+ADV+SINCE$NOUN+A3SG+PNON+NOM^DB" \             # <<<<<<<<<<<<<<
 *                             "+VERB+ZERO+PRES+COP+A3SG":
 *             return "ADJ^DB+VERB+ZERO+PRES+COP+A3SG"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":489
 *             return "ADJ^DB+VERB+ZERO+PRES+COP+A3SG"
 *             # edeceim, ekeceim, koacam, gideceim, savaacam, olacam  #
 *         elif parseString == "POS+FUT+A1SG$POS^DB+ADJ+FUTPART+P1SG$POS^DB+NOUN+FUTPART+A3SG+P1SG+NOM":             # <<<<<<<<<<<<<<
 *             return "POS+FUT+A1SG"
 *             # A #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_POS_FUT_A1SG_POS_DB_ADJ_FUTPART, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 489, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":490
 *             # edeceim, ekeceim, koacam, gideceim, savaacam, olacam  #
 *         elif parseString == "POS+FUT+A1SG$POS^DB+ADJ+FUTPART+P1SG$POS^DB+NOUN+FUTPART+A3SG+P1SG+NOM":
 *             return "POS+FUT+A1SG"             # <<<<<<<<<<<<<<
 *             # A #
 *         elif parseString == "ADJ$INTERJ$NOUN+PROP+A3SG+PNON+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_POS_FUT_A1SG);
    __pyx_r = __pyx_kp_u_POS_FUT_A1SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":489
 *             return "ADJ^DB+VERB+ZERO+PRES+COP+A3SG"
 *             # edeceim, ekeceim, koacam, gideceim, savaacam, olacam  #
 *         elif parseString == "POS+FUT+A1SG$POS^DB+ADJ+FUTPART+P1SG$POS^DB+NOUN+FUTPART+A3SG+P1SG+NOM":             # <<<<<<<<<<<<<<
 *             return "POS+FUT+A1SG"
 *             # A #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":492
 *             return "POS+FUT+A1SG"
 *             # A #
 *         elif parseString == "ADJ$INTERJ$NOUN+PROP+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             return "NOUN+PROP+A3SG+PNON+NOM"
 *             # BZ #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_INTERJ_NOUN_PROP_A3SG_PNON_N, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 492, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":493
 *             # A #
 *         elif parseString == "ADJ$INTERJ$NOUN+PROP+A3SG+PNON+NOM":
 *             return "NOUN+PROP+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *             # BZ #
 *         elif parseString == "NOUN+A3SG+P3SG+NOM$NOUN+A3SG+PNON+ACC$PRON+PERS+A1PL+PNON+ACC":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_PROP_A3SG_PNON_NOM);
    __pyx_r = __pyx_kp_u_NOUN_PROP_A3SG_PNON_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":492
 *             return "POS+FUT+A1SG"
 *             # A #
 *         elif parseString == "ADJ$INTERJ$NOUN+PROP+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             return "NOUN+PROP+A3SG+PNON+NOM"
 *             # BZ #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":495
 *             return "NOUN+PROP+A3SG+PNON+NOM"
 *             # BZ #
 *         elif parseString == "NOUN+A3SG+P3SG+NOM$NOUN+A3SG+PNON+ACC$PRON+PERS+A1PL+PNON+ACC":             # <<<<<<<<<<<<<<
 *             return "PRON+PERS+A1PL+PNON+ACC"
 *             # BZM #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_P3SG_NOM_NOUN_A3SG_PNO, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 495, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":496
 *             # BZ #
 *         elif parseString == "NOUN+A3SG+P3SG+NOM$NOUN+A3SG+PNON+ACC$PRON+PERS+A1PL+PNON+ACC":
 *             return "PRON+PERS+A1PL+PNON+ACC"             # <<<<<<<<<<<<<<
 *             # BZM #
 *         elif parseString == "NOUN+A3SG+P1SG+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A1SG$PRON+PERS+A1PL+PNON" \
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_PRON_PERS_A1PL_PNON_ACC);
    __pyx_r = __pyx_kp_u_PRON_PERS_A1PL_PNON_ACC;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":495
 *             return "NOUN+PROP+A3SG+PNON+NOM"
 *             # BZ #
 *         elif parseString == "NOUN+A3SG+P3SG+NOM$NOUN+A3SG+PNON+ACC$PRON+PERS+A1PL+PNON+ACC":             # <<<<<<<<<<<<<<
 *             return "PRON+PERS+A1PL+PNON+ACC"
 *             # BZM #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":498
 *             return "PRON+PERS+A1PL+PNON+ACC"
 *             # BZM #
 *         elif parseString == "NOUN+A3SG+P1SG+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A1SG$PRON+PERS+A1PL+PNON" \             # <<<<<<<<<<<<<<
 *                             "+GEN$PRON+PERS+A1PL+PNON+NOM^DB+VERB+ZERO+PRES+A1SG":
 *             return "PRON+PERS+A1PL+PNON+GEN"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_P1SG_NOM_NOUN_A3SG_PNO_2, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 498, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":500
 *         elif parseString == "NOUN+A3SG+P1SG+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A1SG$PRON+PERS+A1PL+PNON" \
 *                             "+GEN$PRON+PERS+A1PL+PNON+NOM^DB+VERB+ZERO+PRES+A1SG":
 *             return "PRON+PERS+A1PL+PNON+GEN"             # <<<<<<<<<<<<<<
 *             # erkekler, kadnlar, madenler, uzmanlar#
 *         elif parseString == "ADJ^DB+VERB+ZERO+PRES+A3PL$NOUN+A3PL+PNON+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES" \
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_PRON_PERS_A1PL_PNON_GEN);
    __pyx_r = __pyx_kp_u_PRON_PERS_A1PL_PNON_GEN;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":498
 *             return "PRON+PERS+A1PL+PNON+ACC"
 *             # BZM #
 *         elif parseString == "NOUN+A3SG+P1SG+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A1SG$PRON+PERS+A1PL+PNON" \             # <<<<<<<<<<<<<<
 *                             "+GEN$PRON+PERS+A1PL+PNON+NOM^DB+VERB+ZERO+PRES+A1SG":
 *             return "PRON+PERS+A1PL+PNON+GEN"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":502
 *             return "PRON+PERS+A1PL+PNON+GEN"
 *             # erkekler, kadnlar, madenler, uzmanlar#
 *         elif parseString == "ADJ^DB+VERB+ZERO+PRES+A3PL$NOUN+A3PL+PNON+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES" \             # <<<<<<<<<<<<<<
 *                             "+A3PL$NOUN+PROP+A3PL+PNON+NOM":
 *             return "NOUN+A3PL+PNON+NOM"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_DB_VERB_ZERO_PRES_A3PL_NOUN_2, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 502, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":504
 *         elif parseString == "ADJ^DB+VERB+ZERO+PRES+A3PL$NOUN+A3PL+PNON+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES" \
 *                             "+A3PL$NOUN+PROP+A3PL+PNON+NOM":
 *             return "NOUN+A3PL+PNON+NOM"             # <<<<<<<<<<<<<<
 *             # TAB #
 *         elif parseString == "ADJ$INTERJ":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_A3PL_PNON_NOM);
    __pyx_r = __pyx_kp_u_NOUN_A3PL_PNON_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":502
 *             return "PRON+PERS+A1PL+PNON+GEN"
 *             # erkekler, kadnlar, madenler, uzmanlar#
 *         elif parseString == "ADJ^DB+VERB+ZERO+PRES+A3PL$NOUN+A3PL+PNON+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES" \             # <<<<<<<<<<<<<<
 *                             "+A3PL$NOUN+PROP+A3PL+PNON+NOM":
 *             return "NOUN+A3PL+PNON+NOM"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":506
 *             return "NOUN+A3PL+PNON+NOM"
 *             # TAB #
 *         elif parseString == "ADJ$INTERJ":             # <<<<<<<<<<<<<<
 *             return "ADJ"
 *         elif parseString == "AOR+A2PL$AOR^DB+ADJ+ZERO^DB+ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2PL+NOM":
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_INTERJ, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 506, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":507
 *             # TAB #
 *         elif parseString == "ADJ$INTERJ":
 *             return "ADJ"             # <<<<<<<<<<<<<<
 *         elif parseString == "AOR+A2PL$AOR^DB+ADJ+ZERO^DB+ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2PL+NOM":
 *             return "AOR+A2PL"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ADJ);
    __pyx_r = __pyx_n_u_ADJ;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":506
 *             return "NOUN+A3PL+PNON+NOM"
 *             # TAB #
 *         elif parseString == "ADJ$INTERJ":             # <<<<<<<<<<<<<<
 *             return "ADJ"
 *         elif parseString == "AOR+A2PL$AOR^DB+ADJ+ZERO^DB+ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2PL+NOM":
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":508
 *         elif parseString == "ADJ$INTERJ":
 *             return "ADJ"
 *         elif parseString == "AOR+A2PL$AOR^DB+ADJ+ZERO^DB+ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2PL+NOM":             # <<<<<<<<<<<<<<
 *             return "AOR+A2PL"
 *             # ayn, dnn#
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_AOR_A2PL_AOR_DB_ADJ_ZERO_DB_ADJ, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 508, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":509
 *             return "ADJ"
 *         elif parseString == "AOR+A2PL$AOR^DB+ADJ+ZERO^DB+ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2PL+NOM":
 *             return "AOR+A2PL"             # <<<<<<<<<<<<<<
 *             # ayn, dnn#
 *         elif parseString == "NOUN+A3SG+P2SG+NOM$NOUN+A3SG+PNON+GEN$VERB+POS+IMP+A2PL":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_AOR_A2PL);
    __pyx_r = __pyx_kp_u_AOR_A2PL;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":508
 *         elif parseString == "ADJ$INTERJ":
 *             return "ADJ"
 *         elif parseString == "AOR+A2PL$AOR^DB+ADJ+ZERO^DB+ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2PL+NOM":             # <<<<<<<<<<<<<<
 *             return "AOR+A2PL"
 *             # ayn, dnn#
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":511
 *             return "AOR+A2PL"
 *             # ayn, dnn#
 *         elif parseString == "NOUN+A3SG+P2SG+NOM$NOUN+A3SG+PNON+GEN$VERB+POS+IMP+A2PL":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "VERB+POS+IMP+A2PL"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_P2SG_NOM_NOUN_A3SG_PNO_2, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 511, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":512
 *             # ayn, dnn#
 *         elif parseString == "NOUN+A3SG+P2SG+NOM$NOUN+A3SG+PNON+GEN$VERB+POS+IMP+A2PL":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "VERB+POS+IMP+A2PL"
 *             return "NOUN+A3SG+PNON+GEN"
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isBeforeLastWord); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 512, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 512, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 512, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_2 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 512, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (__pyx_t_1) {
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1); __pyx_t_1 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 512, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 512, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":513
 *         elif parseString == "NOUN+A3SG+P2SG+NOM$NOUN+A3SG+PNON+GEN$VERB+POS+IMP+A2PL":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "VERB+POS+IMP+A2PL"             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+PNON+GEN"
 *             # deyecekler, olacaklar #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_VERB_POS_IMP_A2PL);
      __pyx_r = __pyx_kp_u_VERB_POS_IMP_A2PL;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":512
 *             # ayn, dnn#
 *         elif parseString == "NOUN+A3SG+P2SG+NOM$NOUN+A3SG+PNON+GEN$VERB+POS+IMP+A2PL":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "VERB+POS+IMP+A2PL"
 *             return "NOUN+A3SG+PNON+GEN"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":514
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "VERB+POS+IMP+A2PL"
 *             return "NOUN+A3SG+PNON+GEN"             # <<<<<<<<<<<<<<
 *             # deyecekler, olacaklar #
 *         elif parseString == "POS+FUT+A3PL$POS^DB+NOUN+FUTPART+A3PL+PNON+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_A3SG_PNON_GEN);
    __pyx_r = __pyx_kp_u_NOUN_A3SG_PNON_GEN;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":511
 *             return "AOR+A2PL"
 *             # ayn, dnn#
 *         elif parseString == "NOUN+A3SG+P2SG+NOM$NOUN+A3SG+PNON+GEN$VERB+POS+IMP+A2PL":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "VERB+POS+IMP+A2PL"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":516
 *             return "NOUN+A3SG+PNON+GEN"
 *             # deyecekler, olacaklar #
 *         elif parseString == "POS+FUT+A3PL$POS^DB+NOUN+FUTPART+A3PL+PNON+NOM":             # <<<<<<<<<<<<<<
 *             return "POS+FUT+A3PL"
 *             # 9:30'daki #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_POS_FUT_A3PL_POS_DB_NOUN_FUTPART, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 516, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":517
 *             # deyecekler, olacaklar #
 *         elif parseString == "POS+FUT+A3PL$POS^DB+NOUN+FUTPART+A3PL+PNON+NOM":
 *             return "POS+FUT+A3PL"             # <<<<<<<<<<<<<<
 *             # 9:30'daki #
 *         elif parseString == "P3SG$PNON":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_POS_FUT_A3PL);
    __pyx_r = __pyx_kp_u_POS_FUT_A3PL;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":516
 *             return "NOUN+A3SG+PNON+GEN"
 *             # deyecekler, olacaklar #
 *         elif parseString == "POS+FUT+A3PL$POS^DB+NOUN+FUTPART+A3PL+PNON+NOM":             # <<<<<<<<<<<<<<
 *             return "POS+FUT+A3PL"
 *             # 9:30'daki #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":519
 *             return "POS+FUT+A3PL"
 *             # 9:30'daki #
 *         elif parseString == "P3SG$PNON":             # <<<<<<<<<<<<<<
 *             return "PNON"
 *             # olabilecek, yapabilecek #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_P3SG_PNON, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 519, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":520
 *             # 9:30'daki #
 *         elif parseString == "P3SG$PNON":
 *             return "PNON"             # <<<<<<<<<<<<<<
 *             # olabilecek, yapabilecek #
 *         elif parseString == "ABLE+FUT+A3SG$ABLE^DB+ADJ+FUTPART+PNON":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_PNON);
    __pyx_r = __pyx_n_u_PNON;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":519
 *             return "POS+FUT+A3PL"
 *             # 9:30'daki #
 *         elif parseString == "P3SG$PNON":             # <<<<<<<<<<<<<<
 *             return "PNON"
 *             # olabilecek, yapabilecek #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":522
 *             return "PNON"
 *             # olabilecek, yapabilecek #
 *         elif parseString == "ABLE+FUT+A3SG$ABLE^DB+ADJ+FUTPART+PNON":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ABLE^DB+ADJ+FUTPART+PNON"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ABLE_FUT_A3SG_ABLE_DB_ADJ_FUTPAR, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 522, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":523
 *             # olabilecek, yapabilecek #
 *         elif parseString == "ABLE+FUT+A3SG$ABLE^DB+ADJ+FUTPART+PNON":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ABLE^DB+ADJ+FUTPART+PNON"
 *             return "ABLE+FUT+A3SG"
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNounOrAdjective); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 523, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 523, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 523, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_1 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 523, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__pyx_t_2) {
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2); __pyx_t_2 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 523, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 523, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":524
 *         elif parseString == "ABLE+FUT+A3SG$ABLE^DB+ADJ+FUTPART+PNON":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ABLE^DB+ADJ+FUTPART+PNON"             # <<<<<<<<<<<<<<
 *             return "ABLE+FUT+A3SG"
 *             # dm duymu artm #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_ABLE_DB_ADJ_FUTPART_PNON);
      __pyx_r = __pyx_kp_u_ABLE_DB_ADJ_FUTPART_PNON;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":523
 *             # olabilecek, yapabilecek #
 *         elif parseString == "ABLE+FUT+A3SG$ABLE^DB+ADJ+FUTPART+PNON":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ABLE^DB+ADJ+FUTPART+PNON"
 *             return "ABLE+FUT+A3SG"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":525
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ABLE^DB+ADJ+FUTPART+PNON"
 *             return "ABLE+FUT+A3SG"             # <<<<<<<<<<<<<<
 *             # dm duymu artm #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+NARR+A3SG$VERB+POS+NARR+A3SG$VERB+POS+NARR^DB+ADJ+ZERO":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_ABLE_FUT_A3SG);
    __pyx_r = __pyx_kp_u_ABLE_FUT_A3SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":522
 *             return "PNON"
 *             # olabilecek, yapabilecek #
 *         elif parseString == "ABLE+FUT+A3SG$ABLE^DB+ADJ+FUTPART+PNON":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ABLE^DB+ADJ+FUTPART+PNON"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":527
 *             return "ABLE+FUT+A3SG"
 *             # dm duymu artm #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+NARR+A3SG$VERB+POS+NARR+A3SG$VERB+POS+NARR^DB+ADJ+ZERO":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "VERB+POS+NARR+A3SG"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_3, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 527, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":528
 *             # dm duymu artm #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+NARR+A3SG$VERB+POS+NARR+A3SG$VERB+POS+NARR^DB+ADJ+ZERO":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "VERB+POS+NARR+A3SG"
 *             return "VERB+POS+NARR^DB+ADJ+ZERO"
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isBeforeLastWord); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 528, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 528, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 528, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_2 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 528, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (__pyx_t_1) {
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1); __pyx_t_1 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 528, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 528, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":529
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+NARR+A3SG$VERB+POS+NARR+A3SG$VERB+POS+NARR^DB+ADJ+ZERO":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "VERB+POS+NARR+A3SG"             # <<<<<<<<<<<<<<
 *             return "VERB+POS+NARR^DB+ADJ+ZERO"
 *             # BER, DIARI, AAI #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_VERB_POS_NARR_A3SG);
      __pyx_r = __pyx_kp_u_VERB_POS_NARR_A3SG;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":528
 *             # dm duymu artm #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+NARR+A3SG$VERB+POS+NARR+A3SG$VERB+POS+NARR^DB+ADJ+ZERO":
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "VERB+POS+NARR+A3SG"
 *             return "VERB+POS+NARR^DB+ADJ+ZERO"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":530
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "VERB+POS+NARR+A3SG"
 *             return "VERB+POS+NARR^DB+ADJ+ZERO"             # <<<<<<<<<<<<<<
 *             # BER, DIARI, AAI #
 *         elif parseString == "ADJ$ADV$NOUN+A3SG+PNON+NOM$POSTP+PCABL":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_VERB_POS_NARR_DB_ADJ_ZERO);
    __pyx_r = __pyx_kp_u_VERB_POS_NARR_DB_ADJ_ZERO;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":527
 *             return "ABLE+FUT+A3SG"
 *             # dm duymu artm #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+NARR+A3SG$VERB+POS+NARR+A3SG$VERB+POS+NARR^DB+ADJ+ZERO":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isBeforeLastWord(index, fsmParses):
 *                 return "VERB+POS+NARR+A3SG"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":532
 *             return "VERB+POS+NARR^DB+ADJ+ZERO"
 *             # BER, DIARI, AAI #
 *         elif parseString == "ADJ$ADV$NOUN+A3SG+PNON+NOM$POSTP+PCABL":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_ADV_NOUN_A3SG_PNON_NOM_POSTP_2, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 532, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":533
 *             # BER, DIARI, AAI #
 *         elif parseString == "ADJ$ADV$NOUN+A3SG+PNON+NOM$POSTP+PCABL":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCABL"
 *             return "ADV"
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_hasPreviousWordTag); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 533, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_MorphologicalTag); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 533, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_ABLATIVE); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 533, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_v_index, __pyx_v_correctParses, __pyx_t_1};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 533, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_v_index, __pyx_v_correctParses, __pyx_t_1};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 533, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(3+__pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 533, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__pyx_t_2) {
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2); __pyx_t_2 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_10, __pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_10, __pyx_t_1);
      __pyx_t_1 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 533, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 533, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":534
 *         elif parseString == "ADJ$ADV$NOUN+A3SG+PNON+NOM$POSTP+PCABL":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"             # <<<<<<<<<<<<<<
 *             return "ADV"
 *             # TV, CD #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_POSTP_PCABL);
      __pyx_r = __pyx_kp_u_POSTP_PCABL;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":533
 *             # BER, DIARI, AAI #
 *         elif parseString == "ADJ$ADV$NOUN+A3SG+PNON+NOM$POSTP+PCABL":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCABL"
 *             return "ADV"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":535
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"
 *             return "ADV"             # <<<<<<<<<<<<<<
 *             # TV, CD #
 *         elif parseString == "A3SG+PNON+ACC$PROP+A3SG+PNON+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ADV);
    __pyx_r = __pyx_n_u_ADV;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":532
 *             return "VERB+POS+NARR^DB+ADJ+ZERO"
 *             # BER, DIARI, AAI #
 *         elif parseString == "ADJ$ADV$NOUN+A3SG+PNON+NOM$POSTP+PCABL":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":537
 *             return "ADV"
 *             # TV, CD #
 *         elif parseString == "A3SG+PNON+ACC$PROP+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             return "A3SG+PNON+ACC"
 *             # deinmeyeceim, vermeyeceim #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_A3SG_PNON_ACC_PROP_A3SG_PNON_NOM, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 537, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":538
 *             # TV, CD #
 *         elif parseString == "A3SG+PNON+ACC$PROP+A3SG+PNON+NOM":
 *             return "A3SG+PNON+ACC"             # <<<<<<<<<<<<<<
 *             # deinmeyeceim, vermeyeceim #
 *         elif parseString == "NEG+FUT+A1SG$NEG^DB+ADJ+FUTPART+P1SG$NEG^DB+NOUN+FUTPART+A3SG+P1SG+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_A3SG_PNON_ACC);
    __pyx_r = __pyx_kp_u_A3SG_PNON_ACC;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":537
 *             return "ADV"
 *             # TV, CD #
 *         elif parseString == "A3SG+PNON+ACC$PROP+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             return "A3SG+PNON+ACC"
 *             # deinmeyeceim, vermeyeceim #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":540
 *             return "A3SG+PNON+ACC"
 *             # deinmeyeceim, vermeyeceim #
 *         elif parseString == "NEG+FUT+A1SG$NEG^DB+ADJ+FUTPART+P1SG$NEG^DB+NOUN+FUTPART+A3SG+P1SG+NOM":             # <<<<<<<<<<<<<<
 *             return "NEG+FUT+A1SG"
 *             # grne, sata, durua #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NEG_FUT_A1SG_NEG_DB_ADJ_FUTPART, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 540, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":541
 *             # deinmeyeceim, vermeyeceim #
 *         elif parseString == "NEG+FUT+A1SG$NEG^DB+ADJ+FUTPART+P1SG$NEG^DB+NOUN+FUTPART+A3SG+P1SG+NOM":
 *             return "NEG+FUT+A1SG"             # <<<<<<<<<<<<<<
 *             # grne, sata, durua #
 *         elif parseString == "POS^DB+NOUN+INF3+A3SG+PNON+DAT$RECIP+POS+OPT+A3SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NEG_FUT_A1SG);
    __pyx_r = __pyx_kp_u_NEG_FUT_A1SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":540
 *             return "A3SG+PNON+ACC"
 *             # deinmeyeceim, vermeyeceim #
 *         elif parseString == "NEG+FUT+A1SG$NEG^DB+ADJ+FUTPART+P1SG$NEG^DB+NOUN+FUTPART+A3SG+P1SG+NOM":             # <<<<<<<<<<<<<<
 *             return "NEG+FUT+A1SG"
 *             # grne, sata, durua #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":543
 *             return "NEG+FUT+A1SG"
 *             # grne, sata, durua #
 *         elif parseString == "POS^DB+NOUN+INF3+A3SG+PNON+DAT$RECIP+POS+OPT+A3SG":             # <<<<<<<<<<<<<<
 *             return "POS^DB+NOUN+INF3+A3SG+PNON+DAT"
 *             # YILDIR, AYDIR, YOLDUR #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_POS_DB_NOUN_INF3_A3SG_PNON_DAT_R, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 543, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":544
 *             # grne, sata, durua #
 *         elif parseString == "POS^DB+NOUN+INF3+A3SG+PNON+DAT$RECIP+POS+OPT+A3SG":
 *             return "POS^DB+NOUN+INF3+A3SG+PNON+DAT"             # <<<<<<<<<<<<<<
 *             # YILDIR, AYDIR, YOLDUR #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+ADV+SINCE$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+COP+A3SG$VERB^DB" \
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_POS_DB_NOUN_INF3_A3SG_PNON_DAT);
    __pyx_r = __pyx_kp_u_POS_DB_NOUN_INF3_A3SG_PNON_DAT;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":543
 *             return "NEG+FUT+A1SG"
 *             # grne, sata, durua #
 *         elif parseString == "POS^DB+NOUN+INF3+A3SG+PNON+DAT$RECIP+POS+OPT+A3SG":             # <<<<<<<<<<<<<<
 *             return "POS^DB+NOUN+INF3+A3SG+PNON+DAT"
 *             # YILDIR, AYDIR, YOLDUR #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":546
 *             return "POS^DB+NOUN+INF3+A3SG+PNON+DAT"
 *             # YILDIR, AYDIR, YOLDUR #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+ADV+SINCE$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+COP+A3SG$VERB^DB" \             # <<<<<<<<<<<<<<
 *                             "+VERB+CAUS+POS+IMP+A2SG":
 *             if root == "yl" or root == "ay":
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_ADV_SINCE, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 546, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":548
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+ADV+SINCE$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+COP+A3SG$VERB^DB" \
 *                             "+VERB+CAUS+POS+IMP+A2SG":
 *             if root == "yl" or root == "ay":             # <<<<<<<<<<<<<<
 *                 return "NOUN+A3SG+PNON+NOM^DB+ADV+SINCE"
 *             else:
 */
    __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_root, __pyx_n_u_yl, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 548, __pyx_L1_error)
    __pyx_t_8 = (__pyx_t_11 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_9 = __pyx_t_8;
      goto __pyx_L94_bool_binop_done;
    }
    __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_root, __pyx_n_u_ay, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 548, __pyx_L1_error)
    __pyx_t_11 = (__pyx_t_8 != 0);
    __pyx_t_9 = __pyx_t_11;
    __pyx_L94_bool_binop_done:;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":549
 *                             "+VERB+CAUS+POS+IMP+A2SG":
 *             if root == "yl" or root == "ay":
 *                 return "NOUN+A3SG+PNON+NOM^DB+ADV+SINCE"             # <<<<<<<<<<<<<<
 *             else:
 *                 return "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+COP+A3SG"
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_ADV_SINCE_2);
      __pyx_r = __pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_ADV_SINCE_2;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":548
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+ADV+SINCE$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+COP+A3SG$VERB^DB" \
 *                             "+VERB+CAUS+POS+IMP+A2SG":
 *             if root == "yl" or root == "ay":             # <<<<<<<<<<<<<<
 *                 return "NOUN+A3SG+PNON+NOM^DB+ADV+SINCE"
 *             else:
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":551
 *                 return "NOUN+A3SG+PNON+NOM^DB+ADV+SINCE"
 *             else:
 *                 return "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+COP+A3SG"             # <<<<<<<<<<<<<<
 *             # BEN #
 *         elif parseString == "NOUN+A3SG+P3SG+NOM$NOUN+A3SG+PNON+ACC$PRON+PERS+A1SG+PNON+ACC":
 */
    /*else*/ {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_4);
      __pyx_r = __pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_4;
      goto __pyx_L0;
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":546
 *             return "POS^DB+NOUN+INF3+A3SG+PNON+DAT"
 *             # YILDIR, AYDIR, YOLDUR #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+ADV+SINCE$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+COP+A3SG$VERB^DB" \             # <<<<<<<<<<<<<<
 *                             "+VERB+CAUS+POS+IMP+A2SG":
 *             if root == "yl" or root == "ay":
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":553
 *                 return "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+COP+A3SG"
 *             # BEN #
 *         elif parseString == "NOUN+A3SG+P3SG+NOM$NOUN+A3SG+PNON+ACC$PRON+PERS+A1SG+PNON+ACC":             # <<<<<<<<<<<<<<
 *             return "PRON+PERS+A1SG+PNON+ACC"
 *             # edemezsin, kantlarsn, yapamazsn #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_P3SG_NOM_NOUN_A3SG_PNO_2, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 553, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":554
 *             # BEN #
 *         elif parseString == "NOUN+A3SG+P3SG+NOM$NOUN+A3SG+PNON+ACC$PRON+PERS+A1SG+PNON+ACC":
 *             return "PRON+PERS+A1SG+PNON+ACC"             # <<<<<<<<<<<<<<
 *             # edemezsin, kantlarsn, yapamazsn #
 *         elif parseString == "AOR+A2SG$AOR^DB+ADJ+ZERO^DB+ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2SG+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_PRON_PERS_A1SG_PNON_ACC);
    __pyx_r = __pyx_kp_u_PRON_PERS_A1SG_PNON_ACC;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":553
 *                 return "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+COP+A3SG"
 *             # BEN #
 *         elif parseString == "NOUN+A3SG+P3SG+NOM$NOUN+A3SG+PNON+ACC$PRON+PERS+A1SG+PNON+ACC":             # <<<<<<<<<<<<<<
 *             return "PRON+PERS+A1SG+PNON+ACC"
 *             # edemezsin, kantlarsn, yapamazsn #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":556
 *             return "PRON+PERS+A1SG+PNON+ACC"
 *             # edemezsin, kantlarsn, yapamazsn #
 *         elif parseString == "AOR+A2SG$AOR^DB+ADJ+ZERO^DB+ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2SG+NOM":             # <<<<<<<<<<<<<<
 *             return "AOR+A2SG"
 *             # BYME, ATAMA, KARIMA, KORUMA, TANIMA, REME #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_AOR_A2SG_AOR_DB_ADJ_ZERO_DB_ADJ, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 556, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":557
 *             # edemezsin, kantlarsn, yapamazsn #
 *         elif parseString == "AOR+A2SG$AOR^DB+ADJ+ZERO^DB+ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2SG+NOM":
 *             return "AOR+A2SG"             # <<<<<<<<<<<<<<
 *             # BYME, ATAMA, KARIMA, KORUMA, TANIMA, REME #
 *         elif parseString == "NOUN+A3SG+P1SG+DAT$VERB+NEG+IMP+A2SG$VERB+POS^DB+NOUN+INF2+A3SG+PNON+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_AOR_A2SG);
    __pyx_r = __pyx_kp_u_AOR_A2SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":556
 *             return "PRON+PERS+A1SG+PNON+ACC"
 *             # edemezsin, kantlarsn, yapamazsn #
 *         elif parseString == "AOR+A2SG$AOR^DB+ADJ+ZERO^DB+ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2SG+NOM":             # <<<<<<<<<<<<<<
 *             return "AOR+A2SG"
 *             # BYME, ATAMA, KARIMA, KORUMA, TANIMA, REME #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":559
 *             return "AOR+A2SG"
 *             # BYME, ATAMA, KARIMA, KORUMA, TANIMA, REME #
 *         elif parseString == "NOUN+A3SG+P1SG+DAT$VERB+NEG+IMP+A2SG$VERB+POS^DB+NOUN+INF2+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if root == "kar":
 *                 return "NOUN+A3SG+P1SG+DAT"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_P1SG_DAT_VERB_NEG_IMP, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 559, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":560
 *             # BYME, ATAMA, KARIMA, KORUMA, TANIMA, REME #
 *         elif parseString == "NOUN+A3SG+P1SG+DAT$VERB+NEG+IMP+A2SG$VERB+POS^DB+NOUN+INF2+A3SG+PNON+NOM":
 *             if root == "kar":             # <<<<<<<<<<<<<<
 *                 return "NOUN+A3SG+P1SG+DAT"
 *             return "VERB+POS^DB+NOUN+INF2+A3SG+PNON+NOM"
 */
    __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_root, __pyx_n_u_kar, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 560, __pyx_L1_error)
    __pyx_t_9 = (__pyx_t_11 != 0);
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":561
 *         elif parseString == "NOUN+A3SG+P1SG+DAT$VERB+NEG+IMP+A2SG$VERB+POS^DB+NOUN+INF2+A3SG+PNON+NOM":
 *             if root == "kar":
 *                 return "NOUN+A3SG+P1SG+DAT"             # <<<<<<<<<<<<<<
 *             return "VERB+POS^DB+NOUN+INF2+A3SG+PNON+NOM"
 *             # HANG #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_NOUN_A3SG_P1SG_DAT);
      __pyx_r = __pyx_kp_u_NOUN_A3SG_P1SG_DAT;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":560
 *             # BYME, ATAMA, KARIMA, KORUMA, TANIMA, REME #
 *         elif parseString == "NOUN+A3SG+P1SG+DAT$VERB+NEG+IMP+A2SG$VERB+POS^DB+NOUN+INF2+A3SG+PNON+NOM":
 *             if root == "kar":             # <<<<<<<<<<<<<<
 *                 return "NOUN+A3SG+P1SG+DAT"
 *             return "VERB+POS^DB+NOUN+INF2+A3SG+PNON+NOM"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":562
 *             if root == "kar":
 *                 return "NOUN+A3SG+P1SG+DAT"
 *             return "VERB+POS^DB+NOUN+INF2+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *             # HANG #
 *         elif parseString == "ADJ$PRON+QUESP+A3SG+PNON+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_VERB_POS_DB_NOUN_INF2_A3SG_PNON);
    __pyx_r = __pyx_kp_u_VERB_POS_DB_NOUN_INF2_A3SG_PNON;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":559
 *             return "AOR+A2SG"
 *             # BYME, ATAMA, KARIMA, KORUMA, TANIMA, REME #
 *         elif parseString == "NOUN+A3SG+P1SG+DAT$VERB+NEG+IMP+A2SG$VERB+POS^DB+NOUN+INF2+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if root == "kar":
 *                 return "NOUN+A3SG+P1SG+DAT"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":564
 *             return "VERB+POS^DB+NOUN+INF2+A3SG+PNON+NOM"
 *             # HANG #
 *         elif parseString == "ADJ$PRON+QUESP+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if lastWord == "?":
 *                 return "PRON+QUESP+A3SG+PNON+NOM"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_PRON_QUESP_A3SG_PNON_NOM, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 564, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":565
 *             # HANG #
 *         elif parseString == "ADJ$PRON+QUESP+A3SG+PNON+NOM":
 *             if lastWord == "?":             # <<<<<<<<<<<<<<
 *                 return "PRON+QUESP+A3SG+PNON+NOM"
 *             return "ADJ"
 */
    __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_lastWord, __pyx_kp_u_, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 565, __pyx_L1_error)
    __pyx_t_9 = (__pyx_t_11 != 0);
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":566
 *         elif parseString == "ADJ$PRON+QUESP+A3SG+PNON+NOM":
 *             if lastWord == "?":
 *                 return "PRON+QUESP+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *             return "ADJ"
 *             # GCN, GCNN, ESASINDA #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_PRON_QUESP_A3SG_PNON_NOM);
      __pyx_r = __pyx_kp_u_PRON_QUESP_A3SG_PNON_NOM;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":565
 *             # HANG #
 *         elif parseString == "ADJ$PRON+QUESP+A3SG+PNON+NOM":
 *             if lastWord == "?":             # <<<<<<<<<<<<<<
 *                 return "PRON+QUESP+A3SG+PNON+NOM"
 *             return "ADJ"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":567
 *             if lastWord == "?":
 *                 return "PRON+QUESP+A3SG+PNON+NOM"
 *             return "ADJ"             # <<<<<<<<<<<<<<
 *             # GCN, GCNN, ESASINDA #
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3SG+P2SG$ADJ^DB+NOUN+ZERO+A3SG+P3SG$NOUN+A3SG+P2SG$NOUN+A3SG+P3SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ADJ);
    __pyx_r = __pyx_n_u_ADJ;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":564
 *             return "VERB+POS^DB+NOUN+INF2+A3SG+PNON+NOM"
 *             # HANG #
 *         elif parseString == "ADJ$PRON+QUESP+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             if lastWord == "?":
 *                 return "PRON+QUESP+A3SG+PNON+NOM"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":569
 *             return "ADJ"
 *             # GCN, GCNN, ESASINDA #
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3SG+P2SG$ADJ^DB+NOUN+ZERO+A3SG+P3SG$NOUN+A3SG+P2SG$NOUN+A3SG+P3SG":             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+P3SG"
 *             # YILININ, YOLUNUN, DLNN #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_DB_NOUN_ZERO_A3SG_P2SG_ADJ_D, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 569, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":570
 *             # GCN, GCNN, ESASINDA #
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3SG+P2SG$ADJ^DB+NOUN+ZERO+A3SG+P3SG$NOUN+A3SG+P2SG$NOUN+A3SG+P3SG":
 *             return "NOUN+A3SG+P3SG"             # <<<<<<<<<<<<<<
 *             # YILININ, YOLUNUN, DLNN #
 *         elif parseString == "NOUN+A3SG+P2SG+GEN$NOUN+A3SG+P3SG+GEN$VERB^DB+VERB+PASS+POS+IMP+A2PL":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_A3SG_P3SG);
    __pyx_r = __pyx_kp_u_NOUN_A3SG_P3SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":569
 *             return "ADJ"
 *             # GCN, GCNN, ESASINDA #
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3SG+P2SG$ADJ^DB+NOUN+ZERO+A3SG+P3SG$NOUN+A3SG+P2SG$NOUN+A3SG+P3SG":             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+P3SG"
 *             # YILININ, YOLUNUN, DLNN #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":572
 *             return "NOUN+A3SG+P3SG"
 *             # YILININ, YOLUNUN, DLNN #
 *         elif parseString == "NOUN+A3SG+P2SG+GEN$NOUN+A3SG+P3SG+GEN$VERB^DB+VERB+PASS+POS+IMP+A2PL":             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+P3SG+GEN"
 *             # IKARDI #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_P2SG_GEN_NOUN_A3SG_P3S, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 572, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":573
 *             # YILININ, YOLUNUN, DLNN #
 *         elif parseString == "NOUN+A3SG+P2SG+GEN$NOUN+A3SG+P3SG+GEN$VERB^DB+VERB+PASS+POS+IMP+A2PL":
 *             return "NOUN+A3SG+P3SG+GEN"             # <<<<<<<<<<<<<<
 *             # IKARDI #
 *         elif parseString == "VERB+POS+AOR$VERB^DB+VERB+CAUS+POS":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_A3SG_P3SG_GEN);
    __pyx_r = __pyx_kp_u_NOUN_A3SG_P3SG_GEN;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":572
 *             return "NOUN+A3SG+P3SG"
 *             # YILININ, YOLUNUN, DLNN #
 *         elif parseString == "NOUN+A3SG+P2SG+GEN$NOUN+A3SG+P3SG+GEN$VERB^DB+VERB+PASS+POS+IMP+A2PL":             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+P3SG+GEN"
 *             # IKARDI #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":575
 *             return "NOUN+A3SG+P3SG+GEN"
 *             # IKARDI #
 *         elif parseString == "VERB+POS+AOR$VERB^DB+VERB+CAUS+POS":             # <<<<<<<<<<<<<<
 *             return "VERB+POS+AOR"
 *             # sunucularmz, rakiplerimiz, yaynlarmz #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_VERB_POS_AOR_VERB_DB_VERB_CAUS_P, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 575, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":576
 *             # IKARDI #
 *         elif parseString == "VERB+POS+AOR$VERB^DB+VERB+CAUS+POS":
 *             return "VERB+POS+AOR"             # <<<<<<<<<<<<<<
 *             # sunucularmz, rakiplerimiz, yaynlarmz #
 *         elif parseString == "P1PL+NOM$P1SG+NOM^DB+VERB+ZERO+PRES+A1PL":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_VERB_POS_AOR);
    __pyx_r = __pyx_kp_u_VERB_POS_AOR;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":575
 *             return "NOUN+A3SG+P3SG+GEN"
 *             # IKARDI #
 *         elif parseString == "VERB+POS+AOR$VERB^DB+VERB+CAUS+POS":             # <<<<<<<<<<<<<<
 *             return "VERB+POS+AOR"
 *             # sunucularmz, rakiplerimiz, yaynlarmz #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":578
 *             return "VERB+POS+AOR"
 *             # sunucularmz, rakiplerimiz, yaynlarmz #
 *         elif parseString == "P1PL+NOM$P1SG+NOM^DB+VERB+ZERO+PRES+A1PL":             # <<<<<<<<<<<<<<
 *             return "P1PL+NOM"
 *             # etmitir, artmtr, dnmtr, alnmtr #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_P1PL_NOM_P1SG_NOM_DB_VERB_ZERO_P, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 578, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":579
 *             # sunucularmz, rakiplerimiz, yaynlarmz #
 *         elif parseString == "P1PL+NOM$P1SG+NOM^DB+VERB+ZERO+PRES+A1PL":
 *             return "P1PL+NOM"             # <<<<<<<<<<<<<<
 *             # etmitir, artmtr, dnmtr, alnmtr #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+NARR+A3SG+COP$VERB+POS+NARR+COP+A3SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_P1PL_NOM);
    __pyx_r = __pyx_kp_u_P1PL_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":578
 *             return "VERB+POS+AOR"
 *             # sunucularmz, rakiplerimiz, yaynlarmz #
 *         elif parseString == "P1PL+NOM$P1SG+NOM^DB+VERB+ZERO+PRES+A1PL":             # <<<<<<<<<<<<<<
 *             return "P1PL+NOM"
 *             # etmitir, artmtr, dnmtr, alnmtr #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":581
 *             return "P1PL+NOM"
 *             # etmitir, artmtr, dnmtr, alnmtr #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+NARR+A3SG+COP$VERB+POS+NARR+COP+A3SG":             # <<<<<<<<<<<<<<
 *             return "VERB+POS+NARR+COP+A3SG"
 *             # hazrland, yuvarland, temizlendi #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_5, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 581, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":582
 *             # etmitir, artmtr, dnmtr, alnmtr #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+NARR+A3SG+COP$VERB+POS+NARR+COP+A3SG":
 *             return "VERB+POS+NARR+COP+A3SG"             # <<<<<<<<<<<<<<
 *             # hazrland, yuvarland, temizlendi #
 *         elif parseString == "VERB+REFLEX$VERB^DB+VERB+PASS":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_VERB_POS_NARR_COP_A3SG);
    __pyx_r = __pyx_kp_u_VERB_POS_NARR_COP_A3SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":581
 *             return "P1PL+NOM"
 *             # etmitir, artmtr, dnmtr, alnmtr #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+NARR+A3SG+COP$VERB+POS+NARR+COP+A3SG":             # <<<<<<<<<<<<<<
 *             return "VERB+POS+NARR+COP+A3SG"
 *             # hazrland, yuvarland, temizlendi #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":584
 *             return "VERB+POS+NARR+COP+A3SG"
 *             # hazrland, yuvarland, temizlendi #
 *         elif parseString == "VERB+REFLEX$VERB^DB+VERB+PASS":             # <<<<<<<<<<<<<<
 *             return "VERB^DB+VERB+PASS"
 *             # KARA, EK, SOL, KOCA #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_VERB_REFLEX_VERB_DB_VERB_PASS, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 584, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":585
 *             # hazrland, yuvarland, temizlendi #
 *         elif parseString == "VERB+REFLEX$VERB^DB+VERB+PASS":
 *             return "VERB^DB+VERB+PASS"             # <<<<<<<<<<<<<<
 *             # KARA, EK, SOL, KOCA #
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_VERB_DB_VERB_PASS);
    __pyx_r = __pyx_kp_u_VERB_DB_VERB_PASS;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":584
 *             return "VERB+POS+NARR+COP+A3SG"
 *             # hazrland, yuvarland, temizlendi #
 *         elif parseString == "VERB+REFLEX$VERB^DB+VERB+PASS":             # <<<<<<<<<<<<<<
 *             return "VERB^DB+VERB+PASS"
 *             # KARA, EK, SOL, KOCA #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":587
 *             return "VERB^DB+VERB+PASS"
 *             # KARA, EK, SOL, KOCA #
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             if index > 0:
 *                 if fsmParses[index].getFsmParse(0).isCapitalWord():
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_NOUN_A3SG_PNON_NOM_NOUN_PROP_2, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 587, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":588
 *             # KARA, EK, SOL, KOCA #
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             if index > 0:             # <<<<<<<<<<<<<<
 *                 if fsmParses[index].getFsmParse(0).isCapitalWord():
 *                     return "NOUN+PROP+A3SG+PNON+NOM"
 */
    __pyx_t_3 = PyObject_RichCompare(__pyx_v_index, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 588, __pyx_L1_error)
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 588, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":589
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             if index > 0:
 *                 if fsmParses[index].getFsmParse(0).isCapitalWord():             # <<<<<<<<<<<<<<
 *                     return "NOUN+PROP+A3SG+PNON+NOM"
 *                 return "ADJ"
 */
      if (unlikely(__pyx_v_fsmParses == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 589, __pyx_L1_error)
      }
      __pyx_t_5 = __Pyx_PyObject_GetItem(__pyx_v_fsmParses, __pyx_v_index); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 589, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_getFsmParse); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 589, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
        }
      }
      __pyx_t_4 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_5, __pyx_int_0) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_int_0);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 589, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_isCapitalWord); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 589, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
        }
      }
      __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 589, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 589, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_11) {

        /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":590
 *             if index > 0:
 *                 if fsmParses[index].getFsmParse(0).isCapitalWord():
 *                     return "NOUN+PROP+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *                 return "ADJ"
 *             # YZ #
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_kp_u_NOUN_PROP_A3SG_PNON_NOM);
        __pyx_r = __pyx_kp_u_NOUN_PROP_A3SG_PNON_NOM;
        goto __pyx_L0;

        /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":589
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             if index > 0:
 *                 if fsmParses[index].getFsmParse(0).isCapitalWord():             # <<<<<<<<<<<<<<
 *                     return "NOUN+PROP+A3SG+PNON+NOM"
 *                 return "ADJ"
 */
      }

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":591
 *                 if fsmParses[index].getFsmParse(0).isCapitalWord():
 *                     return "NOUN+PROP+A3SG+PNON+NOM"
 *                 return "ADJ"             # <<<<<<<<<<<<<<
 *             # YZ #
 *         elif parseString == "NOUN+A3SG+PNON+NOM$NUM+CARD$VERB+POS+IMP+A2SG":
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_n_u_ADJ);
      __pyx_r = __pyx_n_u_ADJ;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":588
 *             # KARA, EK, SOL, KOCA #
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             if index > 0:             # <<<<<<<<<<<<<<
 *                 if fsmParses[index].getFsmParse(0).isCapitalWord():
 *                     return "NOUN+PROP+A3SG+PNON+NOM"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":587
 *             return "VERB^DB+VERB+PASS"
 *             # KARA, EK, SOL, KOCA #
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             if index > 0:
 *                 if fsmParses[index].getFsmParse(0).isCapitalWord():
 */
    goto __pyx_L3;
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":593
 *                 return "ADJ"
 *             # YZ #
 *         elif parseString == "NOUN+A3SG+PNON+NOM$NUM+CARD$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNum(index, fsmParses):
 *                 return "NUM+CARD"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_PNON_NOM_NUM_CARD_VERB, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 593, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":594
 *             # YZ #
 *         elif parseString == "NOUN+A3SG+PNON+NOM$NUM+CARD$VERB+POS+IMP+A2SG":
 *             if AutoDisambiguator.isNextWordNum(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "NUM+CARD"
 *             return "NOUN+A3SG+PNON+NOM"
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNum); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 594, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 594, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 594, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 594, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 594, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 594, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":595
 *         elif parseString == "NOUN+A3SG+PNON+NOM$NUM+CARD$VERB+POS+IMP+A2SG":
 *             if AutoDisambiguator.isNextWordNum(index, fsmParses):
 *                 return "NUM+CARD"             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+PNON+NOM"
 *         elif parseString == "ADJ+AGT^DB+ADJ+JUSTLIKE$NOUN+AGT+A3SG+P3SG+NOM$NOUN+AGT^DB+ADJ+ALMOST":
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_NUM_CARD);
      __pyx_r = __pyx_kp_u_NUM_CARD;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":594
 *             # YZ #
 *         elif parseString == "NOUN+A3SG+PNON+NOM$NUM+CARD$VERB+POS+IMP+A2SG":
 *             if AutoDisambiguator.isNextWordNum(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "NUM+CARD"
 *             return "NOUN+A3SG+PNON+NOM"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":596
 *             if AutoDisambiguator.isNextWordNum(index, fsmParses):
 *                 return "NUM+CARD"
 *             return "NOUN+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *         elif parseString == "ADJ+AGT^DB+ADJ+JUSTLIKE$NOUN+AGT+A3SG+P3SG+NOM$NOUN+AGT^DB+ADJ+ALMOST":
 *             return "NOUN+AGT+A3SG+P3SG+NOM"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_A3SG_PNON_NOM);
    __pyx_r = __pyx_kp_u_NOUN_A3SG_PNON_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":593
 *                 return "ADJ"
 *             # YZ #
 *         elif parseString == "NOUN+A3SG+PNON+NOM$NUM+CARD$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNum(index, fsmParses):
 *                 return "NUM+CARD"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":597
 *                 return "NUM+CARD"
 *             return "NOUN+A3SG+PNON+NOM"
 *         elif parseString == "ADJ+AGT^DB+ADJ+JUSTLIKE$NOUN+AGT+A3SG+P3SG+NOM$NOUN+AGT^DB+ADJ+ALMOST":             # <<<<<<<<<<<<<<
 *             return "NOUN+AGT+A3SG+P3SG+NOM"
 *             # artn, dn, ykseliin#
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_AGT_DB_ADJ_JUSTLIKE_NOUN_AGT, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 597, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":598
 *             return "NOUN+A3SG+PNON+NOM"
 *         elif parseString == "ADJ+AGT^DB+ADJ+JUSTLIKE$NOUN+AGT+A3SG+P3SG+NOM$NOUN+AGT^DB+ADJ+ALMOST":
 *             return "NOUN+AGT+A3SG+P3SG+NOM"             # <<<<<<<<<<<<<<
 *             # artn, dn, ykseliin#
 *         elif parseString == "POS^DB+NOUN+INF3+A3SG+P2SG+NOM$POS^DB+NOUN+INF3+A3SG+PNON+GEN$RECIP+POS+IMP+A2PL":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_AGT_A3SG_P3SG_NOM);
    __pyx_r = __pyx_kp_u_NOUN_AGT_A3SG_P3SG_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":597
 *                 return "NUM+CARD"
 *             return "NOUN+A3SG+PNON+NOM"
 *         elif parseString == "ADJ+AGT^DB+ADJ+JUSTLIKE$NOUN+AGT+A3SG+P3SG+NOM$NOUN+AGT^DB+ADJ+ALMOST":             # <<<<<<<<<<<<<<
 *             return "NOUN+AGT+A3SG+P3SG+NOM"
 *             # artn, dn, ykseliin#
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":600
 *             return "NOUN+AGT+A3SG+P3SG+NOM"
 *             # artn, dn, ykseliin#
 *         elif parseString == "POS^DB+NOUN+INF3+A3SG+P2SG+NOM$POS^DB+NOUN+INF3+A3SG+PNON+GEN$RECIP+POS+IMP+A2PL":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "POS^DB+NOUN+INF3+A3SG+P2SG+NOM"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_POS_DB_NOUN_INF3_A3SG_P2SG_NOM_P, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 600, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":601
 *             # artn, dn, ykseliin#
 *         elif parseString == "POS^DB+NOUN+INF3+A3SG+P2SG+NOM$POS^DB+NOUN+INF3+A3SG+PNON+GEN$RECIP+POS+IMP+A2PL":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "POS^DB+NOUN+INF3+A3SG+P2SG+NOM"
 *             return "POS^DB+NOUN+INF3+A3SG+PNON+GEN"
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isAnyWordSecondPerson); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 601, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 601, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 601, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 601, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_10, __pyx_v_correctParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 601, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 601, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":602
 *         elif parseString == "POS^DB+NOUN+INF3+A3SG+P2SG+NOM$POS^DB+NOUN+INF3+A3SG+PNON+GEN$RECIP+POS+IMP+A2PL":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "POS^DB+NOUN+INF3+A3SG+P2SG+NOM"             # <<<<<<<<<<<<<<
 *             return "POS^DB+NOUN+INF3+A3SG+PNON+GEN"
 *             # VARSA #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_POS_DB_NOUN_INF3_A3SG_P2SG_NOM);
      __pyx_r = __pyx_kp_u_POS_DB_NOUN_INF3_A3SG_P2SG_NOM;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":601
 *             # artn, dn, ykseliin#
 *         elif parseString == "POS^DB+NOUN+INF3+A3SG+P2SG+NOM$POS^DB+NOUN+INF3+A3SG+PNON+GEN$RECIP+POS+IMP+A2PL":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "POS^DB+NOUN+INF3+A3SG+P2SG+NOM"
 *             return "POS^DB+NOUN+INF3+A3SG+PNON+GEN"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":603
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "POS^DB+NOUN+INF3+A3SG+P2SG+NOM"
 *             return "POS^DB+NOUN+INF3+A3SG+PNON+GEN"             # <<<<<<<<<<<<<<
 *             # VARSA #
 *         elif parseString == "ADJ^DB+VERB+ZERO+COND$VERB+POS+DESR":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_POS_DB_NOUN_INF3_A3SG_PNON_GEN);
    __pyx_r = __pyx_kp_u_POS_DB_NOUN_INF3_A3SG_PNON_GEN;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":600
 *             return "NOUN+AGT+A3SG+P3SG+NOM"
 *             # artn, dn, ykseliin#
 *         elif parseString == "POS^DB+NOUN+INF3+A3SG+P2SG+NOM$POS^DB+NOUN+INF3+A3SG+PNON+GEN$RECIP+POS+IMP+A2PL":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "POS^DB+NOUN+INF3+A3SG+P2SG+NOM"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":605
 *             return "POS^DB+NOUN+INF3+A3SG+PNON+GEN"
 *             # VARSA #
 *         elif parseString == "ADJ^DB+VERB+ZERO+COND$VERB+POS+DESR":             # <<<<<<<<<<<<<<
 *             return "ADJ^DB+VERB+ZERO+COND"
 *             # DEK #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_DB_VERB_ZERO_COND_VERB_POS_D, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 605, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":606
 *             # VARSA #
 *         elif parseString == "ADJ^DB+VERB+ZERO+COND$VERB+POS+DESR":
 *             return "ADJ^DB+VERB+ZERO+COND"             # <<<<<<<<<<<<<<
 *             # DEK #
 *         elif parseString == "NOUN+A3SG+PNON+NOM$POSTP+PCDAT":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_ADJ_DB_VERB_ZERO_COND);
    __pyx_r = __pyx_kp_u_ADJ_DB_VERB_ZERO_COND;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":605
 *             return "POS^DB+NOUN+INF3+A3SG+PNON+GEN"
 *             # VARSA #
 *         elif parseString == "ADJ^DB+VERB+ZERO+COND$VERB+POS+DESR":             # <<<<<<<<<<<<<<
 *             return "ADJ^DB+VERB+ZERO+COND"
 *             # DEK #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":608
 *             return "ADJ^DB+VERB+ZERO+COND"
 *             # DEK #
 *         elif parseString == "NOUN+A3SG+PNON+NOM$POSTP+PCDAT":             # <<<<<<<<<<<<<<
 *             return "POSTP+PCDAT"
 *             # ALDIK #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_PNON_NOM_POSTP_PCDAT, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 608, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":609
 *             # DEK #
 *         elif parseString == "NOUN+A3SG+PNON+NOM$POSTP+PCDAT":
 *             return "POSTP+PCDAT"             # <<<<<<<<<<<<<<
 *             # ALDIK #
 *         elif parseString == "ADJ^DB+VERB+ZERO+PAST+A1PL$VERB+POS+PAST+A1PL$VERB+POS^DB+ADJ+PASTPART+PNON$VERB+POS^DB" \
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_POSTP_PCDAT);
    __pyx_r = __pyx_kp_u_POSTP_PCDAT;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":608
 *             return "ADJ^DB+VERB+ZERO+COND"
 *             # DEK #
 *         elif parseString == "NOUN+A3SG+PNON+NOM$POSTP+PCDAT":             # <<<<<<<<<<<<<<
 *             return "POSTP+PCDAT"
 *             # ALDIK #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":611
 *             return "POSTP+PCDAT"
 *             # ALDIK #
 *         elif parseString == "ADJ^DB+VERB+ZERO+PAST+A1PL$VERB+POS+PAST+A1PL$VERB+POS^DB+ADJ+PASTPART+PNON$VERB+POS^DB" \             # <<<<<<<<<<<<<<
 *                             "+NOUN+PASTPART+A3SG+PNON+NOM":
 *             return "VERB+POS+PAST+A1PL"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_DB_VERB_ZERO_PAST_A1PL_VERB, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 611, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":613
 *         elif parseString == "ADJ^DB+VERB+ZERO+PAST+A1PL$VERB+POS+PAST+A1PL$VERB+POS^DB+ADJ+PASTPART+PNON$VERB+POS^DB" \
 *                             "+NOUN+PASTPART+A3SG+PNON+NOM":
 *             return "VERB+POS+PAST+A1PL"             # <<<<<<<<<<<<<<
 *             # BRNN, BRNE, BRN, BRNDEN #
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3SG+P2SG$ADJ^DB+NOUN+ZERO+A3SG+P3SG$NUM+CARD^DB+NOUN+ZERO+A3SG" \
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_VERB_POS_PAST_A1PL);
    __pyx_r = __pyx_kp_u_VERB_POS_PAST_A1PL;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":611
 *             return "POSTP+PCDAT"
 *             # ALDIK #
 *         elif parseString == "ADJ^DB+VERB+ZERO+PAST+A1PL$VERB+POS+PAST+A1PL$VERB+POS^DB+ADJ+PASTPART+PNON$VERB+POS^DB" \             # <<<<<<<<<<<<<<
 *                             "+NOUN+PASTPART+A3SG+PNON+NOM":
 *             return "VERB+POS+PAST+A1PL"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":615
 *             return "VERB+POS+PAST+A1PL"
 *             # BRNN, BRNE, BRN, BRNDEN #
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3SG+P2SG$ADJ^DB+NOUN+ZERO+A3SG+P3SG$NUM+CARD^DB+NOUN+ZERO+A3SG" \             # <<<<<<<<<<<<<<
 *                             "+P2SG$NUM+CARD^DB+NOUN+ZERO+A3SG+P3SG":
 *             return "NUM+CARD^DB+NOUN+ZERO+A3SG+P3SG"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_DB_NOUN_ZERO_A3SG_P2SG_ADJ_D_2, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 615, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":617
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3SG+P2SG$ADJ^DB+NOUN+ZERO+A3SG+P3SG$NUM+CARD^DB+NOUN+ZERO+A3SG" \
 *                             "+P2SG$NUM+CARD^DB+NOUN+ZERO+A3SG+P3SG":
 *             return "NUM+CARD^DB+NOUN+ZERO+A3SG+P3SG"             # <<<<<<<<<<<<<<
 *             # ARTIK #
 *         elif parseString == "ADJ$ADV$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NUM_CARD_DB_NOUN_ZERO_A3SG_P3SG);
    __pyx_r = __pyx_kp_u_NUM_CARD_DB_NOUN_ZERO_A3SG_P3SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":615
 *             return "VERB+POS+PAST+A1PL"
 *             # BRNN, BRNE, BRN, BRNDEN #
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3SG+P2SG$ADJ^DB+NOUN+ZERO+A3SG+P3SG$NUM+CARD^DB+NOUN+ZERO+A3SG" \             # <<<<<<<<<<<<<<
 *                             "+P2SG$NUM+CARD^DB+NOUN+ZERO+A3SG+P3SG":
 *             return "NUM+CARD^DB+NOUN+ZERO+A3SG+P3SG"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":619
 *             return "NUM+CARD^DB+NOUN+ZERO+A3SG+P3SG"
 *             # ARTIK #
 *         elif parseString == "ADJ$ADV$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             return "ADV"
 *             # BR #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_ADV_NOUN_A3SG_PNON_NOM_NOUN, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 619, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":620
 *             # ARTIK #
 *         elif parseString == "ADJ$ADV$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM":
 *             return "ADV"             # <<<<<<<<<<<<<<
 *             # BR #
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3SG+P3SG+NOM$ADJ^DB+NOUN+ZERO+A3SG+PNON+ACC$NUM+CARD^DB+NOUN+ZERO+A3SG" \
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ADV);
    __pyx_r = __pyx_n_u_ADV;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":619
 *             return "NUM+CARD^DB+NOUN+ZERO+A3SG+P3SG"
 *             # ARTIK #
 *         elif parseString == "ADJ$ADV$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             return "ADV"
 *             # BR #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":622
 *             return "ADV"
 *             # BR #
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3SG+P3SG+NOM$ADJ^DB+NOUN+ZERO+A3SG+PNON+ACC$NUM+CARD^DB+NOUN+ZERO+A3SG" \             # <<<<<<<<<<<<<<
 *                             "+P3SG+NOM$NUM+CARD^DB+NOUN+ZERO+A3SG+PNON+ACC":
 *             return "NUM+CARD^DB+NOUN+ZERO+A3SG+P3SG+NOM"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_DB_NOUN_ZERO_A3SG_P3SG_NOM_A, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 622, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":624
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3SG+P3SG+NOM$ADJ^DB+NOUN+ZERO+A3SG+PNON+ACC$NUM+CARD^DB+NOUN+ZERO+A3SG" \
 *                             "+P3SG+NOM$NUM+CARD^DB+NOUN+ZERO+A3SG+PNON+ACC":
 *             return "NUM+CARD^DB+NOUN+ZERO+A3SG+P3SG+NOM"             # <<<<<<<<<<<<<<
 *             # DORU #
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM$POSTP+PCDAT":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NUM_CARD_DB_NOUN_ZERO_A3SG_P3SG_2);
    __pyx_r = __pyx_kp_u_NUM_CARD_DB_NOUN_ZERO_A3SG_P3SG_2;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":622
 *             return "ADV"
 *             # BR #
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3SG+P3SG+NOM$ADJ^DB+NOUN+ZERO+A3SG+PNON+ACC$NUM+CARD^DB+NOUN+ZERO+A3SG" \             # <<<<<<<<<<<<<<
 *                             "+P3SG+NOM$NUM+CARD^DB+NOUN+ZERO+A3SG+PNON+ACC":
 *             return "NUM+CARD^DB+NOUN+ZERO+A3SG+P3SG+NOM"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":626
 *             return "NUM+CARD^DB+NOUN+ZERO+A3SG+P3SG+NOM"
 *             # DORU #
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM$POSTP+PCDAT":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.DATIVE):
 *                 return "POSTP+PCDAT"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_NOUN_A3SG_PNON_NOM_NOUN_PROP_3, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 626, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":627
 *             # DORU #
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM$POSTP+PCDAT":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.DATIVE):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCDAT"
 *             return "ADJ"
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_hasPreviousWordTag); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 627, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_MorphologicalTag); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 627, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_DATIVE); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 627, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_index, __pyx_v_correctParses, __pyx_t_5};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 627, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_index, __pyx_v_correctParses, __pyx_t_5};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 627, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    {
      __pyx_t_2 = PyTuple_New(3+__pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 627, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_10, __pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_2, 2+__pyx_t_10, __pyx_t_5);
      __pyx_t_5 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 627, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 627, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":628
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM$POSTP+PCDAT":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.DATIVE):
 *                 return "POSTP+PCDAT"             # <<<<<<<<<<<<<<
 *             return "ADJ"
 *             # demiryollar, havayollar, milletvekilleri #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_POSTP_PCDAT);
      __pyx_r = __pyx_kp_u_POSTP_PCDAT;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":627
 *             # DORU #
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM$POSTP+PCDAT":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.DATIVE):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCDAT"
 *             return "ADJ"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":629
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.DATIVE):
 *                 return "POSTP+PCDAT"
 *             return "ADJ"             # <<<<<<<<<<<<<<
 *             # demiryollar, havayollar, milletvekilleri #
 *         elif parseString == "P3PL+NOM$P3SG+NOM$PNON+ACC":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ADJ);
    __pyx_r = __pyx_n_u_ADJ;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":626
 *             return "NUM+CARD^DB+NOUN+ZERO+A3SG+P3SG+NOM"
 *             # DORU #
 *         elif parseString == "ADJ$NOUN+A3SG+PNON+NOM$NOUN+PROP+A3SG+PNON+NOM$POSTP+PCDAT":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.DATIVE):
 *                 return "POSTP+PCDAT"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":631
 *             return "ADJ"
 *             # demiryollar, havayollar, milletvekilleri #
 *         elif parseString == "P3PL+NOM$P3SG+NOM$PNON+ACC":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "P3PL+NOM"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_P3PL_NOM_P3SG_NOM_PNON_ACC, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 631, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":632
 *             # demiryollar, havayollar, milletvekilleri #
 *         elif parseString == "P3PL+NOM$P3SG+NOM$PNON+ACC":
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "P3PL+NOM"
 *             return "P3SG+NOM"
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isPossessivePlural); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 632, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 632, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 632, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 632, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__pyx_t_2) {
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2); __pyx_t_2 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_10, __pyx_v_correctParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 632, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 632, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":633
 *         elif parseString == "P3PL+NOM$P3SG+NOM$PNON+ACC":
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "P3PL+NOM"             # <<<<<<<<<<<<<<
 *             return "P3SG+NOM"
 *             # GEREK #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_P3PL_NOM);
      __pyx_r = __pyx_kp_u_P3PL_NOM;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":632
 *             # demiryollar, havayollar, milletvekilleri #
 *         elif parseString == "P3PL+NOM$P3SG+NOM$PNON+ACC":
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "P3PL+NOM"
 *             return "P3SG+NOM"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":634
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "P3PL+NOM"
 *             return "P3SG+NOM"             # <<<<<<<<<<<<<<
 *             # GEREK #
 *         elif parseString == "CONJ$NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_P3SG_NOM);
    __pyx_r = __pyx_kp_u_P3SG_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":631
 *             return "ADJ"
 *             # demiryollar, havayollar, milletvekilleri #
 *         elif parseString == "P3PL+NOM$P3SG+NOM$PNON+ACC":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "P3PL+NOM"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":636
 *             return "P3SG+NOM"
 *             # GEREK #
 *         elif parseString == "CONJ$NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.containsTwoNeOrYa(fsmParses, "gerek"):
 *                 return "CONJ"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_CONJ_NOUN_A3SG_PNON_NOM_VERB_POS, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 636, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":637
 *             # GEREK #
 *         elif parseString == "CONJ$NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             if AutoDisambiguator.containsTwoNeOrYa(fsmParses, "gerek"):             # <<<<<<<<<<<<<<
 *                 return "CONJ"
 *             return "NOUN+A3SG+PNON+NOM"
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_containsTwoNeOrYa); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 637, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_fsmParses, __pyx_n_u_gerek};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 637, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_fsmParses, __pyx_n_u_gerek};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 637, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_2 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 637, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_10, __pyx_v_fsmParses);
      __Pyx_INCREF(__pyx_n_u_gerek);
      __Pyx_GIVEREF(__pyx_n_u_gerek);
      PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_10, __pyx_n_u_gerek);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 637, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 637, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":638
 *         elif parseString == "CONJ$NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             if AutoDisambiguator.containsTwoNeOrYa(fsmParses, "gerek"):
 *                 return "CONJ"             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+PNON+NOM"
 *             # bilmediiniz, sevdiiniz, kazandnz #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_n_u_CONJ);
      __pyx_r = __pyx_n_u_CONJ;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":637
 *             # GEREK #
 *         elif parseString == "CONJ$NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             if AutoDisambiguator.containsTwoNeOrYa(fsmParses, "gerek"):             # <<<<<<<<<<<<<<
 *                 return "CONJ"
 *             return "NOUN+A3SG+PNON+NOM"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":639
 *             if AutoDisambiguator.containsTwoNeOrYa(fsmParses, "gerek"):
 *                 return "CONJ"
 *             return "NOUN+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *             # bilmediiniz, sevdiiniz, kazandnz #
 *         elif parseString == "ADJ+PASTPART+P2PL$NOUN+PASTPART+A3SG+P2PL+NOM$NOUN+PASTPART+A3SG+PNON+GEN^DB+VERB+ZERO" \
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_A3SG_PNON_NOM);
    __pyx_r = __pyx_kp_u_NOUN_A3SG_PNON_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":636
 *             return "P3SG+NOM"
 *             # GEREK #
 *         elif parseString == "CONJ$NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.containsTwoNeOrYa(fsmParses, "gerek"):
 *                 return "CONJ"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":641
 *             return "NOUN+A3SG+PNON+NOM"
 *             # bilmediiniz, sevdiiniz, kazandnz #
 *         elif parseString == "ADJ+PASTPART+P2PL$NOUN+PASTPART+A3SG+P2PL+NOM$NOUN+PASTPART+A3SG+PNON+GEN^DB+VERB+ZERO" \             # <<<<<<<<<<<<<<
 *                             "+PRES+A1PL":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_PASTPART_P2PL_NOUN_PASTPART, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 641, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":643
 *         elif parseString == "ADJ+PASTPART+P2PL$NOUN+PASTPART+A3SG+P2PL+NOM$NOUN+PASTPART+A3SG+PNON+GEN^DB+VERB+ZERO" \
 *                             "+PRES+A1PL":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ+PASTPART+P2PL"
 *             return "NOUN+PASTPART+A3SG+P2PL+NOM"
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNounOrAdjective); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 643, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 643, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 643, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 643, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__pyx_t_2) {
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2); __pyx_t_2 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 643, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 643, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":644
 *                             "+PRES+A1PL":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+PASTPART+P2PL"             # <<<<<<<<<<<<<<
 *             return "NOUN+PASTPART+A3SG+P2PL+NOM"
 *             # yapabilecekleri, edebilecekleri, sunabilecekleri #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_ADJ_PASTPART_P2PL);
      __pyx_r = __pyx_kp_u_ADJ_PASTPART_P2PL;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":643
 *         elif parseString == "ADJ+PASTPART+P2PL$NOUN+PASTPART+A3SG+P2PL+NOM$NOUN+PASTPART+A3SG+PNON+GEN^DB+VERB+ZERO" \
 *                             "+PRES+A1PL":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ+PASTPART+P2PL"
 *             return "NOUN+PASTPART+A3SG+P2PL+NOM"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":645
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+PASTPART+P2PL"
 *             return "NOUN+PASTPART+A3SG+P2PL+NOM"             # <<<<<<<<<<<<<<
 *             # yapabilecekleri, edebilecekleri, sunabilecekleri #
 *         elif parseString == "ADJ+FUTPART+P3PL$NOUN+FUTPART+A3PL+P3PL+NOM$NOUN+FUTPART+A3PL+P3SG+NOM$NOUN+FUTPART+A3PL" \
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_PASTPART_A3SG_P2PL_NOM);
    __pyx_r = __pyx_kp_u_NOUN_PASTPART_A3SG_P2PL_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":641
 *             return "NOUN+A3SG+PNON+NOM"
 *             # bilmediiniz, sevdiiniz, kazandnz #
 *         elif parseString == "ADJ+PASTPART+P2PL$NOUN+PASTPART+A3SG+P2PL+NOM$NOUN+PASTPART+A3SG+PNON+GEN^DB+VERB+ZERO" \             # <<<<<<<<<<<<<<
 *                             "+PRES+A1PL":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":647
 *             return "NOUN+PASTPART+A3SG+P2PL+NOM"
 *             # yapabilecekleri, edebilecekleri, sunabilecekleri #
 *         elif parseString == "ADJ+FUTPART+P3PL$NOUN+FUTPART+A3PL+P3PL+NOM$NOUN+FUTPART+A3PL+P3SG+NOM$NOUN+FUTPART+A3PL" \             # <<<<<<<<<<<<<<
 *                             "+PNON+ACC$NOUN+FUTPART+A3SG+P3PL+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_FUTPART_P3PL_NOUN_FUTPART_A3, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 647, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":649
 *         elif parseString == "ADJ+FUTPART+P3PL$NOUN+FUTPART+A3PL+P3PL+NOM$NOUN+FUTPART+A3PL+P3SG+NOM$NOUN+FUTPART+A3PL" \
 *                             "+PNON+ACC$NOUN+FUTPART+A3SG+P3PL+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ+FUTPART+P3PL"
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNounOrAdjective); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 649, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 649, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 649, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_2 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 649, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 649, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 649, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":650
 *                             "+PNON+ACC$NOUN+FUTPART+A3SG+P3PL+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+FUTPART+P3PL"             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "NOUN+FUTPART+A3SG+P3PL+NOM"
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_ADJ_FUTPART_P3PL);
      __pyx_r = __pyx_kp_u_ADJ_FUTPART_P3PL;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":649
 *         elif parseString == "ADJ+FUTPART+P3PL$NOUN+FUTPART+A3PL+P3PL+NOM$NOUN+FUTPART+A3PL+P3SG+NOM$NOUN+FUTPART+A3PL" \
 *                             "+PNON+ACC$NOUN+FUTPART+A3SG+P3PL+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ+FUTPART+P3PL"
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":651
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+FUTPART+P3PL"
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "NOUN+FUTPART+A3SG+P3PL+NOM"
 *             return "NOUN+FUTPART+A3PL+P3SG+NOM"
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isPossessivePlural); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 651, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 651, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 651, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 651, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__pyx_t_2) {
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2); __pyx_t_2 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_10, __pyx_v_correctParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 651, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 651, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":652
 *                 return "ADJ+FUTPART+P3PL"
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "NOUN+FUTPART+A3SG+P3PL+NOM"             # <<<<<<<<<<<<<<
 *             return "NOUN+FUTPART+A3PL+P3SG+NOM"
 *             # KM #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_NOUN_FUTPART_A3SG_P3PL_NOM);
      __pyx_r = __pyx_kp_u_NOUN_FUTPART_A3SG_P3PL_NOM;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":651
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+FUTPART+P3PL"
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "NOUN+FUTPART+A3SG+P3PL+NOM"
 *             return "NOUN+FUTPART+A3PL+P3SG+NOM"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":653
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "NOUN+FUTPART+A3SG+P3PL+NOM"
 *             return "NOUN+FUTPART+A3PL+P3SG+NOM"             # <<<<<<<<<<<<<<
 *             # KM #
 *         elif parseString == "NOUN+PROP$PRON+QUESP":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_FUTPART_A3PL_P3SG_NOM);
    __pyx_r = __pyx_kp_u_NOUN_FUTPART_A3PL_P3SG_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":647
 *             return "NOUN+PASTPART+A3SG+P2PL+NOM"
 *             # yapabilecekleri, edebilecekleri, sunabilecekleri #
 *         elif parseString == "ADJ+FUTPART+P3PL$NOUN+FUTPART+A3PL+P3PL+NOM$NOUN+FUTPART+A3PL+P3SG+NOM$NOUN+FUTPART+A3PL" \             # <<<<<<<<<<<<<<
 *                             "+PNON+ACC$NOUN+FUTPART+A3SG+P3PL+NOM":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":655
 *             return "NOUN+FUTPART+A3PL+P3SG+NOM"
 *             # KM #
 *         elif parseString == "NOUN+PROP$PRON+QUESP":             # <<<<<<<<<<<<<<
 *             if lastWord == "?":
 *                 return "PRON+QUESP"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_PROP_PRON_QUESP, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 655, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":656
 *             # KM #
 *         elif parseString == "NOUN+PROP$PRON+QUESP":
 *             if lastWord == "?":             # <<<<<<<<<<<<<<
 *                 return "PRON+QUESP"
 *             return "NOUN+PROP"
 */
    __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_lastWord, __pyx_kp_u_, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 656, __pyx_L1_error)
    __pyx_t_11 = (__pyx_t_9 != 0);
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":657
 *         elif parseString == "NOUN+PROP$PRON+QUESP":
 *             if lastWord == "?":
 *                 return "PRON+QUESP"             # <<<<<<<<<<<<<<
 *             return "NOUN+PROP"
 *             # ALINDI #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_PRON_QUESP);
      __pyx_r = __pyx_kp_u_PRON_QUESP;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":656
 *             # KM #
 *         elif parseString == "NOUN+PROP$PRON+QUESP":
 *             if lastWord == "?":             # <<<<<<<<<<<<<<
 *                 return "PRON+QUESP"
 *             return "NOUN+PROP"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":658
 *             if lastWord == "?":
 *                 return "PRON+QUESP"
 *             return "NOUN+PROP"             # <<<<<<<<<<<<<<
 *             # ALINDI #
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3SG+P2SG+NOM^DB+VERB+ZERO$ADJ^DB+NOUN+ZERO+A3SG+PNON+GEN^DB+VERB" \
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_PROP);
    __pyx_r = __pyx_kp_u_NOUN_PROP;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":655
 *             return "NOUN+FUTPART+A3PL+P3SG+NOM"
 *             # KM #
 *         elif parseString == "NOUN+PROP$PRON+QUESP":             # <<<<<<<<<<<<<<
 *             if lastWord == "?":
 *                 return "PRON+QUESP"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":660
 *             return "NOUN+PROP"
 *             # ALINDI #
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3SG+P2SG+NOM^DB+VERB+ZERO$ADJ^DB+NOUN+ZERO+A3SG+PNON+GEN^DB+VERB" \             # <<<<<<<<<<<<<<
 *                             "+ZERO$VERB^DB+VERB+PASS+POS":
 *             return "VERB^DB+VERB+PASS+POS"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_DB_NOUN_ZERO_A3SG_P2SG_NOM_D, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 660, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":662
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3SG+P2SG+NOM^DB+VERB+ZERO$ADJ^DB+NOUN+ZERO+A3SG+PNON+GEN^DB+VERB" \
 *                             "+ZERO$VERB^DB+VERB+PASS+POS":
 *             return "VERB^DB+VERB+PASS+POS"             # <<<<<<<<<<<<<<
 *             # KIZIM #
 *         elif parseString == "ADJ^DB+VERB+ZERO+PRES+A1SG$NOUN+A3SG+P1SG+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A1SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_VERB_DB_VERB_PASS_POS);
    __pyx_r = __pyx_kp_u_VERB_DB_VERB_PASS_POS;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":660
 *             return "NOUN+PROP"
 *             # ALINDI #
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3SG+P2SG+NOM^DB+VERB+ZERO$ADJ^DB+NOUN+ZERO+A3SG+PNON+GEN^DB+VERB" \             # <<<<<<<<<<<<<<
 *                             "+ZERO$VERB^DB+VERB+PASS+POS":
 *             return "VERB^DB+VERB+PASS+POS"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":664
 *             return "VERB^DB+VERB+PASS+POS"
 *             # KIZIM #
 *         elif parseString == "ADJ^DB+VERB+ZERO+PRES+A1SG$NOUN+A3SG+P1SG+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A1SG":             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+P1SG+NOM"
 *             # etmeliydi, yaratmalyd #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_DB_VERB_ZERO_PRES_A1SG_NOUN, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 664, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":665
 *             # KIZIM #
 *         elif parseString == "ADJ^DB+VERB+ZERO+PRES+A1SG$NOUN+A3SG+P1SG+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A1SG":
 *             return "NOUN+A3SG+P1SG+NOM"             # <<<<<<<<<<<<<<
 *             # etmeliydi, yaratmalyd #
 *         elif parseString == "POS+NECES$POS^DB+NOUN+INF2+A3SG+PNON+NOM^DB+ADJ+WITH^DB+VERB+ZERO":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_A3SG_P1SG_NOM);
    __pyx_r = __pyx_kp_u_NOUN_A3SG_P1SG_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":664
 *             return "VERB^DB+VERB+PASS+POS"
 *             # KIZIM #
 *         elif parseString == "ADJ^DB+VERB+ZERO+PRES+A1SG$NOUN+A3SG+P1SG+NOM$NOUN+A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A1SG":             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+P1SG+NOM"
 *             # etmeliydi, yaratmalyd #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":667
 *             return "NOUN+A3SG+P1SG+NOM"
 *             # etmeliydi, yaratmalyd #
 *         elif parseString == "POS+NECES$POS^DB+NOUN+INF2+A3SG+PNON+NOM^DB+ADJ+WITH^DB+VERB+ZERO":             # <<<<<<<<<<<<<<
 *             return "POS+NECES"
 *             # HERKESN #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_POS_NECES_POS_DB_NOUN_INF2_A3SG_2, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 667, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":668
 *             # etmeliydi, yaratmalyd #
 *         elif parseString == "POS+NECES$POS^DB+NOUN+INF2+A3SG+PNON+NOM^DB+ADJ+WITH^DB+VERB+ZERO":
 *             return "POS+NECES"             # <<<<<<<<<<<<<<
 *             # HERKESN #
 *         elif parseString == "NOUN+A3SG+P2SG+NOM$NOUN+A3SG+PNON+GEN$PRON+QUANTP+A3PL+P3PL+GEN":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_POS_NECES);
    __pyx_r = __pyx_kp_u_POS_NECES;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":667
 *             return "NOUN+A3SG+P1SG+NOM"
 *             # etmeliydi, yaratmalyd #
 *         elif parseString == "POS+NECES$POS^DB+NOUN+INF2+A3SG+PNON+NOM^DB+ADJ+WITH^DB+VERB+ZERO":             # <<<<<<<<<<<<<<
 *             return "POS+NECES"
 *             # HERKESN #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":670
 *             return "POS+NECES"
 *             # HERKESN #
 *         elif parseString == "NOUN+A3SG+P2SG+NOM$NOUN+A3SG+PNON+GEN$PRON+QUANTP+A3PL+P3PL+GEN":             # <<<<<<<<<<<<<<
 *             return "PRON+QUANTP+A3PL+P3PL+GEN"
 *         elif parseString == "ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2SG$ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+PNON$NOUN+ZERO+A3SG" \
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_P2SG_NOM_NOUN_A3SG_PNO_3, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 670, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":671
 *             # HERKESN #
 *         elif parseString == "NOUN+A3SG+P2SG+NOM$NOUN+A3SG+PNON+GEN$PRON+QUANTP+A3PL+P3PL+GEN":
 *             return "PRON+QUANTP+A3PL+P3PL+GEN"             # <<<<<<<<<<<<<<
 *         elif parseString == "ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2SG$ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+PNON$NOUN+ZERO+A3SG" \
 *                             "+P3SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_PRON_QUANTP_A3PL_P3PL_GEN);
    __pyx_r = __pyx_kp_u_PRON_QUANTP_A3PL_P3PL_GEN;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":670
 *             return "POS+NECES"
 *             # HERKESN #
 *         elif parseString == "NOUN+A3SG+P2SG+NOM$NOUN+A3SG+PNON+GEN$PRON+QUANTP+A3PL+P3PL+GEN":             # <<<<<<<<<<<<<<
 *             return "PRON+QUANTP+A3PL+P3PL+GEN"
 *         elif parseString == "ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2SG$ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+PNON$NOUN+ZERO+A3SG" \
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":672
 *         elif parseString == "NOUN+A3SG+P2SG+NOM$NOUN+A3SG+PNON+GEN$PRON+QUANTP+A3PL+P3PL+GEN":
 *             return "PRON+QUANTP+A3PL+P3PL+GEN"
 *         elif parseString == "ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2SG$ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+PNON$NOUN+ZERO+A3SG" \             # <<<<<<<<<<<<<<
 *                             "+P3SG":
 *             return "NOUN+ZERO+A3SG+P3SG"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_JUSTLIKE_DB_NOUN_ZERO_A3SG_P_2, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 672, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":674
 *         elif parseString == "ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2SG$ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+PNON$NOUN+ZERO+A3SG" \
 *                             "+P3SG":
 *             return "NOUN+ZERO+A3SG+P3SG"             # <<<<<<<<<<<<<<
 *             # milyarlk, milyonluk, belik, ikilik #
 *         elif parseString == "NESS+A3SG+PNON+NOM$ZERO+A3SG+PNON+NOM^DB+ADJ+FITFOR":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_ZERO_A3SG_P3SG);
    __pyx_r = __pyx_kp_u_NOUN_ZERO_A3SG_P3SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":672
 *         elif parseString == "NOUN+A3SG+P2SG+NOM$NOUN+A3SG+PNON+GEN$PRON+QUANTP+A3PL+P3PL+GEN":
 *             return "PRON+QUANTP+A3PL+P3PL+GEN"
 *         elif parseString == "ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+P2SG$ADJ+JUSTLIKE^DB+NOUN+ZERO+A3SG+PNON$NOUN+ZERO+A3SG" \             # <<<<<<<<<<<<<<
 *                             "+P3SG":
 *             return "NOUN+ZERO+A3SG+P3SG"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":676
 *             return "NOUN+ZERO+A3SG+P3SG"
 *             # milyarlk, milyonluk, belik, ikilik #
 *         elif parseString == "NESS+A3SG+PNON+NOM$ZERO+A3SG+PNON+NOM^DB+ADJ+FITFOR":             # <<<<<<<<<<<<<<
 *             return "ZERO+A3SG+PNON+NOM^DB+ADJ+FITFOR"
 *             # alnmamaktadr, koymamaktadr #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NESS_A3SG_PNON_NOM_ZERO_A3SG_PNO, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 676, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":677
 *             # milyarlk, milyonluk, belik, ikilik #
 *         elif parseString == "NESS+A3SG+PNON+NOM$ZERO+A3SG+PNON+NOM^DB+ADJ+FITFOR":
 *             return "ZERO+A3SG+PNON+NOM^DB+ADJ+FITFOR"             # <<<<<<<<<<<<<<
 *             # alnmamaktadr, koymamaktadr #
 *         elif parseString == "NEG+PROG2$NEG^DB+NOUN+INF+A3SG+PNON+LOC^DB+VERB+ZERO+PRES":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_ZERO_A3SG_PNON_NOM_DB_ADJ_FITFOR);
    __pyx_r = __pyx_kp_u_ZERO_A3SG_PNON_NOM_DB_ADJ_FITFOR;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":676
 *             return "NOUN+ZERO+A3SG+P3SG"
 *             # milyarlk, milyonluk, belik, ikilik #
 *         elif parseString == "NESS+A3SG+PNON+NOM$ZERO+A3SG+PNON+NOM^DB+ADJ+FITFOR":             # <<<<<<<<<<<<<<
 *             return "ZERO+A3SG+PNON+NOM^DB+ADJ+FITFOR"
 *             # alnmamaktadr, koymamaktadr #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":679
 *             return "ZERO+A3SG+PNON+NOM^DB+ADJ+FITFOR"
 *             # alnmamaktadr, koymamaktadr #
 *         elif parseString == "NEG+PROG2$NEG^DB+NOUN+INF+A3SG+PNON+LOC^DB+VERB+ZERO+PRES":             # <<<<<<<<<<<<<<
 *             return "NEG+PROG2"
 *             # HEPMZ #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NEG_PROG2_NEG_DB_NOUN_INF_A3SG_P, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 679, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":680
 *             # alnmamaktadr, koymamaktadr #
 *         elif parseString == "NEG+PROG2$NEG^DB+NOUN+INF+A3SG+PNON+LOC^DB+VERB+ZERO+PRES":
 *             return "NEG+PROG2"             # <<<<<<<<<<<<<<
 *             # HEPMZ #
 *         elif parseString == "A1PL+P1PL+NOM$A3SG+P3SG+GEN^DB+VERB+ZERO+PRES+A1PL":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NEG_PROG2);
    __pyx_r = __pyx_kp_u_NEG_PROG2;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":679
 *             return "ZERO+A3SG+PNON+NOM^DB+ADJ+FITFOR"
 *             # alnmamaktadr, koymamaktadr #
 *         elif parseString == "NEG+PROG2$NEG^DB+NOUN+INF+A3SG+PNON+LOC^DB+VERB+ZERO+PRES":             # <<<<<<<<<<<<<<
 *             return "NEG+PROG2"
 *             # HEPMZ #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":682
 *             return "NEG+PROG2"
 *             # HEPMZ #
 *         elif parseString == "A1PL+P1PL+NOM$A3SG+P3SG+GEN^DB+VERB+ZERO+PRES+A1PL":             # <<<<<<<<<<<<<<
 *             return "A1PL+P1PL+NOM"
 *             # KMSENN #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_A1PL_P1PL_NOM_A3SG_P3SG_GEN_DB_V, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 682, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":683
 *             # HEPMZ #
 *         elif parseString == "A1PL+P1PL+NOM$A3SG+P3SG+GEN^DB+VERB+ZERO+PRES+A1PL":
 *             return "A1PL+P1PL+NOM"             # <<<<<<<<<<<<<<
 *             # KMSENN #
 *         elif parseString == "NOUN+A3SG+P2SG$NOUN+A3SG+PNON$PRON+QUANTP+A3SG+P3SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_A1PL_P1PL_NOM);
    __pyx_r = __pyx_kp_u_A1PL_P1PL_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":682
 *             return "NEG+PROG2"
 *             # HEPMZ #
 *         elif parseString == "A1PL+P1PL+NOM$A3SG+P3SG+GEN^DB+VERB+ZERO+PRES+A1PL":             # <<<<<<<<<<<<<<
 *             return "A1PL+P1PL+NOM"
 *             # KMSENN #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":685
 *             return "A1PL+P1PL+NOM"
 *             # KMSENN #
 *         elif parseString == "NOUN+A3SG+P2SG$NOUN+A3SG+PNON$PRON+QUANTP+A3SG+P3SG":             # <<<<<<<<<<<<<<
 *             return "PRON+QUANTP+A3SG+P3SG"
 *             # GEM, ALMI, VARMI #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_P2SG_NOUN_A3SG_PNON_PR, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 685, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":686
 *             # KMSENN #
 *         elif parseString == "NOUN+A3SG+P2SG$NOUN+A3SG+PNON$PRON+QUANTP+A3SG+P3SG":
 *             return "PRON+QUANTP+A3SG+P3SG"             # <<<<<<<<<<<<<<
 *             # GEM, ALMI, VARMI #
 *         elif parseString == "ADJ^DB+VERB+ZERO+NARR+A3SG$VERB+POS+NARR+A3SG$VERB+POS+NARR^DB+ADJ+ZERO":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_PRON_QUANTP_A3SG_P3SG);
    __pyx_r = __pyx_kp_u_PRON_QUANTP_A3SG_P3SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":685
 *             return "A1PL+P1PL+NOM"
 *             # KMSENN #
 *         elif parseString == "NOUN+A3SG+P2SG$NOUN+A3SG+PNON$PRON+QUANTP+A3SG+P3SG":             # <<<<<<<<<<<<<<
 *             return "PRON+QUANTP+A3SG+P3SG"
 *             # GEM, ALMI, VARMI #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":688
 *             return "PRON+QUANTP+A3SG+P3SG"
 *             # GEM, ALMI, VARMI #
 *         elif parseString == "ADJ^DB+VERB+ZERO+NARR+A3SG$VERB+POS+NARR+A3SG$VERB+POS+NARR^DB+ADJ+ZERO":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "VERB+POS+NARR^DB+ADJ+ZERO"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_DB_VERB_ZERO_NARR_A3SG_VERB, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 688, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":689
 *             # GEM, ALMI, VARMI #
 *         elif parseString == "ADJ^DB+VERB+ZERO+NARR+A3SG$VERB+POS+NARR+A3SG$VERB+POS+NARR^DB+ADJ+ZERO":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "VERB+POS+NARR^DB+ADJ+ZERO"
 *             return "VERB+POS+NARR+A3SG"
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNounOrAdjective); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 689, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 689, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 689, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_2 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 689, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 689, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 689, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":690
 *         elif parseString == "ADJ^DB+VERB+ZERO+NARR+A3SG$VERB+POS+NARR+A3SG$VERB+POS+NARR^DB+ADJ+ZERO":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "VERB+POS+NARR^DB+ADJ+ZERO"             # <<<<<<<<<<<<<<
 *             return "VERB+POS+NARR+A3SG"
 *             # yapacanz, konuabileceiniz, olacanz #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_VERB_POS_NARR_DB_ADJ_ZERO);
      __pyx_r = __pyx_kp_u_VERB_POS_NARR_DB_ADJ_ZERO;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":689
 *             # GEM, ALMI, VARMI #
 *         elif parseString == "ADJ^DB+VERB+ZERO+NARR+A3SG$VERB+POS+NARR+A3SG$VERB+POS+NARR^DB+ADJ+ZERO":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "VERB+POS+NARR^DB+ADJ+ZERO"
 *             return "VERB+POS+NARR+A3SG"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":691
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "VERB+POS+NARR^DB+ADJ+ZERO"
 *             return "VERB+POS+NARR+A3SG"             # <<<<<<<<<<<<<<
 *             # yapacanz, konuabileceiniz, olacanz #
 *         elif parseString == "ADJ+FUTPART+P2PL$NOUN+FUTPART+A3SG+P2PL+NOM$NOUN+FUTPART+A3SG+PNON+GEN^DB+VERB+ZERO+PRES" \
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_VERB_POS_NARR_A3SG);
    __pyx_r = __pyx_kp_u_VERB_POS_NARR_A3SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":688
 *             return "PRON+QUANTP+A3SG+P3SG"
 *             # GEM, ALMI, VARMI #
 *         elif parseString == "ADJ^DB+VERB+ZERO+NARR+A3SG$VERB+POS+NARR+A3SG$VERB+POS+NARR^DB+ADJ+ZERO":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "VERB+POS+NARR^DB+ADJ+ZERO"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":693
 *             return "VERB+POS+NARR+A3SG"
 *             # yapacanz, konuabileceiniz, olacanz #
 *         elif parseString == "ADJ+FUTPART+P2PL$NOUN+FUTPART+A3SG+P2PL+NOM$NOUN+FUTPART+A3SG+PNON+GEN^DB+VERB+ZERO+PRES" \             # <<<<<<<<<<<<<<
 *                             "+A1PL":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_FUTPART_P2PL_NOUN_FUTPART_A3, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 693, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":695
 *         elif parseString == "ADJ+FUTPART+P2PL$NOUN+FUTPART+A3SG+P2PL+NOM$NOUN+FUTPART+A3SG+PNON+GEN^DB+VERB+ZERO+PRES" \
 *                             "+A1PL":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ+FUTPART+P2PL"
 *             return "NOUN+FUTPART+A3SG+P2PL+NOM"
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNounOrAdjective); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 695, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 695, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 695, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 695, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__pyx_t_2) {
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2); __pyx_t_2 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 695, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 695, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":696
 *                             "+A1PL":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+FUTPART+P2PL"             # <<<<<<<<<<<<<<
 *             return "NOUN+FUTPART+A3SG+P2PL+NOM"
 *             # YILINA, DLNE, YOLUNA #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_ADJ_FUTPART_P2PL);
      __pyx_r = __pyx_kp_u_ADJ_FUTPART_P2PL;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":695
 *         elif parseString == "ADJ+FUTPART+P2PL$NOUN+FUTPART+A3SG+P2PL+NOM$NOUN+FUTPART+A3SG+PNON+GEN^DB+VERB+ZERO+PRES" \
 *                             "+A1PL":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ+FUTPART+P2PL"
 *             return "NOUN+FUTPART+A3SG+P2PL+NOM"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":697
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+FUTPART+P2PL"
 *             return "NOUN+FUTPART+A3SG+P2PL+NOM"             # <<<<<<<<<<<<<<
 *             # YILINA, DLNE, YOLUNA #
 *         elif parseString == "NOUN+A3SG+P2SG+DAT$NOUN+A3SG+P3SG+DAT$VERB^DB+VERB+PASS+POS+OPT+A3SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_FUTPART_A3SG_P2PL_NOM);
    __pyx_r = __pyx_kp_u_NOUN_FUTPART_A3SG_P2PL_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":693
 *             return "VERB+POS+NARR+A3SG"
 *             # yapacanz, konuabileceiniz, olacanz #
 *         elif parseString == "ADJ+FUTPART+P2PL$NOUN+FUTPART+A3SG+P2PL+NOM$NOUN+FUTPART+A3SG+PNON+GEN^DB+VERB+ZERO+PRES" \             # <<<<<<<<<<<<<<
 *                             "+A1PL":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":699
 *             return "NOUN+FUTPART+A3SG+P2PL+NOM"
 *             # YILINA, DLNE, YOLUNA #
 *         elif parseString == "NOUN+A3SG+P2SG+DAT$NOUN+A3SG+P3SG+DAT$VERB^DB+VERB+PASS+POS+OPT+A3SG":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "NOUN+A3SG+P2SG+DAT"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_P2SG_DAT_NOUN_A3SG_P3S, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 699, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":700
 *             # YILINA, DLNE, YOLUNA #
 *         elif parseString == "NOUN+A3SG+P2SG+DAT$NOUN+A3SG+P3SG+DAT$VERB^DB+VERB+PASS+POS+OPT+A3SG":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "NOUN+A3SG+P2SG+DAT"
 *             return "NOUN+A3SG+P3SG+DAT"
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isAnyWordSecondPerson); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 700, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 700, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 700, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_2 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 700, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_10, __pyx_v_correctParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 700, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 700, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":701
 *         elif parseString == "NOUN+A3SG+P2SG+DAT$NOUN+A3SG+P3SG+DAT$VERB^DB+VERB+PASS+POS+OPT+A3SG":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "NOUN+A3SG+P2SG+DAT"             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+P3SG+DAT"
 *             # MSN, MYD, MSNZ #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_NOUN_A3SG_P2SG_DAT);
      __pyx_r = __pyx_kp_u_NOUN_A3SG_P2SG_DAT;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":700
 *             # YILINA, DLNE, YOLUNA #
 *         elif parseString == "NOUN+A3SG+P2SG+DAT$NOUN+A3SG+P3SG+DAT$VERB^DB+VERB+PASS+POS+OPT+A3SG":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "NOUN+A3SG+P2SG+DAT"
 *             return "NOUN+A3SG+P3SG+DAT"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":702
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "NOUN+A3SG+P2SG+DAT"
 *             return "NOUN+A3SG+P3SG+DAT"             # <<<<<<<<<<<<<<
 *             # MSN, MYD, MSNZ #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO$QUES":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_A3SG_P3SG_DAT);
    __pyx_r = __pyx_kp_u_NOUN_A3SG_P3SG_DAT;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":699
 *             return "NOUN+FUTPART+A3SG+P2PL+NOM"
 *             # YILINA, DLNE, YOLUNA #
 *         elif parseString == "NOUN+A3SG+P2SG+DAT$NOUN+A3SG+P3SG+DAT$VERB^DB+VERB+PASS+POS+OPT+A3SG":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "NOUN+A3SG+P2SG+DAT"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":704
 *             return "NOUN+A3SG+P3SG+DAT"
 *             # MSN, MYD, MSNZ #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO$QUES":             # <<<<<<<<<<<<<<
 *             return "QUES"
 *             # ATAKLAR, GLER, ESASLAR #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_6, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 704, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":705
 *             # MSN, MYD, MSNZ #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO$QUES":
 *             return "QUES"             # <<<<<<<<<<<<<<
 *             # ATAKLAR, GLER, ESASLAR #
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3PL+PNON+NOM$ADJ^DB+VERB+ZERO+PRES+A3PL$NOUN+A3PL+PNON+NOM$NOUN+A3SG" \
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_QUES);
    __pyx_r = __pyx_n_u_QUES;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":704
 *             return "NOUN+A3SG+P3SG+DAT"
 *             # MSN, MYD, MSNZ #
 *         elif parseString == "NOUN+A3SG+PNON+NOM^DB+VERB+ZERO$QUES":             # <<<<<<<<<<<<<<
 *             return "QUES"
 *             # ATAKLAR, GLER, ESASLAR #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":707
 *             return "QUES"
 *             # ATAKLAR, GLER, ESASLAR #
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3PL+PNON+NOM$ADJ^DB+VERB+ZERO+PRES+A3PL$NOUN+A3PL+PNON+NOM$NOUN+A3SG" \             # <<<<<<<<<<<<<<
 *                             "+PNON+NOM^DB+VERB+ZERO+PRES+A3PL":
 *             return "NOUN+A3PL+PNON+NOM"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_DB_NOUN_ZERO_A3PL_PNON_NOM_A, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 707, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":709
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3PL+PNON+NOM$ADJ^DB+VERB+ZERO+PRES+A3PL$NOUN+A3PL+PNON+NOM$NOUN+A3SG" \
 *                             "+PNON+NOM^DB+VERB+ZERO+PRES+A3PL":
 *             return "NOUN+A3PL+PNON+NOM"             # <<<<<<<<<<<<<<
 *         elif parseString == "A3PL+P3SG$A3SG+P3PL$PROP+A3PL+P3PL":
 *             return "PROP+A3PL+P3PL"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_A3PL_PNON_NOM);
    __pyx_r = __pyx_kp_u_NOUN_A3PL_PNON_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":707
 *             return "QUES"
 *             # ATAKLAR, GLER, ESASLAR #
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3PL+PNON+NOM$ADJ^DB+VERB+ZERO+PRES+A3PL$NOUN+A3PL+PNON+NOM$NOUN+A3SG" \             # <<<<<<<<<<<<<<
 *                             "+PNON+NOM^DB+VERB+ZERO+PRES+A3PL":
 *             return "NOUN+A3PL+PNON+NOM"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":710
 *                             "+PNON+NOM^DB+VERB+ZERO+PRES+A3PL":
 *             return "NOUN+A3PL+PNON+NOM"
 *         elif parseString == "A3PL+P3SG$A3SG+P3PL$PROP+A3PL+P3PL":             # <<<<<<<<<<<<<<
 *             return "PROP+A3PL+P3PL"
 *             # pilotunuz, suunuz, haberiniz #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_A3PL_P3SG_A3SG_P3PL_PROP_A3PL_P3, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 710, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":711
 *             return "NOUN+A3PL+PNON+NOM"
 *         elif parseString == "A3PL+P3SG$A3SG+P3PL$PROP+A3PL+P3PL":
 *             return "PROP+A3PL+P3PL"             # <<<<<<<<<<<<<<
 *             # pilotunuz, suunuz, haberiniz #
 *         elif parseString == "P2PL+NOM$PNON+GEN^DB+VERB+ZERO+PRES+A1PL":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_PROP_A3PL_P3PL);
    __pyx_r = __pyx_kp_u_PROP_A3PL_P3PL;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":710
 *                             "+PNON+NOM^DB+VERB+ZERO+PRES+A3PL":
 *             return "NOUN+A3PL+PNON+NOM"
 *         elif parseString == "A3PL+P3SG$A3SG+P3PL$PROP+A3PL+P3PL":             # <<<<<<<<<<<<<<
 *             return "PROP+A3PL+P3PL"
 *             # pilotunuz, suunuz, haberiniz #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":713
 *             return "PROP+A3PL+P3PL"
 *             # pilotunuz, suunuz, haberiniz #
 *         elif parseString == "P2PL+NOM$PNON+GEN^DB+VERB+ZERO+PRES+A1PL":             # <<<<<<<<<<<<<<
 *             return "P2PL+NOM"
 *             # yllarca, aylarca, dmanca #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_P2PL_NOM_PNON_GEN_DB_VERB_ZERO_P, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 713, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":714
 *             # pilotunuz, suunuz, haberiniz #
 *         elif parseString == "P2PL+NOM$PNON+GEN^DB+VERB+ZERO+PRES+A1PL":
 *             return "P2PL+NOM"             # <<<<<<<<<<<<<<
 *             # yllarca, aylarca, dmanca #
 *         elif parseString == "ADJ+ASIF$ADV+LY":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_P2PL_NOM);
    __pyx_r = __pyx_kp_u_P2PL_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":713
 *             return "PROP+A3PL+P3PL"
 *             # pilotunuz, suunuz, haberiniz #
 *         elif parseString == "P2PL+NOM$PNON+GEN^DB+VERB+ZERO+PRES+A1PL":             # <<<<<<<<<<<<<<
 *             return "P2PL+NOM"
 *             # yllarca, aylarca, dmanca #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":716
 *             return "P2PL+NOM"
 *             # yllarca, aylarca, dmanca #
 *         elif parseString == "ADJ+ASIF$ADV+LY":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+ASIF"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_ASIF_ADV_LY, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 716, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":717
 *             # yllarca, aylarca, dmanca #
 *         elif parseString == "ADJ+ASIF$ADV+LY":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ+ASIF"
 *             return "ADV+LY"
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNounOrAdjective); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 717, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 717, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 717, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 717, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__pyx_t_2) {
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2); __pyx_t_2 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 717, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 717, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":718
 *         elif parseString == "ADJ+ASIF$ADV+LY":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+ASIF"             # <<<<<<<<<<<<<<
 *             return "ADV+LY"
 *             # gereki, alc #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_ADJ_ASIF);
      __pyx_r = __pyx_kp_u_ADJ_ASIF;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":717
 *             # yllarca, aylarca, dmanca #
 *         elif parseString == "ADJ+ASIF$ADV+LY":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ+ASIF"
 *             return "ADV+LY"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":719
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+ASIF"
 *             return "ADV+LY"             # <<<<<<<<<<<<<<
 *             # gereki, alc #
 *         elif parseString == "ADJ^DB+NOUN+AGT+A3SG+PNON+NOM$NOUN+A3SG+PNON+NOM^DB+ADJ+AGT":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_ADV_LY);
    __pyx_r = __pyx_kp_u_ADV_LY;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":716
 *             return "P2PL+NOM"
 *             # yllarca, aylarca, dmanca #
 *         elif parseString == "ADJ+ASIF$ADV+LY":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ+ASIF"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":721
 *             return "ADV+LY"
 *             # gereki, alc #
 *         elif parseString == "ADJ^DB+NOUN+AGT+A3SG+PNON+NOM$NOUN+A3SG+PNON+NOM^DB+ADJ+AGT":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "NOUN+A3SG+PNON+NOM^DB+ADJ+AGT"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_DB_NOUN_AGT_A3SG_PNON_NOM_NO, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 721, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":722
 *             # gereki, alc #
 *         elif parseString == "ADJ^DB+NOUN+AGT+A3SG+PNON+NOM$NOUN+A3SG+PNON+NOM^DB+ADJ+AGT":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "NOUN+A3SG+PNON+NOM^DB+ADJ+AGT"
 *             return "ADJ^DB+NOUN+AGT+A3SG+PNON+NOM"
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNounOrAdjective); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 722, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 722, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 722, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_2 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 722, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 722, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 722, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_11) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":723
 *         elif parseString == "ADJ^DB+NOUN+AGT+A3SG+PNON+NOM$NOUN+A3SG+PNON+NOM^DB+ADJ+AGT":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "NOUN+A3SG+PNON+NOM^DB+ADJ+AGT"             # <<<<<<<<<<<<<<
 *             return "ADJ^DB+NOUN+AGT+A3SG+PNON+NOM"
 *             # havayollarna, gzyalarna #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_ADJ_AGT);
      __pyx_r = __pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_ADJ_AGT;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":722
 *             # gereki, alc #
 *         elif parseString == "ADJ^DB+NOUN+AGT+A3SG+PNON+NOM$NOUN+A3SG+PNON+NOM^DB+ADJ+AGT":
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "NOUN+A3SG+PNON+NOM^DB+ADJ+AGT"
 *             return "ADJ^DB+NOUN+AGT+A3SG+PNON+NOM"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":724
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "NOUN+A3SG+PNON+NOM^DB+ADJ+AGT"
 *             return "ADJ^DB+NOUN+AGT+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *             # havayollarna, gzyalarna #
 *         elif parseString == "P2SG$P3PL$P3SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_ADJ_DB_NOUN_AGT_A3SG_PNON_NOM);
    __pyx_r = __pyx_kp_u_ADJ_DB_NOUN_AGT_A3SG_PNON_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":721
 *             return "ADV+LY"
 *             # gereki, alc #
 *         elif parseString == "ADJ^DB+NOUN+AGT+A3SG+PNON+NOM$NOUN+A3SG+PNON+NOM^DB+ADJ+AGT":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "NOUN+A3SG+PNON+NOM^DB+ADJ+AGT"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":726
 *             return "ADJ^DB+NOUN+AGT+A3SG+PNON+NOM"
 *             # havayollarna, gzyalarna #
 *         elif parseString == "P2SG$P3PL$P3SG":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "P2SG"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_P2SG_P3PL_P3SG, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 726, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":727
 *             # havayollarna, gzyalarna #
 *         elif parseString == "P2SG$P3PL$P3SG":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "P2SG"
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isAnyWordSecondPerson); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 727, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 727, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 727, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 727, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__pyx_t_2) {
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2); __pyx_t_2 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_10, __pyx_v_correctParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 727, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 727, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":728
 *         elif parseString == "P2SG$P3PL$P3SG":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "P2SG"             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "P3PL"
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_n_u_P2SG);
      __pyx_r = __pyx_n_u_P2SG;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":727
 *             # havayollarna, gzyalarna #
 *         elif parseString == "P2SG$P3PL$P3SG":
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "P2SG"
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":729
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "P2SG"
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "P3PL"
 *             return "P3SG"
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isPossessivePlural); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 729, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 729, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_index, __pyx_v_correctParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 729, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_2 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 729, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_10, __pyx_v_correctParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 729, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 729, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":730
 *                 return "P2SG"
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "P3PL"             # <<<<<<<<<<<<<<
 *             return "P3SG"
 *             # olun, kurtulun, gelin #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_n_u_P3PL);
      __pyx_r = __pyx_n_u_P3PL;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":729
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "P2SG"
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):             # <<<<<<<<<<<<<<
 *                 return "P3PL"
 *             return "P3SG"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":731
 *             if AutoDisambiguator.isPossessivePlural(index, correctParses):
 *                 return "P3PL"
 *             return "P3SG"             # <<<<<<<<<<<<<<
 *             # olun, kurtulun, gelin #
 *         elif parseString == "VERB+POS+IMP+A2PL$VERB^DB+VERB+PASS+POS+IMP+A2SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_P3SG);
    __pyx_r = __pyx_n_u_P3SG;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":726
 *             return "ADJ^DB+NOUN+AGT+A3SG+PNON+NOM"
 *             # havayollarna, gzyalarna #
 *         elif parseString == "P2SG$P3PL$P3SG":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isAnyWordSecondPerson(index, correctParses):
 *                 return "P2SG"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":733
 *             return "P3SG"
 *             # olun, kurtulun, gelin #
 *         elif parseString == "VERB+POS+IMP+A2PL$VERB^DB+VERB+PASS+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             return "VERB+POS+IMP+A2PL"
 *         elif parseString == "ADJ+JUSTLIKE^DB$NOUN+ZERO+A3SG+P3SG+NOM^DB":
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_VERB_POS_IMP_A2PL_VERB_DB_VERB_P, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 733, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":734
 *             # olun, kurtulun, gelin #
 *         elif parseString == "VERB+POS+IMP+A2PL$VERB^DB+VERB+PASS+POS+IMP+A2SG":
 *             return "VERB+POS+IMP+A2PL"             # <<<<<<<<<<<<<<
 *         elif parseString == "ADJ+JUSTLIKE^DB$NOUN+ZERO+A3SG+P3SG+NOM^DB":
 *             return "NOUN+ZERO+A3SG+P3SG+NOM^DB"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_VERB_POS_IMP_A2PL);
    __pyx_r = __pyx_kp_u_VERB_POS_IMP_A2PL;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":733
 *             return "P3SG"
 *             # olun, kurtulun, gelin #
 *         elif parseString == "VERB+POS+IMP+A2PL$VERB^DB+VERB+PASS+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             return "VERB+POS+IMP+A2PL"
 *         elif parseString == "ADJ+JUSTLIKE^DB$NOUN+ZERO+A3SG+P3SG+NOM^DB":
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":735
 *         elif parseString == "VERB+POS+IMP+A2PL$VERB^DB+VERB+PASS+POS+IMP+A2SG":
 *             return "VERB+POS+IMP+A2PL"
 *         elif parseString == "ADJ+JUSTLIKE^DB$NOUN+ZERO+A3SG+P3SG+NOM^DB":             # <<<<<<<<<<<<<<
 *             return "NOUN+ZERO+A3SG+P3SG+NOM^DB"
 *             # olumaktayd, gerekemekteydi #
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_JUSTLIKE_DB_NOUN_ZERO_A3SG_P_3, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 735, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":736
 *             return "VERB+POS+IMP+A2PL"
 *         elif parseString == "ADJ+JUSTLIKE^DB$NOUN+ZERO+A3SG+P3SG+NOM^DB":
 *             return "NOUN+ZERO+A3SG+P3SG+NOM^DB"             # <<<<<<<<<<<<<<
 *             # olumaktayd, gerekemekteydi #
 *         elif parseString == "POS+PROG2$POS^DB+NOUN+INF+A3SG+PNON+LOC^DB+VERB+ZERO":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_ZERO_A3SG_P3SG_NOM_DB);
    __pyx_r = __pyx_kp_u_NOUN_ZERO_A3SG_P3SG_NOM_DB;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":735
 *         elif parseString == "VERB+POS+IMP+A2PL$VERB^DB+VERB+PASS+POS+IMP+A2SG":
 *             return "VERB+POS+IMP+A2PL"
 *         elif parseString == "ADJ+JUSTLIKE^DB$NOUN+ZERO+A3SG+P3SG+NOM^DB":             # <<<<<<<<<<<<<<
 *             return "NOUN+ZERO+A3SG+P3SG+NOM^DB"
 *             # olumaktayd, gerekemekteydi #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":738
 *             return "NOUN+ZERO+A3SG+P3SG+NOM^DB"
 *             # olumaktayd, gerekemekteydi #
 *         elif parseString == "POS+PROG2$POS^DB+NOUN+INF+A3SG+PNON+LOC^DB+VERB+ZERO":             # <<<<<<<<<<<<<<
 *             return "POS+PROG2"
 *             # BERABER #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_POS_PROG2_POS_DB_NOUN_INF_A3SG_P_2, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 738, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":739
 *             # olumaktayd, gerekemekteydi #
 *         elif parseString == "POS+PROG2$POS^DB+NOUN+INF+A3SG+PNON+LOC^DB+VERB+ZERO":
 *             return "POS+PROG2"             # <<<<<<<<<<<<<<
 *             # BERABER #
 *         elif parseString == "ADJ$ADV$POSTP+PCINS":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_POS_PROG2);
    __pyx_r = __pyx_kp_u_POS_PROG2;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":738
 *             return "NOUN+ZERO+A3SG+P3SG+NOM^DB"
 *             # olumaktayd, gerekemekteydi #
 *         elif parseString == "POS+PROG2$POS^DB+NOUN+INF+A3SG+PNON+LOC^DB+VERB+ZERO":             # <<<<<<<<<<<<<<
 *             return "POS+PROG2"
 *             # BERABER #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":741
 *             return "POS+PROG2"
 *             # BERABER #
 *         elif parseString == "ADJ$ADV$POSTP+PCINS":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.INSTRUMENTAL):
 *                 return "POSTP+PCINS"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_ADV_POSTP_PCINS, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 741, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":742
 *             # BERABER #
 *         elif parseString == "ADJ$ADV$POSTP+PCINS":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.INSTRUMENTAL):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCINS"
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_hasPreviousWordTag); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 742, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_MorphologicalTag); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 742, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_INSTRUMENTAL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 742, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_v_index, __pyx_v_correctParses, __pyx_t_5};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 742, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_v_index, __pyx_v_correctParses, __pyx_t_5};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 742, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(3+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 742, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_2) {
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2); __pyx_t_2 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_10, __pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_4, 2+__pyx_t_10, __pyx_t_5);
      __pyx_t_5 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 742, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 742, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":743
 *         elif parseString == "ADJ$ADV$POSTP+PCINS":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.INSTRUMENTAL):
 *                 return "POSTP+PCINS"             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ"
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_POSTP_PCINS);
      __pyx_r = __pyx_kp_u_POSTP_PCINS;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":742
 *             # BERABER #
 *         elif parseString == "ADJ$ADV$POSTP+PCINS":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.INSTRUMENTAL):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCINS"
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":744
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.INSTRUMENTAL):
 *                 return "POSTP+PCINS"
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ"
 *             return "ADV"
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_isNextWordNounOrAdjective); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 744, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 744, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_index, __pyx_v_fsmParses};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 744, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 744, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_fsmParses);
      __Pyx_GIVEREF(__pyx_v_fsmParses);
      PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_10, __pyx_v_fsmParses);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 744, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 744, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":745
 *                 return "POSTP+PCINS"
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ"             # <<<<<<<<<<<<<<
 *             return "ADV"
 *             # BN, KIRK #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_n_u_ADJ);
      __pyx_r = __pyx_n_u_ADJ;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":744
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.INSTRUMENTAL):
 *                 return "POSTP+PCINS"
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):             # <<<<<<<<<<<<<<
 *                 return "ADJ"
 *             return "ADV"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":746
 *             if AutoDisambiguator.isNextWordNounOrAdjective(index, fsmParses):
 *                 return "ADJ"
 *             return "ADV"             # <<<<<<<<<<<<<<
 *             # BN, KIRK #
 *         elif parseString == "NUM+CARD$VERB+POS+IMP+A2SG":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ADV);
    __pyx_r = __pyx_n_u_ADV;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":741
 *             return "POS+PROG2"
 *             # BERABER #
 *         elif parseString == "ADJ$ADV$POSTP+PCINS":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.INSTRUMENTAL):
 *                 return "POSTP+PCINS"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":748
 *             return "ADV"
 *             # BN, KIRK #
 *         elif parseString == "NUM+CARD$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             return "NUM+CARD"
 *             # TE #
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NUM_CARD_VERB_POS_IMP_A2SG, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 748, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":749
 *             # BN, KIRK #
 *         elif parseString == "NUM+CARD$VERB+POS+IMP+A2SG":
 *             return "NUM+CARD"             # <<<<<<<<<<<<<<
 *             # TE #
 *         elif parseString == "NOUN+A3SG+PNON+NOM$POSTP+PCABL":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NUM_CARD);
    __pyx_r = __pyx_kp_u_NUM_CARD;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":748
 *             return "ADV"
 *             # BN, KIRK #
 *         elif parseString == "NUM+CARD$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             return "NUM+CARD"
 *             # TE #
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":751
 *             return "NUM+CARD"
 *             # TE #
 *         elif parseString == "NOUN+A3SG+PNON+NOM$POSTP+PCABL":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_PNON_NOM_POSTP_PCABL, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 751, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":752
 *             # TE #
 *         elif parseString == "NOUN+A3SG+PNON+NOM$POSTP+PCABL":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCABL"
 *             return "NOUN+A3SG+PNON+NOM"
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_hasPreviousWordTag); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 752, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_MorphologicalTag); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 752, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_ABLATIVE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 752, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_index, __pyx_v_correctParses, __pyx_t_4};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 752, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_index, __pyx_v_correctParses, __pyx_t_4};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_10, 3+__pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 752, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else
    #endif
    {
      __pyx_t_2 = PyTuple_New(3+__pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 752, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_index);
      __Pyx_GIVEREF(__pyx_v_index);
      PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_10, __pyx_v_index);
      __Pyx_INCREF(__pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_v_correctParses);
      PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_10, __pyx_v_correctParses);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_2, 2+__pyx_t_10, __pyx_t_4);
      __pyx_t_4 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 752, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 752, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":753
 *         elif parseString == "NOUN+A3SG+PNON+NOM$POSTP+PCABL":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+PNON+NOM"
 *             # BENMLE #
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_POSTP_PCABL);
      __pyx_r = __pyx_kp_u_POSTP_PCABL;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":752
 *             # TE #
 *         elif parseString == "NOUN+A3SG+PNON+NOM$POSTP+PCABL":
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):             # <<<<<<<<<<<<<<
 *                 return "POSTP+PCABL"
 *             return "NOUN+A3SG+PNON+NOM"
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":754
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"
 *             return "NOUN+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *             # BENMLE #
 *         elif parseString == "NOUN+A3SG+P1SG$PRON+PERS+A1SG+PNON":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_A3SG_PNON_NOM);
    __pyx_r = __pyx_kp_u_NOUN_A3SG_PNON_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":751
 *             return "NUM+CARD"
 *             # TE #
 *         elif parseString == "NOUN+A3SG+PNON+NOM$POSTP+PCABL":             # <<<<<<<<<<<<<<
 *             if AutoDisambiguator.hasPreviousWordTag(index, correctParses, MorphologicalTag.ABLATIVE):
 *                 return "POSTP+PCABL"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":756
 *             return "NOUN+A3SG+PNON+NOM"
 *             # BENMLE #
 *         elif parseString == "NOUN+A3SG+P1SG$PRON+PERS+A1SG+PNON":             # <<<<<<<<<<<<<<
 *             return "PRON+PERS+A1SG+PNON"
 *             # Accusative and Ablative Cases#
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_P1SG_PRON_PERS_A1SG_PN, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 756, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":757
 *             # BENMLE #
 *         elif parseString == "NOUN+A3SG+P1SG$PRON+PERS+A1SG+PNON":
 *             return "PRON+PERS+A1SG+PNON"             # <<<<<<<<<<<<<<
 *             # Accusative and Ablative Cases#
 *         elif parseString == "ADV+WITHOUTHAVINGDONESO$NOUN+INF2+A3SG+PNON+ABL":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_PRON_PERS_A1SG_PNON);
    __pyx_r = __pyx_kp_u_PRON_PERS_A1SG_PNON;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":756
 *             return "NOUN+A3SG+PNON+NOM"
 *             # BENMLE #
 *         elif parseString == "NOUN+A3SG+P1SG$PRON+PERS+A1SG+PNON":             # <<<<<<<<<<<<<<
 *             return "PRON+PERS+A1SG+PNON"
 *             # Accusative and Ablative Cases#
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":759
 *             return "PRON+PERS+A1SG+PNON"
 *             # Accusative and Ablative Cases#
 *         elif parseString == "ADV+WITHOUTHAVINGDONESO$NOUN+INF2+A3SG+PNON+ABL":             # <<<<<<<<<<<<<<
 *             return "ADV+WITHOUTHAVINGDONESO"
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3SG+P3SG+NOM$ADJ^DB+NOUN+ZERO+A3SG+PNON+ACC$NOUN+A3SG+P3SG+NOM$NOUN" \
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADV_WITHOUTHAVINGDONESO_NOUN_INF, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 759, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":760
 *             # Accusative and Ablative Cases#
 *         elif parseString == "ADV+WITHOUTHAVINGDONESO$NOUN+INF2+A3SG+PNON+ABL":
 *             return "ADV+WITHOUTHAVINGDONESO"             # <<<<<<<<<<<<<<
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3SG+P3SG+NOM$ADJ^DB+NOUN+ZERO+A3SG+PNON+ACC$NOUN+A3SG+P3SG+NOM$NOUN" \
 *                             "+A3SG+PNON+ACC":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_ADV_WITHOUTHAVINGDONESO);
    __pyx_r = __pyx_kp_u_ADV_WITHOUTHAVINGDONESO;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":759
 *             return "PRON+PERS+A1SG+PNON"
 *             # Accusative and Ablative Cases#
 *         elif parseString == "ADV+WITHOUTHAVINGDONESO$NOUN+INF2+A3SG+PNON+ABL":             # <<<<<<<<<<<<<<
 *             return "ADV+WITHOUTHAVINGDONESO"
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3SG+P3SG+NOM$ADJ^DB+NOUN+ZERO+A3SG+PNON+ACC$NOUN+A3SG+P3SG+NOM$NOUN" \
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":761
 *         elif parseString == "ADV+WITHOUTHAVINGDONESO$NOUN+INF2+A3SG+PNON+ABL":
 *             return "ADV+WITHOUTHAVINGDONESO"
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3SG+P3SG+NOM$ADJ^DB+NOUN+ZERO+A3SG+PNON+ACC$NOUN+A3SG+P3SG+NOM$NOUN" \             # <<<<<<<<<<<<<<
 *                             "+A3SG+PNON+ACC":
 *             return "ADJ^DB+NOUN+ZERO+A3SG+P3SG+NOM"
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_DB_NOUN_ZERO_A3SG_P3SG_NOM_A_2, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 761, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":763
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3SG+P3SG+NOM$ADJ^DB+NOUN+ZERO+A3SG+PNON+ACC$NOUN+A3SG+P3SG+NOM$NOUN" \
 *                             "+A3SG+PNON+ACC":
 *             return "ADJ^DB+NOUN+ZERO+A3SG+P3SG+NOM"             # <<<<<<<<<<<<<<
 *         elif parseString == "P3SG+NOM$PNON+ACC":
 *             if fsmParses[index].getFsmParse(0).getFinalPos() == "PROP":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_ADJ_DB_NOUN_ZERO_A3SG_P3SG_NOM);
    __pyx_r = __pyx_kp_u_ADJ_DB_NOUN_ZERO_A3SG_P3SG_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":761
 *         elif parseString == "ADV+WITHOUTHAVINGDONESO$NOUN+INF2+A3SG+PNON+ABL":
 *             return "ADV+WITHOUTHAVINGDONESO"
 *         elif parseString == "ADJ^DB+NOUN+ZERO+A3SG+P3SG+NOM$ADJ^DB+NOUN+ZERO+A3SG+PNON+ACC$NOUN+A3SG+P3SG+NOM$NOUN" \             # <<<<<<<<<<<<<<
 *                             "+A3SG+PNON+ACC":
 *             return "ADJ^DB+NOUN+ZERO+A3SG+P3SG+NOM"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":764
 *                             "+A3SG+PNON+ACC":
 *             return "ADJ^DB+NOUN+ZERO+A3SG+P3SG+NOM"
 *         elif parseString == "P3SG+NOM$PNON+ACC":             # <<<<<<<<<<<<<<
 *             if fsmParses[index].getFsmParse(0).getFinalPos() == "PROP":
 *                 return "PNON+ACC"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_P3SG_NOM_PNON_ACC, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 764, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":765
 *             return "ADJ^DB+NOUN+ZERO+A3SG+P3SG+NOM"
 *         elif parseString == "P3SG+NOM$PNON+ACC":
 *             if fsmParses[index].getFsmParse(0).getFinalPos() == "PROP":             # <<<<<<<<<<<<<<
 *                 return "PNON+ACC"
 *             else:
 */
    if (unlikely(__pyx_v_fsmParses == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 765, __pyx_L1_error)
    }
    __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_fsmParses, __pyx_v_index); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 765, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_getFsmParse); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 765, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_2, __pyx_int_0) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_int_0);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 765, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_getFinalPos); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 765, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_3 = (__pyx_t_1) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_1) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 765, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_n_u_PROP, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 765, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":766
 *         elif parseString == "P3SG+NOM$PNON+ACC":
 *             if fsmParses[index].getFsmParse(0).getFinalPos() == "PROP":
 *                 return "PNON+ACC"             # <<<<<<<<<<<<<<
 *             else:
 *                 return "P3SG+NOM"
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_PNON_ACC);
      __pyx_r = __pyx_kp_u_PNON_ACC;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":765
 *             return "ADJ^DB+NOUN+ZERO+A3SG+P3SG+NOM"
 *         elif parseString == "P3SG+NOM$PNON+ACC":
 *             if fsmParses[index].getFsmParse(0).getFinalPos() == "PROP":             # <<<<<<<<<<<<<<
 *                 return "PNON+ACC"
 *             else:
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":768
 *                 return "PNON+ACC"
 *             else:
 *                 return "P3SG+NOM"             # <<<<<<<<<<<<<<
 *         elif parseString == "A3PL+PNON+NOM$A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A3PL":
 *             return "A3PL+PNON+NOM"
 */
    /*else*/ {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_P3SG_NOM);
      __pyx_r = __pyx_kp_u_P3SG_NOM;
      goto __pyx_L0;
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":764
 *                             "+A3SG+PNON+ACC":
 *             return "ADJ^DB+NOUN+ZERO+A3SG+P3SG+NOM"
 *         elif parseString == "P3SG+NOM$PNON+ACC":             # <<<<<<<<<<<<<<
 *             if fsmParses[index].getFsmParse(0).getFinalPos() == "PROP":
 *                 return "PNON+ACC"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":769
 *             else:
 *                 return "P3SG+NOM"
 *         elif parseString == "A3PL+PNON+NOM$A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A3PL":             # <<<<<<<<<<<<<<
 *             return "A3PL+PNON+NOM"
 *         elif parseString == "ADV+SINCE$VERB+ZERO+PRES+COP+A3SG":
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_A3PL_PNON_NOM_A3SG_PNON_NOM_DB_V_2, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 769, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":770
 *                 return "P3SG+NOM"
 *         elif parseString == "A3PL+PNON+NOM$A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A3PL":
 *             return "A3PL+PNON+NOM"             # <<<<<<<<<<<<<<
 *         elif parseString == "ADV+SINCE$VERB+ZERO+PRES+COP+A3SG":
 *             if root == "yl" or root == "sre" or root == "zaman" or root == "ay":
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_A3PL_PNON_NOM);
    __pyx_r = __pyx_kp_u_A3PL_PNON_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":769
 *             else:
 *                 return "P3SG+NOM"
 *         elif parseString == "A3PL+PNON+NOM$A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A3PL":             # <<<<<<<<<<<<<<
 *             return "A3PL+PNON+NOM"
 *         elif parseString == "ADV+SINCE$VERB+ZERO+PRES+COP+A3SG":
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":771
 *         elif parseString == "A3PL+PNON+NOM$A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A3PL":
 *             return "A3PL+PNON+NOM"
 *         elif parseString == "ADV+SINCE$VERB+ZERO+PRES+COP+A3SG":             # <<<<<<<<<<<<<<
 *             if root == "yl" or root == "sre" or root == "zaman" or root == "ay":
 *                 return "ADV+SINCE"
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADV_SINCE_VERB_ZERO_PRES_COP_A3S, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 771, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":772
 *             return "A3PL+PNON+NOM"
 *         elif parseString == "ADV+SINCE$VERB+ZERO+PRES+COP+A3SG":
 *             if root == "yl" or root == "sre" or root == "zaman" or root == "ay":             # <<<<<<<<<<<<<<
 *                 return "ADV+SINCE"
 *             else:
 */
    __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_root, __pyx_n_u_yl, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 772, __pyx_L1_error)
    __pyx_t_8 = (__pyx_t_11 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_9 = __pyx_t_8;
      goto __pyx_L121_bool_binop_done;
    }
    __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_root, __pyx_n_u_sre, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 772, __pyx_L1_error)
    __pyx_t_11 = (__pyx_t_8 != 0);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_9 = __pyx_t_11;
      goto __pyx_L121_bool_binop_done;
    }
    __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_root, __pyx_n_u_zaman, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 772, __pyx_L1_error)
    __pyx_t_8 = (__pyx_t_11 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_9 = __pyx_t_8;
      goto __pyx_L121_bool_binop_done;
    }
    __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_root, __pyx_n_u_ay, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 772, __pyx_L1_error)
    __pyx_t_11 = (__pyx_t_8 != 0);
    __pyx_t_9 = __pyx_t_11;
    __pyx_L121_bool_binop_done:;
    if (__pyx_t_9) {

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":773
 *         elif parseString == "ADV+SINCE$VERB+ZERO+PRES+COP+A3SG":
 *             if root == "yl" or root == "sre" or root == "zaman" or root == "ay":
 *                 return "ADV+SINCE"             # <<<<<<<<<<<<<<
 *             else:
 *                 return "VERB+ZERO+PRES+COP+A3SG"
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_ADV_SINCE);
      __pyx_r = __pyx_kp_u_ADV_SINCE;
      goto __pyx_L0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":772
 *             return "A3PL+PNON+NOM"
 *         elif parseString == "ADV+SINCE$VERB+ZERO+PRES+COP+A3SG":
 *             if root == "yl" or root == "sre" or root == "zaman" or root == "ay":             # <<<<<<<<<<<<<<
 *                 return "ADV+SINCE"
 *             else:
 */
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":775
 *                 return "ADV+SINCE"
 *             else:
 *                 return "VERB+ZERO+PRES+COP+A3SG"             # <<<<<<<<<<<<<<
 *         elif parseString == "CONJ$VERB+POS+IMP+A2SG":
 *             return "CONJ"
 */
    /*else*/ {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_kp_u_VERB_ZERO_PRES_COP_A3SG);
      __pyx_r = __pyx_kp_u_VERB_ZERO_PRES_COP_A3SG;
      goto __pyx_L0;
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":771
 *         elif parseString == "A3PL+PNON+NOM$A3SG+PNON+NOM^DB+VERB+ZERO+PRES+A3PL":
 *             return "A3PL+PNON+NOM"
 *         elif parseString == "ADV+SINCE$VERB+ZERO+PRES+COP+A3SG":             # <<<<<<<<<<<<<<
 *             if root == "yl" or root == "sre" or root == "zaman" or root == "ay":
 *                 return "ADV+SINCE"
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":776
 *             else:
 *                 return "VERB+ZERO+PRES+COP+A3SG"
 *         elif parseString == "CONJ$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             return "CONJ"
 *         elif parseString == "NEG+IMP+A2SG$POS^DB+NOUN+INF2+A3SG+PNON+NOM":
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_CONJ_VERB_POS_IMP_A2SG, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 776, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":777
 *                 return "VERB+ZERO+PRES+COP+A3SG"
 *         elif parseString == "CONJ$VERB+POS+IMP+A2SG":
 *             return "CONJ"             # <<<<<<<<<<<<<<
 *         elif parseString == "NEG+IMP+A2SG$POS^DB+NOUN+INF2+A3SG+PNON+NOM":
 *             return "POS^DB+NOUN+INF2+A3SG+PNON+NOM"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_CONJ);
    __pyx_r = __pyx_n_u_CONJ;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":776
 *             else:
 *                 return "VERB+ZERO+PRES+COP+A3SG"
 *         elif parseString == "CONJ$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             return "CONJ"
 *         elif parseString == "NEG+IMP+A2SG$POS^DB+NOUN+INF2+A3SG+PNON+NOM":
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":778
 *         elif parseString == "CONJ$VERB+POS+IMP+A2SG":
 *             return "CONJ"
 *         elif parseString == "NEG+IMP+A2SG$POS^DB+NOUN+INF2+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             return "POS^DB+NOUN+INF2+A3SG+PNON+NOM"
 *         elif parseString == "NEG+OPT+A3SG$POS^DB+NOUN+INF2+A3SG+PNON+DAT":
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NEG_IMP_A2SG_POS_DB_NOUN_INF2_A3, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 778, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":779
 *             return "CONJ"
 *         elif parseString == "NEG+IMP+A2SG$POS^DB+NOUN+INF2+A3SG+PNON+NOM":
 *             return "POS^DB+NOUN+INF2+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *         elif parseString == "NEG+OPT+A3SG$POS^DB+NOUN+INF2+A3SG+PNON+DAT":
 *             return "POS^DB+NOUN+INF2+A3SG+PNON+DAT"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_POS_DB_NOUN_INF2_A3SG_PNON_NOM);
    __pyx_r = __pyx_kp_u_POS_DB_NOUN_INF2_A3SG_PNON_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":778
 *         elif parseString == "CONJ$VERB+POS+IMP+A2SG":
 *             return "CONJ"
 *         elif parseString == "NEG+IMP+A2SG$POS^DB+NOUN+INF2+A3SG+PNON+NOM":             # <<<<<<<<<<<<<<
 *             return "POS^DB+NOUN+INF2+A3SG+PNON+NOM"
 *         elif parseString == "NEG+OPT+A3SG$POS^DB+NOUN+INF2+A3SG+PNON+DAT":
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":780
 *         elif parseString == "NEG+IMP+A2SG$POS^DB+NOUN+INF2+A3SG+PNON+NOM":
 *             return "POS^DB+NOUN+INF2+A3SG+PNON+NOM"
 *         elif parseString == "NEG+OPT+A3SG$POS^DB+NOUN+INF2+A3SG+PNON+DAT":             # <<<<<<<<<<<<<<
 *             return "POS^DB+NOUN+INF2+A3SG+PNON+DAT"
 *         elif parseString == "NOUN+A3SG+P3SG+NOM$NOUN^DB+ADJ+ALMOST":
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NEG_OPT_A3SG_POS_DB_NOUN_INF2_A3, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 780, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":781
 *             return "POS^DB+NOUN+INF2+A3SG+PNON+NOM"
 *         elif parseString == "NEG+OPT+A3SG$POS^DB+NOUN+INF2+A3SG+PNON+DAT":
 *             return "POS^DB+NOUN+INF2+A3SG+PNON+DAT"             # <<<<<<<<<<<<<<
 *         elif parseString == "NOUN+A3SG+P3SG+NOM$NOUN^DB+ADJ+ALMOST":
 *             return "NOUN+A3SG+P3SG+NOM"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_POS_DB_NOUN_INF2_A3SG_PNON_DAT);
    __pyx_r = __pyx_kp_u_POS_DB_NOUN_INF2_A3SG_PNON_DAT;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":780
 *         elif parseString == "NEG+IMP+A2SG$POS^DB+NOUN+INF2+A3SG+PNON+NOM":
 *             return "POS^DB+NOUN+INF2+A3SG+PNON+NOM"
 *         elif parseString == "NEG+OPT+A3SG$POS^DB+NOUN+INF2+A3SG+PNON+DAT":             # <<<<<<<<<<<<<<
 *             return "POS^DB+NOUN+INF2+A3SG+PNON+DAT"
 *         elif parseString == "NOUN+A3SG+P3SG+NOM$NOUN^DB+ADJ+ALMOST":
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":782
 *         elif parseString == "NEG+OPT+A3SG$POS^DB+NOUN+INF2+A3SG+PNON+DAT":
 *             return "POS^DB+NOUN+INF2+A3SG+PNON+DAT"
 *         elif parseString == "NOUN+A3SG+P3SG+NOM$NOUN^DB+ADJ+ALMOST":             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+P3SG+NOM"
 *         elif parseString == "ADJ$VERB+POS+IMP+A2SG":
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_P3SG_NOM_NOUN_DB_ADJ_A, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 782, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":783
 *             return "POS^DB+NOUN+INF2+A3SG+PNON+DAT"
 *         elif parseString == "NOUN+A3SG+P3SG+NOM$NOUN^DB+ADJ+ALMOST":
 *             return "NOUN+A3SG+P3SG+NOM"             # <<<<<<<<<<<<<<
 *         elif parseString == "ADJ$VERB+POS+IMP+A2SG":
 *             return "ADJ"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_A3SG_P3SG_NOM);
    __pyx_r = __pyx_kp_u_NOUN_A3SG_P3SG_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":782
 *         elif parseString == "NEG+OPT+A3SG$POS^DB+NOUN+INF2+A3SG+PNON+DAT":
 *             return "POS^DB+NOUN+INF2+A3SG+PNON+DAT"
 *         elif parseString == "NOUN+A3SG+P3SG+NOM$NOUN^DB+ADJ+ALMOST":             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+P3SG+NOM"
 *         elif parseString == "ADJ$VERB+POS+IMP+A2SG":
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":784
 *         elif parseString == "NOUN+A3SG+P3SG+NOM$NOUN^DB+ADJ+ALMOST":
 *             return "NOUN+A3SG+P3SG+NOM"
 *         elif parseString == "ADJ$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             return "ADJ"
 *         elif parseString == "NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_ADJ_VERB_POS_IMP_A2SG, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 784, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":785
 *             return "NOUN+A3SG+P3SG+NOM"
 *         elif parseString == "ADJ$VERB+POS+IMP+A2SG":
 *             return "ADJ"             # <<<<<<<<<<<<<<
 *         elif parseString == "NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             return "NOUN+A3SG+PNON+NOM"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ADJ);
    __pyx_r = __pyx_n_u_ADJ;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":784
 *         elif parseString == "NOUN+A3SG+P3SG+NOM$NOUN^DB+ADJ+ALMOST":
 *             return "NOUN+A3SG+P3SG+NOM"
 *         elif parseString == "ADJ$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             return "ADJ"
 *         elif parseString == "NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":786
 *         elif parseString == "ADJ$VERB+POS+IMP+A2SG":
 *             return "ADJ"
 *         elif parseString == "NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+PNON+NOM"
 *         elif parseString == "INF2+A3SG+P3SG+NOM$INF2^DB+ADJ+ALMOST$":
 */
  __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_NOUN_A3SG_PNON_NOM_VERB_POS_IMP, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 786, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_11 != 0);
  if (__pyx_t_9) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":787
 *             return "ADJ"
 *         elif parseString == "NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             return "NOUN+A3SG+PNON+NOM"             # <<<<<<<<<<<<<<
 *         elif parseString == "INF2+A3SG+P3SG+NOM$INF2^DB+ADJ+ALMOST$":
 *             return "INF2+A3SG+P3SG+NOM"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_NOUN_A3SG_PNON_NOM);
    __pyx_r = __pyx_kp_u_NOUN_A3SG_PNON_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":786
 *         elif parseString == "ADJ$VERB+POS+IMP+A2SG":
 *             return "ADJ"
 *         elif parseString == "NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":             # <<<<<<<<<<<<<<
 *             return "NOUN+A3SG+PNON+NOM"
 *         elif parseString == "INF2+A3SG+P3SG+NOM$INF2^DB+ADJ+ALMOST$":
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":788
 *         elif parseString == "NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             return "NOUN+A3SG+PNON+NOM"
 *         elif parseString == "INF2+A3SG+P3SG+NOM$INF2^DB+ADJ+ALMOST$":             # <<<<<<<<<<<<<<
 *             return "INF2+A3SG+P3SG+NOM"
 *         return None
 */
  __pyx_t_9 = (__Pyx_PyUnicode_Equals(__pyx_v_parseString, __pyx_kp_u_INF2_A3SG_P3SG_NOM_INF2_DB_ADJ_A, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 788, __pyx_L1_error)
  __pyx_t_11 = (__pyx_t_9 != 0);
  if (__pyx_t_11) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":789
 *             return "NOUN+A3SG+PNON+NOM"
 *         elif parseString == "INF2+A3SG+P3SG+NOM$INF2^DB+ADJ+ALMOST$":
 *             return "INF2+A3SG+P3SG+NOM"             # <<<<<<<<<<<<<<
 *         return None
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_INF2_A3SG_P3SG_NOM);
    __pyx_r = __pyx_kp_u_INF2_A3SG_P3SG_NOM;
    goto __pyx_L0;

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":788
 *         elif parseString == "NOUN+A3SG+PNON+NOM$VERB+POS+IMP+A2SG":
 *             return "NOUN+A3SG+PNON+NOM"
 *         elif parseString == "INF2+A3SG+P3SG+NOM$INF2^DB+ADJ+ALMOST$":             # <<<<<<<<<<<<<<
 *             return "INF2+A3SG+P3SG+NOM"
 *         return None
 */
  }
  __pyx_L3:;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":790
 *         elif parseString == "INF2+A3SG+P3SG+NOM$INF2^DB+ADJ+ALMOST$":
 *             return "INF2+A3SG+P3SG+NOM"
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = ((PyObject*)Py_None); __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":77
 * 
 *     @staticmethod
 *     def selectCaseForParseString(parseString: str, index: int, fsmParses: list, correctParses: list) -> str:             # <<<<<<<<<<<<<<
 *         cdef str surfaceForm, root, lastWord
 *         surfaceForm = fsmParses[index].getFsmParse(0).getSurfaceForm()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.selectCaseForParseString", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_surfaceForm);
  __Pyx_XDECREF(__pyx_v_root);
  __Pyx_XDECREF(__pyx_v_lastWord);
  __Pyx_XDECREF(__pyx_v_previousWord);
  __Pyx_XDECREF(__pyx_v_nextWord);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "MorphologicalDisambiguation/AutoDisambiguator.pyx":793
 * 
 *     @staticmethod
 *     def caseDisambiguator(index: int, fsmParses: list, correctParses: list) -> FsmParse:             # <<<<<<<<<<<<<<
 *         cdef FsmParseList fsmParseList
 *         cdef str defaultCase
 */

/* Python wrapper */
static struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_25caseDisambiguator(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_25caseDisambiguator = {"caseDisambiguator", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_25caseDisambiguator, METH_VARARGS|METH_KEYWORDS, 0};
static struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_25caseDisambiguator(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_fsmParses = 0;
  PyObject *__pyx_v_correctParses = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("caseDisambiguator (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_fsmParses,&__pyx_n_s_correctParses,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_fsmParses)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("caseDisambiguator", 1, 3, 3, 1); __PYX_ERR(0, 793, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_correctParses)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("caseDisambiguator", 1, 3, 3, 2); __PYX_ERR(0, 793, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "caseDisambiguator") < 0)) __PYX_ERR(0, 793, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_index = values[0];
    __pyx_v_fsmParses = ((PyObject*)values[1]);
    __pyx_v_correctParses = ((PyObject*)values[2]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("caseDisambiguator", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 793, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.caseDisambiguator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fsmParses), (&PyList_Type), 1, "fsmParses", 1))) __PYX_ERR(0, 793, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_correctParses), (&PyList_Type), 1, "correctParses", 1))) __PYX_ERR(0, 793, __pyx_L1_error)
  __pyx_r = __pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_24caseDisambiguator(__pyx_v_index, __pyx_v_fsmParses, __pyx_v_correctParses);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_24caseDisambiguator(PyObject *__pyx_v_index, PyObject *__pyx_v_fsmParses, PyObject *__pyx_v_correctParses) {
  struct __pyx_obj_21MorphologicalAnalysis_12FsmParseList_FsmParseList *__pyx_v_fsmParseList = 0;
  PyObject *__pyx_v_defaultCase = 0;
  int __pyx_v_i;
  struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *__pyx_v_fsmParse = 0;
  struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("caseDisambiguator", 0);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":798
 *         cdef int i
 *         cdef FsmParse fsmParse
 *         fsmParseList = fsmParses[index]             # <<<<<<<<<<<<<<
 *         defaultCase = AutoDisambiguator.selectCaseForParseString(fsmParses[index].parsesWithoutPrefixAndSuffix(), index,
 *                                                                  fsmParses, correctParses)
 */
  if (unlikely(__pyx_v_fsmParses == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 798, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_fsmParses, __pyx_v_index); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 798, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_21MorphologicalAnalysis_12FsmParseList_FsmParseList))))) __PYX_ERR(0, 798, __pyx_L1_error)
  __pyx_v_fsmParseList = ((struct __pyx_obj_21MorphologicalAnalysis_12FsmParseList_FsmParseList *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":799
 *         cdef FsmParse fsmParse
 *         fsmParseList = fsmParses[index]
 *         defaultCase = AutoDisambiguator.selectCaseForParseString(fsmParses[index].parsesWithoutPrefixAndSuffix(), index,             # <<<<<<<<<<<<<<
 *                                                                  fsmParses, correctParses)
 *         if defaultCase is not None:
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_selectCaseForParseString); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 799, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (unlikely(__pyx_v_fsmParses == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 799, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyObject_GetItem(__pyx_v_fsmParses, __pyx_v_index); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 799, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_parsesWithoutPrefixAndSuffix); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 799, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 799, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":800
 *         fsmParseList = fsmParses[index]
 *         defaultCase = AutoDisambiguator.selectCaseForParseString(fsmParses[index].parsesWithoutPrefixAndSuffix(), index,
 *                                                                  fsmParses, correctParses)             # <<<<<<<<<<<<<<
 *         if defaultCase is not None:
 *             for i in range(fsmParseList.size()):
 */
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_5, __pyx_t_3, __pyx_v_index, __pyx_v_fsmParses, __pyx_v_correctParses};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 4+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 799, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_5, __pyx_t_3, __pyx_v_index, __pyx_v_fsmParses, __pyx_v_correctParses};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 4+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 799, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_4 = PyTuple_New(4+__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 799, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_6, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_index);
    __Pyx_GIVEREF(__pyx_v_index);
    PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_6, __pyx_v_index);
    __Pyx_INCREF(__pyx_v_fsmParses);
    __Pyx_GIVEREF(__pyx_v_fsmParses);
    PyTuple_SET_ITEM(__pyx_t_4, 2+__pyx_t_6, __pyx_v_fsmParses);
    __Pyx_INCREF(__pyx_v_correctParses);
    __Pyx_GIVEREF(__pyx_v_correctParses);
    PyTuple_SET_ITEM(__pyx_t_4, 3+__pyx_t_6, __pyx_v_correctParses);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 799, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":799
 *         cdef FsmParse fsmParse
 *         fsmParseList = fsmParses[index]
 *         defaultCase = AutoDisambiguator.selectCaseForParseString(fsmParses[index].parsesWithoutPrefixAndSuffix(), index,             # <<<<<<<<<<<<<<
 *                                                                  fsmParses, correctParses)
 *         if defaultCase is not None:
 */
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 799, __pyx_L1_error)
  __pyx_v_defaultCase = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":801
 *         defaultCase = AutoDisambiguator.selectCaseForParseString(fsmParses[index].parsesWithoutPrefixAndSuffix(), index,
 *                                                                  fsmParses, correctParses)
 *         if defaultCase is not None:             # <<<<<<<<<<<<<<
 *             for i in range(fsmParseList.size()):
 *                 fsmParse = fsmParseList.getFsmParse(i)
 */
  __pyx_t_7 = (__pyx_v_defaultCase != ((PyObject*)Py_None));
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":802
 *                                                                  fsmParses, correctParses)
 *         if defaultCase is not None:
 *             for i in range(fsmParseList.size()):             # <<<<<<<<<<<<<<
 *                 fsmParse = fsmParseList.getFsmParse(i)
 *                 if defaultCase in fsmParse.transitionList():
 */
    __pyx_t_6 = ((struct __pyx_vtabstruct_21MorphologicalAnalysis_12FsmParseList_FsmParseList *)__pyx_v_fsmParseList->__pyx_vtab)->size(__pyx_v_fsmParseList, 0);
    __pyx_t_9 = __pyx_t_6;
    for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
      __pyx_v_i = __pyx_t_10;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":803
 *         if defaultCase is not None:
 *             for i in range(fsmParseList.size()):
 *                 fsmParse = fsmParseList.getFsmParse(i)             # <<<<<<<<<<<<<<
 *                 if defaultCase in fsmParse.transitionList():
 *                     return fsmParse
 */
      __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_21MorphologicalAnalysis_12FsmParseList_FsmParseList *)__pyx_v_fsmParseList->__pyx_vtab)->getFsmParse(__pyx_v_fsmParseList, __pyx_v_i, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 803, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_fsmParse, ((struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":804
 *             for i in range(fsmParseList.size()):
 *                 fsmParse = fsmParseList.getFsmParse(i)
 *                 if defaultCase in fsmParse.transitionList():             # <<<<<<<<<<<<<<
 *                     return fsmParse
 *         return None
 */
      __pyx_t_1 = ((struct __pyx_vtabstruct_21MorphologicalAnalysis_8FsmParse_FsmParse *)__pyx_v_fsmParse->__pyx_base.__pyx_vtab)->transitionList(__pyx_v_fsmParse, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 804, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (unlikely(__pyx_t_1 == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
        __PYX_ERR(0, 804, __pyx_L1_error)
      }
      __pyx_t_8 = (__Pyx_PyUnicode_ContainsTF(__pyx_v_defaultCase, __pyx_t_1, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 804, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_7 = (__pyx_t_8 != 0);
      if (__pyx_t_7) {

        /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":805
 *                 fsmParse = fsmParseList.getFsmParse(i)
 *                 if defaultCase in fsmParse.transitionList():
 *                     return fsmParse             # <<<<<<<<<<<<<<
 *         return None
 */
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(((PyObject *)__pyx_v_fsmParse));
        __pyx_r = __pyx_v_fsmParse;
        goto __pyx_L0;

        /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":804
 *             for i in range(fsmParseList.size()):
 *                 fsmParse = fsmParseList.getFsmParse(i)
 *                 if defaultCase in fsmParse.transitionList():             # <<<<<<<<<<<<<<
 *                     return fsmParse
 *         return None
 */
      }
    }

    /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":801
 *         defaultCase = AutoDisambiguator.selectCaseForParseString(fsmParses[index].parsesWithoutPrefixAndSuffix(), index,
 *                                                                  fsmParses, correctParses)
 *         if defaultCase is not None:             # <<<<<<<<<<<<<<
 *             for i in range(fsmParseList.size()):
 *                 fsmParse = fsmParseList.getFsmParse(i)
 */
  }

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":806
 *                 if defaultCase in fsmParse.transitionList():
 *                     return fsmParse
 *         return None             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_r = ((struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse *)Py_None); __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":793
 * 
 *     @staticmethod
 *     def caseDisambiguator(index: int, fsmParses: list, correctParses: list) -> FsmParse:             # <<<<<<<<<<<<<<
 *         cdef FsmParseList fsmParseList
 *         cdef str defaultCase
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.caseDisambiguator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_fsmParseList);
  __Pyx_XDECREF(__pyx_v_defaultCase);
  __Pyx_XDECREF((PyObject *)__pyx_v_fsmParse);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_27__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_27__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_26__reduce_cython__(((struct __pyx_obj_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_26__reduce_cython__(struct __pyx_obj_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.morphologicalAnalyzer,)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->morphologicalAnalyzer));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->morphologicalAnalyzer));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self->morphologicalAnalyzer));
  __pyx_v_state = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.morphologicalAnalyzer,)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":7
 *     state = (self.morphologicalAnalyzer,)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_2 = (__pyx_v__dict != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.morphologicalAnalyzer is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.morphologicalAnalyzer,)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.morphologicalAnalyzer is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_AutoDisambiguator, (type(self), 0x8b03656, None), state
 */
  /*else*/ {
    __pyx_t_3 = (((PyObject *)__pyx_v_self->morphologicalAnalyzer) != Py_None);
    __pyx_v_use_setstate = __pyx_t_3;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.morphologicalAnalyzer is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_AutoDisambiguator, (type(self), 0x8b03656, None), state
 *     else:
 */
  __pyx_t_3 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":13
 *         use_setstate = self.morphologicalAnalyzer is not None
 *     if use_setstate:
 *         return __pyx_unpickle_AutoDisambiguator, (type(self), 0x8b03656, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_AutoDisambiguator, (type(self), 0x8b03656, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_AutoDisambiguator); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_145765974);
    __Pyx_GIVEREF(__pyx_int_145765974);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_145765974);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.morphologicalAnalyzer is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_AutoDisambiguator, (type(self), 0x8b03656, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_AutoDisambiguator, (type(self), 0x8b03656, None), state
 *     else:
 *         return __pyx_unpickle_AutoDisambiguator, (type(self), 0x8b03656, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_AutoDisambiguator__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle_AutoDisambiguator); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_145765974);
    __Pyx_GIVEREF(__pyx_int_145765974);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_145765974);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
    __pyx_t_5 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_AutoDisambiguator, (type(self), 0x8b03656, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_AutoDisambiguator__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_29__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_29__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_28__setstate_cython__(((struct __pyx_obj_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_28__setstate_cython__(struct __pyx_obj_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_AutoDisambiguator, (type(self), 0x8b03656, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_AutoDisambiguator__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_27MorphologicalDisambiguation_17AutoDisambiguator___pyx_unpickle_AutoDisambiguator__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_AutoDisambiguator, (type(self), 0x8b03656, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_AutoDisambiguator__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_AutoDisambiguator(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_1__pyx_unpickle_AutoDisambiguator(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_1__pyx_unpickle_AutoDisambiguator = {"__pyx_unpickle_AutoDisambiguator", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_1__pyx_unpickle_AutoDisambiguator, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_1__pyx_unpickle_AutoDisambiguator(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_AutoDisambiguator (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_AutoDisambiguator", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_AutoDisambiguator", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_AutoDisambiguator") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_AutoDisambiguator", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.__pyx_unpickle_AutoDisambiguator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator___pyx_unpickle_AutoDisambiguator(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_27MorphologicalDisambiguation_17AutoDisambiguator___pyx_unpickle_AutoDisambiguator(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_AutoDisambiguator", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x8b03656, 0xbc895e2, 0x8b1515e):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x8b03656, 0xbc895e2, 0x8b1515e) = (morphologicalAnalyzer))" % __pyx_checksum)
 */
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_t_1, __pyx_tuple__3, Py_NE)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x8b03656, 0xbc895e2, 0x8b1515e):
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x8b03656, 0xbc895e2, 0x8b1515e) = (morphologicalAnalyzer))" % __pyx_checksum)
 *     __pyx_result = AutoDisambiguator.__new__(__pyx_type)
 */
    __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_PickleError);
    __pyx_t_4 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_1, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_4, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v___pyx_PickleError = __pyx_t_1;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum not in (0x8b03656, 0xbc895e2, 0x8b1515e):
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x8b03656, 0xbc895e2, 0x8b1515e) = (morphologicalAnalyzer))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = AutoDisambiguator.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_0x_x_vs_0, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_1 = __pyx_v___pyx_PickleError; __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_5);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x8b03656, 0xbc895e2, 0x8b1515e):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x8b03656, 0xbc895e2, 0x8b1515e) = (morphologicalAnalyzer))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x8b03656, 0xbc895e2, 0x8b1515e) = (morphologicalAnalyzer))" % __pyx_checksum)
 *     __pyx_result = AutoDisambiguator.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_AutoDisambiguator__set_state(<AutoDisambiguator> __pyx_result, __pyx_state)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), __pyx_n_s_new); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_4 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_5, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x8b03656, 0xbc895e2, 0x8b1515e) = (morphologicalAnalyzer))" % __pyx_checksum)
 *     __pyx_result = AutoDisambiguator.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_AutoDisambiguator__set_state(<AutoDisambiguator> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_3 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {

    /* "(tree fragment)":9
 *     __pyx_result = AutoDisambiguator.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_AutoDisambiguator__set_state(<AutoDisambiguator> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_AutoDisambiguator__set_state(AutoDisambiguator __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_4 = __pyx_f_27MorphologicalDisambiguation_17AutoDisambiguator___pyx_unpickle_AutoDisambiguator__set_state(((struct __pyx_obj_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x8b03656, 0xbc895e2, 0x8b1515e) = (morphologicalAnalyzer))" % __pyx_checksum)
 *     __pyx_result = AutoDisambiguator.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_AutoDisambiguator__set_state(<AutoDisambiguator> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_AutoDisambiguator__set_state(<AutoDisambiguator> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_AutoDisambiguator__set_state(AutoDisambiguator __pyx_result, tuple __pyx_state):
 *     __pyx_result.morphologicalAnalyzer = __pyx_state[0]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_AutoDisambiguator(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.__pyx_unpickle_AutoDisambiguator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_AutoDisambiguator__set_state(<AutoDisambiguator> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_AutoDisambiguator__set_state(AutoDisambiguator __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.morphologicalAnalyzer = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_27MorphologicalDisambiguation_17AutoDisambiguator___pyx_unpickle_AutoDisambiguator__set_state(struct __pyx_obj_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_AutoDisambiguator__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_AutoDisambiguator__set_state(AutoDisambiguator __pyx_result, tuple __pyx_state):
 *     __pyx_result.morphologicalAnalyzer = __pyx_state[0]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->morphologicalAnalyzer);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->morphologicalAnalyzer));
  __pyx_v___pyx_result->morphologicalAnalyzer = ((struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_AutoDisambiguator__set_state(AutoDisambiguator __pyx_result, tuple __pyx_state):
 *     __pyx_result.morphologicalAnalyzer = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 1) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.morphologicalAnalyzer = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[1])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_AutoDisambiguator__set_state(AutoDisambiguator __pyx_result, tuple __pyx_state):
 *     __pyx_result.morphologicalAnalyzer = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_AutoDisambiguator__set_state(<AutoDisambiguator> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_AutoDisambiguator__set_state(AutoDisambiguator __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.morphologicalAnalyzer = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("MorphologicalDisambiguation.AutoDisambiguator.__pyx_unpickle_AutoDisambiguator__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_tp_new_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator *)o);
  p->morphologicalAnalyzer = ((struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator(PyObject *o) {
  struct __pyx_obj_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator *p = (struct __pyx_obj_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->morphologicalAnalyzer);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator *p = (struct __pyx_obj_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator *)o;
  if (p->morphologicalAnalyzer) {
    e = (*v)(((PyObject *)p->morphologicalAnalyzer), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator *p = (struct __pyx_obj_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator *)o;
  tmp = ((PyObject*)p->morphologicalAnalyzer);
  p->morphologicalAnalyzer = ((struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator[] = {
  {"isAnyWordSecondPerson", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_1isAnyWordSecondPerson, METH_VARARGS|METH_KEYWORDS, 0},
  {"isPossessivePlural", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_3isPossessivePlural, METH_VARARGS|METH_KEYWORDS, 0},
  {"nextWordPos", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_5nextWordPos, METH_VARARGS|METH_KEYWORDS, 0},
  {"isBeforeLastWord", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_7isBeforeLastWord, METH_VARARGS|METH_KEYWORDS, 0},
  {"nextWordExists", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_9nextWordExists, METH_VARARGS|METH_KEYWORDS, 0},
  {"isNextWordNoun", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_11isNextWordNoun, METH_VARARGS|METH_KEYWORDS, 0},
  {"isNextWordNum", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_13isNextWordNum, METH_VARARGS|METH_KEYWORDS, 0},
  {"isNextWordNounOrAdjective", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_15isNextWordNounOrAdjective, METH_VARARGS|METH_KEYWORDS, 0},
  {"isFirstWord", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_17isFirstWord, METH_VARARGS|METH_KEYWORDS, 0},
  {"containsTwoNeOrYa", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_19containsTwoNeOrYa, METH_VARARGS|METH_KEYWORDS, 0},
  {"hasPreviousWordTag", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_21hasPreviousWordTag, METH_VARARGS|METH_KEYWORDS, 0},
  {"selectCaseForParseString", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_23selectCaseForParseString, METH_VARARGS|METH_KEYWORDS, 0},
  {"caseDisambiguator", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_25caseDisambiguator, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_27__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_29__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator = {
  PyVarObject_HEAD_INIT(0, 0)
  "MorphologicalDisambiguation.AutoDisambiguator.AutoDisambiguator", /*tp_name*/
  sizeof(struct __pyx_obj_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator, /*tp_traverse*/
  __pyx_tp_clear_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_AutoDisambiguator(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_AutoDisambiguator},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "AutoDisambiguator",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_u_, __pyx_k_, sizeof(__pyx_k_), 0, 1, 0, 0},
  {&__pyx_kp_u_A1PL_P1PL_NOM, __pyx_k_A1PL_P1PL_NOM, sizeof(__pyx_k_A1PL_P1PL_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_A1PL_P1PL_NOM_A3SG_P3SG_GEN_DB_V, __pyx_k_A1PL_P1PL_NOM_A3SG_P3SG_GEN_DB_V, sizeof(__pyx_k_A1PL_P1PL_NOM_A3SG_P3SG_GEN_DB_V), 0, 1, 0, 0},
  {&__pyx_n_s_A2SG, __pyx_k_A2SG, sizeof(__pyx_k_A2SG), 0, 0, 1, 1},
  {&__pyx_kp_u_A2SG_P2SG, __pyx_k_A2SG_P2SG, sizeof(__pyx_k_A2SG_P2SG), 0, 1, 0, 0},
  {&__pyx_kp_u_A2SG_P2SG_A3SG_P3SG, __pyx_k_A2SG_P2SG_A3SG_P3SG, sizeof(__pyx_k_A2SG_P2SG_A3SG_P3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_A3PL_P2SG, __pyx_k_A3PL_P2SG, sizeof(__pyx_k_A3PL_P2SG), 0, 1, 0, 0},
  {&__pyx_kp_u_A3PL_P2SG_A3PL_P3PL_A3PL_P3SG_A3, __pyx_k_A3PL_P2SG_A3PL_P3PL_A3PL_P3SG_A3, sizeof(__pyx_k_A3PL_P2SG_A3PL_P3PL_A3PL_P3SG_A3), 0, 1, 0, 0},
  {&__pyx_kp_u_A3PL_P3PL_A3PL_P3SG_A3SG_P3PL, __pyx_k_A3PL_P3PL_A3PL_P3SG_A3SG_P3PL, sizeof(__pyx_k_A3PL_P3PL_A3PL_P3SG_A3SG_P3PL), 0, 1, 0, 0},
  {&__pyx_kp_u_A3PL_P3PL_NOM_A3PL_P3SG_NOM_A3PL, __pyx_k_A3PL_P3PL_NOM_A3PL_P3SG_NOM_A3PL, sizeof(__pyx_k_A3PL_P3PL_NOM_A3PL_P3SG_NOM_A3PL), 0, 1, 0, 0},
  {&__pyx_kp_u_A3PL_P3SG, __pyx_k_A3PL_P3SG, sizeof(__pyx_k_A3PL_P3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_A3PL_P3SG_A3SG_P3PL_PROP_A3PL_P3, __pyx_k_A3PL_P3SG_A3SG_P3PL_PROP_A3PL_P3, sizeof(__pyx_k_A3PL_P3SG_A3SG_P3PL_PROP_A3PL_P3), 0, 1, 0, 0},
  {&__pyx_kp_u_A3PL_P3SG_NOM, __pyx_k_A3PL_P3SG_NOM, sizeof(__pyx_k_A3PL_P3SG_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_A3PL_PNON_NOM, __pyx_k_A3PL_PNON_NOM, sizeof(__pyx_k_A3PL_PNON_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_A3PL_PNON_NOM_A3SG_PNON_NOM_DB_V, __pyx_k_A3PL_PNON_NOM_A3SG_PNON_NOM_DB_V, sizeof(__pyx_k_A3PL_PNON_NOM_A3SG_PNON_NOM_DB_V), 0, 1, 0, 0},
  {&__pyx_kp_u_A3PL_PNON_NOM_A3SG_PNON_NOM_DB_V_2, __pyx_k_A3PL_PNON_NOM_A3SG_PNON_NOM_DB_V_2, sizeof(__pyx_k_A3PL_PNON_NOM_A3SG_PNON_NOM_DB_V_2), 0, 1, 0, 0},
  {&__pyx_kp_u_A3PL_PNON_NOM_DB_ADV_SINCE, __pyx_k_A3PL_PNON_NOM_DB_ADV_SINCE, sizeof(__pyx_k_A3PL_PNON_NOM_DB_ADV_SINCE), 0, 1, 0, 0},
  {&__pyx_kp_u_A3PL_PNON_NOM_DB_ADV_SINCE_A3PL, __pyx_k_A3PL_PNON_NOM_DB_ADV_SINCE_A3PL, sizeof(__pyx_k_A3PL_PNON_NOM_DB_ADV_SINCE_A3PL), 0, 1, 0, 0},
  {&__pyx_kp_u_A3PL_PNON_NOM_DB_VERB_ZERO_PRES, __pyx_k_A3PL_PNON_NOM_DB_VERB_ZERO_PRES, sizeof(__pyx_k_A3PL_PNON_NOM_DB_VERB_ZERO_PRES), 0, 1, 0, 0},
  {&__pyx_kp_u_A3SG_P3PL, __pyx_k_A3SG_P3PL, sizeof(__pyx_k_A3SG_P3PL), 0, 1, 0, 0},
  {&__pyx_kp_u_A3SG_P3PL_NOM, __pyx_k_A3SG_P3PL_NOM, sizeof(__pyx_k_A3SG_P3PL_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_A3SG_P3SG, __pyx_k_A3SG_P3SG, sizeof(__pyx_k_A3SG_P3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_A3SG_PNON_ACC, __pyx_k_A3SG_PNON_ACC, sizeof(__pyx_k_A3SG_PNON_ACC), 0, 1, 0, 0},
  {&__pyx_kp_u_A3SG_PNON_ACC_PROP_A3SG_PNON_NOM, __pyx_k_A3SG_PNON_ACC_PROP_A3SG_PNON_NOM, sizeof(__pyx_k_A3SG_PNON_ACC_PROP_A3SG_PNON_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_A3SG_PROP_A3SG, __pyx_k_A3SG_PROP_A3SG, sizeof(__pyx_k_A3SG_PROP_A3SG), 0, 1, 0, 0},
  {&__pyx_n_s_ABLATIVE, __pyx_k_ABLATIVE, sizeof(__pyx_k_ABLATIVE), 0, 0, 1, 1},
  {&__pyx_kp_u_ABLE_DB_ADJ_FUTPART_PNON, __pyx_k_ABLE_DB_ADJ_FUTPART_PNON, sizeof(__pyx_k_ABLE_DB_ADJ_FUTPART_PNON), 0, 1, 0, 0},
  {&__pyx_kp_u_ABLE_FUT_A3SG, __pyx_k_ABLE_FUT_A3SG, sizeof(__pyx_k_ABLE_FUT_A3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_ABLE_FUT_A3SG_ABLE_DB_ADJ_FUTPAR, __pyx_k_ABLE_FUT_A3SG_ABLE_DB_ADJ_FUTPAR, sizeof(__pyx_k_ABLE_FUT_A3SG_ABLE_DB_ADJ_FUTPAR), 0, 1, 0, 0},
  {&__pyx_n_u_ADJ, __pyx_k_ADJ, sizeof(__pyx_k_ADJ), 0, 1, 0, 1},
  {&__pyx_kp_u_ADJ_ADV, __pyx_k_ADJ_ADV, sizeof(__pyx_k_ADJ_ADV), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_ADV_CONJ_PRON_QUESP_A3SG_PNO, __pyx_k_ADJ_ADV_CONJ_PRON_QUESP_A3SG_PNO, sizeof(__pyx_k_ADJ_ADV_CONJ_PRON_QUESP_A3SG_PNO), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_ADV_DET_NUM_CARD, __pyx_k_ADJ_ADV_DET_NUM_CARD, sizeof(__pyx_k_ADJ_ADV_DET_NUM_CARD), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_ADV_DET_POSTP_PCABL, __pyx_k_ADJ_ADV_DET_POSTP_PCABL, sizeof(__pyx_k_ADJ_ADV_DET_POSTP_PCABL), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_ADV_NOUN_A3SG_PNON_NOM, __pyx_k_ADJ_ADV_NOUN_A3SG_PNON_NOM, sizeof(__pyx_k_ADJ_ADV_NOUN_A3SG_PNON_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_ADV_NOUN_A3SG_PNON_NOM_NOUN, __pyx_k_ADJ_ADV_NOUN_A3SG_PNON_NOM_NOUN, sizeof(__pyx_k_ADJ_ADV_NOUN_A3SG_PNON_NOM_NOUN), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_ADV_NOUN_A3SG_PNON_NOM_POSTP, __pyx_k_ADJ_ADV_NOUN_A3SG_PNON_NOM_POSTP, sizeof(__pyx_k_ADJ_ADV_NOUN_A3SG_PNON_NOM_POSTP), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_ADV_NOUN_A3SG_PNON_NOM_POSTP_2, __pyx_k_ADJ_ADV_NOUN_A3SG_PNON_NOM_POSTP_2, sizeof(__pyx_k_ADJ_ADV_NOUN_A3SG_PNON_NOM_POSTP_2), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_ADV_POSTP_PCABL, __pyx_k_ADJ_ADV_POSTP_PCABL, sizeof(__pyx_k_ADJ_ADV_POSTP_PCABL), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_ADV_POSTP_PCABL_VERB_POS_IMP, __pyx_k_ADJ_ADV_POSTP_PCABL_VERB_POS_IMP, sizeof(__pyx_k_ADJ_ADV_POSTP_PCABL_VERB_POS_IMP), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_ADV_POSTP_PCINS, __pyx_k_ADJ_ADV_POSTP_PCINS, sizeof(__pyx_k_ADJ_ADV_POSTP_PCINS), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_ADV_VERB_POS_IMP_A2SG, __pyx_k_ADJ_ADV_VERB_POS_IMP_A2SG, sizeof(__pyx_k_ADJ_ADV_VERB_POS_IMP_A2SG), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_AGT, __pyx_k_ADJ_AGT, sizeof(__pyx_k_ADJ_AGT), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_AGT_DB_ADJ_JUSTLIKE_NOUN_AGT, __pyx_k_ADJ_AGT_DB_ADJ_JUSTLIKE_NOUN_AGT, sizeof(__pyx_k_ADJ_AGT_DB_ADJ_JUSTLIKE_NOUN_AGT), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_AGT_NOUN_AGT_A3SG_PNON_NOM, __pyx_k_ADJ_AGT_NOUN_AGT_A3SG_PNON_NOM, sizeof(__pyx_k_ADJ_AGT_NOUN_AGT_A3SG_PNON_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_ASIF, __pyx_k_ADJ_ASIF, sizeof(__pyx_k_ADJ_ASIF), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_ASIF_ADV_LY, __pyx_k_ADJ_ASIF_ADV_LY, sizeof(__pyx_k_ADJ_ASIF_ADV_LY), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_ASIF_ADV_LY_NOUN_ZERO_A3SG_P, __pyx_k_ADJ_ASIF_ADV_LY_NOUN_ZERO_A3SG_P, sizeof(__pyx_k_ADJ_ASIF_ADV_LY_NOUN_ZERO_A3SG_P), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB, __pyx_k_ADJ_DB, sizeof(__pyx_k_ADJ_DB), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_NOUN_A3SG_PNON_NOM_DB, __pyx_k_ADJ_DB_NOUN_A3SG_PNON_NOM_DB, sizeof(__pyx_k_ADJ_DB_NOUN_A3SG_PNON_NOM_DB), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_NOUN_AGT_A3SG_PNON_NOM, __pyx_k_ADJ_DB_NOUN_AGT_A3SG_PNON_NOM, sizeof(__pyx_k_ADJ_DB_NOUN_AGT_A3SG_PNON_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_NOUN_AGT_A3SG_PNON_NOM_NO, __pyx_k_ADJ_DB_NOUN_AGT_A3SG_PNON_NOM_NO, sizeof(__pyx_k_ADJ_DB_NOUN_AGT_A3SG_PNON_NOM_NO), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_NOUN_ZERO, __pyx_k_ADJ_DB_NOUN_ZERO, sizeof(__pyx_k_ADJ_DB_NOUN_ZERO), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_NOUN_ZERO_A3PL_PNON_NOM_A, __pyx_k_ADJ_DB_NOUN_ZERO_A3PL_PNON_NOM_A, sizeof(__pyx_k_ADJ_DB_NOUN_ZERO_A3PL_PNON_NOM_A), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_NOUN_ZERO_A3SG_P2SG_ADJ_D, __pyx_k_ADJ_DB_NOUN_ZERO_A3SG_P2SG_ADJ_D, sizeof(__pyx_k_ADJ_DB_NOUN_ZERO_A3SG_P2SG_ADJ_D), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_NOUN_ZERO_A3SG_P2SG_ADJ_D_2, __pyx_k_ADJ_DB_NOUN_ZERO_A3SG_P2SG_ADJ_D_2, sizeof(__pyx_k_ADJ_DB_NOUN_ZERO_A3SG_P2SG_ADJ_D_2), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_NOUN_ZERO_A3SG_P2SG_NOM_D, __pyx_k_ADJ_DB_NOUN_ZERO_A3SG_P2SG_NOM_D, sizeof(__pyx_k_ADJ_DB_NOUN_ZERO_A3SG_P2SG_NOM_D), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_NOUN_ZERO_A3SG_P3SG_NOM, __pyx_k_ADJ_DB_NOUN_ZERO_A3SG_P3SG_NOM, sizeof(__pyx_k_ADJ_DB_NOUN_ZERO_A3SG_P3SG_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_NOUN_ZERO_A3SG_P3SG_NOM_A, __pyx_k_ADJ_DB_NOUN_ZERO_A3SG_P3SG_NOM_A, sizeof(__pyx_k_ADJ_DB_NOUN_ZERO_A3SG_P3SG_NOM_A), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_NOUN_ZERO_A3SG_P3SG_NOM_A_2, __pyx_k_ADJ_DB_NOUN_ZERO_A3SG_P3SG_NOM_A_2, sizeof(__pyx_k_ADJ_DB_NOUN_ZERO_A3SG_P3SG_NOM_A_2), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_NOUN_ZERO_NOUN, __pyx_k_ADJ_DB_NOUN_ZERO_NOUN, sizeof(__pyx_k_ADJ_DB_NOUN_ZERO_NOUN), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_POSTP_PCABL_DB, __pyx_k_ADJ_DB_POSTP_PCABL_DB, sizeof(__pyx_k_ADJ_DB_POSTP_PCABL_DB), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_VERB_ZERO, __pyx_k_ADJ_DB_VERB_ZERO, sizeof(__pyx_k_ADJ_DB_VERB_ZERO), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_VERB_ZERO_COND, __pyx_k_ADJ_DB_VERB_ZERO_COND, sizeof(__pyx_k_ADJ_DB_VERB_ZERO_COND), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_VERB_ZERO_COND_VERB_POS_D, __pyx_k_ADJ_DB_VERB_ZERO_COND_VERB_POS_D, sizeof(__pyx_k_ADJ_DB_VERB_ZERO_COND_VERB_POS_D), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_VERB_ZERO_NARR_A3SG_VERB, __pyx_k_ADJ_DB_VERB_ZERO_NARR_A3SG_VERB, sizeof(__pyx_k_ADJ_DB_VERB_ZERO_NARR_A3SG_VERB), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_VERB_ZERO_NOUN_A3SG_PNON, __pyx_k_ADJ_DB_VERB_ZERO_NOUN_A3SG_PNON, sizeof(__pyx_k_ADJ_DB_VERB_ZERO_NOUN_A3SG_PNON), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_VERB_ZERO_PAST_A1PL_VERB, __pyx_k_ADJ_DB_VERB_ZERO_PAST_A1PL_VERB, sizeof(__pyx_k_ADJ_DB_VERB_ZERO_PAST_A1PL_VERB), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_VERB_ZERO_POSTP_PCABL_DB, __pyx_k_ADJ_DB_VERB_ZERO_POSTP_PCABL_DB, sizeof(__pyx_k_ADJ_DB_VERB_ZERO_POSTP_PCABL_DB), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_VERB_ZERO_PRES_A1SG_NOUN, __pyx_k_ADJ_DB_VERB_ZERO_PRES_A1SG_NOUN, sizeof(__pyx_k_ADJ_DB_VERB_ZERO_PRES_A1SG_NOUN), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_VERB_ZERO_PRES_A3PL_NOUN, __pyx_k_ADJ_DB_VERB_ZERO_PRES_A3PL_NOUN, sizeof(__pyx_k_ADJ_DB_VERB_ZERO_PRES_A3PL_NOUN), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_VERB_ZERO_PRES_A3PL_NOUN_2, __pyx_k_ADJ_DB_VERB_ZERO_PRES_A3PL_NOUN_2, sizeof(__pyx_k_ADJ_DB_VERB_ZERO_PRES_A3PL_NOUN_2), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_VERB_ZERO_PRES_COP_A3SG, __pyx_k_ADJ_DB_VERB_ZERO_PRES_COP_A3SG, sizeof(__pyx_k_ADJ_DB_VERB_ZERO_PRES_COP_A3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_VERB_ZERO_PRES_COP_A3SG_N, __pyx_k_ADJ_DB_VERB_ZERO_PRES_COP_A3SG_N, sizeof(__pyx_k_ADJ_DB_VERB_ZERO_PRES_COP_A3SG_N), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_VERB_ZERO_PRES_COP_A3SG_V, __pyx_k_ADJ_DB_VERB_ZERO_PRES_COP_A3SG_V, sizeof(__pyx_k_ADJ_DB_VERB_ZERO_PRES_COP_A3SG_V), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DB_VERB_ZERO_VERB_POS, __pyx_k_ADJ_DB_VERB_ZERO_VERB_POS, sizeof(__pyx_k_ADJ_DB_VERB_ZERO_VERB_POS), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_DET_PRON_QUANTP_A3SG_P3SG_NO, __pyx_k_ADJ_DET_PRON_QUANTP_A3SG_P3SG_NO, sizeof(__pyx_k_ADJ_DET_PRON_QUANTP_A3SG_P3SG_NO), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_FUTPART_P2PL, __pyx_k_ADJ_FUTPART_P2PL, sizeof(__pyx_k_ADJ_FUTPART_P2PL), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_FUTPART_P2PL_NOUN_FUTPART_A3, __pyx_k_ADJ_FUTPART_P2PL_NOUN_FUTPART_A3, sizeof(__pyx_k_ADJ_FUTPART_P2PL_NOUN_FUTPART_A3), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_FUTPART_P3PL, __pyx_k_ADJ_FUTPART_P3PL, sizeof(__pyx_k_ADJ_FUTPART_P3PL), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_FUTPART_P3PL_NOUN_FUTPART_A3, __pyx_k_ADJ_FUTPART_P3PL_NOUN_FUTPART_A3, sizeof(__pyx_k_ADJ_FUTPART_P3PL_NOUN_FUTPART_A3), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_FUTPART_P3SG, __pyx_k_ADJ_FUTPART_P3SG, sizeof(__pyx_k_ADJ_FUTPART_P3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_FUTPART_P3SG_NOUN_FUTPART_A3, __pyx_k_ADJ_FUTPART_P3SG_NOUN_FUTPART_A3, sizeof(__pyx_k_ADJ_FUTPART_P3SG_NOUN_FUTPART_A3), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_INTERJ, __pyx_k_ADJ_INTERJ, sizeof(__pyx_k_ADJ_INTERJ), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_INTERJ_NOUN_PROP_A3SG_PNON_N, __pyx_k_ADJ_INTERJ_NOUN_PROP_A3SG_PNON_N, sizeof(__pyx_k_ADJ_INTERJ_NOUN_PROP_A3SG_PNON_N), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_JUSTLIKE_DB_NOUN_ZERO_A3SG_P, __pyx_k_ADJ_JUSTLIKE_DB_NOUN_ZERO_A3SG_P, sizeof(__pyx_k_ADJ_JUSTLIKE_DB_NOUN_ZERO_A3SG_P), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_JUSTLIKE_DB_NOUN_ZERO_A3SG_P_2, __pyx_k_ADJ_JUSTLIKE_DB_NOUN_ZERO_A3SG_P_2, sizeof(__pyx_k_ADJ_JUSTLIKE_DB_NOUN_ZERO_A3SG_P_2), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_JUSTLIKE_DB_NOUN_ZERO_A3SG_P_3, __pyx_k_ADJ_JUSTLIKE_DB_NOUN_ZERO_A3SG_P_3, sizeof(__pyx_k_ADJ_JUSTLIKE_DB_NOUN_ZERO_A3SG_P_3), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_JUSTLIKE_NOUN_ZERO_A3SG_P3SG, __pyx_k_ADJ_JUSTLIKE_NOUN_ZERO_A3SG_P3SG, sizeof(__pyx_k_ADJ_JUSTLIKE_NOUN_ZERO_A3SG_P3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_JUSTLIKE_NOUN_ZERO_A3SG_P3SG_2, __pyx_k_ADJ_JUSTLIKE_NOUN_ZERO_A3SG_P3SG_2, sizeof(__pyx_k_ADJ_JUSTLIKE_NOUN_ZERO_A3SG_P3SG_2), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_NOUN_A3SG_PNON_NOM, __pyx_k_ADJ_NOUN_A3SG_PNON_NOM, sizeof(__pyx_k_ADJ_NOUN_A3SG_PNON_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_NOUN_A3SG_PNON_NOM_NOUN_PROP, __pyx_k_ADJ_NOUN_A3SG_PNON_NOM_NOUN_PROP, sizeof(__pyx_k_ADJ_NOUN_A3SG_PNON_NOM_NOUN_PROP), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_NOUN_A3SG_PNON_NOM_NOUN_PROP_2, __pyx_k_ADJ_NOUN_A3SG_PNON_NOM_NOUN_PROP_2, sizeof(__pyx_k_ADJ_NOUN_A3SG_PNON_NOM_NOUN_PROP_2), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_NOUN_A3SG_PNON_NOM_NOUN_PROP_3, __pyx_k_ADJ_NOUN_A3SG_PNON_NOM_NOUN_PROP_3, sizeof(__pyx_k_ADJ_NOUN_A3SG_PNON_NOM_NOUN_PROP_3), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_NOUN_A3SG_PNON_NOM_POSTP_PCD, __pyx_k_ADJ_NOUN_A3SG_PNON_NOM_POSTP_PCD, sizeof(__pyx_k_ADJ_NOUN_A3SG_PNON_NOM_POSTP_PCD), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_NOUN_A3SG_PNON_NOM_VERB_POS, __pyx_k_ADJ_NOUN_A3SG_PNON_NOM_VERB_POS, sizeof(__pyx_k_ADJ_NOUN_A3SG_PNON_NOM_VERB_POS), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_NOUN_PROP_A3SG_PNON_NOM, __pyx_k_ADJ_NOUN_PROP_A3SG_PNON_NOM, sizeof(__pyx_k_ADJ_NOUN_PROP_A3SG_PNON_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_NUM_ORD, __pyx_k_ADJ_NUM_ORD, sizeof(__pyx_k_ADJ_NUM_ORD), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_PASTPART_P1PL, __pyx_k_ADJ_PASTPART_P1PL, sizeof(__pyx_k_ADJ_PASTPART_P1PL), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_PASTPART_P1PL_NOUN_PASTPART, __pyx_k_ADJ_PASTPART_P1PL_NOUN_PASTPART, sizeof(__pyx_k_ADJ_PASTPART_P1PL_NOUN_PASTPART), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_PASTPART_P1SG, __pyx_k_ADJ_PASTPART_P1SG, sizeof(__pyx_k_ADJ_PASTPART_P1SG), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_PASTPART_P1SG_NOUN_PASTPART, __pyx_k_ADJ_PASTPART_P1SG_NOUN_PASTPART, sizeof(__pyx_k_ADJ_PASTPART_P1SG_NOUN_PASTPART), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_PASTPART_P2PL, __pyx_k_ADJ_PASTPART_P2PL, sizeof(__pyx_k_ADJ_PASTPART_P2PL), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_PASTPART_P2PL_NOUN_PASTPART, __pyx_k_ADJ_PASTPART_P2PL_NOUN_PASTPART, sizeof(__pyx_k_ADJ_PASTPART_P2PL_NOUN_PASTPART), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_PASTPART_P3PL, __pyx_k_ADJ_PASTPART_P3PL, sizeof(__pyx_k_ADJ_PASTPART_P3PL), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_PASTPART_P3PL_NOUN_PASTPART, __pyx_k_ADJ_PASTPART_P3PL_NOUN_PASTPART, sizeof(__pyx_k_ADJ_PASTPART_P3PL_NOUN_PASTPART), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_PASTPART_P3SG, __pyx_k_ADJ_PASTPART_P3SG, sizeof(__pyx_k_ADJ_PASTPART_P3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_PASTPART_P3SG_NOUN_PASTPART, __pyx_k_ADJ_PASTPART_P3SG_NOUN_PASTPART, sizeof(__pyx_k_ADJ_PASTPART_P3SG_NOUN_PASTPART), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_POSTP_PCABL, __pyx_k_ADJ_POSTP_PCABL, sizeof(__pyx_k_ADJ_POSTP_PCABL), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_PRON_QUESP_A3SG_PNON_NOM, __pyx_k_ADJ_PRON_QUESP_A3SG_PNON_NOM, sizeof(__pyx_k_ADJ_PRON_QUESP_A3SG_PNON_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_ADJ_VERB_POS_IMP_A2SG, __pyx_k_ADJ_VERB_POS_IMP_A2SG, sizeof(__pyx_k_ADJ_VERB_POS_IMP_A2SG), 0, 1, 0, 0},
  {&__pyx_n_u_ADV, __pyx_k_ADV, sizeof(__pyx_k_ADV), 0, 1, 0, 1},
  {&__pyx_kp_u_ADV_CONJ, __pyx_k_ADV_CONJ, sizeof(__pyx_k_ADV_CONJ), 0, 1, 0, 0},
  {&__pyx_kp_u_ADV_LY, __pyx_k_ADV_LY, sizeof(__pyx_k_ADV_LY), 0, 1, 0, 0},
  {&__pyx_kp_u_ADV_NOUN_A3SG_PNON_NOM, __pyx_k_ADV_NOUN_A3SG_PNON_NOM, sizeof(__pyx_k_ADV_NOUN_A3SG_PNON_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_ADV_NOUN_A3SG_PNON_NOM_POSTP_PCA, __pyx_k_ADV_NOUN_A3SG_PNON_NOM_POSTP_PCA, sizeof(__pyx_k_ADV_NOUN_A3SG_PNON_NOM_POSTP_PCA), 0, 1, 0, 0},
  {&__pyx_kp_u_ADV_POSTP_PCINS, __pyx_k_ADV_POSTP_PCINS, sizeof(__pyx_k_ADV_POSTP_PCINS), 0, 1, 0, 0},
  {&__pyx_kp_u_ADV_PRON_QUANTP_A3SG_P3SG_NOM, __pyx_k_ADV_PRON_QUANTP_A3SG_P3SG_NOM, sizeof(__pyx_k_ADV_PRON_QUANTP_A3SG_P3SG_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_ADV_SINCE, __pyx_k_ADV_SINCE, sizeof(__pyx_k_ADV_SINCE), 0, 1, 0, 0},
  {&__pyx_kp_u_ADV_SINCE_VERB_ZERO_PRES_COP_A3S, __pyx_k_ADV_SINCE_VERB_ZERO_PRES_COP_A3S, sizeof(__pyx_k_ADV_SINCE_VERB_ZERO_PRES_COP_A3S), 0, 1, 0, 0},
  {&__pyx_kp_u_ADV_VERB_POS_IMP_A2SG, __pyx_k_ADV_VERB_POS_IMP_A2SG, sizeof(__pyx_k_ADV_VERB_POS_IMP_A2SG), 0, 1, 0, 0},
  {&__pyx_kp_u_ADV_WITHOUTHAVINGDONESO, __pyx_k_ADV_WITHOUTHAVINGDONESO, sizeof(__pyx_k_ADV_WITHOUTHAVINGDONESO), 0, 1, 0, 0},
  {&__pyx_kp_u_ADV_WITHOUTHAVINGDONESO_NOUN_INF, __pyx_k_ADV_WITHOUTHAVINGDONESO_NOUN_INF, sizeof(__pyx_k_ADV_WITHOUTHAVINGDONESO_NOUN_INF), 0, 1, 0, 0},
  {&__pyx_kp_u_AOR_A1SG, __pyx_k_AOR_A1SG, sizeof(__pyx_k_AOR_A1SG), 0, 1, 0, 0},
  {&__pyx_kp_u_AOR_A1SG_AOR_DB_ADJ_ZERO_DB_NOUN, __pyx_k_AOR_A1SG_AOR_DB_ADJ_ZERO_DB_NOUN, sizeof(__pyx_k_AOR_A1SG_AOR_DB_ADJ_ZERO_DB_NOUN), 0, 1, 0, 0},
  {&__pyx_kp_u_AOR_A2PL, __pyx_k_AOR_A2PL, sizeof(__pyx_k_AOR_A2PL), 0, 1, 0, 0},
  {&__pyx_kp_u_AOR_A2PL_AOR_DB_ADJ_ZERO_DB_ADJ, __pyx_k_AOR_A2PL_AOR_DB_ADJ_ZERO_DB_ADJ, sizeof(__pyx_k_AOR_A2PL_AOR_DB_ADJ_ZERO_DB_ADJ), 0, 1, 0, 0},
  {&__pyx_kp_u_AOR_A2SG, __pyx_k_AOR_A2SG, sizeof(__pyx_k_AOR_A2SG), 0, 1, 0, 0},
  {&__pyx_kp_u_AOR_A2SG_AOR_DB_ADJ_ZERO_DB_ADJ, __pyx_k_AOR_A2SG_AOR_DB_ADJ_ZERO_DB_ADJ, sizeof(__pyx_k_AOR_A2SG_AOR_DB_ADJ_ZERO_DB_ADJ), 0, 1, 0, 0},
  {&__pyx_kp_u_AOR_A3PL, __pyx_k_AOR_A3PL, sizeof(__pyx_k_AOR_A3PL), 0, 1, 0, 0},
  {&__pyx_kp_u_AOR_A3PL_AOR_DB_ADJ_ZERO_DB_NOUN, __pyx_k_AOR_A3PL_AOR_DB_ADJ_ZERO_DB_NOUN, sizeof(__pyx_k_AOR_A3PL_AOR_DB_ADJ_ZERO_DB_NOUN), 0, 1, 0, 0},
  {&__pyx_kp_u_AOR_A3SG, __pyx_k_AOR_A3SG, sizeof(__pyx_k_AOR_A3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_AOR_A3SG_AOR_DB_ADJ_ZERO, __pyx_k_AOR_A3SG_AOR_DB_ADJ_ZERO, sizeof(__pyx_k_AOR_A3SG_AOR_DB_ADJ_ZERO), 0, 1, 0, 0},
  {&__pyx_kp_u_AOR_DB_ADJ_ZERO, __pyx_k_AOR_DB_ADJ_ZERO, sizeof(__pyx_k_AOR_DB_ADJ_ZERO), 0, 1, 0, 0},
  {&__pyx_n_s_AutoDisambiguator, __pyx_k_AutoDisambiguator, sizeof(__pyx_k_AutoDisambiguator), 0, 0, 1, 1},
  {&__pyx_n_u_CONJ, __pyx_k_CONJ, sizeof(__pyx_k_CONJ), 0, 1, 0, 1},
  {&__pyx_kp_u_CONJ_INTERJ, __pyx_k_CONJ_INTERJ, sizeof(__pyx_k_CONJ_INTERJ), 0, 1, 0, 0},
  {&__pyx_kp_u_CONJ_NOUN_A3SG_PNON_NOM_VERB_POS, __pyx_k_CONJ_NOUN_A3SG_PNON_NOM_VERB_POS, sizeof(__pyx_k_CONJ_NOUN_A3SG_PNON_NOM_VERB_POS), 0, 1, 0, 0},
  {&__pyx_kp_u_CONJ_NOUN_PROP_A3SG_PNON_NOM_VER, __pyx_k_CONJ_NOUN_PROP_A3SG_PNON_NOM_VER, sizeof(__pyx_k_CONJ_NOUN_PROP_A3SG_PNON_NOM_VER), 0, 1, 0, 0},
  {&__pyx_kp_u_CONJ_POSTP_PCNOM, __pyx_k_CONJ_POSTP_PCNOM, sizeof(__pyx_k_CONJ_POSTP_PCNOM), 0, 1, 0, 0},
  {&__pyx_kp_u_CONJ_VERB_POS_IMP_A2SG, __pyx_k_CONJ_VERB_POS_IMP_A2SG, sizeof(__pyx_k_CONJ_VERB_POS_IMP_A2SG), 0, 1, 0, 0},
  {&__pyx_kp_u_DATE_NUM_FRACTION, __pyx_k_DATE_NUM_FRACTION, sizeof(__pyx_k_DATE_NUM_FRACTION), 0, 1, 0, 0},
  {&__pyx_n_s_DATIVE, __pyx_k_DATIVE, sizeof(__pyx_k_DATIVE), 0, 0, 1, 1},
  {&__pyx_kp_u_DEMONSP_PERS, __pyx_k_DEMONSP_PERS, sizeof(__pyx_k_DEMONSP_PERS), 0, 1, 0, 0},
  {&__pyx_n_u_DET, __pyx_k_DET, sizeof(__pyx_k_DET), 0, 1, 0, 1},
  {&__pyx_kp_u_DET_NOUN_A3SG_PNON_NOM, __pyx_k_DET_NOUN_A3SG_PNON_NOM, sizeof(__pyx_k_DET_NOUN_A3SG_PNON_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_DET_NOUN_PROP_A3SG_PNON_NOM_PRON, __pyx_k_DET_NOUN_PROP_A3SG_PNON_NOM_PRON, sizeof(__pyx_k_DET_NOUN_PROP_A3SG_PNON_NOM_PRON), 0, 1, 0, 0},
  {&__pyx_kp_u_DET_PRON_DEMONSP_A3SG_PNON_NOM, __pyx_k_DET_PRON_DEMONSP_A3SG_PNON_NOM, sizeof(__pyx_k_DET_PRON_DEMONSP_A3SG_PNON_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_DET_PRON_DEMONSP_A3SG_PNON_NOM_P, __pyx_k_DET_PRON_DEMONSP_A3SG_PNON_NOM_P, sizeof(__pyx_k_DET_PRON_DEMONSP_A3SG_PNON_NOM_P), 0, 1, 0, 0},
  {&__pyx_kp_u_INF2_A3SG_P3SG_NOM, __pyx_k_INF2_A3SG_P3SG_NOM, sizeof(__pyx_k_INF2_A3SG_P3SG_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_INF2_A3SG_P3SG_NOM_INF2_DB_ADJ_A, __pyx_k_INF2_A3SG_P3SG_NOM_INF2_DB_ADJ_A, sizeof(__pyx_k_INF2_A3SG_P3SG_NOM_INF2_DB_ADJ_A), 0, 1, 0, 0},
  {&__pyx_n_s_INSTRUMENTAL, __pyx_k_INSTRUMENTAL, sizeof(__pyx_k_INSTRUMENTAL), 0, 0, 1, 1},
  {&__pyx_n_u_INTERJ, __pyx_k_INTERJ, sizeof(__pyx_k_INTERJ), 0, 1, 0, 1},
  {&__pyx_kp_u_INTERJ_NOUN_A3SG_PNON_NOM_VERB_P, __pyx_k_INTERJ_NOUN_A3SG_PNON_NOM_VERB_P, sizeof(__pyx_k_INTERJ_NOUN_A3SG_PNON_NOM_VERB_P), 0, 1, 0, 0},
  {&__pyx_kp_s_Incompatible_checksums_0x_x_vs_0, __pyx_k_Incompatible_checksums_0x_x_vs_0, sizeof(__pyx_k_Incompatible_checksums_0x_x_vs_0), 0, 0, 1, 0},
  {&__pyx_n_s_MorphologicalAnalysis_FsmMorphol, __pyx_k_MorphologicalAnalysis_FsmMorphol, sizeof(__pyx_k_MorphologicalAnalysis_FsmMorphol), 0, 0, 1, 1},
  {&__pyx_kp_s_MorphologicalDisambiguation_Auto, __pyx_k_MorphologicalDisambiguation_Auto, sizeof(__pyx_k_MorphologicalDisambiguation_Auto), 0, 0, 1, 0},
  {&__pyx_n_s_MorphologicalDisambiguation_Auto_2, __pyx_k_MorphologicalDisambiguation_Auto_2, sizeof(__pyx_k_MorphologicalDisambiguation_Auto_2), 0, 0, 1, 1},
  {&__pyx_n_s_MorphologicalTag, __pyx_k_MorphologicalTag, sizeof(__pyx_k_MorphologicalTag), 0, 0, 1, 1},
  {&__pyx_kp_u_NARR_A3SG, __pyx_k_NARR_A3SG, sizeof(__pyx_k_NARR_A3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_NARR_A3SG_NARR_DB_ADJ_ZERO, __pyx_k_NARR_A3SG_NARR_DB_ADJ_ZERO, sizeof(__pyx_k_NARR_A3SG_NARR_DB_ADJ_ZERO), 0, 1, 0, 0},
  {&__pyx_kp_u_NARR_DB_ADJ_ZERO, __pyx_k_NARR_DB_ADJ_ZERO, sizeof(__pyx_k_NARR_DB_ADJ_ZERO), 0, 1, 0, 0},
  {&__pyx_kp_u_NEG_AOR_A1SG, __pyx_k_NEG_AOR_A1SG, sizeof(__pyx_k_NEG_AOR_A1SG), 0, 1, 0, 0},
  {&__pyx_kp_u_NEG_AOR_A1SG_POS_DB_NOUN_INF2_A3, __pyx_k_NEG_AOR_A1SG_POS_DB_NOUN_INF2_A3, sizeof(__pyx_k_NEG_AOR_A1SG_POS_DB_NOUN_INF2_A3), 0, 1, 0, 0},
  {&__pyx_kp_u_NEG_DB_ADJ_PASTPART_PNON, __pyx_k_NEG_DB_ADJ_PASTPART_PNON, sizeof(__pyx_k_NEG_DB_ADJ_PASTPART_PNON), 0, 1, 0, 0},
  {&__pyx_kp_u_NEG_FUT_A1SG, __pyx_k_NEG_FUT_A1SG, sizeof(__pyx_k_NEG_FUT_A1SG), 0, 1, 0, 0},
  {&__pyx_kp_u_NEG_FUT_A1SG_NEG_DB_ADJ_FUTPART, __pyx_k_NEG_FUT_A1SG_NEG_DB_ADJ_FUTPART, sizeof(__pyx_k_NEG_FUT_A1SG_NEG_DB_ADJ_FUTPART), 0, 1, 0, 0},
  {&__pyx_kp_u_NEG_FUT_A3SG, __pyx_k_NEG_FUT_A3SG, sizeof(__pyx_k_NEG_FUT_A3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_NEG_FUT_A3SG_NEG_DB_ADJ_FUTPART, __pyx_k_NEG_FUT_A3SG_NEG_DB_ADJ_FUTPART, sizeof(__pyx_k_NEG_FUT_A3SG_NEG_DB_ADJ_FUTPART), 0, 1, 0, 0},
  {&__pyx_kp_u_NEG_IMP_A2SG_POS_DB_NOUN_INF2_A3, __pyx_k_NEG_IMP_A2SG_POS_DB_NOUN_INF2_A3, sizeof(__pyx_k_NEG_IMP_A2SG_POS_DB_NOUN_INF2_A3), 0, 1, 0, 0},
  {&__pyx_kp_u_NEG_OPT_A3SG_POS_DB_NOUN_INF2_A3, __pyx_k_NEG_OPT_A3SG_POS_DB_NOUN_INF2_A3, sizeof(__pyx_k_NEG_OPT_A3SG_POS_DB_NOUN_INF2_A3), 0, 1, 0, 0},
  {&__pyx_kp_u_NEG_PAST_A1PL, __pyx_k_NEG_PAST_A1PL, sizeof(__pyx_k_NEG_PAST_A1PL), 0, 1, 0, 0},
  {&__pyx_kp_u_NEG_PAST_A1PL_NEG_DB_ADJ_PASTPAR, __pyx_k_NEG_PAST_A1PL_NEG_DB_ADJ_PASTPAR, sizeof(__pyx_k_NEG_PAST_A1PL_NEG_DB_ADJ_PASTPAR), 0, 1, 0, 0},
  {&__pyx_kp_u_NEG_PROG2, __pyx_k_NEG_PROG2, sizeof(__pyx_k_NEG_PROG2), 0, 1, 0, 0},
  {&__pyx_kp_u_NEG_PROG2_NEG_DB_NOUN_INF_A3SG_P, __pyx_k_NEG_PROG2_NEG_DB_NOUN_INF_A3SG_P, sizeof(__pyx_k_NEG_PROG2_NEG_DB_NOUN_INF_A3SG_P), 0, 1, 0, 0},
  {&__pyx_kp_u_NESS_A3SG_PNON_NOM_ZERO_A3SG_PNO, __pyx_k_NESS_A3SG_PNON_NOM_ZERO_A3SG_PNO, sizeof(__pyx_k_NESS_A3SG_PNON_NOM_ZERO_A3SG_PNO), 0, 1, 0, 0},
  {&__pyx_n_u_NOUN, __pyx_k_NOUN, sizeof(__pyx_k_NOUN), 0, 1, 0, 1},
  {&__pyx_kp_u_NOUN_A3PL_PNON_NOM, __pyx_k_NOUN_A3PL_PNON_NOM, sizeof(__pyx_k_NOUN_A3PL_PNON_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_NOUN_PROP_A3SG_PRON_PE, __pyx_k_NOUN_A3SG_NOUN_PROP_A3SG_PRON_PE, sizeof(__pyx_k_NOUN_A3SG_NOUN_PROP_A3SG_PRON_PE), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_P1SG_DAT, __pyx_k_NOUN_A3SG_P1SG_DAT, sizeof(__pyx_k_NOUN_A3SG_P1SG_DAT), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_P1SG_DAT_VERB_NEG_IMP, __pyx_k_NOUN_A3SG_P1SG_DAT_VERB_NEG_IMP, sizeof(__pyx_k_NOUN_A3SG_P1SG_DAT_VERB_NEG_IMP), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_P1SG_NOM, __pyx_k_NOUN_A3SG_P1SG_NOM, sizeof(__pyx_k_NOUN_A3SG_P1SG_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_P1SG_NOM_NOUN_A3SG_PNO, __pyx_k_NOUN_A3SG_P1SG_NOM_NOUN_A3SG_PNO, sizeof(__pyx_k_NOUN_A3SG_P1SG_NOM_NOUN_A3SG_PNO), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_P1SG_NOM_NOUN_A3SG_PNO_2, __pyx_k_NOUN_A3SG_P1SG_NOM_NOUN_A3SG_PNO_2, sizeof(__pyx_k_NOUN_A3SG_P1SG_NOM_NOUN_A3SG_PNO_2), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_P1SG_PRON_PERS_A1PL_PN, __pyx_k_NOUN_A3SG_P1SG_PRON_PERS_A1PL_PN, sizeof(__pyx_k_NOUN_A3SG_P1SG_PRON_PERS_A1PL_PN), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_P1SG_PRON_PERS_A1SG_PN, __pyx_k_NOUN_A3SG_P1SG_PRON_PERS_A1SG_PN, sizeof(__pyx_k_NOUN_A3SG_P1SG_PRON_PERS_A1SG_PN), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_P2SG_DAT, __pyx_k_NOUN_A3SG_P2SG_DAT, sizeof(__pyx_k_NOUN_A3SG_P2SG_DAT), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_P2SG_DAT_NOUN_A3SG_P3S, __pyx_k_NOUN_A3SG_P2SG_DAT_NOUN_A3SG_P3S, sizeof(__pyx_k_NOUN_A3SG_P2SG_DAT_NOUN_A3SG_P3S), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_P2SG_GEN_NOUN_A3SG_P3S, __pyx_k_NOUN_A3SG_P2SG_GEN_NOUN_A3SG_P3S, sizeof(__pyx_k_NOUN_A3SG_P2SG_GEN_NOUN_A3SG_P3S), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_P2SG_NOM, __pyx_k_NOUN_A3SG_P2SG_NOM, sizeof(__pyx_k_NOUN_A3SG_P2SG_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_P2SG_NOM_NOUN_A3SG_PNO, __pyx_k_NOUN_A3SG_P2SG_NOM_NOUN_A3SG_PNO, sizeof(__pyx_k_NOUN_A3SG_P2SG_NOM_NOUN_A3SG_PNO), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_P2SG_NOM_NOUN_A3SG_PNO_2, __pyx_k_NOUN_A3SG_P2SG_NOM_NOUN_A3SG_PNO_2, sizeof(__pyx_k_NOUN_A3SG_P2SG_NOM_NOUN_A3SG_PNO_2), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_P2SG_NOM_NOUN_A3SG_PNO_3, __pyx_k_NOUN_A3SG_P2SG_NOM_NOUN_A3SG_PNO_3, sizeof(__pyx_k_NOUN_A3SG_P2SG_NOM_NOUN_A3SG_PNO_3), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_P2SG_NOUN_A3SG_PNON_PR, __pyx_k_NOUN_A3SG_P2SG_NOUN_A3SG_PNON_PR, sizeof(__pyx_k_NOUN_A3SG_P2SG_NOUN_A3SG_PNON_PR), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_P3SG, __pyx_k_NOUN_A3SG_P3SG, sizeof(__pyx_k_NOUN_A3SG_P3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_P3SG_DAT, __pyx_k_NOUN_A3SG_P3SG_DAT, sizeof(__pyx_k_NOUN_A3SG_P3SG_DAT), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_P3SG_GEN, __pyx_k_NOUN_A3SG_P3SG_GEN, sizeof(__pyx_k_NOUN_A3SG_P3SG_GEN), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_P3SG_NOM, __pyx_k_NOUN_A3SG_P3SG_NOM, sizeof(__pyx_k_NOUN_A3SG_P3SG_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_P3SG_NOM_NOUN_A3SG_PNO, __pyx_k_NOUN_A3SG_P3SG_NOM_NOUN_A3SG_PNO, sizeof(__pyx_k_NOUN_A3SG_P3SG_NOM_NOUN_A3SG_PNO), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_P3SG_NOM_NOUN_A3SG_PNO_2, __pyx_k_NOUN_A3SG_P3SG_NOM_NOUN_A3SG_PNO_2, sizeof(__pyx_k_NOUN_A3SG_P3SG_NOM_NOUN_A3SG_PNO_2), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_P3SG_NOM_NOUN_DB_ADJ_A, __pyx_k_NOUN_A3SG_P3SG_NOM_NOUN_DB_ADJ_A, sizeof(__pyx_k_NOUN_A3SG_P3SG_NOM_NOUN_DB_ADJ_A), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PNON_DAT, __pyx_k_NOUN_A3SG_PNON_DAT, sizeof(__pyx_k_NOUN_A3SG_PNON_DAT), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PNON_DAT_VERB_POS_OPT, __pyx_k_NOUN_A3SG_PNON_DAT_VERB_POS_OPT, sizeof(__pyx_k_NOUN_A3SG_PNON_DAT_VERB_POS_OPT), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PNON_GEN, __pyx_k_NOUN_A3SG_PNON_GEN, sizeof(__pyx_k_NOUN_A3SG_PNON_GEN), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PNON_NOM, __pyx_k_NOUN_A3SG_PNON_NOM, sizeof(__pyx_k_NOUN_A3SG_PNON_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB, __pyx_k_NOUN_A3SG_PNON_NOM_DB, sizeof(__pyx_k_NOUN_A3SG_PNON_NOM_DB), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_ADJ_AGT, __pyx_k_NOUN_A3SG_PNON_NOM_DB_ADJ_AGT, sizeof(__pyx_k_NOUN_A3SG_PNON_NOM_DB_ADJ_AGT), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_ADV_SINCE, __pyx_k_NOUN_A3SG_PNON_NOM_DB_ADV_SINCE, sizeof(__pyx_k_NOUN_A3SG_PNON_NOM_DB_ADV_SINCE), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_ADV_SINCE_2, __pyx_k_NOUN_A3SG_PNON_NOM_DB_ADV_SINCE_2, sizeof(__pyx_k_NOUN_A3SG_PNON_NOM_DB_ADV_SINCE_2), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO, __pyx_k_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO, sizeof(__pyx_k_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_2, __pyx_k_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_2, sizeof(__pyx_k_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_2), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_3, __pyx_k_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_3, sizeof(__pyx_k_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_3), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_4, __pyx_k_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_4, sizeof(__pyx_k_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_4), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_5, __pyx_k_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_5, sizeof(__pyx_k_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_5), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_6, __pyx_k_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_6, sizeof(__pyx_k_NOUN_A3SG_PNON_NOM_DB_VERB_ZERO_6), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PNON_NOM_NOUN_PROP_A3S, __pyx_k_NOUN_A3SG_PNON_NOM_NOUN_PROP_A3S, sizeof(__pyx_k_NOUN_A3SG_PNON_NOM_NOUN_PROP_A3S), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PNON_NOM_NUM_CARD_VERB, __pyx_k_NOUN_A3SG_PNON_NOM_NUM_CARD_VERB, sizeof(__pyx_k_NOUN_A3SG_PNON_NOM_NUM_CARD_VERB), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PNON_NOM_POSTP_PCABL, __pyx_k_NOUN_A3SG_PNON_NOM_POSTP_PCABL, sizeof(__pyx_k_NOUN_A3SG_PNON_NOM_POSTP_PCABL), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PNON_NOM_POSTP_PCDAT, __pyx_k_NOUN_A3SG_PNON_NOM_POSTP_PCDAT, sizeof(__pyx_k_NOUN_A3SG_PNON_NOM_POSTP_PCDAT), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PNON_NOM_QUES_PRES_A3S, __pyx_k_NOUN_A3SG_PNON_NOM_QUES_PRES_A3S, sizeof(__pyx_k_NOUN_A3SG_PNON_NOM_QUES_PRES_A3S), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PNON_NOM_VERB_POS_IMP, __pyx_k_NOUN_A3SG_PNON_NOM_VERB_POS_IMP, sizeof(__pyx_k_NOUN_A3SG_PNON_NOM_VERB_POS_IMP), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PNON_PRON_QUANTP_A3PL, __pyx_k_NOUN_A3SG_PNON_PRON_QUANTP_A3PL, sizeof(__pyx_k_NOUN_A3SG_PNON_PRON_QUANTP_A3PL), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PNON_PRON_QUANTP_A3SG, __pyx_k_NOUN_A3SG_PNON_PRON_QUANTP_A3SG, sizeof(__pyx_k_NOUN_A3SG_PNON_PRON_QUANTP_A3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PRON_PERS_A1PL, __pyx_k_NOUN_A3SG_PRON_PERS_A1PL, sizeof(__pyx_k_NOUN_A3SG_PRON_PERS_A1PL), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_A3SG_PRON_PERS_A1SG, __pyx_k_NOUN_A3SG_PRON_PERS_A1SG, sizeof(__pyx_k_NOUN_A3SG_PRON_PERS_A1SG), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_AGT_A3SG_P3SG_NOM, __pyx_k_NOUN_AGT_A3SG_P3SG_NOM, sizeof(__pyx_k_NOUN_AGT_A3SG_P3SG_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_AGT_A3SG_PNON_NOM, __pyx_k_NOUN_AGT_A3SG_PNON_NOM, sizeof(__pyx_k_NOUN_AGT_A3SG_PNON_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_FUTPART_A3PL_P3SG_NOM, __pyx_k_NOUN_FUTPART_A3PL_P3SG_NOM, sizeof(__pyx_k_NOUN_FUTPART_A3PL_P3SG_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_FUTPART_A3SG_P2PL_NOM, __pyx_k_NOUN_FUTPART_A3SG_P2PL_NOM, sizeof(__pyx_k_NOUN_FUTPART_A3SG_P2PL_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_FUTPART_A3SG_P3PL_NOM, __pyx_k_NOUN_FUTPART_A3SG_P3PL_NOM, sizeof(__pyx_k_NOUN_FUTPART_A3SG_P3PL_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_FUTPART_A3SG_P3SG_NOM, __pyx_k_NOUN_FUTPART_A3SG_P3SG_NOM, sizeof(__pyx_k_NOUN_FUTPART_A3SG_P3SG_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_NUM_CARD_DB_NOUN_ZERO, __pyx_k_NOUN_NUM_CARD_DB_NOUN_ZERO, sizeof(__pyx_k_NOUN_NUM_CARD_DB_NOUN_ZERO), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_PASTPART_A3PL_P3SG_NOM, __pyx_k_NOUN_PASTPART_A3PL_P3SG_NOM, sizeof(__pyx_k_NOUN_PASTPART_A3PL_P3SG_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_PASTPART_A3SG_P1SG_NOM, __pyx_k_NOUN_PASTPART_A3SG_P1SG_NOM, sizeof(__pyx_k_NOUN_PASTPART_A3SG_P1SG_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_PASTPART_A3SG_P2PL_NOM, __pyx_k_NOUN_PASTPART_A3SG_P2PL_NOM, sizeof(__pyx_k_NOUN_PASTPART_A3SG_P2PL_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_PASTPART_A3SG_P3PL_NOM, __pyx_k_NOUN_PASTPART_A3SG_P3PL_NOM, sizeof(__pyx_k_NOUN_PASTPART_A3SG_P3PL_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_PASTPART_A3SG_P3SG_NOM, __pyx_k_NOUN_PASTPART_A3SG_P3SG_NOM, sizeof(__pyx_k_NOUN_PASTPART_A3SG_P3SG_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_PROP, __pyx_k_NOUN_PROP, sizeof(__pyx_k_NOUN_PROP), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_PROP_A3SG_PNON_NOM, __pyx_k_NOUN_PROP_A3SG_PNON_NOM, sizeof(__pyx_k_NOUN_PROP_A3SG_PNON_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_PROP_A3SG_PNON_NOM_VERB_POS, __pyx_k_NOUN_PROP_A3SG_PNON_NOM_VERB_POS, sizeof(__pyx_k_NOUN_PROP_A3SG_PNON_NOM_VERB_POS), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_PROP_PRON_QUESP, __pyx_k_NOUN_PROP_PRON_QUESP, sizeof(__pyx_k_NOUN_PROP_PRON_QUESP), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_ZERO_A3PL_PNON_NOM, __pyx_k_NOUN_ZERO_A3PL_PNON_NOM, sizeof(__pyx_k_NOUN_ZERO_A3PL_PNON_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_ZERO_A3PL_PNON_NOM_VERB_ZER, __pyx_k_NOUN_ZERO_A3PL_PNON_NOM_VERB_ZER, sizeof(__pyx_k_NOUN_ZERO_A3PL_PNON_NOM_VERB_ZER), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_ZERO_A3SG_P1SG_NOM_VERB_ZER, __pyx_k_NOUN_ZERO_A3SG_P1SG_NOM_VERB_ZER, sizeof(__pyx_k_NOUN_ZERO_A3SG_P1SG_NOM_VERB_ZER), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_ZERO_A3SG_P3SG, __pyx_k_NOUN_ZERO_A3SG_P3SG, sizeof(__pyx_k_NOUN_ZERO_A3SG_P3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_ZERO_A3SG_P3SG_NOM, __pyx_k_NOUN_ZERO_A3SG_P3SG_NOM, sizeof(__pyx_k_NOUN_ZERO_A3SG_P3SG_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_NOUN_ZERO_A3SG_P3SG_NOM_DB, __pyx_k_NOUN_ZERO_A3SG_P3SG_NOM_DB, sizeof(__pyx_k_NOUN_ZERO_A3SG_P3SG_NOM_DB), 0, 1, 0, 0},
  {&__pyx_n_u_NUM, __pyx_k_NUM, sizeof(__pyx_k_NUM), 0, 1, 0, 1},
  {&__pyx_kp_u_NUM_CARD, __pyx_k_NUM_CARD, sizeof(__pyx_k_NUM_CARD), 0, 1, 0, 0},
  {&__pyx_kp_u_NUM_CARD_DB_NOUN_ZERO_A3SG_P3SG, __pyx_k_NUM_CARD_DB_NOUN_ZERO_A3SG_P3SG, sizeof(__pyx_k_NUM_CARD_DB_NOUN_ZERO_A3SG_P3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_NUM_CARD_DB_NOUN_ZERO_A3SG_P3SG_2, __pyx_k_NUM_CARD_DB_NOUN_ZERO_A3SG_P3SG_2, sizeof(__pyx_k_NUM_CARD_DB_NOUN_ZERO_A3SG_P3SG_2), 0, 1, 0, 0},
  {&__pyx_kp_u_NUM_CARD_VERB_POS_IMP_A2SG, __pyx_k_NUM_CARD_VERB_POS_IMP_A2SG, sizeof(__pyx_k_NUM_CARD_VERB_POS_IMP_A2SG), 0, 1, 0, 0},
  {&__pyx_kp_u_NUM_FRACTION, __pyx_k_NUM_FRACTION, sizeof(__pyx_k_NUM_FRACTION), 0, 1, 0, 0},
  {&__pyx_kp_u_P1PL_NOM, __pyx_k_P1PL_NOM, sizeof(__pyx_k_P1PL_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_P1PL_NOM_P1SG_NOM_DB_VERB_ZERO_P, __pyx_k_P1PL_NOM_P1SG_NOM_DB_VERB_ZERO_P, sizeof(__pyx_k_P1PL_NOM_P1SG_NOM_DB_VERB_ZERO_P), 0, 1, 0, 0},
  {&__pyx_kp_u_P1SG_NOM, __pyx_k_P1SG_NOM, sizeof(__pyx_k_P1SG_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_P1SG_NOM_PNON_NOM_DB_VERB_ZERO_P, __pyx_k_P1SG_NOM_PNON_NOM_DB_VERB_ZERO_P, sizeof(__pyx_k_P1SG_NOM_PNON_NOM_DB_VERB_ZERO_P), 0, 1, 0, 0},
  {&__pyx_kp_u_P2PL_NOM, __pyx_k_P2PL_NOM, sizeof(__pyx_k_P2PL_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_P2PL_NOM_PNON_GEN_DB_VERB_ZERO_P, __pyx_k_P2PL_NOM_PNON_GEN_DB_VERB_ZERO_P, sizeof(__pyx_k_P2PL_NOM_PNON_GEN_DB_VERB_ZERO_P), 0, 1, 0, 0},
  {&__pyx_n_s_P2SG, __pyx_k_P2SG, sizeof(__pyx_k_P2SG), 0, 0, 1, 1},
  {&__pyx_n_u_P2SG, __pyx_k_P2SG, sizeof(__pyx_k_P2SG), 0, 1, 0, 1},
  {&__pyx_kp_u_P2SG_NOM, __pyx_k_P2SG_NOM, sizeof(__pyx_k_P2SG_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_P2SG_NOM_PNON_GEN, __pyx_k_P2SG_NOM_PNON_GEN, sizeof(__pyx_k_P2SG_NOM_PNON_GEN), 0, 1, 0, 0},
  {&__pyx_kp_u_P2SG_P3PL_P3SG, __pyx_k_P2SG_P3PL_P3SG, sizeof(__pyx_k_P2SG_P3PL_P3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_P2SG_P3SG, __pyx_k_P2SG_P3SG, sizeof(__pyx_k_P2SG_P3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_P2SG_PNON, __pyx_k_P2SG_PNON, sizeof(__pyx_k_P2SG_PNON), 0, 1, 0, 0},
  {&__pyx_n_u_P3PL, __pyx_k_P3PL, sizeof(__pyx_k_P3PL), 0, 1, 0, 1},
  {&__pyx_kp_u_P3PL_NOM, __pyx_k_P3PL_NOM, sizeof(__pyx_k_P3PL_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_P3PL_NOM_P3SG_NOM_PNON_ACC, __pyx_k_P3PL_NOM_P3SG_NOM_PNON_ACC, sizeof(__pyx_k_P3PL_NOM_P3SG_NOM_PNON_ACC), 0, 1, 0, 0},
  {&__pyx_n_u_P3SG, __pyx_k_P3SG, sizeof(__pyx_k_P3SG), 0, 1, 0, 1},
  {&__pyx_kp_u_P3SG_NOM, __pyx_k_P3SG_NOM, sizeof(__pyx_k_P3SG_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_P3SG_NOM_PNON_ACC, __pyx_k_P3SG_NOM_PNON_ACC, sizeof(__pyx_k_P3SG_NOM_PNON_ACC), 0, 1, 0, 0},
  {&__pyx_kp_u_P3SG_PNON, __pyx_k_P3SG_PNON, sizeof(__pyx_k_P3SG_PNON), 0, 1, 0, 0},
  {&__pyx_n_u_PERS, __pyx_k_PERS, sizeof(__pyx_k_PERS), 0, 1, 0, 1},
  {&__pyx_n_u_PNON, __pyx_k_PNON, sizeof(__pyx_k_PNON), 0, 1, 0, 1},
  {&__pyx_kp_u_PNON_ACC, __pyx_k_PNON_ACC, sizeof(__pyx_k_PNON_ACC), 0, 1, 0, 0},
  {&__pyx_kp_u_PNON_GEN, __pyx_k_PNON_GEN, sizeof(__pyx_k_PNON_GEN), 0, 1, 0, 0},
  {&__pyx_kp_u_PNON_NOM_DB_VERB_ZERO_PRES_A1SG, __pyx_k_PNON_NOM_DB_VERB_ZERO_PRES_A1SG, sizeof(__pyx_k_PNON_NOM_DB_VERB_ZERO_PRES_A1SG), 0, 1, 0, 0},
  {&__pyx_kp_u_POSTP_PCABL, __pyx_k_POSTP_PCABL, sizeof(__pyx_k_POSTP_PCABL), 0, 1, 0, 0},
  {&__pyx_kp_u_POSTP_PCABL_DB, __pyx_k_POSTP_PCABL_DB, sizeof(__pyx_k_POSTP_PCABL_DB), 0, 1, 0, 0},
  {&__pyx_kp_u_POSTP_PCDAT, __pyx_k_POSTP_PCDAT, sizeof(__pyx_k_POSTP_PCDAT), 0, 1, 0, 0},
  {&__pyx_kp_u_POSTP_PCINS, __pyx_k_POSTP_PCINS, sizeof(__pyx_k_POSTP_PCINS), 0, 1, 0, 0},
  {&__pyx_kp_u_POSTP_PCNOM, __pyx_k_POSTP_PCNOM, sizeof(__pyx_k_POSTP_PCNOM), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_DB_ADJ_FUTPART_PNON, __pyx_k_POS_DB_ADJ_FUTPART_PNON, sizeof(__pyx_k_POS_DB_ADJ_FUTPART_PNON), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_DB_NOUN_INF2_A3SG_PNON_DAT, __pyx_k_POS_DB_NOUN_INF2_A3SG_PNON_DAT, sizeof(__pyx_k_POS_DB_NOUN_INF2_A3SG_PNON_DAT), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_DB_NOUN_INF2_A3SG_PNON_NOM, __pyx_k_POS_DB_NOUN_INF2_A3SG_PNON_NOM, sizeof(__pyx_k_POS_DB_NOUN_INF2_A3SG_PNON_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_DB_NOUN_INF2_A3SG_PNON_NOM_D, __pyx_k_POS_DB_NOUN_INF2_A3SG_PNON_NOM_D, sizeof(__pyx_k_POS_DB_NOUN_INF2_A3SG_PNON_NOM_D), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_DB_NOUN_INF3_A3SG_P2SG_NOM, __pyx_k_POS_DB_NOUN_INF3_A3SG_P2SG_NOM, sizeof(__pyx_k_POS_DB_NOUN_INF3_A3SG_P2SG_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_DB_NOUN_INF3_A3SG_P2SG_NOM_P, __pyx_k_POS_DB_NOUN_INF3_A3SG_P2SG_NOM_P, sizeof(__pyx_k_POS_DB_NOUN_INF3_A3SG_P2SG_NOM_P), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_DB_NOUN_INF3_A3SG_PNON_DAT, __pyx_k_POS_DB_NOUN_INF3_A3SG_PNON_DAT, sizeof(__pyx_k_POS_DB_NOUN_INF3_A3SG_PNON_DAT), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_DB_NOUN_INF3_A3SG_PNON_DAT_R, __pyx_k_POS_DB_NOUN_INF3_A3SG_PNON_DAT_R, sizeof(__pyx_k_POS_DB_NOUN_INF3_A3SG_PNON_DAT_R), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_DB_NOUN_INF3_A3SG_PNON_GEN, __pyx_k_POS_DB_NOUN_INF3_A3SG_PNON_GEN, sizeof(__pyx_k_POS_DB_NOUN_INF3_A3SG_PNON_GEN), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_DB_NOUN_INF3_A3SG_PNON_NOM, __pyx_k_POS_DB_NOUN_INF3_A3SG_PNON_NOM, sizeof(__pyx_k_POS_DB_NOUN_INF3_A3SG_PNON_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_DB_NOUN_INF3_A3SG_PNON_NOM_R, __pyx_k_POS_DB_NOUN_INF3_A3SG_PNON_NOM_R, sizeof(__pyx_k_POS_DB_NOUN_INF3_A3SG_PNON_NOM_R), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_FUT_A1SG, __pyx_k_POS_FUT_A1SG, sizeof(__pyx_k_POS_FUT_A1SG), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_FUT_A1SG_POS_DB_ADJ_FUTPART, __pyx_k_POS_FUT_A1SG_POS_DB_ADJ_FUTPART, sizeof(__pyx_k_POS_FUT_A1SG_POS_DB_ADJ_FUTPART), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_FUT_A3PL, __pyx_k_POS_FUT_A3PL, sizeof(__pyx_k_POS_FUT_A3PL), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_FUT_A3PL_POS_DB_NOUN_FUTPART, __pyx_k_POS_FUT_A3PL_POS_DB_NOUN_FUTPART, sizeof(__pyx_k_POS_FUT_A3PL_POS_DB_NOUN_FUTPART), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_FUT_A3SG, __pyx_k_POS_FUT_A3SG, sizeof(__pyx_k_POS_FUT_A3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_FUT_A3SG_POS_DB_ADJ_FUTPART, __pyx_k_POS_FUT_A3SG_POS_DB_ADJ_FUTPART, sizeof(__pyx_k_POS_FUT_A3SG_POS_DB_ADJ_FUTPART), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_NECES, __pyx_k_POS_NECES, sizeof(__pyx_k_POS_NECES), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_NECES_A3SG, __pyx_k_POS_NECES_A3SG, sizeof(__pyx_k_POS_NECES_A3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_NECES_A3SG_POS_DB_NOUN_INF2, __pyx_k_POS_NECES_A3SG_POS_DB_NOUN_INF2, sizeof(__pyx_k_POS_NECES_A3SG_POS_DB_NOUN_INF2), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_NECES_POS_DB_NOUN_INF2_A3SG, __pyx_k_POS_NECES_POS_DB_NOUN_INF2_A3SG, sizeof(__pyx_k_POS_NECES_POS_DB_NOUN_INF2_A3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_NECES_POS_DB_NOUN_INF2_A3SG_2, __pyx_k_POS_NECES_POS_DB_NOUN_INF2_A3SG_2, sizeof(__pyx_k_POS_NECES_POS_DB_NOUN_INF2_A3SG_2), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_PAST_A1PL, __pyx_k_POS_PAST_A1PL, sizeof(__pyx_k_POS_PAST_A1PL), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_PAST_A1PL_POS_DB_ADJ_PASTPAR, __pyx_k_POS_PAST_A1PL_POS_DB_ADJ_PASTPAR, sizeof(__pyx_k_POS_PAST_A1PL_POS_DB_ADJ_PASTPAR), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_PROG2, __pyx_k_POS_PROG2, sizeof(__pyx_k_POS_PROG2), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_PROG2_A3SG, __pyx_k_POS_PROG2_A3SG, sizeof(__pyx_k_POS_PROG2_A3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_PROG2_A3SG_POS_DB_NOUN_INF_A, __pyx_k_POS_PROG2_A3SG_POS_DB_NOUN_INF_A, sizeof(__pyx_k_POS_PROG2_A3SG_POS_DB_NOUN_INF_A), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_PROG2_POS_DB_NOUN_INF_A3SG_P, __pyx_k_POS_PROG2_POS_DB_NOUN_INF_A3SG_P, sizeof(__pyx_k_POS_PROG2_POS_DB_NOUN_INF_A3SG_P), 0, 1, 0, 0},
  {&__pyx_kp_u_POS_PROG2_POS_DB_NOUN_INF_A3SG_P_2, __pyx_k_POS_PROG2_POS_DB_NOUN_INF_A3SG_P_2, sizeof(__pyx_k_POS_PROG2_POS_DB_NOUN_INF_A3SG_P_2), 0, 1, 0, 0},
  {&__pyx_kp_u_PRON_DEMONSP_A3SG_PNON_NOM, __pyx_k_PRON_DEMONSP_A3SG_PNON_NOM, sizeof(__pyx_k_PRON_DEMONSP_A3SG_PNON_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_PRON_PERS_A1PL, __pyx_k_PRON_PERS_A1PL, sizeof(__pyx_k_PRON_PERS_A1PL), 0, 1, 0, 0},
  {&__pyx_kp_u_PRON_PERS_A1PL_PNON, __pyx_k_PRON_PERS_A1PL_PNON, sizeof(__pyx_k_PRON_PERS_A1PL_PNON), 0, 1, 0, 0},
  {&__pyx_kp_u_PRON_PERS_A1PL_PNON_ACC, __pyx_k_PRON_PERS_A1PL_PNON_ACC, sizeof(__pyx_k_PRON_PERS_A1PL_PNON_ACC), 0, 1, 0, 0},
  {&__pyx_kp_u_PRON_PERS_A1PL_PNON_GEN, __pyx_k_PRON_PERS_A1PL_PNON_GEN, sizeof(__pyx_k_PRON_PERS_A1PL_PNON_GEN), 0, 1, 0, 0},
  {&__pyx_kp_u_PRON_PERS_A1SG, __pyx_k_PRON_PERS_A1SG, sizeof(__pyx_k_PRON_PERS_A1SG), 0, 1, 0, 0},
  {&__pyx_kp_u_PRON_PERS_A1SG_PNON, __pyx_k_PRON_PERS_A1SG_PNON, sizeof(__pyx_k_PRON_PERS_A1SG_PNON), 0, 1, 0, 0},
  {&__pyx_kp_u_PRON_PERS_A1SG_PNON_ACC, __pyx_k_PRON_PERS_A1SG_PNON_ACC, sizeof(__pyx_k_PRON_PERS_A1SG_PNON_ACC), 0, 1, 0, 0},
  {&__pyx_kp_u_PRON_PERS_A1SG_PNON_GEN, __pyx_k_PRON_PERS_A1SG_PNON_GEN, sizeof(__pyx_k_PRON_PERS_A1SG_PNON_GEN), 0, 1, 0, 0},
  {&__pyx_kp_u_PRON_PERS_A3SG_PNON_NOM, __pyx_k_PRON_PERS_A3SG_PNON_NOM, sizeof(__pyx_k_PRON_PERS_A3SG_PNON_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_PRON_QUANTP_A3PL_P3PL, __pyx_k_PRON_QUANTP_A3PL_P3PL, sizeof(__pyx_k_PRON_QUANTP_A3PL_P3PL), 0, 1, 0, 0},
  {&__pyx_kp_u_PRON_QUANTP_A3PL_P3PL_GEN, __pyx_k_PRON_QUANTP_A3PL_P3PL_GEN, sizeof(__pyx_k_PRON_QUANTP_A3PL_P3PL_GEN), 0, 1, 0, 0},
  {&__pyx_kp_u_PRON_QUANTP_A3SG_P3SG, __pyx_k_PRON_QUANTP_A3SG_P3SG, sizeof(__pyx_k_PRON_QUANTP_A3SG_P3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_PRON_QUESP, __pyx_k_PRON_QUESP, sizeof(__pyx_k_PRON_QUESP), 0, 1, 0, 0},
  {&__pyx_kp_u_PRON_QUESP_A3SG_PNON_NOM, __pyx_k_PRON_QUESP_A3SG_PNON_NOM, sizeof(__pyx_k_PRON_QUESP_A3SG_PNON_NOM), 0, 1, 0, 0},
  {&__pyx_n_u_PROP, __pyx_k_PROP, sizeof(__pyx_k_PROP), 0, 1, 0, 1},
  {&__pyx_kp_u_PROP_A3PL_P3PL, __pyx_k_PROP_A3PL_P3PL, sizeof(__pyx_k_PROP_A3PL_P3PL), 0, 1, 0, 0},
  {&__pyx_kp_u_PROP_A3PL_PNON_NOM, __pyx_k_PROP_A3PL_PNON_NOM, sizeof(__pyx_k_PROP_A3PL_PNON_NOM), 0, 1, 0, 0},
  {&__pyx_kp_u_PROP_A3SG, __pyx_k_PROP_A3SG, sizeof(__pyx_k_PROP_A3SG), 0, 1, 0, 0},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_n_u_QUES, __pyx_k_QUES, sizeof(__pyx_k_QUES), 0, 1, 0, 1},
  {&__pyx_kp_u_QUES_PRES_A3SG, __pyx_k_QUES_PRES_A3SG, sizeof(__pyx_k_QUES_PRES_A3SG), 0, 1, 0, 0},
  {&__pyx_n_u_VERB, __pyx_k_VERB, sizeof(__pyx_k_VERB), 0, 1, 0, 1},
  {&__pyx_kp_u_VERB_DB_VERB_PASS, __pyx_k_VERB_DB_VERB_PASS, sizeof(__pyx_k_VERB_DB_VERB_PASS), 0, 1, 0, 0},
  {&__pyx_kp_u_VERB_DB_VERB_PASS_POS, __pyx_k_VERB_DB_VERB_PASS_POS, sizeof(__pyx_k_VERB_DB_VERB_PASS_POS), 0, 1, 0, 0},
  {&__pyx_kp_u_VERB_POS, __pyx_k_VERB_POS, sizeof(__pyx_k_VERB_POS), 0, 1, 0, 0},
  {&__pyx_kp_u_VERB_POS_AOR, __pyx_k_VERB_POS_AOR, sizeof(__pyx_k_VERB_POS_AOR), 0, 1, 0, 0},
  {&__pyx_kp_u_VERB_POS_AOR_VERB_DB_VERB_CAUS_P, __pyx_k_VERB_POS_AOR_VERB_DB_VERB_CAUS_P, sizeof(__pyx_k_VERB_POS_AOR_VERB_DB_VERB_CAUS_P), 0, 1, 0, 0},
  {&__pyx_kp_u_VERB_POS_DB_NOUN_INF2_A3SG_PNON, __pyx_k_VERB_POS_DB_NOUN_INF2_A3SG_PNON, sizeof(__pyx_k_VERB_POS_DB_NOUN_INF2_A3SG_PNON), 0, 1, 0, 0},
  {&__pyx_kp_u_VERB_POS_IMP_A2PL, __pyx_k_VERB_POS_IMP_A2PL, sizeof(__pyx_k_VERB_POS_IMP_A2PL), 0, 1, 0, 0},
  {&__pyx_kp_u_VERB_POS_IMP_A2PL_VERB_DB_VERB_P, __pyx_k_VERB_POS_IMP_A2PL_VERB_DB_VERB_P, sizeof(__pyx_k_VERB_POS_IMP_A2PL_VERB_DB_VERB_P), 0, 1, 0, 0},
  {&__pyx_kp_u_VERB_POS_NARR_A3SG, __pyx_k_VERB_POS_NARR_A3SG, sizeof(__pyx_k_VERB_POS_NARR_A3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_VERB_POS_NARR_COP_A3SG, __pyx_k_VERB_POS_NARR_COP_A3SG, sizeof(__pyx_k_VERB_POS_NARR_COP_A3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_VERB_POS_NARR_DB_ADJ_ZERO, __pyx_k_VERB_POS_NARR_DB_ADJ_ZERO, sizeof(__pyx_k_VERB_POS_NARR_DB_ADJ_ZERO), 0, 1, 0, 0},
  {&__pyx_kp_u_VERB_POS_PAST_A1PL, __pyx_k_VERB_POS_PAST_A1PL, sizeof(__pyx_k_VERB_POS_PAST_A1PL), 0, 1, 0, 0},
  {&__pyx_kp_u_VERB_REFLEX_VERB_DB_VERB_PASS, __pyx_k_VERB_REFLEX_VERB_DB_VERB_PASS, sizeof(__pyx_k_VERB_REFLEX_VERB_DB_VERB_PASS), 0, 1, 0, 0},
  {&__pyx_kp_u_VERB_ZERO_PRES_A1SG, __pyx_k_VERB_ZERO_PRES_A1SG, sizeof(__pyx_k_VERB_ZERO_PRES_A1SG), 0, 1, 0, 0},
  {&__pyx_kp_u_VERB_ZERO_PRES_COP_A3SG, __pyx_k_VERB_ZERO_PRES_COP_A3SG, sizeof(__pyx_k_VERB_ZERO_PRES_COP_A3SG), 0, 1, 0, 0},
  {&__pyx_kp_u_ZERO_A3SG_PNON_NOM_DB_ADJ_FITFOR, __pyx_k_ZERO_A3SG_PNON_NOM_DB_ADJ_FITFOR, sizeof(__pyx_k_ZERO_A3SG_PNON_NOM_DB_ADJ_FITFOR), 0, 1, 0, 0},
  {&__pyx_kp_u__2, __pyx_k__2, sizeof(__pyx_k__2), 0, 1, 0, 0},
  {&__pyx_n_u_allmadk, __pyx_k_allmadk, sizeof(__pyx_k_allmadk), 0, 1, 0, 1},
  {&__pyx_n_u_artk, __pyx_k_artk, sizeof(__pyx_k_artk), 0, 1, 0, 1},
  {&__pyx_n_u_ay, __pyx_k_ay, sizeof(__pyx_k_ay), 0, 1, 0, 1},
  {&__pyx_n_s_caseDisambiguator, __pyx_k_caseDisambiguator, sizeof(__pyx_k_caseDisambiguator), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_containsTag, __pyx_k_containsTag, sizeof(__pyx_k_containsTag), 0, 0, 1, 1},
  {&__pyx_n_s_containsTwoNeOrYa, __pyx_k_containsTwoNeOrYa, sizeof(__pyx_k_containsTwoNeOrYa), 0, 0, 1, 1},
  {&__pyx_n_s_correctParses, __pyx_k_correctParses, sizeof(__pyx_k_correctParses), 0, 0, 1, 1},
  {&__pyx_n_s_count, __pyx_k_count, sizeof(__pyx_k_count), 0, 0, 1, 1},
  {&__pyx_n_u_da, __pyx_k_da, sizeof(__pyx_k_da), 0, 1, 0, 1},
  {&__pyx_n_s_defaultCase, __pyx_k_defaultCase, sizeof(__pyx_k_defaultCase), 0, 0, 1, 1},
  {&__pyx_n_u_deil, __pyx_k_deil, sizeof(__pyx_k_deil), 0, 1, 0, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_u_dk, __pyx_k_dk, sizeof(__pyx_k_dk), 0, 1, 0, 1},
  {&__pyx_n_u_eksik, __pyx_k_eksik, sizeof(__pyx_k_eksik), 0, 1, 0, 1},
  {&__pyx_n_s_fsmPars, __pyx_k_fsmPars, sizeof(__pyx_k_fsmPars), 0, 0, 1, 1},
  {&__pyx_n_s_fsmParse, __pyx_k_fsmParse, sizeof(__pyx_k_fsmParse), 0, 0, 1, 1},
  {&__pyx_n_s_fsmParseList, __pyx_k_fsmParseList, sizeof(__pyx_k_fsmParseList), 0, 0, 1, 1},
  {&__pyx_n_s_fsmParses, __pyx_k_fsmParses, sizeof(__pyx_k_fsmParses), 0, 0, 1, 1},
  {&__pyx_n_u_gerek, __pyx_k_gerek, sizeof(__pyx_k_gerek), 0, 1, 0, 1},
  {&__pyx_n_s_getFinalPos, __pyx_k_getFinalPos, sizeof(__pyx_k_getFinalPos), 0, 0, 1, 1},
  {&__pyx_n_s_getFsmParse, __pyx_k_getFsmParse, sizeof(__pyx_k_getFsmParse), 0, 0, 1, 1},
  {&__pyx_n_s_getName, __pyx_k_getName, sizeof(__pyx_k_getName), 0, 0, 1, 1},
  {&__pyx_n_s_getSurfaceForm, __pyx_k_getSurfaceForm, sizeof(__pyx_k_getSurfaceForm), 0, 0, 1, 1},
  {&__pyx_n_s_getWord, __pyx_k_getWord, sizeof(__pyx_k_getWord), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_hasPreviousWordTag, __pyx_k_hasPreviousWordTag, sizeof(__pyx_k_hasPreviousWordTag), 0, 0, 1, 1},
  {&__pyx_n_s_i, __pyx_k_i, sizeof(__pyx_k_i), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_index, __pyx_k_index, sizeof(__pyx_k_index), 0, 0, 1, 1},
  {&__pyx_n_s_isAnyWordSecondPerson, __pyx_k_isAnyWordSecondPerson, sizeof(__pyx_k_isAnyWordSecondPerson), 0, 0, 1, 1},
  {&__pyx_n_s_isBeforeLastWord, __pyx_k_isBeforeLastWord, sizeof(__pyx_k_isBeforeLastWord), 0, 0, 1, 1},
  {&__pyx_n_s_isCapitalWord, __pyx_k_isCapitalWord, sizeof(__pyx_k_isCapitalWord), 0, 0, 1, 1},
  {&__pyx_n_s_isFirstWord, __pyx_k_isFirstWord, sizeof(__pyx_k_isFirstWord), 0, 0, 1, 1},
  {&__pyx_n_s_isNextWordNoun, __pyx_k_isNextWordNoun, sizeof(__pyx_k_isNextWordNoun), 0, 0, 1, 1},
  {&__pyx_n_s_isNextWordNounOrAdjective, __pyx_k_isNextWordNounOrAdjective, sizeof(__pyx_k_isNextWordNounOrAdjective), 0, 0, 1, 1},
  {&__pyx_n_s_isNextWordNum, __pyx_k_isNextWordNum, sizeof(__pyx_k_isNextWordNum), 0, 0, 1, 1},
  {&__pyx_n_s_isNoun, __pyx_k_isNoun, sizeof(__pyx_k_isNoun), 0, 0, 1, 1},
  {&__pyx_n_s_isPlural, __pyx_k_isPlural, sizeof(__pyx_k_isPlural), 0, 0, 1, 1},
  {&__pyx_n_s_isPossessivePlural, __pyx_k_isPossessivePlural, sizeof(__pyx_k_isPossessivePlural), 0, 0, 1, 1},
  {&__pyx_n_u_kar, __pyx_k_kar, sizeof(__pyx_k_kar), 0, 1, 0, 1},
  {&__pyx_n_s_lastWord, __pyx_k_lastWord, sizeof(__pyx_k_lastWord), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_map, __pyx_k_map, sizeof(__pyx_k_map), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_u_ne, __pyx_k_ne, sizeof(__pyx_k_ne), 0, 1, 0, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_nextParseList, __pyx_k_nextParseList, sizeof(__pyx_k_nextParseList), 0, 0, 1, 1},
  {&__pyx_n_s_nextWord, __pyx_k_nextWord, sizeof(__pyx_k_nextWord), 0, 0, 1, 1},
  {&__pyx_n_s_nextWordExists, __pyx_k_nextWordExists, sizeof(__pyx_k_nextWordExists), 0, 0, 1, 1},
  {&__pyx_n_s_nextWordPos, __pyx_k_nextWordPos, sizeof(__pyx_k_nextWordPos), 0, 0, 1, 1},
  {&__pyx_n_u_orta, __pyx_k_orta, sizeof(__pyx_k_orta), 0, 1, 0, 1},
  {&__pyx_n_s_parseString, __pyx_k_parseString, sizeof(__pyx_k_parseString), 0, 0, 1, 1},
  {&__pyx_n_s_parsesWithoutPrefixAndSuffix, __pyx_k_parsesWithoutPrefixAndSuffix, sizeof(__pyx_k_parsesWithoutPrefixAndSuffix), 0, 0, 1, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_previousWord, __pyx_k_previousWord, sizeof(__pyx_k_previousWord), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_AutoDisambiguator, __pyx_k_pyx_unpickle_AutoDisambiguator, sizeof(__pyx_k_pyx_unpickle_AutoDisambiguator), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_u_rahat, __pyx_k_rahat, sizeof(__pyx_k_rahat), 0, 1, 0, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_root, __pyx_k_root, sizeof(__pyx_k_root), 0, 0, 1, 1},
  {&__pyx_n_s_selectCaseForParseString, __pyx_k_selectCaseForParseString, sizeof(__pyx_k_selectCaseForParseString), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_u_sk, __pyx_k_sk, sizeof(__pyx_k_sk), 0, 1, 0, 1},
  {&__pyx_n_u_sre, __pyx_k_sre, sizeof(__pyx_k_sre), 0, 1, 0, 1},
  {&__pyx_n_s_staticmethod, __pyx_k_staticmethod, sizeof(__pyx_k_staticmethod), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_surfaceForm, __pyx_k_surfaceForm, sizeof(__pyx_k_surfaceForm), 0, 0, 1, 1},
  {&__pyx_n_s_tag, __pyx_k_tag, sizeof(__pyx_k_tag), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_u_var, __pyx_k_var, sizeof(__pyx_k_var), 0, 1, 0, 1},
  {&__pyx_n_u_vasat, __pyx_k_vasat, sizeof(__pyx_k_vasat), 0, 1, 0, 1},
  {&__pyx_n_s_word, __pyx_k_word, sizeof(__pyx_k_word), 0, 0, 1, 1},
  {&__pyx_n_u_ya, __pyx_k_ya, sizeof(__pyx_k_ya), 0, 1, 0, 1},
  {&__pyx_n_u_yl, __pyx_k_yl, sizeof(__pyx_k_yl), 0, 1, 0, 1},
  {&__pyx_n_u_yok, __pyx_k_yok, sizeof(__pyx_k_yok), 0, 1, 0, 1},
  {&__pyx_n_u_zaman, __pyx_k_zaman, sizeof(__pyx_k_zaman), 0, 1, 0, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_staticmethod = __Pyx_GetBuiltinName(__pyx_n_s_staticmethod); if (!__pyx_builtin_staticmethod) __PYX_ERR(0, 7, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 11, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0x8b03656, 0xbc895e2, 0x8b1515e):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0x8b03656, 0xbc895e2, 0x8b1515e) = (morphologicalAnalyzer))" % __pyx_checksum)
 */
  __pyx_tuple__3 = PyTuple_Pack(3, __pyx_int_145765974, __pyx_int_197694946, __pyx_int_145838430); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":8
 * 
 *     @staticmethod
 *     def isAnyWordSecondPerson(index: int, correctParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         cdef int count, i
 *         count = 0
 */
  __pyx_tuple__4 = PyTuple_Pack(4, __pyx_n_s_index, __pyx_n_s_correctParses, __pyx_n_s_count, __pyx_n_s_i); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);
  __pyx_codeobj__5 = (PyObject*)__Pyx_PyCode_New(2, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__4, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_MorphologicalDisambiguation_Auto, __pyx_n_s_isAnyWordSecondPerson, 8, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__5)) __PYX_ERR(0, 8, __pyx_L1_error)

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":18
 * 
 *     @staticmethod
 *     def isPossessivePlural(index: int, correctParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         cdef int i
 *         for i in range(index - 1, -1, -1):
 */
  __pyx_tuple__6 = PyTuple_Pack(3, __pyx_n_s_index, __pyx_n_s_correctParses, __pyx_n_s_i); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);
  __pyx_codeobj__7 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__6, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_MorphologicalDisambiguation_Auto, __pyx_n_s_isPossessivePlural, 18, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__7)) __PYX_ERR(0, 18, __pyx_L1_error)

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":26
 * 
 *     @staticmethod
 *     def nextWordPos(nextParseList: FsmParseList) -> str:             # <<<<<<<<<<<<<<
 *         cdef CounterHashMap _map
 *         cdef int i
 */
  __pyx_tuple__8 = PyTuple_Pack(3, __pyx_n_s_nextParseList, __pyx_n_s_map, __pyx_n_s_i); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);
  __pyx_codeobj__9 = (PyObject*)__Pyx_PyCode_New(1, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__8, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_MorphologicalDisambiguation_Auto, __pyx_n_s_nextWordPos, 26, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__9)) __PYX_ERR(0, 26, __pyx_L1_error)

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":35
 * 
 *     @staticmethod
 *     def isBeforeLastWord(index: int, fsmParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         return index + 2 == len(fsmParses)
 * 
 */
  __pyx_tuple__10 = PyTuple_Pack(2, __pyx_n_s_index, __pyx_n_s_fsmParses); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);
  __pyx_codeobj__11 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__10, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_MorphologicalDisambiguation_Auto, __pyx_n_s_isBeforeLastWord, 35, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__11)) __PYX_ERR(0, 35, __pyx_L1_error)

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":39
 * 
 *     @staticmethod
 *     def nextWordExists(index: int, fsmParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         return index + 1 < len(fsmParses)
 * 
 */
  __pyx_tuple__12 = PyTuple_Pack(2, __pyx_n_s_index, __pyx_n_s_fsmParses); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);
  __pyx_codeobj__13 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__12, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_MorphologicalDisambiguation_Auto, __pyx_n_s_nextWordExists, 39, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__13)) __PYX_ERR(0, 39, __pyx_L1_error)

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":43
 * 
 *     @staticmethod
 *     def isNextWordNoun(index: int, fsmParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         return index + 1 < len(fsmParses) and AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN"
 * 
 */
  __pyx_tuple__14 = PyTuple_Pack(2, __pyx_n_s_index, __pyx_n_s_fsmParses); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);
  __pyx_codeobj__15 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__14, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_MorphologicalDisambiguation_Auto, __pyx_n_s_isNextWordNoun, 43, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__15)) __PYX_ERR(0, 43, __pyx_L1_error)

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":47
 * 
 *     @staticmethod
 *     def isNextWordNum(index: int, fsmParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         return index + 1 < len(fsmParses) and AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NUM"
 * 
 */
  __pyx_tuple__16 = PyTuple_Pack(2, __pyx_n_s_index, __pyx_n_s_fsmParses); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);
  __pyx_codeobj__17 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__16, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_MorphologicalDisambiguation_Auto, __pyx_n_s_isNextWordNum, 47, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__17)) __PYX_ERR(0, 47, __pyx_L1_error)

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":51
 * 
 *     @staticmethod
 *     def isNextWordNounOrAdjective(index: int, fsmParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         return index + 1 < len(fsmParses) and (AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN" or
 *                                                AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "ADJ" or
 */
  __pyx_tuple__18 = PyTuple_Pack(2, __pyx_n_s_index, __pyx_n_s_fsmParses); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);
  __pyx_codeobj__19 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__18, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_MorphologicalDisambiguation_Auto, __pyx_n_s_isNextWordNounOrAdjective, 51, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__19)) __PYX_ERR(0, 51, __pyx_L1_error)

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":57
 * 
 *     @staticmethod
 *     def isFirstWord(index: int) -> bool:             # <<<<<<<<<<<<<<
 *         return index == 0
 * 
 */
  __pyx_tuple__20 = PyTuple_Pack(1, __pyx_n_s_index); if (unlikely(!__pyx_tuple__20)) __PYX_ERR(0, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__20);
  __Pyx_GIVEREF(__pyx_tuple__20);
  __pyx_codeobj__21 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__20, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_MorphologicalDisambiguation_Auto, __pyx_n_s_isFirstWord, 57, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__21)) __PYX_ERR(0, 57, __pyx_L1_error)

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":61
 * 
 *     @staticmethod
 *     def containsTwoNeOrYa(fsmParses: list, word: str) -> bool:             # <<<<<<<<<<<<<<
 *         cdef int count
 *         cdef FsmParseList fsmPars
 */
  __pyx_tuple__22 = PyTuple_Pack(5, __pyx_n_s_fsmParses, __pyx_n_s_word, __pyx_n_s_count, __pyx_n_s_fsmPars, __pyx_n_s_surfaceForm); if (unlikely(!__pyx_tuple__22)) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);
  __pyx_codeobj__23 = (PyObject*)__Pyx_PyCode_New(2, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__22, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_MorphologicalDisambiguation_Auto, __pyx_n_s_containsTwoNeOrYa, 61, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__23)) __PYX_ERR(0, 61, __pyx_L1_error)

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":73
 * 
 *     @staticmethod
 *     def hasPreviousWordTag(index: int, correctParses: list, tag: MorphologicalTag) -> bool:             # <<<<<<<<<<<<<<
 *         return index > 0 and correctParses[index - 1].containsTag(tag)
 * 
 */
  __pyx_tuple__24 = PyTuple_Pack(3, __pyx_n_s_index, __pyx_n_s_correctParses, __pyx_n_s_tag); if (unlikely(!__pyx_tuple__24)) __PYX_ERR(0, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);
  __pyx_codeobj__25 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__24, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_MorphologicalDisambiguation_Auto, __pyx_n_s_hasPreviousWordTag, 73, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__25)) __PYX_ERR(0, 73, __pyx_L1_error)

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":77
 * 
 *     @staticmethod
 *     def selectCaseForParseString(parseString: str, index: int, fsmParses: list, correctParses: list) -> str:             # <<<<<<<<<<<<<<
 *         cdef str surfaceForm, root, lastWord
 *         surfaceForm = fsmParses[index].getFsmParse(0).getSurfaceForm()
 */
  __pyx_tuple__26 = PyTuple_Pack(9, __pyx_n_s_parseString, __pyx_n_s_index, __pyx_n_s_fsmParses, __pyx_n_s_correctParses, __pyx_n_s_surfaceForm, __pyx_n_s_root, __pyx_n_s_lastWord, __pyx_n_s_previousWord, __pyx_n_s_nextWord); if (unlikely(!__pyx_tuple__26)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__26);
  __Pyx_GIVEREF(__pyx_tuple__26);
  __pyx_codeobj__27 = (PyObject*)__Pyx_PyCode_New(4, 0, 9, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__26, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_MorphologicalDisambiguation_Auto, __pyx_n_s_selectCaseForParseString, 77, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__27)) __PYX_ERR(0, 77, __pyx_L1_error)

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":793
 * 
 *     @staticmethod
 *     def caseDisambiguator(index: int, fsmParses: list, correctParses: list) -> FsmParse:             # <<<<<<<<<<<<<<
 *         cdef FsmParseList fsmParseList
 *         cdef str defaultCase
 */
  __pyx_tuple__28 = PyTuple_Pack(7, __pyx_n_s_index, __pyx_n_s_fsmParses, __pyx_n_s_correctParses, __pyx_n_s_fsmParseList, __pyx_n_s_defaultCase, __pyx_n_s_i, __pyx_n_s_fsmParse); if (unlikely(!__pyx_tuple__28)) __PYX_ERR(0, 793, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__28);
  __Pyx_GIVEREF(__pyx_tuple__28);
  __pyx_codeobj__29 = (PyObject*)__Pyx_PyCode_New(3, 0, 7, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__28, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_MorphologicalDisambiguation_Auto, __pyx_n_s_caseDisambiguator, 793, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__29)) __PYX_ERR(0, 793, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __pyx_unpickle_AutoDisambiguator(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_tuple__30 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__30)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__30);
  __Pyx_GIVEREF(__pyx_tuple__30);
  __pyx_codeobj__31 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__30, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_AutoDisambiguator, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__31)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_145765974 = PyInt_FromLong(145765974L); if (unlikely(!__pyx_int_145765974)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_145838430 = PyInt_FromLong(145838430L); if (unlikely(!__pyx_int_145838430)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_197694946 = PyInt_FromLong(197694946L); if (unlikely(!__pyx_int_197694946)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_1 = PyInt_FromLong(-1); if (unlikely(!__pyx_int_neg_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  if (PyType_Ready(&__pyx_type_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator) < 0) __PYX_ERR(0, 5, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator.tp_dictoffset && __pyx_type_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_AutoDisambiguator, (PyObject *)&__pyx_type_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator) < 0) __PYX_ERR(0, 5, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator) < 0) __PYX_ERR(0, 5, __pyx_L1_error)
  __pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator = &__pyx_type_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule("Dictionary.Word"); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_10Dictionary_4Word_Word = __Pyx_ImportType(__pyx_t_1, "Dictionary.Word", "Word", sizeof(struct __pyx_obj_10Dictionary_4Word_Word), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_10Dictionary_4Word_Word) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_vtabptr_10Dictionary_4Word_Word = (struct __pyx_vtabstruct_10Dictionary_4Word_Word*)__Pyx_GetVtable(__pyx_ptype_10Dictionary_4Word_Word->tp_dict); if (unlikely(!__pyx_vtabptr_10Dictionary_4Word_Word)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("Corpus.Sentence"); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_6Corpus_8Sentence_Sentence = __Pyx_ImportType(__pyx_t_1, "Corpus.Sentence", "Sentence", sizeof(struct __pyx_obj_6Corpus_8Sentence_Sentence), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_6Corpus_8Sentence_Sentence) __PYX_ERR(3, 4, __pyx_L1_error)
  __pyx_vtabptr_6Corpus_8Sentence_Sentence = (struct __pyx_vtabstruct_6Corpus_8Sentence_Sentence*)__Pyx_GetVtable(__pyx_ptype_6Corpus_8Sentence_Sentence->tp_dict); if (unlikely(!__pyx_vtabptr_6Corpus_8Sentence_Sentence)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("Dictionary.Trie.TrieNode"); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_10Dictionary_4Trie_8TrieNode_TrieNode = __Pyx_ImportType(__pyx_t_1, "Dictionary.Trie.TrieNode", "TrieNode", sizeof(struct __pyx_obj_10Dictionary_4Trie_8TrieNode_TrieNode), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_10Dictionary_4Trie_8TrieNode_TrieNode) __PYX_ERR(4, 4, __pyx_L1_error)
  __pyx_vtabptr_10Dictionary_4Trie_8TrieNode_TrieNode = (struct __pyx_vtabstruct_10Dictionary_4Trie_8TrieNode_TrieNode*)__Pyx_GetVtable(__pyx_ptype_10Dictionary_4Trie_8TrieNode_TrieNode->tp_dict); if (unlikely(!__pyx_vtabptr_10Dictionary_4Trie_8TrieNode_TrieNode)) __PYX_ERR(4, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("Dictionary.TxtWord"); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_10Dictionary_7TxtWord_TxtWord = __Pyx_ImportType(__pyx_t_1, "Dictionary.TxtWord", "TxtWord", sizeof(struct __pyx_obj_10Dictionary_7TxtWord_TxtWord), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_10Dictionary_7TxtWord_TxtWord) __PYX_ERR(5, 4, __pyx_L1_error)
  __pyx_vtabptr_10Dictionary_7TxtWord_TxtWord = (struct __pyx_vtabstruct_10Dictionary_7TxtWord_TxtWord*)__Pyx_GetVtable(__pyx_ptype_10Dictionary_7TxtWord_TxtWord->tp_dict); if (unlikely(!__pyx_vtabptr_10Dictionary_7TxtWord_TxtWord)) __PYX_ERR(5, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("Dictionary.Trie.Trie"); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_10Dictionary_4Trie_4Trie_Trie = __Pyx_ImportType(__pyx_t_1, "Dictionary.Trie.Trie", "Trie", sizeof(struct __pyx_obj_10Dictionary_4Trie_4Trie_Trie), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_10Dictionary_4Trie_4Trie_Trie) __PYX_ERR(6, 6, __pyx_L1_error)
  __pyx_vtabptr_10Dictionary_4Trie_4Trie_Trie = (struct __pyx_vtabstruct_10Dictionary_4Trie_4Trie_Trie*)__Pyx_GetVtable(__pyx_ptype_10Dictionary_4Trie_4Trie_Trie->tp_dict); if (unlikely(!__pyx_vtabptr_10Dictionary_4Trie_4Trie_Trie)) __PYX_ERR(6, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("Dictionary.Dictionary"); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_10Dictionary_10Dictionary_Dictionary = __Pyx_ImportType(__pyx_t_1, "Dictionary.Dictionary", "Dictionary", sizeof(struct __pyx_obj_10Dictionary_10Dictionary_Dictionary), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_10Dictionary_10Dictionary_Dictionary) __PYX_ERR(7, 4, __pyx_L1_error)
  __pyx_vtabptr_10Dictionary_10Dictionary_Dictionary = (struct __pyx_vtabstruct_10Dictionary_10Dictionary_Dictionary*)__Pyx_GetVtable(__pyx_ptype_10Dictionary_10Dictionary_Dictionary->tp_dict); if (unlikely(!__pyx_vtabptr_10Dictionary_10Dictionary_Dictionary)) __PYX_ERR(7, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("Dictionary.TxtDictionary"); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_10Dictionary_13TxtDictionary_TxtDictionary = __Pyx_ImportType(__pyx_t_1, "Dictionary.TxtDictionary", "TxtDictionary", sizeof(struct __pyx_obj_10Dictionary_13TxtDictionary_TxtDictionary), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_10Dictionary_13TxtDictionary_TxtDictionary) __PYX_ERR(8, 7, __pyx_L1_error)
  __pyx_vtabptr_10Dictionary_13TxtDictionary_TxtDictionary = (struct __pyx_vtabstruct_10Dictionary_13TxtDictionary_TxtDictionary*)__Pyx_GetVtable(__pyx_ptype_10Dictionary_13TxtDictionary_TxtDictionary->tp_dict); if (unlikely(!__pyx_vtabptr_10Dictionary_13TxtDictionary_TxtDictionary)) __PYX_ERR(8, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("DataStructure.Cache.LRUCache"); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_13DataStructure_5Cache_8LRUCache_LRUCache = __Pyx_ImportType(__pyx_t_1, "DataStructure.Cache.LRUCache", "LRUCache", sizeof(struct __pyx_obj_13DataStructure_5Cache_8LRUCache_LRUCache), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_13DataStructure_5Cache_8LRUCache_LRUCache) __PYX_ERR(9, 1, __pyx_L1_error)
  __pyx_vtabptr_13DataStructure_5Cache_8LRUCache_LRUCache = (struct __pyx_vtabstruct_13DataStructure_5Cache_8LRUCache_LRUCache*)__Pyx_GetVtable(__pyx_ptype_13DataStructure_5Cache_8LRUCache_LRUCache->tp_dict); if (unlikely(!__pyx_vtabptr_13DataStructure_5Cache_8LRUCache_LRUCache)) __PYX_ERR(9, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("MorphologicalAnalysis.InflectionalGroup"); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup = __Pyx_ImportType(__pyx_t_1, "MorphologicalAnalysis.InflectionalGroup", "InflectionalGroup", sizeof(struct __pyx_obj_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup) __PYX_ERR(10, 1, __pyx_L1_error)
  __pyx_vtabptr_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup = (struct __pyx_vtabstruct_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup*)__Pyx_GetVtable(__pyx_ptype_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup->tp_dict); if (unlikely(!__pyx_vtabptr_21MorphologicalAnalysis_17InflectionalGroup_InflectionalGroup)) __PYX_ERR(10, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("MorphologicalAnalysis.MorphologicalParse"); if (unlikely(!__pyx_t_1)) __PYX_ERR(11, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse = __Pyx_ImportType(__pyx_t_1, "MorphologicalAnalysis.MorphologicalParse", "MorphologicalParse", sizeof(struct __pyx_obj_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse) __PYX_ERR(11, 7, __pyx_L1_error)
  __pyx_vtabptr_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse = (struct __pyx_vtabstruct_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse*)__Pyx_GetVtable(__pyx_ptype_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse->tp_dict); if (unlikely(!__pyx_vtabptr_21MorphologicalAnalysis_18MorphologicalParse_MorphologicalParse)) __PYX_ERR(11, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("MorphologicalAnalysis.State"); if (unlikely(!__pyx_t_1)) __PYX_ERR(12, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_21MorphologicalAnalysis_5State_State = __Pyx_ImportType(__pyx_t_1, "MorphologicalAnalysis.State", "State", sizeof(struct __pyx_obj_21MorphologicalAnalysis_5State_State), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_21MorphologicalAnalysis_5State_State) __PYX_ERR(12, 1, __pyx_L1_error)
  __pyx_vtabptr_21MorphologicalAnalysis_5State_State = (struct __pyx_vtabstruct_21MorphologicalAnalysis_5State_State*)__Pyx_GetVtable(__pyx_ptype_21MorphologicalAnalysis_5State_State->tp_dict); if (unlikely(!__pyx_vtabptr_21MorphologicalAnalysis_5State_State)) __PYX_ERR(12, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("MorphologicalAnalysis.FsmParse"); if (unlikely(!__pyx_t_1)) __PYX_ERR(13, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_21MorphologicalAnalysis_8FsmParse_FsmParse = __Pyx_ImportType(__pyx_t_1, "MorphologicalAnalysis.FsmParse", "FsmParse", sizeof(struct __pyx_obj_21MorphologicalAnalysis_8FsmParse_FsmParse), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_21MorphologicalAnalysis_8FsmParse_FsmParse) __PYX_ERR(13, 5, __pyx_L1_error)
  __pyx_vtabptr_21MorphologicalAnalysis_8FsmParse_FsmParse = (struct __pyx_vtabstruct_21MorphologicalAnalysis_8FsmParse_FsmParse*)__Pyx_GetVtable(__pyx_ptype_21MorphologicalAnalysis_8FsmParse_FsmParse->tp_dict); if (unlikely(!__pyx_vtabptr_21MorphologicalAnalysis_8FsmParse_FsmParse)) __PYX_ERR(13, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("MorphologicalAnalysis.FiniteStateMachine"); if (unlikely(!__pyx_t_1)) __PYX_ERR(14, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_21MorphologicalAnalysis_18FiniteStateMachine_FiniteStateMachine = __Pyx_ImportType(__pyx_t_1, "MorphologicalAnalysis.FiniteStateMachine", "FiniteStateMachine", sizeof(struct __pyx_obj_21MorphologicalAnalysis_18FiniteStateMachine_FiniteStateMachine), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_21MorphologicalAnalysis_18FiniteStateMachine_FiniteStateMachine) __PYX_ERR(14, 4, __pyx_L1_error)
  __pyx_vtabptr_21MorphologicalAnalysis_18FiniteStateMachine_FiniteStateMachine = (struct __pyx_vtabstruct_21MorphologicalAnalysis_18FiniteStateMachine_FiniteStateMachine*)__Pyx_GetVtable(__pyx_ptype_21MorphologicalAnalysis_18FiniteStateMachine_FiniteStateMachine->tp_dict); if (unlikely(!__pyx_vtabptr_21MorphologicalAnalysis_18FiniteStateMachine_FiniteStateMachine)) __PYX_ERR(14, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("MorphologicalAnalysis.MetamorphicParse"); if (unlikely(!__pyx_t_1)) __PYX_ERR(15, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_21MorphologicalAnalysis_16MetamorphicParse_MetamorphicParse = __Pyx_ImportType(__pyx_t_1, "MorphologicalAnalysis.MetamorphicParse", "MetamorphicParse", sizeof(struct __pyx_obj_21MorphologicalAnalysis_16MetamorphicParse_MetamorphicParse), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_21MorphologicalAnalysis_16MetamorphicParse_MetamorphicParse) __PYX_ERR(15, 6, __pyx_L1_error)
  __pyx_vtabptr_21MorphologicalAnalysis_16MetamorphicParse_MetamorphicParse = (struct __pyx_vtabstruct_21MorphologicalAnalysis_16MetamorphicParse_MetamorphicParse*)__Pyx_GetVtable(__pyx_ptype_21MorphologicalAnalysis_16MetamorphicParse_MetamorphicParse->tp_dict); if (unlikely(!__pyx_vtabptr_21MorphologicalAnalysis_16MetamorphicParse_MetamorphicParse)) __PYX_ERR(15, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("MorphologicalAnalysis.FsmMorphologicalAnalyzer"); if (unlikely(!__pyx_t_1)) __PYX_ERR(16, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer = __Pyx_ImportType(__pyx_t_1, "MorphologicalAnalysis.FsmMorphologicalAnalyzer", "FsmMorphologicalAnalyzer", sizeof(struct __pyx_obj_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer) __PYX_ERR(16, 13, __pyx_L1_error)
  __pyx_vtabptr_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer = (struct __pyx_vtabstruct_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer*)__Pyx_GetVtable(__pyx_ptype_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer->tp_dict); if (unlikely(!__pyx_vtabptr_21MorphologicalAnalysis_24FsmMorphologicalAnalyzer_FsmMorphologicalAnalyzer)) __PYX_ERR(16, 13, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("DataStructure.CounterHashMap"); if (unlikely(!__pyx_t_1)) __PYX_ERR(17, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_13DataStructure_14CounterHashMap_CounterHashMap = __Pyx_ImportType(__pyx_t_1, "DataStructure.CounterHashMap", "CounterHashMap", sizeof(struct __pyx_obj_13DataStructure_14CounterHashMap_CounterHashMap), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_13DataStructure_14CounterHashMap_CounterHashMap) __PYX_ERR(17, 1, __pyx_L1_error)
  __pyx_vtabptr_13DataStructure_14CounterHashMap_CounterHashMap = (struct __pyx_vtabstruct_13DataStructure_14CounterHashMap_CounterHashMap*)__Pyx_GetVtable(__pyx_ptype_13DataStructure_14CounterHashMap_CounterHashMap->tp_dict); if (unlikely(!__pyx_vtabptr_13DataStructure_14CounterHashMap_CounterHashMap)) __PYX_ERR(17, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("MorphologicalAnalysis.FsmParseList"); if (unlikely(!__pyx_t_1)) __PYX_ERR(18, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_21MorphologicalAnalysis_12FsmParseList_FsmParseList = __Pyx_ImportType(__pyx_t_1, "MorphologicalAnalysis.FsmParseList", "FsmParseList", sizeof(struct __pyx_obj_21MorphologicalAnalysis_12FsmParseList_FsmParseList), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_21MorphologicalAnalysis_12FsmParseList_FsmParseList) __PYX_ERR(18, 4, __pyx_L1_error)
  __pyx_vtabptr_21MorphologicalAnalysis_12FsmParseList_FsmParseList = (struct __pyx_vtabstruct_21MorphologicalAnalysis_12FsmParseList_FsmParseList*)__Pyx_GetVtable(__pyx_ptype_21MorphologicalAnalysis_12FsmParseList_FsmParseList->tp_dict); if (unlikely(!__pyx_vtabptr_21MorphologicalAnalysis_12FsmParseList_FsmParseList)) __PYX_ERR(18, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initAutoDisambiguator(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initAutoDisambiguator(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_AutoDisambiguator(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_AutoDisambiguator(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_AutoDisambiguator(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'AutoDisambiguator' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_AutoDisambiguator(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("AutoDisambiguator", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_MorphologicalDisambiguation__AutoDisambiguator) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "MorphologicalDisambiguation.AutoDisambiguator")) {
      if (unlikely(PyDict_SetItemString(modules, "MorphologicalDisambiguation.AutoDisambiguator", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":2
 * from DataStructure.CounterHashMap cimport CounterHashMap
 * from MorphologicalAnalysis.FsmMorphologicalAnalyzer import MorphologicalTag             # <<<<<<<<<<<<<<
 * from MorphologicalAnalysis.FsmParseList cimport FsmParseList
 * 
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_MorphologicalTag);
  __Pyx_GIVEREF(__pyx_n_s_MorphologicalTag);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_MorphologicalTag);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_MorphologicalAnalysis_FsmMorphol, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_MorphologicalTag); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_MorphologicalTag, __pyx_t_1) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":8
 * 
 *     @staticmethod
 *     def isAnyWordSecondPerson(index: int, correctParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         cdef int count, i
 *         count = 0
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_1isAnyWordSecondPerson, NULL, __pyx_n_s_MorphologicalDisambiguation_Auto_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_isAnyWordSecondPerson, __pyx_t_2) < 0) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":7
 * cdef class AutoDisambiguator:
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def isAnyWordSecondPerson(index: int, correctParses: list) -> bool:
 *         cdef int count, i
 */
  __Pyx_GetNameInClass(__pyx_t_2, (PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator, __pyx_n_s_isAnyWordSecondPerson); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_staticmethod, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_isAnyWordSecondPerson, __pyx_t_1) < 0) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":18
 * 
 *     @staticmethod
 *     def isPossessivePlural(index: int, correctParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         cdef int i
 *         for i in range(index - 1, -1, -1):
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_3isPossessivePlural, NULL, __pyx_n_s_MorphologicalDisambiguation_Auto_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_isPossessivePlural, __pyx_t_1) < 0) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":17
 *         return count >= 1
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def isPossessivePlural(index: int, correctParses: list) -> bool:
 *         cdef int i
 */
  __Pyx_GetNameInClass(__pyx_t_1, (PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator, __pyx_n_s_isPossessivePlural); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_staticmethod, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_isPossessivePlural, __pyx_t_2) < 0) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":26
 * 
 *     @staticmethod
 *     def nextWordPos(nextParseList: FsmParseList) -> str:             # <<<<<<<<<<<<<<
 *         cdef CounterHashMap _map
 *         cdef int i
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_5nextWordPos, NULL, __pyx_n_s_MorphologicalDisambiguation_Auto_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_nextWordPos, __pyx_t_2) < 0) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":25
 *         return False
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def nextWordPos(nextParseList: FsmParseList) -> str:
 *         cdef CounterHashMap _map
 */
  __Pyx_GetNameInClass(__pyx_t_2, (PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator, __pyx_n_s_nextWordPos); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_staticmethod, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_nextWordPos, __pyx_t_1) < 0) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":35
 * 
 *     @staticmethod
 *     def isBeforeLastWord(index: int, fsmParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         return index + 2 == len(fsmParses)
 * 
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_7isBeforeLastWord, NULL, __pyx_n_s_MorphologicalDisambiguation_Auto_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_isBeforeLastWord, __pyx_t_1) < 0) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":34
 *         return _map.max()
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def isBeforeLastWord(index: int, fsmParses: list) -> bool:
 *         return index + 2 == len(fsmParses)
 */
  __Pyx_GetNameInClass(__pyx_t_1, (PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator, __pyx_n_s_isBeforeLastWord); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_staticmethod, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 34, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_isBeforeLastWord, __pyx_t_2) < 0) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":39
 * 
 *     @staticmethod
 *     def nextWordExists(index: int, fsmParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         return index + 1 < len(fsmParses)
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_9nextWordExists, NULL, __pyx_n_s_MorphologicalDisambiguation_Auto_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_nextWordExists, __pyx_t_2) < 0) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":38
 *         return index + 2 == len(fsmParses)
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def nextWordExists(index: int, fsmParses: list) -> bool:
 *         return index + 1 < len(fsmParses)
 */
  __Pyx_GetNameInClass(__pyx_t_2, (PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator, __pyx_n_s_nextWordExists); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_staticmethod, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_nextWordExists, __pyx_t_1) < 0) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":43
 * 
 *     @staticmethod
 *     def isNextWordNoun(index: int, fsmParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         return index + 1 < len(fsmParses) and AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN"
 * 
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_11isNextWordNoun, NULL, __pyx_n_s_MorphologicalDisambiguation_Auto_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_isNextWordNoun, __pyx_t_1) < 0) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":42
 *         return index + 1 < len(fsmParses)
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def isNextWordNoun(index: int, fsmParses: list) -> bool:
 *         return index + 1 < len(fsmParses) and AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN"
 */
  __Pyx_GetNameInClass(__pyx_t_1, (PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator, __pyx_n_s_isNextWordNoun); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_staticmethod, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_isNextWordNoun, __pyx_t_2) < 0) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":47
 * 
 *     @staticmethod
 *     def isNextWordNum(index: int, fsmParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         return index + 1 < len(fsmParses) and AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NUM"
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_13isNextWordNum, NULL, __pyx_n_s_MorphologicalDisambiguation_Auto_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_isNextWordNum, __pyx_t_2) < 0) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":46
 *         return index + 1 < len(fsmParses) and AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN"
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def isNextWordNum(index: int, fsmParses: list) -> bool:
 *         return index + 1 < len(fsmParses) and AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NUM"
 */
  __Pyx_GetNameInClass(__pyx_t_2, (PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator, __pyx_n_s_isNextWordNum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_staticmethod, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 46, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_isNextWordNum, __pyx_t_1) < 0) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":51
 * 
 *     @staticmethod
 *     def isNextWordNounOrAdjective(index: int, fsmParses: list) -> bool:             # <<<<<<<<<<<<<<
 *         return index + 1 < len(fsmParses) and (AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN" or
 *                                                AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "ADJ" or
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_15isNextWordNounOrAdjective, NULL, __pyx_n_s_MorphologicalDisambiguation_Auto_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_isNextWordNounOrAdjective, __pyx_t_1) < 0) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":50
 *         return index + 1 < len(fsmParses) and AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NUM"
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def isNextWordNounOrAdjective(index: int, fsmParses: list) -> bool:
 *         return index + 1 < len(fsmParses) and (AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "NOUN" or
 */
  __Pyx_GetNameInClass(__pyx_t_1, (PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator, __pyx_n_s_isNextWordNounOrAdjective); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_staticmethod, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_isNextWordNounOrAdjective, __pyx_t_2) < 0) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":57
 * 
 *     @staticmethod
 *     def isFirstWord(index: int) -> bool:             # <<<<<<<<<<<<<<
 *         return index == 0
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_17isFirstWord, NULL, __pyx_n_s_MorphologicalDisambiguation_Auto_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_isFirstWord, __pyx_t_2) < 0) __PYX_ERR(0, 57, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":56
 *                                                AutoDisambiguator.nextWordPos(fsmParses[index + 1]) == "DET")
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def isFirstWord(index: int) -> bool:
 *         return index == 0
 */
  __Pyx_GetNameInClass(__pyx_t_2, (PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator, __pyx_n_s_isFirstWord); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_staticmethod, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_isFirstWord, __pyx_t_1) < 0) __PYX_ERR(0, 57, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":61
 * 
 *     @staticmethod
 *     def containsTwoNeOrYa(fsmParses: list, word: str) -> bool:             # <<<<<<<<<<<<<<
 *         cdef int count
 *         cdef FsmParseList fsmPars
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_19containsTwoNeOrYa, NULL, __pyx_n_s_MorphologicalDisambiguation_Auto_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_containsTwoNeOrYa, __pyx_t_1) < 0) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":60
 *         return index == 0
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def containsTwoNeOrYa(fsmParses: list, word: str) -> bool:
 *         cdef int count
 */
  __Pyx_GetNameInClass(__pyx_t_1, (PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator, __pyx_n_s_containsTwoNeOrYa); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_staticmethod, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_containsTwoNeOrYa, __pyx_t_2) < 0) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":73
 * 
 *     @staticmethod
 *     def hasPreviousWordTag(index: int, correctParses: list, tag: MorphologicalTag) -> bool:             # <<<<<<<<<<<<<<
 *         return index > 0 and correctParses[index - 1].containsTag(tag)
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_21hasPreviousWordTag, NULL, __pyx_n_s_MorphologicalDisambiguation_Auto_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_hasPreviousWordTag, __pyx_t_2) < 0) __PYX_ERR(0, 73, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":72
 *         return count == 2
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def hasPreviousWordTag(index: int, correctParses: list, tag: MorphologicalTag) -> bool:
 *         return index > 0 and correctParses[index - 1].containsTag(tag)
 */
  __Pyx_GetNameInClass(__pyx_t_2, (PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator, __pyx_n_s_hasPreviousWordTag); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_staticmethod, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 72, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_hasPreviousWordTag, __pyx_t_1) < 0) __PYX_ERR(0, 73, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":77
 * 
 *     @staticmethod
 *     def selectCaseForParseString(parseString: str, index: int, fsmParses: list, correctParses: list) -> str:             # <<<<<<<<<<<<<<
 *         cdef str surfaceForm, root, lastWord
 *         surfaceForm = fsmParses[index].getFsmParse(0).getSurfaceForm()
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_23selectCaseForParseString, NULL, __pyx_n_s_MorphologicalDisambiguation_Auto_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_selectCaseForParseString, __pyx_t_1) < 0) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":76
 *         return index > 0 and correctParses[index - 1].containsTag(tag)
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def selectCaseForParseString(parseString: str, index: int, fsmParses: list, correctParses: list) -> str:
 *         cdef str surfaceForm, root, lastWord
 */
  __Pyx_GetNameInClass(__pyx_t_1, (PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator, __pyx_n_s_selectCaseForParseString); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_staticmethod, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_selectCaseForParseString, __pyx_t_2) < 0) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":793
 * 
 *     @staticmethod
 *     def caseDisambiguator(index: int, fsmParses: list, correctParses: list) -> FsmParse:             # <<<<<<<<<<<<<<
 *         cdef FsmParseList fsmParseList
 *         cdef str defaultCase
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_17AutoDisambiguator_25caseDisambiguator, NULL, __pyx_n_s_MorphologicalDisambiguation_Auto_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 793, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_caseDisambiguator, __pyx_t_2) < 0) __PYX_ERR(0, 793, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":792
 *         return None
 * 
 *     @staticmethod             # <<<<<<<<<<<<<<
 *     def caseDisambiguator(index: int, fsmParses: list, correctParses: list) -> FsmParse:
 *         cdef FsmParseList fsmParseList
 */
  __Pyx_GetNameInClass(__pyx_t_2, (PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator, __pyx_n_s_caseDisambiguator); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 793, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_staticmethod, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 792, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem((PyObject *)__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator->tp_dict, __pyx_n_s_caseDisambiguator, __pyx_t_1) < 0) __PYX_ERR(0, 793, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  PyType_Modified(__pyx_ptype_27MorphologicalDisambiguation_17AutoDisambiguator_AutoDisambiguator);

  /* "(tree fragment)":1
 * def __pyx_unpickle_AutoDisambiguator(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_27MorphologicalDisambiguation_17AutoDisambiguator_1__pyx_unpickle_AutoDisambiguator, NULL, __pyx_n_s_MorphologicalDisambiguation_Auto_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_AutoDisambiguator, __pyx_t_1) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "MorphologicalDisambiguation/AutoDisambiguator.pyx":1
 * from DataStructure.CounterHashMap cimport CounterHashMap             # <<<<<<<<<<<<<<
 * from MorphologicalAnalysis.FsmMorphologicalAnalyzer import MorphologicalTag
 * from MorphologicalAnalysis.FsmParseList cimport FsmParseList
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init MorphologicalDisambiguation.AutoDisambiguator", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init MorphologicalDisambiguation.AutoDisambiguator");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_SubtractObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a - b);
            if (likely((x^a) >= 0 || (x^~b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
            }
        }
                x = a - b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla - llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("subtract", return NULL)
            result = ((double)a) - (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceSubtract : PyNumber_Subtract)(op1, op2);
}
#endif

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (__Pyx_PyFastCFunction_Check(func)) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* PyIntCompare */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED long inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = Py_SIZE(op1);
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        if (intval == 0) {
            if (size == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
        } else if (intval < 0) {
            if (size >= 0)
                Py_RETURN_FALSE;
            intval = -intval;
            size = -size;
        } else {
            if (size <= 0)
                Py_RETURN_FALSE;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        if (unequal == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
        if ((double)a == (double)b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    return (
        PyObject_RichCompare(op1, op2, Py_EQ));
}

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* BytesEquals */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS && (PY_VERSION_HEX < 0x030B0000)
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* PyObjectGetAttrStrNoError */
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_getstate = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
    PyObject *getstate = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    getstate = _PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate);
#else
    getstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_getstate);
    if (!getstate && PyErr_Occurred()) {
        goto __PYX_BAD;
    }
#endif
    if (getstate) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_getstate = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_getstate);
#else
        object_getstate = __Pyx_PyObject_GetAttrStrNoError((PyObject*)&PyBaseObject_Type, __pyx_n_s_getstate);
        if (!object_getstate && PyErr_Occurred()) {
            goto __PYX_BAD;
        }
#endif
        if (object_getstate != getstate) {
            goto __PYX_GOOD;
        }
    }
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
    Py_XDECREF(object_getstate);
    Py_XDECREF(getstate);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* GetVTable */
static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* GetNameInClass */
static PyObject *__Pyx_GetGlobalNameAfterAttributeLookup(PyObject *name) {
    PyObject *result;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    __Pyx_GetModuleGlobalNameUncached(result, name);
    return result;
}
static PyObject *__Pyx__GetNameInClass(PyObject *nmspace, PyObject *name) {
    PyObject *result;
    result = __Pyx_PyObject_GetAttrStr(nmspace, name);
    if (!result) {
        result = __Pyx_GetGlobalNameAfterAttributeLookup(name);
    }
    return result;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);  // XDECREF since it's only set on Py3 if cline
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[5];
    int same=1, i, found_dot;
    const char* rt_from_call = Py_GetVersion();
    PyOS_snprintf(ctversion, 5, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    found_dot = 0;
    for (i = 0; i < 4; i++) {
        if (!ctversion[i]) {
            same = (rt_from_call[i] < '0' || rt_from_call[i] > '9');
            break;
        }
        if (rt_from_call[i] != ctversion[i]) {
            same = 0;
            break;
        }
    }
    if (!same) {
        char rtversion[5] = {'\0'};
        char message[200];
        for (i=0; i<4; ++i) {
            if (rt_from_call[i] == '.') {
                if (found_dot) break;
                found_dot = 1;
            } else if (rt_from_call[i] < '0' || rt_from_call[i] > '9') {
                break;
            }
            rtversion[i] = rt_from_call[i];
        }
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
